/* 
 * This file is part of XCmail and is subject to the 
 * XCmail license agreement
 */

/* $Id: binhex_encoding.cxx,v 2.2 1999/01/07 16:08:44 beckerr Exp $ */


//
// binhex encoding fuer xc-mail
// (C)1997 Ralf Becker (Ra.Becker@fh-trier.de)
//
// erzeugt: 13.04.97 (r.b.)
//
#include "../configure.h" 

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include "../include/encoder.h"

//--------------------------------------------------------------------

class encoder_binhex : public encoder {

	public:
		encoder_binhex(void) : encoder("binhex;x-binhex", "Ralf Becker", EF_DECODE) {};
		~encoder_binhex() {};
		virtual char *data2mail(char *,int size,int &newsize,char *path,char *filename) {return NULL;};
		virtual char *mail2data(char *,int size,int &newsize,char *path,char *filename);
		virtual void dispose(char *b) { free(b);}
};

static encoder_binhex binhex;

//--------------------------------------------------------------------

#define CRCCONSTANT	0x1021
#define RUNCHAR		0x90

#define BYTEMASK	0xff
#define BYTEBIT		0x100
#define WORDMASK	0xffff
#define WORDBIT		0x10000

#ifndef BUFSIZ
#define BUFSIZ		400
#endif

//--------------------------------------------------------------------


static unsigned char trlookup[256] = {
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 
	0xFF, 0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06,
	0x07, 0x08, 0x09, 0x0A, 0x0B, 0x0C, 0xFF, 0xFF,
	0x0D, 0x0E, 0x0F, 0x10, 0x11, 0x12, 0x13, 0xFF,
	0x14, 0x15, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0x16, 0x17, 0x18, 0x19, 0x1A, 0x1B, 0x1C, 0x1D,
	0x1E, 0x1F, 0x20, 0x21, 0x22, 0x23, 0x24, 0xFF,
	0x25, 0x26, 0x27, 0x28, 0x29, 0x2A, 0x2B, 0xFF,
	0x2C, 0x2D, 0x2E, 0x2F, 0xFF, 0xFF, 0xFF, 0xFF,
	0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0xFF,
	0x37, 0x38, 0x39, 0x3A, 0x3B, 0x3C, 0xFF, 0xFF,
	0x3D, 0x3E, 0x3F, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
};
static int eof;
static int rep, lastc;
static char obuf[3];
static char *op, *oend;
static unsigned int crc;

//--------------------------------------------------------------------

static void comp_q_crc(unsigned int c);

static int get6bits(char **inP) {
	int c;
	while (c = (unsigned char)*((*inP)++), c && c != ':') {
		if (c == '\n' || c == '\r') continue;
		if (trlookup[c] == 0xff) {
			fprintf(stderr, "binhex: bad char: '%c'\n", c);
			return EOF;
		}
		return trlookup[c];
	}
	return EOF;
}

static int getq_raw(char **inP) {
	int c;

	if (op == obuf) {
		char ibuf[4], *ebuf = ibuf + 4;
		for (char *iP = ibuf; iP < ebuf; iP++) {
			if (c = get6bits(inP), c == EOF) {
				if (iP <= ibuf + 1)
					return EOF;
				else
				if (iP == ibuf + 2)
					eof = 1;
				else
					eof = 2;
			}
			*iP = c;
		}
		obuf[0] = (ibuf[0] << 2 | ibuf[1] >> 4);
		obuf[1] = (ibuf[1] << 4 | ibuf[2] >> 2);
		obuf[2] = (ibuf[2] << 6 | ibuf[3]);
	}
	// Im original Code steht:
	//    if ((eof) & (op >= &obuf[eof]))
	// ist das wirklich so gemeint?
	// eof kann 0, 1 oder 2 sein...der Ausdruck wuerde false sein bei
	// eof=2...koennte ein Fehler sein
	//
	if (eof && op >= obuf + eof)
		return EOF;
	c = *(op++);
	if (op >= oend)
		op = obuf;
	return c & BYTEMASK;
}

static int getq_nocrc(char **inP) {
	int c;
	if (rep) {
		rep--;
		return lastc;
	}
	if (c = getq_raw(inP), c == EOF)
		return EOF;
	if (c == RUNCHAR) {
		if (rep = getq_raw(inP), rep == EOF)
			return EOF;
		if (rep != 0) {
			rep -= 2;
			return lastc;
		}
		lastc = RUNCHAR;
		return RUNCHAR;
	}
	lastc = c;
	return c;
}


static int getq(char **inP) {
	int c = getq_nocrc(inP);
	if (c != EOF) comp_q_crc((unsigned int)c);
	return c;
}

static short get2q(char **inP) {
	int c;
	short value;
	c = getq(inP); value  = (c & BYTEMASK) << 8;
	c = getq(inP); value |= (c & BYTEMASK);
	return value;
}

static long get4q(char **inP) {
	long value = 0L;
	for (int i = 0; i < 4; i++) {
		int c = getq(inP);
		value <<= 8;
		value |= (c & BYTEMASK);
	}
	return value;
}

static int getqstring(char *buf, int n, char **inP) {
	for (int i = 0; i < n; i++) {
		int c = getq(inP);
		if (c == EOF) return EOF;
		*buf++ = c;
	}
	*buf = 0;
	return n;
}

static void q_init(void) {
	lastc = 0;
	rep = 0;
	crc = 0;
	eof = 0;
	op = obuf;
	oend = obuf + sizeof(obuf);
}

static void comp_q_crc(unsigned int c) {
	unsigned long temp = crc;
	for (int i = 0; i < 8; i++) {
		c <<= 1;
		if ((temp <<= 1) & WORDBIT)
			temp = (temp & WORDMASK) ^ CRCCONSTANT;
		temp ^= (c >> 8);
		c &= BYTEMASK;
	}
	crc = temp;
}

static int verify_crc(char **inP) {
	comp_q_crc(0);
	comp_q_crc(0);
	unsigned int calc_crc = crc;
	unsigned int file_crc = get2q(inP);
	crc = 0;
	return (calc_crc & WORDMASK) == (file_crc & WORDMASK);
}

// binhex decode
//
char *encoder_binhex::mail2data(char *inbuf,int size,int &newsize,char *path,char *filename) {

	char	*outbuf, *cP;
	char 	ibuf[BUFSIZ];
	char	m_name[256];
	char 	m_type[5];
	char 	m_author[5];
	short 	m_flags;
	long 	m_datalen;
	long 	m_rsrclen;
	int 	c, n;

	q_init();
		
	newsize = 0;
	outbuf = cP = (char *)malloc(size);
	if (!outbuf) {
		fprintf(stderr, "binhex: out of memory!\n");
		return NULL;
	}
	
	// "(This file ...)" Zeile suchen
	//
	while (egets(ibuf, BUFSIZ, &inbuf) != NULL)
		if (strncmp(ibuf, "(This file", 10) == 0)
			break;

	// Anfang suchen
	//
	while (c = *(inbuf++), c)
		if ((c == '\n' || c == '\r') && *inbuf == ':')
			break;
	
	// Fehler
	//
	if (!c) {
		fprintf(stderr, "binhex: unexpected EOF\n");
		free(outbuf);
		return NULL;
	}
	
	// Headerinformationen auslesen:
	// - Dateiname
	// - Dateityp
	// - Autor
	// - Flags
	// - Laenge des Datenbereichs
	// - Laenge des Resourcebereichs
	//
	getqstring(m_name,  getq(&inbuf), &inbuf);	// Name
	getqstring(m_type, 4, &inbuf);				// Typ
	getqstring(m_author, 4, &inbuf);			// Autor
	m_flags   = get2q(&inbuf);					// Flags
	m_datalen = get4q(&inbuf);					// Datenlaenge
	m_rsrclen = get4q(&inbuf);					// Resourcelaenge

	// CRC pruefen
	//
	if (!verify_crc(&inbuf)) {
		fprintf(stderr, "binhex: CRC error\n");
		free(outbuf);
		return NULL;
	} 

	// Data-Fork ist leer -> nichts zu tun!
	//
	if (!m_datalen) {
		fprintf(stderr, "binhex: empty data fork, nothing to do!\n");
		free(outbuf);
		return NULL;
	}

	//
	// Decode Data-Fork (Resource-Fork wird ignoriert!)
	//
	for (int i = 0; i < m_datalen; i++) {
		if (c = getq(&inbuf), c == EOF) {
			fprintf(stderr, "binhex: unexpected EOF\n");
			free(outbuf);
			return NULL;
		}
		*(cP++) = c;
		newsize++;
	}
			
	//
	// CRC pruefen
	//
	if (!verify_crc(&inbuf)) {
		fprintf(stderr, "binhex: CRC error\n");
		free(outbuf);
		return NULL;
	} 

	// Dateiname pruefen
	// keine Endung -> Datei Typ wird zur Endung
	//
	if (!strchr(m_name, '.')) {
		char *s = strchr(m_name, 0);
		*(s++) = '.';
		for (int i = 0; i < 4; i++)
			if (isalnum(m_type[i]))
				*(s++) = m_type[i];
		s[(s[-1] == '.')? -1 : 0] = 0;
	}
	
	
	*cP = 0;
	return (cP = (char *)realloc(outbuf, newsize), cP) ? cP : outbuf;
}
