/* 
 * This file is part of XCmail and is subject to the 
 * XCmail license agreement
 */

/* $Id: check_new_release.cxx,v 2.22 2002/04/25 09:11:30 schmitzj Exp $ */


#include "../configure.h"

//
// Format:
// XCmail-<type>-Version: VERSION
// XCmail-<type>-URL: (Bin-)URL
// XCmail-<type>-List-URL: (Bin-)URL (Bin-)URL ... (Spaces getrennt)
//
//  <type>: Linux, SunOS, FreeBSD, ....
//          develop
//
// XCmail-OtherServer: URL
// XCmail-News-Number: INT
// XCmail-News-Text: TEXT
//

#include "../../src/mailer.h"
#include "check_new_release.h"
#include <fcntl.h>

// #define TESTHOST

#ifndef TESTHOST
#define HOSTNAME		"www.JS-Home.org"
#define HOSTPORT		"80"
#define HOSTFILE		"/LATEST"
#define HOSTNAME2		"www.trier.linux.de"
#define HOSTPORT2		"80"
#define HOSTFILE2		"/~schmitzj/LATEST"
#else
#define HOSTNAME		"localhost"
#define HOSTPORT		"80"
#define HOSTFILE		"/LATEST"
#define HOSTNAME2		"localhost"
#define HOSTPORT2		"80"
#define HOSTFILE2		"/LATEST"
#endif

#ifdef XC_UNAME
#undef OSNAME
#define OSNAME	XC_UNAME
#else
#undef OSNAME
#define OSNAME	XC_UNAME
#endif

#include <strings.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>

check_new_release::check_new_release(void) : mainplugin("check new release","Jrgen Schmitz",PLF_CONFIGURABLE,"Automatically checks for new XCmail releases and allows to transfer (get) them.")
{
	sock=0;
	cnbuf=NULL;
	cnbufsize=0;
	cnbufused=0;
	writefile=NULL;
	mywin=NULL;
	latest_news=0;
	askdevelopment=1;
	lastdevelopmentversion[0]=0;
	strncat(lastdevelopmentversion,programversion(),sizeof(lastdevelopmentversion));
	LATESTurl[0]=0;
	cfgsetup();
}
check_new_release::~check_new_release()
{
	delete cfg;
	if (cnbuf)
	{
		free(cnbuf);
		cnbuf=NULL;
		cnbufsize=0;
		cnbufused=0;
	}
}

void check_new_release::init(void)
{
	if (!enable) return;
	switch (mode)
	{
		case 0:	// start
			check();
			break;
		case 1: // menu
			addmenu(MPMENU_GENERAL,1,"check for new release");
			break;
	}
}
void check_new_release::finish(void)
{
	if (mywin)
	{
		mywin->Close();
		DeleteObjectTree(mywin->FullName());
		mywin=NULL;
	}
	if (writefile)
	{
		fclose(writefile);
		writefile=NULL;
		remove(filename);
	}
	if (sock)
	{
		gadget::RemoveFD(sock);
		close(sock);
		sock=0;
	}
	if (!enable) return;
}
void check_new_release::handle(int id)
{
	switch (id)
	{
		case 1:
			check();
			break;
	}
}

group *check_new_release::makeconfig(void)
{
	group *gp=new group;
	gp->Name("check_new_release_group");
	gp->Vert();
	 checkbox *cb=new checkbox;
	 cb->Name("enable");
	 cb->Text("enable check");
	 cb->State(enable?PRESSED:RELEASED);
	gp->Add(cb,"");
	 static char *itemstxt[]={"at start","by menu",NULL};
	 choice *ch=new choice;
	 ch->Name("mode");
	 ch->Items(itemstxt);
	 ch->Selected(mode);
	 ch->HelpText("Change will work after restart!");
	gp->Add(ch,"check for new release");
	 cb=new checkbox;
	 cb->Name("askdevelopment");
	 cb->Text("also check for new development releases");
	 cb->State(askdevelopment?PRESSED:RELEASED);
	 cb->HelpText("This will also check for new development\n(beta, etc) releases.");
	gp->Add(cb,"");
	 cb=new checkbox;
	 cb->Name("ask");
	 cb->Text("ask before start transfer new file");
	 cb->State(ask?PRESSED:RELEASED);
	 cb->HelpText("If disabled, the new release will be\ntransfered at once.");
	gp->Add(cb,"");
	 cb=new checkbox;
	 cb->Name("askplace");
	 cb->Text("ask for directory to save new release into");
	 cb->State(askplace?PRESSED:RELEASED);
	 cb->HelpText("If disabled, the new release will be\nplaces in the default directory set below.");
	gp->Add(cb,"");
	 fileinput *fip=new fileinput;
	 fip->Name("defaultdir");
	 fip->ModeDir();
	 fip->SetFile(defaultdir);
	gp->Add(fip,"Default directory");
	return gp;
}
void check_new_release::readconfig(group *gp)
{
	enable=((checkbox*)FindObject(gp->FullName(),".enable"))->State()==PRESSED?1:0;
	mode=((choice*)FindObject(gp->FullName(),".mode"))->Selected();
	ask=((checkbox*)FindObject(gp->FullName(),".ask"))->State()==PRESSED?1:0;
	askplace=((checkbox*)FindObject(gp->FullName(),".askplace"))->State()==PRESSED?1:0;
	askdevelopment=((checkbox*)FindObject(gp->FullName(),".askdevelopment"))->State()==PRESSED?1:0;
	((fileinput*)FindObject(gp->FullName(),".defaultdir"))->GetFile(defaultdir,sizeof(defaultdir));
}
void check_new_release::readconfigfile(void)
{
	cfg->Read();
}
void check_new_release::writeconfigfile(void)
{
	cfg->Write();
}

void check_new_release::cfgsetup(void) {

	static configopts cfgitems[] = {
		{"B:enable",(char*)&enable},
		{"Estart,menu:mode",(char*)&mode},
		{"B:ask",(char*)&ask},
		{"B:askplace",(char*)&askplace},
		{"B:askdevelopment",(char*)&askdevelopment},
		{"S200:defaultdir",defaultdir},
		{"S50:lastdevelopmentversion",lastdevelopmentversion},
		{"I:latest_news",(char*)&latest_news},
		{"T200:LATESTurl",LATESTurl},
		{NULL,NULL}
	};
	cfg = new config("~/.XCmail/plugin-preferences-check_new_release");
	cfg->SetItems(cfgitems);

	mode=1;
	enable=1;
	ask=1;
	askplace=1;
	defaultdir[0]='~';
	defaultdir[1]=0;
	latest_news=0;
	askdevelopment=1;
	lastdevelopmentversion[0]=0;
	strncat(lastdevelopmentversion,programversion(),sizeof(lastdevelopmentversion));
	LATESTurl[0]=0;
}
void check_new_release::error(char *txt)
{
	stophandler(NULL);
	easy_dialog ed;
	ed.Text(txt);
	ed.Options("Ok");
	ed.Start(readerwin);
}

void check_new_release::check(void)
{
	if (sock)
	{
		easy_dialog ed;
		ed.Text("Already checking!");
		ed.Options("continue|abort");
		if (ed.Start(readerwin)==2)
		{
			stophandler(NULL);
			return;
		}
		return;
	}
	getcount=0;
	getbycount();
}
int check_new_release::getbycount(void)
{
	int r=0;
	switch (getcount)
	{
		case 0:
			getcount=1;
			runmode=0;
			if (!get("http://"HOSTNAME":"HOSTPORT""HOSTFILE))
			break;
		case 1:
			getcount=2;
			runmode=0;
			if (!get("http://"HOSTNAME2":"HOSTPORT2""HOSTFILE2))
			break;
		case 2:
			getcount=3;
			runmode=0;
			if (LATESTurl[0])
				if (get(LATESTurl))
					r=1;
			break;
	}
	return r;
}
int check_new_release::get(char *url)
{ // 0: ok
  // 1: can't connect
  // 2: unknown host
	if (cnbuf)
	{
		free(cnbuf);
	}
	cnbuf=(char*)malloc(1000);
	cnbufsize=1000;
	cnbufused=0;

	int port=80;
	char *c=url;
	while (*c && *c!='/') c++;
	while (*c && *c=='/') c++;

	char *hostname_c=c;
	while (*c && *c!='/') c++;

	char *uri=c;
	char hostname[c-hostname_c+1];
	hostname[0]=0;
	strncat(hostname,hostname_c,c-hostname_c);

	c=strchr(hostname,':');
	if (c)
	{
		*c=0;
		port=atoi(c+1);
	}

	struct hostent	*host;
	struct sockaddr_in	sain;

	if ((host=gethostbyname(hostname))==0)
	{
		error("Can't resolve hostname!");
		return 2;
    }
	if ((sock=socket(AF_INET,SOCK_STREAM,0))<0)
	{
		error("Error: can't get socket.");
		return 1;
    }
    bzero(&sain,sizeof(sain));
    sain.sin_family=AF_INET;
    sain.sin_port  =htons(port);

	bcopy(host->h_addr,&sain.sin_addr,host->h_length);

	if (connect(sock,(struct sockaddr*)&sain,sizeof(sain))<0)
	{
		error("Can't connect to host!");
		return 1;
    }
	fcntl(sock,F_SETFL,O_NONBLOCK);

	char buf[200];
	sprintf(buf,"GET %s HTTP/1.0\r\nUser-Agent: XCmail %s check_new_release "OSNAME"\r\n\r\n",uri,programversion());
	write(sock,buf,strlen(buf));
	asciimode=1;
	gadget::AddReadFD(sock,this,(FDHANDLER)&check_new_release::sockhandler);

	return 0;
}
void check_new_release::sockhandler(int s)
{
	if (!sock) return;
	char b;
	int n=recv(sock,&b,1,0);
	if (asciimode && n>0)
	{
		if (cnbufused+1>=cnbufsize)
		{
			cnbufsize+=1000;
			cnbuf=(char*)realloc(cnbuf,cnbufsize);
		}
		if (b!='\r')
			cnbuf[cnbufused++]=b;
 	}

	switch (runmode)
	{
		case 0: // warte auf Antwort LATEST file
		case 3: // warte auf Antwort von XCMAIL
			if (b=='\n')
			{ // erste Zeile
				cnbuf[cnbufused]=0;

				char *c=cnbuf;
				while (*c && *c!=' ') c++;
				while (*c && *c==' ') c++;

				int res=atoi(c);

				if (res==200)
				{ // ok
					runmode++;
				}
				else if (res>=400)
				{ // error
					if (runmode==0)
						runmode=410;
					else
						runmode=400;
				}
				else if (res>=301 && res<=302)
				{ // Location:
					runmode+=300;
				}
			}
			break;
		case 1: // warte auf LATEST file
			if (cnbuf[cnbufused-1]=='\n' && cnbuf[cnbufused-2]=='\n')
			{ // leere Zeile
				cnbuf[cnbufused]=0;
				runmode=2;
				cnbufused=0;
			}
			break;
		case 2: // lese LATEST file
			if (n==0)
			{
				excepthandler(s);
			}
			break;

		case 4: // warte auf XCMAIL
			if (cnbuf[cnbufused-1]=='\n' && cnbuf[cnbufused-2]=='\n')
			{ // leere Zeile
				cnbuf[cnbufused]=0;

				mail *m=new mail(cnbuf);
				m->addreferer();
				cnbuf=NULL;
				cnbufsize=0;
				cnbufused=0;

				readsize=0;
				char *len=m->getTag("Content-Length:");
				if (len && *len)
				{
					writesize=atoi(len);
					((indicator*)FindObject(".check_new_release_window.transfer"))->MaxValue(writesize/1024);
					char buf[30];
					sprintf(buf,"size: %d bytes",writesize);
					((output*)FindObject(".check_new_release_window.size"))->Output(buf);
				}
				else
				{
					writesize=0;
					((output*)FindObject(".check_new_release_window.size"))->Output("size unknown");
				}

				m->remreferer();
				runmode=5;
				asciimode=0;
			}
			break;
		case 5: // lese XCMAIL
			if (n==0)
			{
				excepthandler(s);
			}
			else
			{
				recbuf[recpos++]=b;
				if (recpos>1000)
				{
					fwrite(recbuf,1,recpos,writefile);
					readsize+=recpos-1;
					if (!writesize && readsize/1024>10) readsize=-10*1024;

					char buf[30];
					sprintf(buf,"got %d kbytes...",readsize/1024);
					((indicator*)FindObject(".check_new_release_window.transfer"))->Value(abs(readsize)/1024,buf);
					recpos=0;
				}
			}
			break;

		case 300: // redirect LATEST
		case 303: // redirect XCMAIL
			if (cnbuf[cnbufused-1]=='\n' && cnbuf[cnbufused-2]=='\n')
			{ // leere Zeile
				cnbuf[cnbufused]=0;

				mail *m=new mail(cnbuf);
				cnbuf=NULL;
				cnbufsize=0;
				cnbufused=0;

				gadget::RemoveFD(sock);
				close(sock);
				sock=0;

				char *u=m->getTag("Location:");
				if (u && *u)
				{
					runmode=runmode-300;
					get(u);
				}
				else
				{
					error("Unknown URL!");
				}
				delete m;
			}
			break;
		case 400: // lese bis Ende
		case 410:
			if (cnbuf[cnbufused-1]=='\n' && cnbuf[cnbufused-2]=='\n')
			{ // leere Zeile
				cnbuf[cnbufused]=0;

				mail *m=new mail(cnbuf);
				m->addreferer();
				cnbuf=NULL;
				cnbufsize=0;
				cnbufused=0;
				m->remreferer();

				runmode++;
				asciimode=0;
				if (writefile)
				{
					fclose(writefile);
					writefile=NULL;
				}
			}
			break;
		case 401:
			if (n==0)
			{
				gadget::RemoveFD(sock);
				close(sock);
				sock=0;
				error("HTTP transfer error:\nFile not found.");
			}
			break;
		case 411: // LATEST not found -> check next update server
			if (n==0)
			{
				gadget::RemoveFD(sock);
				close(sock);
				sock=0;
				if (getbycount())
					error("HTTP transfer error:\nFile not found.");
			}
			break;
	}
}
void check_new_release::excepthandler(int s)
{
	switch (runmode)
	{
		case 2: // LATEST da
			{
				gadget::RemoveFD(sock);
				close(sock);
				sock=0;
				cnbuf[cnbufused]=0;

				mail *m=new mail(cnbuf);
				m->addreferer();
				cnbuf=NULL;
				cnbufsize=0;
				cnbufused=0;

				char *redir=m->getTag("XCmail-Redirect:");
				if (redir && *redir)
				{
					runmode=0;
					get(redir);
					m->remreferer();
					break;
				}

				char *newLATESTurl=m->getTag("XCmail-OtherServer:");
				if (newLATESTurl)
				{
					strcpy(LATESTurl,newLATESTurl);
				}

				char *lnews=m->getTag("XCmail-News-Number:");
				if (lnews && *lnews && atoi(lnews)>latest_news)
				{
					lnews=strdup(lnews);
					char *ntext=m->getTag("XCmail-News-Text:");
					if (ntext && *ntext)
					{
						latest_news=atoi(lnews);
						writeconfigfile();
						Alert(&readerwin,"%s|Ok",ntext);
					}
					free(lnews);
				}

				char *dlme=NULL;
				char *ver=m->getTag("XCmail-"OSNAME"-Version:");
				if (ver && *ver && strcmp(ver,programversion())>0)
				{ // neue Version
					if (!ask || Alert(&readerwin,"There is a new version of XCmail\navailable: %s|Get it|Ignore",ver)==1)
					{
						dlme=OSNAME;
					}
				}
				if (!dlme && askdevelopment)
				{
					char *ver=m->getTag("XCmail-develop-Version:");
					if (ver && *ver && strcmp(ver,lastdevelopmentversion)>0)
					{ // neue Version
						if (!ask || Alert(&readerwin,"There is a new development release of XCmail\navailable: %s|Get it|Ignore",ver)==1)
						{
							dlme="develop";
						}
						strcpy(lastdevelopmentversion,ver);
					}
				}

				if (dlme)
				{
					runmode=3;
					char *u=askBinary(m,dlme);
					char *c1=u,*c2=u;
					file_dialog fd;
					fd.SaveMode();
					fd.SetDir(defaultdir);
					while (*c1)
					{
						if (*c1=='/') c2=c1+1;
						c1++;
					}
					fd.SetFile(c2);
					if (!askplace || fd.Start(readerwin))
					{
						char buf[200];
						fd.GetDir(filename,sizeof(filename));
						fd.GetFile(buf,sizeof(buf));
						strcat(filename,buf);
						if (writefile=fopen(filename,"w"))
						{
							setvbuf(writefile,NULL,_IOFBF,5000);
							recpos=0;
							get(u);

							mywin=new appwindow;
							spacegroup *gp=new spacegroup;
							indicator *id=new indicator;
							output *op=new output;
							fbutton *fb=new fbutton;
							int x,y,w,h;
							readerwin.Position(x,y);
							readerwin.Size(w,h);
							mywin->Name("check_new_release_window");
							mywin->InitSize(50);
							mywin->Init();
							mywin->Adjust(x,y,w,h,ADJUST_LEFT|ADJUST_TOP);
							mywin->Title("getting new XCmail");
							mywin->IconTitle("getting new XCmail");
							mywin->TransientFor(readerwin);
							gp->Name(NULL);
							gp->Vert();
							gp->YDistance(4);
							gp->Border(4);
							 op->Output(filename);
							gp->Add(op);
							 op=new output;
							 op->Name("size");
							gp->Add(op);
							 group *gp2=new group;
							 gp2->Name(NULL);
							 gp2->Horiz();
							 gp2->UseSmallest();
							 gp2->Center();
							 gp2->XDistance(20);
							  id->Name("transfer");
							  id->MaxValue(10);
							  id->Value(0,"getting file...");
							 gp2->Add(id);
							  fb->Text("stop");
							  fb->Callback(this,(CALLBACK)&check_new_release::stophandler);
							  fb->Weight(50);
							 gp2->Add(fb);
							gp->Add(gp2);
							mywin->Callback(this,(CLOSECALLBACK)&check_new_release::stophandler);
							mywin->SetGroup(gp);
							mywin->Create();
						}
						else
						{
							Alert(&readerwin,"Error writing file!");
						}
					}
					else
					{
						runmode=0;
					}
				}
				else if (mode==1)
				{
					Alert(&readerwin,"No new version of XCmail.");
					runmode=0;
				}
				m->remreferer();
				cnbufused=0;
				writeconfigfile();
			}
			break;
		case 5: // XCMAIL da
			gadget::RemoveFD(sock);
			close(sock);
			sock=0;
			cnbufused=0;
			if (recpos>0)
			{
				fwrite(recbuf,1,recpos,writefile);
			}
			fclose(writefile);
			writefile=NULL;
			if (mywin)
			{
				mywin->Close();
				DeleteObjectTree(mywin->FullName());
				mywin=NULL;
			}
			Alert(&readerwin,"All fine!\nYour new XCmail archiv has been loaded\nand you may install it now.");
			runmode=0;
			break;
	}
}
void check_new_release::stophandler(appwindow *)
{
	gadget::RemoveFD(sock);
	close(sock);
	sock=0;
	cnbufused=0;
	if (writefile)
	{
		fclose(writefile);
		remove(filename);
	}
	writefile=NULL;
	if (mywin)
	{
		mywin->Close();
		DeleteObjectTree(mywin->FullName());
		mywin=NULL;
	}
	runmode=0;
}
char *check_new_release::askBinary(mail *m,char *osname)
{
	char buf[200];
	xc_snprintf(buf,sizeof(buf),"XCmail-%s-List-URL:",osname);
	char *u=m->getTag(buf);
	if (!u || !u[0])
	{
		xc_snprintf(buf,sizeof(buf),"XCmail-%s-URL:",osname);
		return m->getTag(buf);
	}

	char *files[10];
	int filescount=0;

	appwindow *aw=new appwindow;
	aw->Name("check_new_release_window");
	aw->ParentClass(readerwin);
	aw->InitSize(150);
	aw->Init();
	aw->Title("Choose binary");
	aw->IconTitle("Choose binary");
	aw->TransientFor(readerwin);

	spacegroup *mgp=new spacegroup;
	mgp->Vert();
	mgp->Name("maingroup");
	mgp->Border(4);
	 listview *lv=new listview;
	 lv->ShowSelected();
	 lv->ID(2);

	 char *c1=u,*c2=u,*c3=u;
	 while (*c1)
	 {
		if (*c1=='/') c2=c1+1;
		if (*c1==' ')
		{
			*c1=0;
			if (strlen(c3))
			{
				files[filescount]=c3;
				lv->AddAtEnd(c2);
				filescount++;
			}
			c3=c1+1;
		}
		c1++;
	 }
	 *c1=0;
	 if (strlen(c3))
	 {
		files[filescount]=c3;
		lv->AddAtEnd(c2);
		filescount++;
	 }
	 lv->Selected(1);
	mgp->Add(lv);
	mgp->Add(new ruler);
	 fbutton *fb=new fbutton;
	 fb->Text("Ok");
	 fb->ID(1);
	mgp->Add(fb);
	aw->SetGroup(mgp);
	if (filescount>1)
	{
		aw->Create();
		aw->BlockExcept();

		int r=0,ev;
		do
		{
			ev=appwindow::NextEvent();
			switch (ev)
			{
				case EV_GADGET:
					r=appwindow::ID();
					if (r==2)
					{
						if (!appwindow::DoubleClick()) r=0;
					}
					break;
			}
		}
		while (!r);
		aw->UnblockExcept();
		aw->Close();
	}
	u=files[lv->Selected()-1];
	DeleteObjectTree(aw->FullName());
	return u;
}

static check_new_release t1;
