/* 
 * This file is part of XCmail and is subject to the 
 * XCmail license agreement
 */

/* $Id: dictionary.cxx,v 1.25 2000/07/23 17:59:37 schmitzj Exp $ */

#include "configure.h"

#include <Xclasses/Xclasses.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdarg.h>
#include <unistd.h>

#include "../../src/mailer.h"
#include "../../src/xc_snprintf.h"
#include "../../src/helpers/chartable.h"
#include "../../src/stringTokenizer.h"

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

#include "dictionary.h"

#define MAXQUERYLEN 255

Dictionary::Dictionary()
	: mainplugin(
		"Dictionary",
		"Ralf Becker",
		PLF_STATICUSE|PLF_CONFIGURABLE,
		"Interface for various online dictionaries"
	),
	profile(NULL,"\n"," \t")
{
	sockfd=-1;
	aw=NULL;
	html=NULL;
	cfgsetup();
	plist=NULL;
	pronum=0;
	gp=NULL;
	logfile=NULL;
}
Dictionary::~Dictionary()
{
	finish();
}

void Dictionary::init(void)
{
	addmenu(MPMENU_GENERAL,1,"Dictionary...");
}
void Dictionary::finish(void)
{
	closeCB(aw);
	closeSocket();
}
void Dictionary::handle(int id)
{
	switch (id)
	{
		case 1:
			createwin();
			break;
	}
}
void Dictionary::closeSocket(void)
{
	if (sockfd>=0)
	{
		gadget::RemoveFD(sockfd);
		close(sockfd);
		sockfd=-1;
	}
}
void Dictionary::cfgsetup(void)
{
	int i=0;
	profile.removeall();
	profile.set(i++,
		"German <-> English (TU Chemnitz):"
		"www.tu-chemnitz.de:80:"
		"/cgi-bin/wb?query=:"
		"German :<TR BGCOLOR=\\\"#FFFFDD\\\"><TD VALIGN=top WIDTH=\\\"40%\\\">:</TD>:"
		"English:<TD VALIGN=top WIDTH=\\\"40%\\\">:</TD>");
	profile.set(i++,
		"English <-> German (Leo.org):"
		"dict.leo.org:80:"
		"/?search=:"
		"English:<TR VALIGN=\\\"top\\\" BGCOLOR=\\\"#FFFCDF\\\"><TD>:</TD>:"
		"German :<TD>:</TD>");
	profile.set(i++,
		"English -> French (www.june29.com):"
		"www.june29.com:80:"
		"/~chambers/cgi/IDPsearch.cgi?translate=English+to+French&how=exact&word=:"
		"English:<dt>:<dd>:"
		"French :<dd>:<");
	profile.set(i++,
		"French -> English (www.june29.com):"
		"www.june29.com:80:"
		"/~chambers/cgi/IDPsearch.cgi?translate=French+to+English&how=exact&word=:"
		"English:<dt>:<dd>:"
		"French :<dd>:<");
	profile.set(i++,
		"English -> Spanish (www.june29.com):"
		"www.june29.com:80:"
		"/~chambers/cgi/IDPsearch.cgi?translate=English+to+Spanish&how=exact&word=:"
		"English:<dt>:<dd>:"
		"Spanish:<dd>:<");
	profile.set(i++,
		"Spanish -> English (www.june29.com):"
		"www.june29.com:80:"
		"/~chambers/cgi/IDPsearch.cgi?translate=Spanish+to+English&how=exact&word=:"
		"English:<dt>:<dd>:"
		"Spanish:<dd>:<");
	profile.set(i++,
		"English -> Italian (www.june29.com):"
		"www.june29.com:80:"
		"/~chambers/cgi/IDPsearch.cgi?translate=English+to+Italian&how=exact&word=:"
		"English:<dt>:<dd>:"
		"Italian:<dd>:<");
	profile.set(i++,
		"Italian -> English (www.june29.com):"
		"www.june29.com:80:"
		"/~chambers/cgi/IDPsearch.cgi?translate=Italian+to+English&how=exact&word=:"
		"Italian:<dt>:<dd>:"
		"French :<dd>:<");
}
void Dictionary::readconfigfile(void)
{
	char *h=getenv("HOME");
	char buf[strlen(h)+50];

	sprintf(buf,"%s/.XCmail/plugin-dictionary-profiles",h);
	
	FILE *file=fopen(buf,"r");
	if (file)
	{
		profile.removeall();

		fseek(file,0,SEEK_END);
		int len=ftell(file);
		char *m=(char*)malloc(len+1);
		if (m)
		{
			fseek(file,0,SEEK_SET);
			fread(m,1,len,file);
			m[len]=0;
			profile.string(m);
			profile.removeempty();
			free(m);
		}
		fclose(file);
	}
}
void Dictionary::writeconfigfile(void)
{
	char *h=getenv("HOME");
	char buf1[strlen(h)+50];
	char buf2[strlen(h)+50];
	
	sprintf(buf1,"%s/.XCmail/plugin-dictionary-profiles.tmp",h);
	sprintf(buf2,"%s/.XCmail/plugin-dictionary-profiles",h);

	FILE *file=fopen(buf1,"w");
	int err=1;
	if (file)
	{
		err=0;
		char *s=profile.list();
		if (s)
		{
			err=fwrite(s,1,strlen(s),file)!=strlen(s);
			err=err || fwrite("\n",1,1,file)!=1;
		}
		err=fclose(file)!=0 || err;
	}
	if (!err) rename(buf1,buf2);
}
void Dictionary::newprofile(gadget *gg, int)
{
	if (strcmp(gg->Name(),"useprofile")==0)
	{
		tokenizer t(profile[((choice*)gg)->Selected()].tokenize(),":",NULL);

		column_spec cs;
		cs.flags=CSF_USEWEIGHT|CSF_SELECTABLE|CSF_SCALEABLE|CSF_TITLE;
		cs.weight=50;
		cs.title=t[4];

		((multilistview*)FindObject(aw->FullName(),".result"))->ColumnConfigure(1,&cs);

		cs.flags=CSF_USEWEIGHT|CSF_SELECTABLE|CSF_SCALEABLE|CSF_TITLE;
		cs.weight=1;
		cs.title=t[7];
		((multilistview*)FindObject(aw->FullName(),".result"))->ColumnConfigure(2,&cs);
	}
	else if (strcmp(gg->Name(),"profile")==0)
	{
		readconfig(gp);
		pronum=((choice*)gg)->Selected();
		setupconfig(gp);
	}
	else if (strcmp(gg->Name(),"del")==0)
	{
		if (((appwindow*)FindObject(readerwin.FullName(),"mimeprefs"))->Alert("Delete Profile?","Yes|No")==1)
		{
			pronum=((choice*)FindObject(gp->FullName(),".profile"))->Selected();
			profile.remove(pronum--);
			if (pronum<0) pronum=0;
			setupconfig(gp);
		}
	}
	else if (strcmp(gg->Name(),"new")==0)
	{
		appwindow *a=new appwindow;
		appwindow *aw=(appwindow*)FindObject(readerwin.FullName(),"mimeprefs");
		int x,y;

		GetPositionOnRoot(gg->display(),gg->gg_win(),x,y);
		a->InitSize(x-200,y,160);
		if (!a->Init()) return;

		a->TransientFor(aw);
		a->Name("newprowindow");
		a->Title("Profile name...");
		a->IconTitle("Profile name...");

		spacegroup *sg1=new spacegroup;
		sg1->Border(4);
		sg1->Vert();
		sg1->Name(NULL);
			input *ip=new input;
			ip->Name("name");
			ip->Text("Profile name");
			ip->MaxChars(50);
			ip->Default("New Profile");
			ip->Callback(this,(CALLBACK)&Dictionary::newprofile);
		sg1->Add(ip);
		sg1->Add(new ruler);
			group *g1=new group;
			g1->Horiz();
			g1->Name(NULL);
			g1->DialogMode();
				fbutton *fb1=new fbutton;
				fb1->Name("okbutton");
				fb1->Text("_Ok");
	 			fb1->Callback(this,(CALLBACK)&Dictionary::newprofile);
			g1->Add(fb1);
				fbutton *fb2=new fbutton;
				fb2->Name("cancelbutton");
				fb2->Text("_Cancel");
	 			fb2->Callback(this,(CALLBACK)&Dictionary::newprofile);
				fb2->ApplyXKey(XK_Escape);
			g1->Add(fb2);
		sg1->Add(g1);
		a->SetGroup(sg1);
		a->Create();
		ip->ActivateKey();
		aw->Block();
	}
	else if (strcmp(gg->Name(),"okbutton")==0 || strcmp(gg->Name(),"cancelbutton")==0 || strcmp(gg->Name(),"name")==0)
	{
		appwindow *aw=(appwindow*)FindObject(readerwin.FullName(),".mimeprefs");
		appwindow *a=(appwindow*)FindObject(readerwin.FullName(),".newprowindow");
		aw->Unblock();
	
		if (strcmp(gg->Name(),"cancelbutton")!=0)
		{
			char buf[256];
			((input*)FindObject(a->FullName(),".name"))->Input(buf);
			pronum=profile.count();
			profile.set(pronum,buf);
			setupconfig(gp);
		}		
		a->Close();
		DeleteObjectTree(a->FullName());
	}
	else
	{
		readconfig(gp);
		setupconfig(gp);
	}
}
char **Dictionary::getplist(void)
{
	static char *empty[]={NULL};
	int i;
	if (plist)
	{
		for (i=0;plist[i];i++)
			free(plist[i]);
		free(plist);
	}
	plist=(char**)malloc(sizeof(char*)*(profile.count()+1));
	if (plist)
	{
		for (i=0;i<profile.count();i++)
		{
			tokenizer t(profile[i].tokenize(),":",NULL);
			plist[i]=strdup(t[0]);
		}
		plist[i]=NULL;
	}
	return plist?plist:empty;
}
group *Dictionary::makeconfig(void)
{
	static char *dummy[]={"",NULL};
	gp=new group;
	gp->Name("dictionary_group");
	gp->Vert();
		group *cg = new group;
		cg->Name(NULL);
		cg->Horiz();
			choice *choice_profile=new choice;
			choice_profile->Name("profile");
			choice_profile->Items(dummy);
			choice_profile->Callback(this,(CALLBACK)&Dictionary::newprofile);
		cg->Add(choice_profile);
			fbutton *cb = new fbutton;
			cb->Name("new");
			cb->Text("_New");
			cb->Weight(20);
			cb->Callback(this,(CALLBACK)&Dictionary::newprofile);
		cg->Add(cb);
			fbutton *fbutton_del=new fbutton;
			fbutton_del->Name("del");
			fbutton_del->Text("_Delete");
			fbutton_del->Weight(20);
			fbutton_del->Callback(this,(CALLBACK)&Dictionary::newprofile);
		cg->Add(fbutton_del);
	gp->Add(cg);
	gp->Add(new ruler);
		group *ug=new group;
		ug->Name(NULL);
		ug->Horiz();
			input *input_url=new input;
			input_url->Name("url");
		ug->Add(input_url);
			input *input_port=new input;
			input_port->Text("port");
			input_port->Name("port");
			input_port->Weight(20);
		ug->Add(input_port);
	gp->Add(ug,"server");
		input *input_getstring = new input;
		input_getstring->Text("get string");
		input_getstring->Name("getstring");
		input_getstring->MaxChars(200);
	gp->Add(input_getstring);
		group *g=new group;
		g->Horiz();
		g->Name(NULL);
			input *input_col1name=new input;
			input_col1name->Text("name");
			input_col1name->Name("col1name");
			input_col1name->Weight(20);
		g->Add(input_col1name);
			input *input_col1start = new input;
			input_col1start->Text("start");
			input_col1start->Name("col1start");
			input_col1start->Weight(35);
		g->Add(input_col1start);
			input *input_col1end = new input;
			input_col1end->Text("end");
			input_col1end->Name("col1end");
			input_col1end->Weight(10);
		g->Add(input_col1end);
	gp->Add(g, "first column,");
		g=new group;
		g->Horiz();
		g->Name(NULL);
			input *input_col2name=new input;
			input_col2name->Text("name");
			input_col2name->Name("col2name");
			input_col2name->Weight(20);
		g->Add(input_col2name);
			input *input_col2start=new input;
			input_col2start->Text("start");
			input_col2start->Name("col2start");
			input_col2start->Weight(35);
		g->Add(input_col2start);
			input *input_col2end=new input;
			input_col2end->Text("end");
			input_col2end->Name("col2end");
			input_col2end->Weight(10);
		g->Add(input_col2end);
	gp->Add(g, "secound column,");
	gp->Add(new ruler);
		fbutton *b=new fbutton;
		b->Name("apply");
		b->Text("_Apply Changes");
		b->Callback(this,(CALLBACK)&Dictionary::newprofile);
	gp->Add(b);

	return gp;
}
void Dictionary::setupconfig(group *gg)
{
	char *dummy[]={"",NULL};

	tokenizer t(profile[pronum].tokenize(),":",NULL);

	if (aw)
	{
		choice *ch=(choice*)FindObject(aw->FullName(),".useprofile");
		ch->Selected(0);
		ch->Items(dummy);
	}

	((choice*)FindObject(gg->FullName(),".profile"))->Items(dummy);
	((choice*)FindObject(gg->FullName(),".profile"))->Items(getplist());
	((choice*)FindObject(gg->FullName(),".profile"))->Selected(pronum);
	((input*)FindObject(gg->FullName(),".url"))->Default(t[1]);
	((input*)FindObject(gg->FullName(),".port"))->Default(t[2]);
	((input*)FindObject(gg->FullName(),".getstring"))->Default(t[3]);
	((input*)FindObject(gg->FullName(),".col1name"))->Default(t[4]);
	((input*)FindObject(gg->FullName(),".col1start"))->Default(t[5]);
	((input*)FindObject(gg->FullName(),".col1end"))->Default(t[6]);
	((input*)FindObject(gg->FullName(),".col2name"))->Default(t[7]);
	((input*)FindObject(gg->FullName(),".col2start"))->Default(t[8]);
	((input*)FindObject(gg->FullName(),".col2end"))->Default(t[9]);

	if (profile.count()==0)
	{
		((choice*)FindObject(gg->FullName(),".profile"))->Lock();
		((input*)FindObject(gg->FullName(),".url"))->Lock();
		((input*)FindObject(gg->FullName(),".port"))->Lock();
		((input*)FindObject(gg->FullName(),".getstring"))->Lock();
		((input*)FindObject(gg->FullName(),".col1name"))->Lock();
		((input*)FindObject(gg->FullName(),".col1start"))->Lock();
		((input*)FindObject(gg->FullName(),".col1end"))->Lock();
		((input*)FindObject(gg->FullName(),".col2name"))->Lock();
		((input*)FindObject(gg->FullName(),".col2start"))->Lock();
		((input*)FindObject(gg->FullName(),".col2end"))->Lock();
		((fbutton*)FindObject(gg->FullName(),".del"))->Lock();
	}
	else
	{
		((choice*)FindObject(gg->FullName(),".profile"))->Unlock();
		((input*)FindObject(gg->FullName(),".url"))->Unlock();
		((input*)FindObject(gg->FullName(),".port"))->Unlock();
		((input*)FindObject(gg->FullName(),".getstring"))->Unlock();
		((input*)FindObject(gg->FullName(),".col1name"))->Unlock();
		((input*)FindObject(gg->FullName(),".col1start"))->Unlock();
		((input*)FindObject(gg->FullName(),".col1end"))->Unlock();
		((input*)FindObject(gg->FullName(),".col2name"))->Unlock();
		((input*)FindObject(gg->FullName(),".col2start"))->Unlock();
		((input*)FindObject(gg->FullName(),".col2end"))->Unlock();
		((fbutton*)FindObject(gg->FullName(),".del"))->Unlock();
	}
	if (aw)
	{
		choice *ch=(choice*)FindObject(aw->FullName(),".useprofile");
		ch->Items(plist);
		ch->Selected(pronum);
		newprofile(ch,0);
	}
}
void Dictionary::readconfig(group *gg)
{
	char buf[256];
	tokenizer t(profile[pronum].tokenize(),":",NULL);
	((input*)FindObject(gg->FullName(),".url"))->Input(buf);t.set(1,buf);
	((input*)FindObject(gg->FullName(),".port"))->Input(buf);t.set(2,buf);
	((input*)FindObject(gg->FullName(),".getstring"))->Input(buf);t.set(3,buf);
	((input*)FindObject(gg->FullName(),".col1name"))->Input(buf);t.set(4,buf);
	((input*)FindObject(gg->FullName(),".col1start"))->Input(buf);t.set(5,buf);
	((input*)FindObject(gg->FullName(),".col1end"))->Input(buf);t.set(6,buf);
	((input*)FindObject(gg->FullName(),".col2name"))->Input(buf);t.set(7,buf);
	((input*)FindObject(gg->FullName(),".col2start"))->Input(buf);t.set(8,buf);
	((input*)FindObject(gg->FullName(),".col2end"))->Input(buf);t.set(9,buf);
	profile.set(pronum,t.string());
}
void Dictionary::closeCB(appwindow *)
{
	if (aw)
	{
		aw->Close();
		DeleteObjectTree(aw->FullName());
		aw = NULL;
	}
}

void Dictionary::finishCB(gadget *, int)
{
	closeCB(aw);
}

int Dictionary::createwin(void)
{
	if (profile.count()==0)
	{
		readerwin.Alert("There's no online dictionary defined!","Cancel");
		return 0;
	}

	tokenizer pro(profile[pronum].tokenize(),":",NULL);
	
	// Open Window
	//
	aw = new appwindow;
	if (!aw) return 0;
	
	aw->Name("dictionarywindow");
	aw->InitSize(15);
	if (!aw->Init())
	{
		delete aw;
		pstatus("Error in opening window!");
		return 0;
	}

	aw->Title(PROGRAMNAME": Online Dictionary");
	aw->IconTitle("Dictionary");

	spacegroup *g = new spacegroup;
	g->Vert();
	g->Name("maingroup");
	g->YDistance(5);
	g->XDistance(5);
	g->Border(5);
		::text *t = new ::text;
		t->Text(
			"This function uses online dictionaries provided by different internet sites.\n"
			"Each dictionary is represented by a profile below...");
		t->Font(FONT_SMALL);
	g->Add(t);
		choice *ch=new choice;
		ch->Name("useprofile");
		ch->Items(getplist());
		ch->Selected(pronum);
		ch->Callback(this,(CALLBACK)&Dictionary::newprofile);
	g->Add(ch,"Profile");
		group *g1 = new group;
		g1->Name(NULL);
		g1->Horiz();
			input *in1 = new input;
			in1->Name("query");
			in1->Text("Query");
			in1->MaxChars(MAXQUERYLEN);
			in1->Callback(this,(CALLBACK) &Dictionary::lookupCB);
			in1->Weight(40);
		g1->Add(in1);
			fbutton *fb1 = new fbutton;
			fb1->Name("lookup");
			fb1->Text("_Lookup");
			fb1->Callback(this,(CALLBACK) &Dictionary::lookupCB);
			fb1->Weight(10);
		g1->Add(fb1);
	g->Add(g1);

		column_spec cs[2];
		cs[0].flags=CSF_USEWEIGHT|CSF_SELECTABLE|CSF_SCALEABLE|CSF_TITLE;
		cs[0].weight=50;
		cs[0].title=pro[4];
		cs[1].flags=CSF_USEWEIGHT|CSF_SELECTABLE|CSF_SCALEABLE|CSF_TITLE;
		cs[1].weight=1;
		cs[1].title=pro[7];

		multilistview *mlv = new multilistview;
		mlv->Name("result");
		mlv->Font(FONT_FIXED); 
		mlv->Columns(2,cs);
		mlv->Weight(20);
		mlv->Spacing(2);
	g->Add(mlv);		
		g1 = new group;
		g1->Horiz();
		g1->Name(NULL);
			output *status = new output;
			status->Name("state");
			status->Text("State");
			status->Font(FONT_SMALL);
			status->Output("");
			status->Weight(40);
		g1->Add(status);
			fbutton *fb2 = new fbutton;
			fb2->Name("close");
			fb2->Text("_Close");
			fb2->Callback(this,(CALLBACK) &Dictionary::finishCB);
			fb2->Weight(10);
		g1->Add(fb2);
	g->Add(g1);
	
	aw->SetGroup(g);
	aw->Callback(this,(CLOSECALLBACK) &Dictionary::closeCB);
	aw->Create();

	return 1;
}

void Dictionary::state(char *s, ...)
{
	static char buf[255];
	va_list valist;
	va_start(valist,s);
	xc_vsnprintf(buf,sizeof(buf),s,valist);
	va_end(valist);
	((output *)FindObject(aw->FullName(),".state"))->Output(buf);
	aw->Update();
}

void Dictionary::lookupCB(gadget *, int)
{
	int num=((choice*)FindObject(aw->FullName(),".useprofile"))->Selected();

	tokenizer p(profile[num].tokenize(),":",NULL);
	
	char *host        = strdup(p[1]);
	int port   		  = atoi(p[2]);		// port
	char querytempl[] = "GET %s%s%s HTTP/1.0\n"
						"User-Agent: XCmail dictionary plugin\n"
						"Accept: text/html\n"
						"Accept-Language: de,en\n"
						"Accept-Charset: iso-8859-1,*\n"
						"\n";
	char proxytempl[] = "http://%s:%d";
	char iword[MAXQUERYLEN+1];
	char qword[3*MAXQUERYLEN+1];
	int i,n;

	closeSocket();

	if (!host) return;

	multilistview *mlv=(multilistview *)FindObject(aw->FullName(),".result");
	input *in=(input *)FindObject(aw->FullName(),".query");
	
	in->Input(iword);
	mlv->RemoveAll();

	for (i=0,n=0; iword[i]; i++)
	{
		if (!isalnum(iword[i]))
		{
			n+=sprintf(qword+n, "%%%0.2X", (int)(unsigned char)iword[i]);
		}
		else
		{
			qword[n++]=iword[i];
		}
	}
	qword[n]=0;

	char *query=NULL;
	char empty[]="";
	char *opthost=empty;
		
	char *proxy=getenv("http_proxy");
	if (proxy)
	{
		rextokenizer hp(proxy,"http://([^/:]+):?([0-9]*)/?");

		char *t=new char[strlen(proxytempl)+strlen(host)+10];
		if (t)
		{
			opthost=t;
			sprintf(t,proxytempl,host,port);
			free(host);
			host=strdup(hp[0]);
			port=atoi(hp[1]);
			if (!port) port=80;
		}
	}
	if (!host) return;
	
	query=new char[strlen(querytempl)+strlen(opthost)+strlen(p[3])+strlen(qword)+2];
	if (query)
	{
		sprintf(query,querytempl,opthost,(char*)p[3],qword);
	}
	if (opthost!=empty)
	{
		delete[] opthost;
	}
	if (!query)
	{
		free(host);
		aw->Alert("out of memory","OK");
		return;
	}

	
	// connect
	//
	int rc;
	struct hostent *host_info;
	struct sockaddr_in cli_addr, serv_addr;
	
	state("Looking up host %s", host);

	if (!(host_info = gethostbyname(host)))
	{
		aw->Alert("unable to locate \"%s:%i\"","OK", host,port);
		state("");
		delete [] query;
		free(host);
		return;
	}

	serv_addr.sin_family = host_info->h_addrtype;
	memcpy((char*)&serv_addr.sin_addr.s_addr, host_info->h_addr, host_info->h_length);
	serv_addr.sin_port = htons(port);
	for(i=0; i<8; i++) serv_addr.sin_zero[i]=0;
			
	cli_addr.sin_family = AF_INET;
	cli_addr.sin_addr.s_addr = htonl(INADDR_ANY);
	cli_addr.sin_port = htons(0);
	for(i=0; i<8; i++) cli_addr.sin_zero[i]=0;

	// build query body
	//
	state("Connecting host %s", host);

	if ((sockfd = socket(AF_INET, SOCK_STREAM, 0))<0)
	{
		aw->Alert("unable to create socket!","OK");
		state("");
		delete [] query;
		free(host);
		return;
	}

	rc = bind(sockfd, (struct sockaddr *)&cli_addr, sizeof(cli_addr));
	if (rc)
	{
		aw->Alert("unable to bind socket!","OK");
		state("");
		close(sockfd);
		sockfd=-1;
		free(host);
		delete [] query;
	}
	
	rc = connect(sockfd, (struct sockaddr *)&serv_addr, sizeof(serv_addr));
	if (rc)
	{
		aw->Alert("unable to connect to \"%s:%i\"","OK", host, port);
		state("");
		close(sockfd);
		sockfd=-1;
		delete [] query;
		free(host);
		return;
	}
	fcntl(sockfd,F_SETFL,O_NONBLOCK);

	state("Sending Query");

	rc = send(sockfd, query, strlen(query), 0);
	delete [] query;

	if (rc<0)
	{
		aw->Alert("error while sending data to \"%s\"","OK", host);
		state("");
		close(sockfd);
		sockfd=-1;
		delete [] query;
		free(host);
		return;
	}
	
	state("Waiting for response");

	gadget::AddReadFD(sockfd,this,(FDHANDLER)&Dictionary::handleSocket);
	html=NULL;
	free(host);
}

void Dictionary::handleSocket(int)
{
	int num=((choice*)FindObject(aw->FullName(),".useprofile"))->Selected();
	tokenizer p(profile[num].tokenize(),":",NULL);

	multilistview *mlv=(multilistview *)FindObject(aw->FullName(),".result");
	if (!html)
	{
		state("Receiving text");
		htmls = 512;
		htmli = 0;
		html = (char*)malloc(htmls); // wg. realloc(!)

		char *s=getenv("XC_DICT_LOG");
		if (s && s[0])
		{
			logfile=fopen(s,"w");
		}
	}
	char b;
	int n = recv(sockfd, &b, 1, 0);

	if (n>0)
	{
		if (logfile)
		{
			fwrite(&b,1,1,logfile);
		}
		if (n + 1 + htmli > htmls)
		{
			html = (char *)realloc(html, htmls+512);
			if (html) htmls+=512;
		}
		if (html)
		{
			if (htmli>0 && html[htmli-1]==' ')
			{
				switch(b)
				{
					case ' ':
					case '\t':
					case '\r':
					case '\n':
						break;
						
					default:
						html[htmli++]=b;
						break;
				}
			}
			else
			{
				switch(b)
				{
					case '\t':
					case '\r':
					case '\n':
						b=' ';
						break;
				}
				html[htmli++]=b;
			}
		}
		if (htmli%100==0)
		{
			state("Receiving text (%i bytes)", htmli);
		}
	}
	else if (n<0)
	{
		if (logfile) fclose(logfile);
		aw->Alert("Error while reading data","OK");
		state("");
		closeSocket();
		return;
	}
	else
	{
		if (logfile) fclose(logfile);
		state("Transforming text");

		if (html)
		{
			html[htmli++]=0;
			char *d1 = html;

			CharTable ct;

			char *t[2] = {NULL,NULL};

			while (*d1)
			{
				char *c,*s;

				if (!t[0] && strcncmp(p[5],d1)==0)
				{
					c=s=d1+strlen(p[5]);
					while (*c && strcncmp(p[6],c)!=0)
						c++;
					d1=c;					
					c=t[0]=new char[c-s+1];
					if (t[0])
					{
						while (s<d1)
						{
							if (*s=='<')
							{
								while (s<d1 && *s!='>') s++;
								if (s<d1) s++;
								continue;
							}
							*(c++)=*(s++);
						}
						*c=0;
					}
				}
				else if (t[0] && strcncmp(p[8],d1)==0)
				{
					c=s=d1+strlen(p[8]);
					while (*c && strcncmp(p[9],c)!=0)
						c++;
					d1=c;
					c=t[1]=new char[c-s+1];
					if (t[1])
					{
						while (s<d1)
						{
							if (*s=='<')
							{
								while (s<d1 && *s!='>') s++;
								if (s<d1) s++;
								continue;
							}
							*(c++)=*(s++);
						}
						*c=0;
					}
					if (t[0] && t[1])
					{
						ct.Translate(t[0]);
						ct.Translate(t[1]);
						mlv->AddAtEnd(t);
					}
					delete[] t[0]; t[0]=NULL;
					delete[] t[1]; t[1]=NULL;
				}
				else
				{
					d1++;
				}
			}
		}			
		if (html) free(html);
		html=NULL;

		state("");
		closeSocket();
		aw->Update();
	}
}

static Dictionary _dic;
