/* 
 * This file is part of XCmail and is subject to the 
 * XCmail license agreement
 */

/* $Id: encoder.h,v 2.5 1999/02/01 19:00:19 beckerr Exp $ */


#include "../../src/encoder.h"

#if 0
//#ifndef ENCODER_H
#define ENCODER_H

#include <stdlib.h>


/*---------------------------------------------------------------------------
 * Encoders:
 *
 * What are encoders?
 * Encoders code and decode any data into ASCII for sending and back to data.
 *
 * How it works:
 * Any encoder has to exist once declared static.
 *   static class demo_encoder;
 * By this the encoder is recognized by XCmail. Every time a mail was encoded
 * with an encoder whose name is recognized the mail2data method is called
 * and later on free to free its memory. Every time an user wants to decode
 * a mail (or binary) data2mail is called and later on free again.
 *
 * Syntax:
 * Any encoder may have multiple names devided by a semicolon
 * (eg. "base64;x-base64").
 * Any kind of information (description) may be added.
 *
 * Flags:
 * Some encoders may only code or decode because of some reasons. This is
 * given by flags which may be logically added (OR).
 *
 *  - EF_ENCODE .......... encoder can code data to mail
 *  - EF_DECODE .......... encoder can decode mail to data
 *  - EF_DIRECT_ENCODE ... encoder should be added to "encode part" menu
 *                         in sendmail window (not yet supported)
 *  - EF_DIRECT_DECODE ... encoder should be added to "decode part" menu
 *                         in readmail window (not yet supported)
 * 
 *
 */

#define EF_ENCODE			0x00000001	// can encode
#define EF_DECODE			0x00000002	// can decode
#define EF_DIRECT_ENCODE	0x00000004	// in read mail menu, encode
#define EF_DIRECT_DECODE	0x00000008  // in read mail menu, decode

class encoder	// abstract
{
friend class encoding;
	char *name;
	char *author;
	char *information;
public:
	int  flags;

	encoder(char *name,char *author,int flags=(EF_ENCODE|EF_DECODE),char *information=NULL);
	virtual ~encoder();

	char *encoder_name(void);
	char *encoder_author(void);
	char *encoder_information(void);

	char *egets(char *,int,char **buf);
	void eputs(char *,char **buf);

	virtual char *data2mail(char *,int size,int &newsize,char *path,char *filename);
	virtual char *mail2data(char *,int size,int &newsize,char *path,char *filename);
	virtual void dispose(char *);
};

class encoder_command : public encoder
{
	char *codecom;
	char *decodecom;
public:
	encoder_command(void);
	~encoder_command();

	void command(char *,char *);

	virtual char *data2mail(char *,int size,int &newsize,char *path,char *filename);
	virtual char *mail2data(char *,int size,int &newsize,char *path,char *filename);
	virtual void dispose(char *);
};

#endif
