/* 
 * This file is part of XCmail and is subject to the 
 * XCmail license agreement
 */

/* $Id: helper.h,v 2.4 1999/02/01 19:00:19 beckerr Exp $ */


#include "../../src/helper.h"

#if 0
//#ifndef HELPER_H
#define HELPER_H

#include <stdlib.h>
#include <Xclasses/Xclasses.h>

/*---------------------------------------------------------------------------
 * Helpers:
 *
 * What are helpers:
 * Helpers convert mimetyped mailparts to an ASCII visible form or make some
 * other crazy things.
 *
 * How it works: (idea, not completely realized)
 * Any helper has to exist once declared static like the encoders.
 *   static class demo_helper;
 * By this the helper is recognized by XCmail. The user may set up any
 * helper for any mimetype. The helper need not support them but must be able
 * to ignore unknown types.
 *
 * If a helper is required its "clone" is called which must create a new
 * object of this type.
 *
 * "go" is called to convert and must return so text, maybe only some kind of
 * "nothing to display".
 *
 * Then "makegadget" is called if HL_GADGETS is set,
 * next "makemenu" is called if HL_PDMENU is set, "menuhandler" is called
 * if required. The object is deleted if it is no longer needed.
 *
 * Syntax:
 * Every helper may only have one name. A list of only supported mime types
 * may be specified, separated by semicolons (eg. "text/html;text/rtf").
 *
 * Config options:
 * Helpers may offer a page in the general preferences window (HL_SETUP).
 * makeconfig() is called to open a group, readconfig() to read it, all
 * objects must be dynamically created and must not cleared from the helper.
 * readconfigfile() is called after program start (or later...)
 * writeconfigfile() is called by user's "Save preferences"
 *
 * Example config class setup
 * static config cfg_file("~/."PROGRAMNAME"/helpername");
 */

#define HL_PDMENU		0x00000001		// PD menu
#define HL_GADGETS		0x00000002		// replace textbox gadget
#define HL_SETUP		0x00000004		// own card in general setup

class helper	// abstract
{
friend class helpers;
friend class helperinfo;
	char *name;
	char *author;
	char *mimes;
	char *information;
public:
	int  flags;
	void *mail;
	int  partnb;

	// global contructor
	helper(char *name,char *author,char *mimes,int flags=0,char *information=NULL);
	// clone contructor
	helper(class helper *);
	virtual ~helper();								// free your gadgets!!

	char *helper_name(void);
	char *helper_author(void);
	char *helper_information(void);

	char *egets(char *,int,char **buf);
	void eputs(char *,char **buf);
	char *getheader(char *h);
	char *mime1(void);
	char *mime2(void);

	virtual class helper *clone(char *in,int insize);

	virtual char *go(int &newsize);					// return NULL if text is unchanged
	virtual int makegadget(class visible *);		// return 1 for show applied gadgets, else 0
	virtual int makemenu(class menu *,int menu);	// return 1 if menus added, else 0
	virtual void menuhandler(class menu *,int mid);

	virtual group *makeconfig(void);
	virtual void readconfig(group *);

	virtual void readconfigfile(void);
	virtual void writeconfigfile(void);
};

#endif
