/* 
 * This file is part of XCmail and is subject to the 
 * XCmail license agreement
 */

/* $Id: rot13_encoding.cxx,v 2.5 1999/01/31 16:52:52 beckerr Exp $ */


//
// Rot13 encoding fuer xc-mail
// (C)1997 Ralf Becker (Ra.Becker@fh-trier.de)
//
// erzeugt: 13.04.97 (r.b.)
//
#include "../configure.h" 

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include "../include/encoder.h"
#include "../include/plugins.h"

//--------------------------------------------------------------------

// rot13 -> encode = decode
//
static char *R13(char *inbuf, int size, int &newsize) {

	char *outbuf, *cP, c;
	
	newsize = size;
	outbuf = cP = new char[size + 1];
	if (!outbuf) {
		fprintf(stderr, "rot13: out of memory!\n");
		return NULL;
	}
	
	for (; size > 0; size--) {
		c = *(inbuf++);
		if (islower(c))
			*(cP++) = 'a' + ((c - 'a' + 13) % 26);
		else
		if (isupper(c))
			*(cP++) = 'A' + ((c - 'A' + 13) % 26);
		else
			*(cP++) = c;
	}
	*cP = 0;
		
	return outbuf;
}

//--------------------------------------------------------------------

class encoder_rot13 : public encoder {

	public:
		encoder_rot13(void) : encoder("rot13;x-rot13", "Ralf Becker", EF_ENCODE|EF_DECODE|EF_DIRECT_ENCODE|EF_DIRECT_DECODE) {};
		~encoder_rot13() {};
		virtual char *data2mail(char *i,int s,int &n,char *p,char *f) {return R13(i,s,n);};
		virtual char *mail2data(char *i,int s,int &n,char *p,char *f) {return R13(i,s,n);};
		virtual void dispose(char *b) {::delete[] b;}
};

static encoder_rot13 rot13;

//--------------------------------------------------------------------

class Rot13
{
public:
	void handle(int id, Xclasses *);
};

void Rot13::handle(int id, Xclasses *xcl)
{
	if (id!=1) return;

	textbox *t=(textbox*)FindObject(xcl->FullName(),".mailbody");
	if (!t) return;

	int marked=1;
	int size=t->MarkedTextSize();
	if (!size)
	{
		size=t->TextSize();
		marked=0;
	}

	char *s=new char[size+1];
	if (!s)
	{
		return;
	}
	if (marked)
	{
		t->CopyMarkedText(s,size+1);
	}
	else
	{
		t->CopyText(s,size+1);
	}
	unsigned char *c=(unsigned char*)s;
	while (size>0)
	{
		if (islower(*c) && (*c-'a')<26)
		{
			*c='a'+((*c-'a'+13)%26);
		}
		else if (isupper(*c) && (*c-'A')<26)
		{
			*c='A'+((*c-'A'+13)%26);
		}
		c++;
		size--;
	}
	if (marked)
	{
		t->ReplaceMarkedText(s);
	}
	else
	{
		t->ReplaceText(s);
	}
	delete[] s;
}

class Rot13showmail : public showmailplugin, Rot13
{
public:
	Rot13showmail():showmailplugin("Rot13 (hidden)","Ralf Becker",PLF_STATICUSE|PLF_INVISIBLE,"(hidden)")
	{
	};
	void init(Xclasses *x)
	{
		addmenu(1,"Rot13 decode/encode","3");
	};
	void handle(int i, Xclasses *x)
	{
		Rot13::handle(i,x);
	};
};
static Rot13showmail _r13s;


class Rot13newmail : public newmailplugin, Rot13
{
public:
	Rot13newmail():newmailplugin("Rot13 encoder/decoder","Ralf Becker",PLF_STATICUSE,"Rot13 encode/decode enhancement")
	{
	};
	void init(Xclasses *x)
	{
		addmenu(1,"Rot13 decode/encode","3");
	};
	void handle(int i, Xclasses *x)
	{
		Rot13::handle(i,x);
	};
};
static Rot13newmail _r13n;
