/* 
 * This file is part of XCmail and is subject to the 
 * XCmail license agreement
 */

/* $Id: buildin_conditions.cxx,v 1.11 2002/07/31 08:44:45 schmitzj Exp $ */


#include "../../src/mailer.h"
#include "condition.h"


#define MAXLEN	40

class cond_subject_startcmp : public condition
{
	char *subj;
public:
	cond_subject_startcmp(void);
	cond_subject_startcmp(class cond_subject_startcmp *,char *);
	~cond_subject_startcmp();

	class cond_subject_startcmp *clone(char *);
	int work(class mail *);
	int load(tokenizer *);
	int save(tokenizer *);

	group *makeprefs(void);
	void setupprefs(group *);
	void readprefs(group *);
};

cond_subject_startcmp::cond_subject_startcmp(void) : condition("subject starts with","Jrgen Schmitz")
{
	subj=(char*)malloc(MAXLEN+1);
	subj[0]=0;
}
cond_subject_startcmp::cond_subject_startcmp(cond_subject_startcmp *c,char *n) : condition(c,n)
{ // copy global values
	subj=(char*)malloc(MAXLEN+1);
	subj[0]=0;
}
cond_subject_startcmp::~cond_subject_startcmp()
{
	if (subj) free(subj);
}
cond_subject_startcmp *cond_subject_startcmp::clone(char *n)
{
	return new cond_subject_startcmp(this,n);
}
int cond_subject_startcmp::work(mail *m)
{
	return !strcncmp(subj,m->subject());
}
int cond_subject_startcmp::load(tokenizer *tk)
{
	xc_strncpy(subj,(*tk)[0],MAXLEN+1);
	return 0;
}
int cond_subject_startcmp::save(tokenizer *tk)
{
	tk->append(new token(subj));
	return 0;
}

group *cond_subject_startcmp::makeprefs(void)
{
	group *gp=new group;
	gp->Name("group");
	gp->Vert();
	 input *ip=new input;
	 ip->Name("subject");
	 ip->MaxChars(MAXLEN);
	 ip->Text("first letters of subject");
	gp->Add(ip);
	return gp;
}
void cond_subject_startcmp::setupprefs(group *g)
{
	((input*)FindObject(g->FullName(),".subject"))->Default(subj);
}
void cond_subject_startcmp::readprefs(group *g)
{
	((input*)FindObject(g->FullName(),".subject"))->Input(subj);
}

static cond_subject_startcmp csusc;

// --------------------------------------------------------------------------

class cond_subject_partcmp : public condition
{
	char *subj;
public:
	cond_subject_partcmp(void);
	cond_subject_partcmp(class cond_subject_partcmp *,char *);
	~cond_subject_partcmp();

	class cond_subject_partcmp *clone(char *);
	int work(class mail *);
	int load(tokenizer *);
	int save(tokenizer *);

	group *makeprefs(void);
	void setupprefs(group *);
	void readprefs(group *);
};

cond_subject_partcmp::cond_subject_partcmp(void) : condition("subject contains","Jrgen Schmitz")
{
	subj=(char*)malloc(MAXLEN+1);
	subj[0]=0;
}
cond_subject_partcmp::cond_subject_partcmp(cond_subject_partcmp *c,char *n) : condition(c,n)
{ // copy global values
	subj=(char*)malloc(MAXLEN+1);
	subj[0]=0;
}
cond_subject_partcmp::~cond_subject_partcmp()
{
	if (subj) free(subj);
}
cond_subject_partcmp *cond_subject_partcmp::clone(char *n)
{
	return new cond_subject_partcmp(this,n);
}
int cond_subject_partcmp::work(mail *m)
{
	return !strscncmp(subj,m->subject());
}
int cond_subject_partcmp::load(tokenizer *tk)
{
	xc_strncpy(subj,(*tk)[0],MAXLEN+1);
	return 0;
}
int cond_subject_partcmp::save(tokenizer *tk)
{
	tk->append(new token(subj));
	return 0;
}

group *cond_subject_partcmp::makeprefs(void)
{
	group *gp=new group;
	gp->Name("group");
	gp->Vert();
	 input *ip=new input;
	 ip->Name("subject");
	 ip->MaxChars(MAXLEN);
	 ip->Text("look for string in subject");
	gp->Add(ip);
	return gp;
}
void cond_subject_partcmp::setupprefs(group *g)
{
	((input*)FindObject(g->FullName(),".subject"))->Default(subj);
}
void cond_subject_partcmp::readprefs(group *g)
{
	((input*)FindObject(g->FullName(),".subject"))->Input(subj);
}

static cond_subject_partcmp csupc;

// --------------------------------------------------------------------------

class cond_sender_cmp : public condition
{
	int email;
	int partial;
	char *from;
public:
	cond_sender_cmp(void);
	cond_sender_cmp(class cond_sender_cmp *,char *);
	~cond_sender_cmp();

	class cond_sender_cmp *clone(char *);
	int work(class mail *);
	int load(tokenizer *);
	int save(tokenizer *);

	group *makeprefs(void);
	void setupprefs(group *);
	void readprefs(group *);
};

cond_sender_cmp::cond_sender_cmp(void) : condition("sender (from)","Jrgen Schmitz")
{
	from=(char*)malloc(MAXLEN+1);
	from[0]=0;
}
cond_sender_cmp::cond_sender_cmp(cond_sender_cmp *c,char *n) : condition(c,n)
{ // copy global values
	from=(char*)malloc(MAXLEN+1);
	from[0]=0;
	email=0;
	partial=0;
}
cond_sender_cmp::~cond_sender_cmp()
{
	if (from) free(from);
}
cond_sender_cmp *cond_sender_cmp::clone(char *n)
{
	return new cond_sender_cmp(this,n);
}
int cond_sender_cmp::work(mail *m)
{
	int (*cmpf)(char*,char*)=&strcncmp;
	if (partial)
		cmpf=&strscncmp;
	if (email)
		return !cmpf(from,m->from_email());
	else
		return !cmpf(from,m->from());
}
int cond_sender_cmp::load(tokenizer *tk)
{
	int i=0;
	while (i<tk->Count()-1)
	{
		if (!strcscmp("email",(*tk)[i]))
		{
			email=1;
		}
		else if (!strcscmp("name",(*tk)[i]))
		{
			email=0;
		}
		else if (!strcscmp("partial",(*tk)[i]))
		{
			partial=1;
		}
		i++;
	}
	xc_strncpy(from,(*tk)[tk->Count()-1],MAXLEN+1);
	return 0;
}
int cond_sender_cmp::save(tokenizer *tk)
{
	if (partial)
		tk->append(new token("partial"));

	if (email)
		tk->append(new token("email"));
	else
		tk->append(new token("name"));
	tk->append(new token(from));
	return 0;
}

group *cond_sender_cmp::makeprefs(void)
{
	group *gp=new group;
	gp->Name("group");
	gp->Vert();
	 input *ip=new input;
	 ip->Name("from");
	 ip->MaxChars(MAXLEN);
	 ip->Text("From");
	gp->Add(ip);
	 choice *ch=new choice;
	 static char *pits[]={"compare first characters","look for partial string",NULL};
	 ch->Name("partial");
	 ch->Items(pits);
	gp->Add(ch,"");
	 checkbox *cb=new checkbox;
	 cb->Name("email");
	 cb->Text("compare sender email address");
	gp->Add(cb,"");
	return gp;
}
void cond_sender_cmp::setupprefs(group *g)
{
	((input*)FindObject(g->FullName(),".from"))->Default(from);
	((checkbox*)FindObject(g->FullName(),".email"))->State(email?PRESSED:RELEASED);
	((choice*)FindObject(g->FullName(),".partial"))->Selected(partial?1:0);
}
void cond_sender_cmp::readprefs(group *g)
{
	((input*)FindObject(g->FullName(),".from"))->Input(from);
	email=(((checkbox*)FindObject(g->FullName(),".email"))->State()==PRESSED)?1:0;
	partial=((choice*)FindObject(g->FullName(),".partial"))->Selected();
}

static cond_sender_cmp csnc;

// --------------------------------------------------------------------------

class cond_recipient_cmp : public condition
{
	char *to;
	int partial;
public:
	cond_recipient_cmp(void);
	cond_recipient_cmp(class cond_recipient_cmp *,char *);
	~cond_recipient_cmp();

	class cond_recipient_cmp *clone(char *);
	int work(class mail *);
	int load(tokenizer *);
	int save(tokenizer *);

	group *makeprefs(void);
	void setupprefs(group *);
	void readprefs(group *);
};

cond_recipient_cmp::cond_recipient_cmp(void) : condition("recipient (to)","Jrgen Schmitz")
{
	to=(char*)malloc(MAXLEN+1);
	to[0]=0;
	partial=0;
}
cond_recipient_cmp::cond_recipient_cmp(cond_recipient_cmp *c,char *n) : condition(c,n)
{ // copy global values
	to=(char*)malloc(MAXLEN+1);
	to[0]=0;
	partial=0;
}
cond_recipient_cmp::~cond_recipient_cmp()
{
	if (to) free(to);
}
cond_recipient_cmp *cond_recipient_cmp::clone(char *n)
{
	return new cond_recipient_cmp(this,n);
}
int cond_recipient_cmp::work(mail *m)
{
	int (*cmpf)(char*,char*)=&strcncmp;
	if (partial)
		cmpf=&strscncmp;

	char *toe=strdup(m->to());
	mail::parse_from_address(toe);
	return !cmpf(to,toe);
}
int cond_recipient_cmp::load(tokenizer *tk)
{
	int i=0;
	while (i<tk->Count()-1)
	{
		if (!strcscmp("partial",(*tk)[i]))
		{
			partial=1;
		}
		i++;
	}
	xc_strncpy(to,(*tk)[tk->Count()-1],MAXLEN+1);
	return 0;
}
int cond_recipient_cmp::save(tokenizer *tk)
{
	if (partial)
		tk->append(new token("partial"));

	tk->append(new token(to));
	return 0;
}

group *cond_recipient_cmp::makeprefs(void)
{
	group *gp=new group;
	gp->Name("group");
	gp->Vert();
	 input *ip=new input;
	 ip->Name("to");
	 ip->MaxChars(MAXLEN);
	 ip->Text("To (email address)");
	gp->Add(ip);
	 choice *ch=new choice;
	 static char *pits[]={"compare first characters","look for partial string",NULL};
	 ch->Name("partial");
	 ch->Items(pits);
	gp->Add(ch,"");
	return gp;
}
void cond_recipient_cmp::setupprefs(group *g)
{
	((input*)FindObject(g->FullName(),".to"))->Default(to);
	((choice*)FindObject(g->FullName(),".partial"))->Selected(partial?1:0);
}
void cond_recipient_cmp::readprefs(group *g)
{
	((input*)FindObject(g->FullName(),".to"))->Input(to);
	partial=((choice*)FindObject(g->FullName(),".partial"))->Selected();
}

static cond_recipient_cmp crcc;
