/* 
 * This file is part of XCmail and is subject to the 
 * XCmail license agreement
 */

/* $Id: filterlist.cxx,v 1.17 2001/10/10 17:11:57 schmitzj Exp $ */


#include "../configure.h" 
#include "simple_filter.h"
#include "../../src/mailer.h"
#include "../../src/strbuf.h"
#include "filter.h"
#include "condition.h"

filterlist::filterlist(simple_filter *p) : workabledlist()
{
	parent=p;
}
filterlist::~filterlist()
{
}

int filterlist::load(tokenizer *tk)
{ // <name> COND <cName>+ ORDER <jName>
	filter *f=new filter((*tk)[0],parent);
	AddTail(f);
	tk->remove(0);
	return f->load(tk);
}
int filterlist::save(tokenizer *tk,int l)
{
	filter *n=(filter*)Find(l+1);
	if (n)
	{
		tk->append(new token(n->getname())); 
		return workabledlist::save(tk,l);
	}
	return 0;
}
// ----------------------------------------------------------------------------

class filter_visitor : public named_visitor
{
public:
	int visit(workablenode *wn,class mail *m)
	{
		return ((condition*)wn)->work(m);
	};
};
filter::filter(char *n,class simple_filter *p) : workablenode(n), conds(p->getCondlst()), buf()
{
	parent=p;
	order=NULL;
	enabled=0;
}
filter::~filter()
{
	namenode *nn;
	if (order) delete order;
	while (nn=(namenode*)conds.RemTail())
	{
		delete nn;
	}
}
int filter::load(tokenizer *tk)
{ // [ENABLE] COND <cName>+ ORDER <jName>
	int md=0;
	for (int i=0;i<tk->count();i++)
	{
		if (md==0 && !strcmp("ENABLE",(*tk)[i]))
		{
			enabled=1;
		}
		else if (md==0 && !strcmp("COND",(*tk)[i]))
		{
			md=1;
		}
		else if ((md==0 || md==1) && !strcmp("ORDER",(*tk)[i]))
		{
			md=2;
		}
		else if (md==1)
		{ // a new condition
			namenode *nn=new namenode((*tk)[i]);
			conds.AddTail(nn);
		}
		else if (md==2)
		{ // the order
			order=new namenode((*tk)[i]);
			md=0;
		}
	}
}
int filter::save(tokenizer *tk)
{
	if (enabled)
		tk->append(new token("ENABLE"));

	if (conds.Count()>0)
	{
		tk->append(new token("COND"));
		conds.save(tk,0);
	}
	if (order)
	{
		tk->append(new token("ORDER"));
		tk->append(new token(*order));
	}
	return 0;
}

int filter::work(class mail *m)
{ // check conds & start job
	filter_visitor v;
	if (enabled && conds.workAnd(m,&v))
	{
		class order *o=parent->getOrder(order);
		return o?o->work(m):0;
	}
	return 0;
}

int filter::getenable(void)
{
	return enabled;
}
void filter::setenable(int e)
{
	enabled=e!=0;
}
char *filter::getcondname(void)
{
	namenode *nn=(namenode*)conds.head;

	buf.clear();
	while (nn->next)
	{
		buf+=(char*)*nn;
		nn=(namenode*)nn->next;
		if (nn->next) buf+=",";
	}
	return buf();
}
char *filter::getordername(void)
{
	if (order)
		return (char*)*order;
	else
		return "";
}
void filter::renameRef(char *old,char *newn)
{
	if (order)
	{
		if (!strcmp(order->text,old))
		{
			order->setname(newn);
		}
	}
}
