/* 
 * This file is part of XCmail and is subject to the 
 * XCmail license agreement
 */

/* $Id: misc.h,v 1.14 2001/10/10 17:11:57 schmitzj Exp $ */


#ifndef MISC_H
#define MISC_H

#include "../include/plugins.h"
#include "../include/filter_spec.h"
#include <Xclasses/Xclasses.h>
#include "../../src/stringTokenizer.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

class saveablenode : public node
{
	char	*name;
public:
	saveablenode(char *n);
	virtual ~saveablenode();

	virtual int load(tokenizer *);
	virtual int save(tokenizer *);

	char *getname(void) { return name; };
	void setname(char *);

	virtual void renameRef(char *old,char *newn);
};

class saveabledlist : public dlist
{
public:
	saveabledlist(void);
	virtual ~saveabledlist();

	virtual int load(tokenizer *);
	virtual int save(tokenizer *,int);

	void renameRef(char *old,char *newn);
};


class workablenode : public saveablenode
{
public:
	workablenode(char *n);
	virtual ~workablenode();

	virtual int work(class mail *);
};

class workabledlist : public saveabledlist
{
public:
	workabledlist(void);
	virtual ~workabledlist();

	virtual int workOr(class mail *);
	virtual int workAnd(class mail *);
};

class namenode : public node
{
public:
	namenode(char *n)
	{
		if (n)
			text=strdup(n);
		else
			text=strdup("");
	};
	namenode(namenode *n)
	{
		if (n->text)
			text=strdup(n->text);
		else
			text=strdup("");
	};
	virtual ~namenode()
	{
		if (text)
			free(text);
	}
	operator char*()
	{
		return text;
	};
	void setname(char *n)
	{
		if (text)
			free(text);
		text=strdup(n);
	};
	char *getname(void)
	{
		return text;
	};
};

class named_visitor
{
public:
	virtual int visit(workablenode *,class mail *)=0;
};
class nameabledlist : public saveabledlist
{
	saveabledlist	*worklist;
public:
	nameabledlist(saveabledlist *);
	virtual ~nameabledlist();

	virtual int workOr(class mail *,named_visitor *);
	virtual int workAnd(class mail *,named_visitor *);

	int save(tokenizer *,int);
};

class cfg_plugin : public plugins, public workablenode
{
public:
	cfg_plugin(char *name,char *author,enum PLUGINTYPE t,unsigned int flags);
	cfg_plugin(class cfg_plugin *,char *n);
	virtual ~cfg_plugin();

	virtual class cfg_plugin *clone(char *n)=0;		// clone plugin

	virtual int work(class mail *)=0;				// do the job
	virtual int load(tokenizer *)=0;				// gets parameters
	virtual int save(tokenizer *)=0;				// sets parameters

	virtual group *makeprefs(void)=0;				// NULL if no settings
	virtual void setupprefs(group *)=0;
	virtual void readprefs(group *)=0;
};

#define FILTER_VERSION		"1.0"

#define FILTERNAMELEN		40
#define CONDITIONNAMELEN	40
#define JOBSNAMELEN			40
#define PREFSFILENAME		"simple_filter-preferences"
// der Name wird in der Final gendert, es wird vordefinierte Orders
// geben, die dann jeder beim ersten Start hat - auch Beta-Nutzer

#endif
