/* 
 * This file is part of XCmail and is subject to the 
 * XCmail license agreement
 */

/* $Id: simple_filter.cxx,v 1.37 2001/10/10 17:11:57 schmitzj Exp $ */


#include "../configure.h" 
#include <Xclasses/Xclasses.h>
#include "../../src/pluginsmgr.h"
#include "../../src/xc_snprintf.h"
#include "simple_filter.h"
#include "filter.h"
#include "condition.h"
#include "job.h"

static unsigned int ccount=0;

simple_filter::simple_filter(void) : filter_spec("simple filter","Jrgen Schmitz"), flst(this), jlst(this), clst(this), blst(this)
{
	allconds=NULL;
	alljob=NULL;

	j_window=NULL;
	c_window=NULL;
}
simple_filter::~simple_filter()
{
	delete c_window;
	delete j_window;
	delete [] alljob;
	delete [] allconds;
}

plugins *simple_filter::findPlugin(char *name,enum plugins::PLUGINTYPE t)
{
	plugins *pl=NULL;
	while (pl=prgplugins->nextOfType(t,pl))
	{
		if (!strcmp(pl->plugin_name(),name))
			return pl;
	}
	return NULL;
}
class job *simple_filter::findJob(char *name)
{
	return (job*)findPlugin(name,PT_SJob);
}
class condition *simple_filter::findCondition(char *name)
{
	return (condition*)findPlugin(name,PT_SCondition);
}

int simple_filter::save(void)
{
	FILE *f;
	char buf[200];
	strcpy(buf,settingspath());
	strcat(buf,PREFSFILENAME);

	if (f=fopen(buf,"w"))
	{
		tokenizer tk(NULL," \t"," \t\n\r");
		int i;

		fprintf(f,"## This is a generated file, Do not edit!\n");
		i=0;
		while (i=flst.save(&tk,i))
		{
			tk.insert(new token("FILTER"),0);
			fprintf(f,"%s\n",tk.string());
			tk.removeall();
		}

		i=0;
		while (i=clst.save(&tk,i))
		{
			tk.insert(new token("CONDITION"),0);
			fprintf(f,"%s\n",tk.string());
			tk.removeall();
		}
		
		i=0;
		while (i=jlst.save(&tk,i))
		{
			tk.insert(new token("ORDER"),0);
			fprintf(f,"%s\n",tk.string());
			tk.removeall();
		}

		i=0;
		while (i=blst.save(&tk,i))
		{
			tk.insert(new token("JOB"),0);
			fprintf(f,"%s\n",tk.string());
			tk.removeall();
		}
		fclose(f);
	}
}
int simple_filter::load(void)
{
	plugins *pl=NULL;

	int cond_count=0;
	while (pl=prgplugins->nextOfType(PT_SCondition,pl)) cond_count++;
	allconds=new char*[cond_count+1];

	cond_count=0;
	pl=NULL;
	while (pl=prgplugins->nextOfType(PT_SCondition,pl))
	{
		allconds[cond_count]=pl->plugin_name();
		cond_count++;
	}
	allconds[cond_count]=NULL;


	int job_count=0;
	while (pl=prgplugins->nextOfType(PT_SJob,pl)) job_count++;
	alljob=new char*[job_count+1];

	job_count=0;
	pl=NULL;
	while (pl=prgplugins->nextOfType(PT_SJob,pl))
	{
		alljob[job_count]=pl->plugin_name();
		job_count++;
	}
	alljob[job_count]=NULL;


	FILE *f;
	int buf_size=50000;
	char *buf=new char[buf_size];
	strcpy(buf,settingspath());
	strcat(buf,PREFSFILENAME);

	if (f=fopen(buf,"r"))
	{
		tokenizer tk(NULL," \t"," \t\n\r");
		while (fgets(buf,buf_size,f)>0)
		{
			if (buf[0]!='#')
			{
				tk.string(buf);
				if (!strcmp(tk[0],"FILTER"))
				{ // FILTER <name> COND <cName>+ ORDER <jName>
					tk.remove(0);
					flst.load(&tk);
				}
				else if (!strcmp(tk[0],"CONDITION"))
				{ // CONDITION <cName> <pluginName> <parameters>
					tk.remove(0);
					clst.load(&tk);
				}
				else if (!strcmp(tk[0],"ORDER"))
				{ // ORDER <jName> <bName>+
					tk.remove(0);
					jlst.load(&tk);
				}
				else if (!strcmp(tk[0],"JOB"))
				{ // JOB <bName> <pluginName> <parameter>
					tk.remove(0);
					blst.load(&tk);
				}
			}
		}
		fclose(f);
	}
	delete [] buf;
}
int simple_filter::filter(class mail *m) // 0: add mail, 1: don't add, free memory
{
	return flst.workOr(m);
}

// GUI
group *simple_filter::genprefs(void)
{
	bordergroup *mgp=new bordergroup;
	mgp->Name("simple_filter");
	mgp->Vert();
	mgp->Title("Simple filter");
	 rubbergroup *gp1=new rubbergroup;
	 gp1->Name(NULL);
	 gp1->Horiz();
	  group *gp2=new group;
	  gp2->Name("listgroup");
	  gp2->Vert();
	   listview *lv=new listview;
	   lv->Name("filterlist");
	   lv->ShowSelected();
	   lv->HelpText("installed filters");
	   lv->Callback(this,(CALLBACK)&simple_filter::lvhandler);
//	   lv->SetHook(this,(SETHOOK)&simple_filter::lvhook,NULL);
	   saveablenode *n=(saveablenode*)flst.head;
	   while (n->next)
	   {
			lv->AddAtEnd(n->getname());
		 	n=(saveablenode*)n->next;
	   }
	  gp2->Add(lv);
	   group *gp3=new group;
	   gp3->Horiz();
	   gp3->Name(NULL);
	    fbutton *fb1=new fbutton;
	    fb1->Name("addbutton");
	    fb1->Text("_Add new");
	    fb1->Callback(this,(CALLBACK)&simple_filter::lvhandler);
	   gp3->Add(fb1);
	    fb1=new fbutton;
	    fb1->Name("delbutton");
	    fb1->Text("_Delete");
	    fb1->Callback(this,(CALLBACK)&simple_filter::lvhandler);
		fb1->Lock();
	   gp3->Add(fb1);
	  gp2->Add(gp3);
	 gp1->Add(gp2);
	  group *sgp1=new group;
	  sgp1->Name("datagroup");
	  sgp1->Vert();
	   checkbox *cb=new checkbox;
	   cb->Name("filterenable");
	   cb->Text("_Enable filter");
	   cb->Callback(this,(CALLBACK)&simple_filter::lvhandler);
	   cb->Lock();
	  sgp1->Add(cb,"");
	  sgp1->YDistance(5);
	   input *ip=new input;
	   ip->Name("filtername");
	   ip->MaxChars(FILTERNAMELEN);
	   ip->Text("_Name");
	   ip->Lock();
	   ip->Callback(this,(CALLBACK)&simple_filter::lvhandler);
	  sgp1->Add(ip);
	   gp2=new group;
	   gp2->Horiz();
	   gp2->Name(NULL);
	   gp2->UseSmallest();
	    output *op=new output;
	    op->Name("conditionname");
	   gp2->Add(op);
	    fb1=new fbutton;
	    fb1->Name("conditionselect");
	    fb1->Text("edit...");
	    fb1->Weight(1);
		fb1->HelpText("edit filter condition");
	    fb1->Callback(this,(CALLBACK)&simple_filter::condhandler);
		fb1->Lock();
	   gp2->Add(fb1);
	  sgp1->Add(gp2,"Condition");
	   gp2=new group;
	   gp2->Horiz();
	   gp2->Name(NULL);
	   gp2->UseSmallest();
	    op=new output;
	    op->Name("ordername");
	   gp2->Add(op);
	    fb1=new fbutton;
	    fb1->Name("orderselect");
	    fb1->Text("edit...");
	    fb1->Weight(1);
		fb1->HelpText("edit/choose filter order list");
	    fb1->Callback(this,(CALLBACK)&simple_filter::orderhandler);
		fb1->Lock();
	   gp2->Add(fb1);
	  sgp1->Add(gp2,"Order");
	   Xclasses_text *txt=new Xclasses_text;
	   txt->Name("info");
	   txt->Center();
	   txt->Text("\n{FONT_BIG}XCmail{/FONT}\n\nsimple filter - easy to use\n\nVersion: "FILTER_VERSION"\n");
//	  sgp1->Add(txt,"");
	 gp1->Add(sgp1);
	mgp->Add(gp1);

	dellock=0;
	old_selected=0;
	return mgp;
}
void simple_filter::readprefs(group *g)
{
	if (c_window)
	{
		c_window->close(NULL);
		delete c_window;
		c_window=NULL;
	}
	if (j_window)
	{
		j_window->close(NULL);
		delete j_window;
		j_window=NULL;
	}
	if ((appwindow*)FindObject(mygroup->FullName(),".orderwindow"))
		orderwin_close(NULL);
	checkvalues(mygroup);
}

int simple_filter::lvhook(struct hook_data *hd)
{
//	if (!enabled)
//	{
//		XSetForeground(hd->drawable->Xdisplay(),hd->drawable->col_shadow);
//	}
	return 1;
}
void simple_filter::main_lockcheck(group *g)
{
	if (((listview*)FindObject(g->FullName(),".filterlist"))->Selected())
	{
		if (!dellock) ((fbutton*)FindObject(g->FullName(),".listgroup.delbutton"))->Unlock();
		if (!dellock) ((fbutton*)FindObject(g->FullName(),".listgroup.addbutton"))->Unlock();
		((fbutton*)FindObject(g->FullName(),".datagroup.conditionselect"))->Unlock();
		((fbutton*)FindObject(g->FullName(),".datagroup.orderselect"))->Unlock();
		((input*)FindObject(g->FullName(),".datagroup.filtername"))->Unlock();
		((checkbox*)FindObject(g->FullName(),".datagroup.filterenable"))->Unlock();
	}
	else
	{
		((fbutton*)FindObject(g->FullName(),".listgroup.delbutton"))->Lock();
		((fbutton*)FindObject(g->FullName(),".datagroup.conditionselect"))->Lock();
		((fbutton*)FindObject(g->FullName(),".datagroup.orderselect"))->Lock();
		((input*)FindObject(g->FullName(),".datagroup.filtername"))->Lock();
		((checkbox*)FindObject(g->FullName(),".datagroup.filterenable"))->Lock();
	}
}
void simple_filter::checkvalues(group *g)
{ // alte Werte (Namen, etc.) prfen und in vorher gewhlts bernehmen
	class filter *n=filter_selected;
	if (old_selected)
	{
		char buf[FILTERNAMELEN+1];
		n->setenable(((checkbox*)FindObject(g->FullName(),".datagroup.filterenable"))->State()==PRESSED?1:0);
		((input*)FindObject(g->FullName(),".datagroup.filtername"))->Input(buf);
		n->setname(buf);
		((listview*)FindObject(g->FullName(),".filterlist"))->Replace(old_selected,n->getname());
	}
}
void simple_filter::showvalues(group *g)
{
	class filter *n=filter_selected;
	if (old_selected)
	{
		((checkbox*)FindObject(g->FullName(),".datagroup.filterenable"))->State(n->getenable()?PRESSED:RELEASED);
		((input*)FindObject(g->FullName(),".datagroup.filtername"))->Default(n->getname());
		((output*)FindObject(g->FullName(),".datagroup.conditionname"))->Output(n->getcondname());
		((output*)FindObject(g->FullName(),".datagroup.ordername"))->Output(n->getordername());
	}
}
void simple_filter::lvhandler(gadget *g,int i)
{
	if (!strcmp(g->Name(),"filterlist"))
	{ // Name gendert prfen!
		checkvalues(mygroup);

		old_selected=((listview*)g)->Selected();
		filter_selected=(class filter*)flst.Find(old_selected);
		showvalues(mygroup);
	}
	else if (!strcmp(g->Name(),"addbutton"))
	{
		listview *lv=(listview*)FindObject(mygroup,".filterlist");

		checkvalues(mygroup);
		class filter *f=new class filter("new filter",this);
		flst.AddTail(f);
		lv->AddAtEnd(f->getname());
		lv->Selected(flst.Count());
		old_selected=flst.Count();
		filter_selected=f;
		showvalues(mygroup);
	}
	else if (!strcmp(g->Name(),"delbutton"))
	{
		listview *lv=(listview*)FindObject(mygroup,".filterlist");

		class filter *n=filter_selected;
		if (n)
		{
			flst.Remove(n);
			lv->Remove(lv->Selected());
			lv->Selected(0);
			old_selected=0;
		    namenode *c;
	   		while (c=(namenode*)n->conds.RemTail())
			{
				condition *co=(condition*)(workablenode*)clst.FindText((char*)*c);
				if (co)
				{
					clst.Remove((workablenode*)co);
					delete co;
				}
				delete c;
			}
			delete n;
		}
	}
	else if (!strcmp(g->Name(),"filterenable"))
	{
		checkvalues(mygroup);
	}
	else if (!strcmp(g->Name(),"filtername"))
	{
		checkvalues(mygroup);
	}
	main_lockcheck(mygroup);
}
void simple_filter::condhandler(gadget *g,int i)
{
	((fbutton*)FindObject(mygroup->FullName(),".listgroup.delbutton"))->Lock();
	((fbutton*)FindObject(mygroup->FullName(),".listgroup.addbutton"))->Lock();
	((listview*)FindObject(mygroup->FullName(),".filterlist"))->Lock();
	dellock++;
	g->Lock();

	// open condition window
	int x,y;
	GetPositionOnRoot(g->display(),g->gg_win(),x,y);

	c_window=new c_cfg(this,mygroup,filter_selected,allconds);
	c_window->open(x,y);
}
void simple_filter::SUFcondhandler(void)
{
	((fbutton*)FindObject(mygroup->FullName(),".datagroup.conditionselect"))->Unlock();
	if (!--dellock)
	{
		((listview*)FindObject(mygroup->FullName(),".filterlist"))->Unlock();
		((fbutton*)FindObject(mygroup->FullName(),".listgroup.delbutton"))->Unlock();
		((fbutton*)FindObject(mygroup->FullName(),".listgroup.addbutton"))->Unlock();
	}
	delete c_window;
	c_window=NULL;

	checkvalues(mygroup);
	showvalues(mygroup);
}
void simple_filter::orderhandler(gadget *g,int i)
{
	((fbutton*)FindObject(mygroup->FullName(),".listgroup.delbutton"))->Lock();
	((fbutton*)FindObject(mygroup->FullName(),".listgroup.addbutton"))->Lock();
	((listview*)FindObject(mygroup->FullName(),".filterlist"))->Lock();
	dellock++;
	g->Lock();

	// open order window
	int x,y;
	GetPositionOnRoot(g->display(),g->gg_win(),x,y);
	openorder(x,y);
}
void simple_filter::SUForderhandler(class order *sel)
{
	((fbutton*)FindObject(mygroup->FullName(),".datagroup.orderselect"))->Unlock();
	if (!--dellock)
	{
		((listview*)FindObject(mygroup->FullName(),".filterlist"))->Unlock();
		((fbutton*)FindObject(mygroup->FullName(),".listgroup.delbutton"))->Unlock();
		((fbutton*)FindObject(mygroup->FullName(),".listgroup.addbutton"))->Unlock();
	}
	checkvalues(mygroup);
	delete filter_selected->order;
	filter_selected->order=NULL;
	if (sel)
	{
		filter_selected->order=new namenode(sel->getname());
	}
	showvalues(mygroup);
}

// ----------------------------------------------------------------------------
void simple_filter::openorder(int x,int y)
{
	old_oselected=0;

	appwindow *aw=new appwindow;
	aw->Name("orderwindow");
	aw->ParentClass(mygroup);

	aw->InitSize(x,y,70);
	aw->Init();
	aw->Title("Order");
	aw->IconTitle("Order");
	aw->TransientFor((appwindow*)mygroup->ParentClassType("appwindow"));
	aw->Adjust(x,y,10,10,ADJUST_CENTER|ADJUST_MIDDLE);

	spacegroup *mgp=new spacegroup;
	mgp->Name("mainordergroup");
	mgp->Vert();
	mgp->Border(4);
	 rubbergroup *rgp1=new rubbergroup;
	 rgp1->Horiz();
	 rgp1->Name(NULL);
	  listview *lv=new listview;
	  lv->Name("orderlist");
	  lv->ShowSelected();
	  lv->Callback(this,(CALLBACK)&simple_filter::orderwin_lvhandler);
	  order *n=(order*)jlst.head;
	  while (n->next)
	  {
			lv->AddAtEnd(n->getname());
		 	n=(order*)n->next;
	  }
	  if (filter_selected->getordername())
	  {
		  n=(order*)jlst.FindText(filter_selected->getordername());
		  if (n)
		  {
		  	  old_oselected=jlst.Find(n);
			  lv->Selected(old_oselected);
		  }
	  }
	 rgp1->Add(lv);
	  group *gp2=new group;
	  gp2->Name(NULL);
	  gp2->Vert();
	   input *ip=new input;
	   ip->Name("ordername");
	   ip->MaxChars(FILTERNAMELEN);
	   ip->Text("Name");
	   ip->Callback(this,(CALLBACK)&simple_filter::orderwin_lvhandler);
	  gp2->Add(ip);
	   lv=new listview;
	   lv->Name("joblist");
	   lv->ReadOnly();
	  gp2->Add(lv);
	  gp2->Add(new ruler);
	   fbutton *fb=new fbutton;
	   fb->Name("addorder");
	   fb->Text("_Create new order...");
	   fb->Callback(this,(CALLBACK)&simple_filter::orderwin_lvhandler);
	  gp2->Add(fb);
	   fb=new fbutton;
	   fb->Name("editorder");
	   fb->Text("_Edit order...");
	   fb->Lock();
	   fb->Callback(this,(CALLBACK)&simple_filter::orderwin_lvhandler);
	  gp2->Add(fb);
	   fb=new fbutton;
	   fb->Name("delorder");
	   fb->Text("_Remove order");
	   fb->Lock();
	   fb->Callback(this,(CALLBACK)&simple_filter::orderwin_lvhandler);
	  gp2->Add(fb);
	 rgp1->Add(gp2);
	mgp->Add(rgp1);

	mgp->Add(new ruler);
	 group *gp1=new group;
	 gp1->Name("");
	 gp1->DialogMode();
	 gp1->Horiz();
	  fb=new fbutton;
	  fb->Name("okorder");
	  fb->Text(" _Set ");
	  fb->Callback(this,(CALLBACK)(void (simple_filter::*)(gadget *,int))&simple_filter::orderwin_close);
	 gp1->Add(fb);
	mgp->Add(gp1);

	aw->Callback(this,(CLOSECALLBACK)(void (simple_filter::*)(appwindow *))&simple_filter::orderwin_close);
	aw->SetGroup(mgp);
	aw->Create();
	odellock=0;
	orderwin_showvalues();
	orderwin_main_lockcheck();
}
void simple_filter::orderwin_showvalues(void)
{
	appwindow *aw=(appwindow*)FindObject(mygroup->FullName(),".orderwindow");

	if (old_oselected)
	{
		listview *jlv=(listview*)FindObject(aw->FullName(),".joblist");
		class order *n=(order*)jlst.Find(old_oselected);
		((input*)FindObject(aw->FullName(),".ordername"))->Default(n->getname());
		jlv->RemoveAll();

	    namenode *c=(namenode*)n->myorder.head;
		char buf[100];
   		while (c->next)
		{
			job *j=(job*)(workablenode*)blst.FindText((char*)*c);
			xc_snprintf(buf,sizeof(buf),"%s: %s",j?j->plugin_name():"---",(char*)*c);
			jlv->AddAtEnd(buf);
			c=(namenode*)c->next;
		}
	}
}
void simple_filter::orderwin_checkvalues(void)
{
	appwindow *aw=(appwindow*)FindObject(mygroup->FullName(),".orderwindow");

	if (old_oselected)
	{
		class order *n=(order*)jlst.Find(old_oselected);
		char buf[FILTERNAMELEN+1];
		((input*)FindObject(aw->FullName(),".ordername"))->Input(buf);
		// Verweise anpassen
		flst.renameRef(n->text,buf);
		blst.renameRef(n->text,buf);	// JOBs mit Verweisen auf ORDERs?

		n->setname(buf);
		((listview*)FindObject(aw->FullName(),".orderlist"))->Replace(old_oselected,n->getname());
	}
}
void simple_filter::orderwin_main_lockcheck(void)
{
	appwindow *aw=(appwindow*)FindObject(mygroup->FullName(),".orderwindow");

	if (old_oselected && !odellock)
	{
		((gadget*)FindObject(aw->FullName(),".ordername"))->Unlock();
		((gadget*)FindObject(aw->FullName(),".editorder"))->Unlock();
		((gadget*)FindObject(aw->FullName(),".delorder"))->Unlock();
	}
	else
	{
		((gadget*)FindObject(aw->FullName(),".ordername"))->Lock();
		((gadget*)FindObject(aw->FullName(),".editorder"))->Lock();
		((gadget*)FindObject(aw->FullName(),".delorder"))->Lock();
	}
	if (odellock)
	{
		((gadget*)FindObject(aw->FullName(),".orderlist"))->Lock();
		((gadget*)FindObject(aw->FullName(),".addorder"))->Lock();
		((gadget*)FindObject(aw->FullName(),".okorder"))->Lock();
	}
	else
	{
		((gadget*)FindObject(aw->FullName(),".orderlist"))->Unlock();
		((gadget*)FindObject(aw->FullName(),".addorder"))->Unlock();
		((gadget*)FindObject(aw->FullName(),".okorder"))->Unlock();
	}
}
void simple_filter::orderwin_close(gadget *,int)
{
	orderwin_checkvalues();
	appwindow *aw=(appwindow*)FindObject(mygroup->FullName(),".orderwindow");
	aw->Close();
	DeleteObjectTree(aw->FullName(),0);
	SUForderhandler((order*)jlst.Find(old_oselected));
}
void simple_filter::orderwin_close(appwindow *aw)
{
	if (!odellock) orderwin_close(NULL,0);
}
void simple_filter::orderwin_lvhandler(gadget *g,int)
{
	if (!strcmp(g->Name(),"orderlist"))
	{
		orderwin_checkvalues();
		listview *lv=(listview*)FindObject(mygroup->FullName(),".orderlist");
		old_oselected=lv->Selected();
		orderwin_showvalues();
		if (g->DoubleClick())
		{
			orderwin_lvhandler((gadget*)FindObject(mygroup->FullName(),".editorder"),0);
		}
	}
	else if (!strcmp(g->Name(),"ordername"))
	{
		orderwin_checkvalues();
	}
	else if (!strcmp(g->Name(),"addorder"))
	{
		char buf[50];
		orderwin_checkvalues();
		listview *lv=(listview*)FindObject(mygroup->FullName(),".orderlist");

		sprintf(buf,"order_%d%d",time(NULL),ccount++);
		class order *f=new class order(buf,this);

		jlst.AddTail(f);
		lv->AddAtEnd((char*)f->getname());
		old_oselected=jlst.Count();
		lv->Selected(old_oselected);
		orderwin_showvalues();

		odellock++;
		int x,y;
		GetPositionOnRoot(g->display(),g->gg_win(),x,y);

		j_window=new j_cfg(this,(group*)FindObject(mygroup->FullName(),".mainordergroup"),(order*)jlst.Find(old_oselected),alljob);
		j_window->open(x,y);
	}
	else if (!strcmp(g->Name(),"editorder"))
	{
		orderwin_checkvalues();

		odellock++;
		int x,y;
		GetPositionOnRoot(g->display(),g->gg_win(),x,y);

		j_window=new j_cfg(this,(group*)FindObject(mygroup->FullName(),".mainordergroup"),(order*)jlst.Find(old_oselected),alljob);
		j_window->open(x,y);
	}
	else if (!strcmp(g->Name(),"delorder"))
	{
		listview *lv=(listview*)FindObject(mygroup,".orderlist");

		class order *n=(order*)jlst.Find(old_oselected);
		if (n)
		{
			jlst.Remove(n);
			lv->Remove(lv->Selected());
			lv->Selected(0);
			old_oselected=0;
		    namenode *c;
	   		while (c=(namenode*)n->myorder.RemTail())
			{
				job *j=(job*)(workablenode*)blst.FindText((char*)*c);
				if (j)
				{
					blst.Remove((workablenode*)j);
					delete j;
				}
				delete c;
			}
			delete n;
		}
	}
	orderwin_main_lockcheck();
}
void simple_filter::orderwin_SUFlvhandler(void)
{
	delete j_window;
	j_window=NULL;
	odellock--;

	orderwin_checkvalues();
	orderwin_showvalues();
	orderwin_main_lockcheck();
}

static simple_filter sflt;

simple_filter *simple_filter::getMaster(void)
{
	return &sflt;
}

// ----------------------------------------------------------------------------

jc_cfg::jc_cfg(simple_filter *p,group *pg) : callable()
{
	parent=p;
	parentgroup=pg;
}
jc_cfg::~jc_cfg()
{
}
void jc_cfg::open(int x,int y)
{
	appwindow *aw=new appwindow;
	aw->Name("jcwindow");
	aw->ParentClass(parentgroup);

	aw->InitSize(x,y,70);
	aw->Init();
	aw->Title(title);
	aw->IconTitle(title);
	aw->TransientFor((appwindow*)parentgroup->ParentClassType("appwindow"));
	aw->Adjust(x,y,10,10,ADJUST_CENTER|ADJUST_MIDDLE);
	
	spacegroup *mgp=new spacegroup;
	mgp->Name("maingroup");
	mgp->Vert();
	mgp->Border(4);
	 rubbergroup *rg1=new rubbergroup;
	 rg1->Name(NULL);
	 rg1->Horiz();
	  group *gp1=new group;
	  gp1->Name("jcgroup");
	  gp1->Vert();
	   listview *lv=new listview;
	   lv->Name("jclist");
	   lv->ShowSelected();
	   lv->Callback(this,(CALLBACK)&jc_cfg::lvhandler);
	   namenode *n=(namenode*)nlst->head;
	   while (n->next)
	   {
			lv->AddAtEnd(*n);
		 	n=(namenode*)n->next;
	   }
	  gp1->Add(lv);
	   group *gp2=new group;
	   gp2->Name(NULL);
	   gp2->Horiz();
	    fbutton *fb=new fbutton;
	    fb->Name("jcaddbutton");
	    fb->Text("_Add");
		fb->Callback(this,(CALLBACK)&jc_cfg::lvhandler);
	   gp2->Add(fb);
	    fb=new fbutton;
	    fb->Name("jcdelbutton");
	    fb->Text("_Del");
		fb->Lock();
	    fb->Callback(this,(CALLBACK)&jc_cfg::lvhandler);
	   gp2->Add(fb);
	  gp1->Add(gp2);
	 rg1->Add(gp1);
	  gp1=new group;
	  gp1->Name("jcgroup");
	  gp1->Vert();
	   input *ip=new input;
	   ip->Name("jcname");
	   ip->MaxChars(FILTERNAMELEN);
	   ip->Lock();
	   ip->Callback(this,(CALLBACK)&jc_cfg::lvhandler);
	  gp1->Add(ip,"Name");
	   choice *ch=new choice;
	   ch->Name("jctype");
	   ch->Items(alljc);
	   ch->Lock();
	   ch->Callback(this,(CALLBACK)&jc_cfg::lvhandler);
	  gp1->Add(ch,"Type");
	   visible *vb=new visible;
	   vb->Name("jcopts");
	  gp1->Add(vb,"");
	 rg1->Add(gp1);
	mgp->Add(rg1);

	mgp->Add(new ruler);
	 gp1=new group;
	 gp1->Name("");
	 gp1->DialogMode();
	 gp1->Horiz();
	  fb=new fbutton;
	  fb->Name("okbutton");
	  fb->Text(" _Ok ");
	  fb->Callback(this,(CALLBACK)(void (jc_cfg::*)(gadget *,int))&jc_cfg::close);
	 gp1->Add(fb);
	mgp->Add(gp1);

	aw->Callback(this,(CLOSECALLBACK)(void (jc_cfg::*)(appwindow *))&jc_cfg::close);
	aw->SetGroup(mgp);
	aw->Create();
	old_selected=0;
	pl_group=NULL;
	old_jctype=-1;
}
void jc_cfg::showvalues(void)
{
	appwindow *aw=(appwindow*)FindObject(parentgroup->FullName(),".jcwindow");

//	if (old_selected)
	if (((listview*)FindObject(parentgroup->FullName(),".jclist"))->Selected())
	{
		int tp=0,i=0;
		char **c=alljc;
		while (sel_plugin && *c)
		{
			if (!strcmp(*c,sel_plugin->plugin_name()))
			{
				tp=i;
				break;
			}
			c++; i++;
		}

		((input*)FindObject(parentgroup->FullName(),".jcname"))->Default(*sel_name);
		((choice*)FindObject(parentgroup->FullName(),".jctype"))->Selected(tp);

		if (tp!=old_jctype)
		{
			((visible*)FindObject(parentgroup->FullName(),".jcopts"))->Apply(NULL);
			if (pl_group) DeleteObjectTree(pl_group->FullName());
			pl_group=NULL;
			if (sel_plugin)
			{
				pl_group=sel_plugin->makeprefs();
				if (pl_group)
				{
					((visible*)FindObject(parentgroup->FullName(),".jcopts"))->Apply(pl_group);
					pl_group->FullName();
				}
			}
			old_jctype=tp;
		}
		if (pl_group && sel_plugin)
		{
			sel_plugin->setupprefs(pl_group);
		}
	}
}
void jc_cfg::checkvalues(void)
{
	appwindow *aw=(appwindow*)FindObject(parentgroup->FullName(),".jcwindow");
	if (old_selected)
	{
		char buf[FILTERNAMELEN+1];
		((input*)FindObject(parentgroup->FullName(),".jcname"))->Input(buf);
		// check name in lst

		if (strcmp(buf,*sel_name))
		{
			if (!lst->FindText(buf))
			{
				sel_name->setname(buf);
				if (sel_plugin)
					sel_plugin->setname(buf);
				((listview*)FindObject(parentgroup->FullName(),".jclist"))->Replace(old_selected,buf);
			}
			else
			{
				((input*)FindObject(parentgroup->FullName(),".jcname"))->Default(*sel_name);
			}
		}
		if (sel_plugin && pl_group)
			sel_plugin->readprefs(pl_group);
	}
}
void jc_cfg::main_lockcheck(void)
{
	appwindow *aw=(appwindow*)FindObject(parentgroup->FullName(),".jcwindow");

	if (old_selected)
	{
		((gadget*)FindObject(parentgroup->FullName(),".jcname"))->Unlock();
		((gadget*)FindObject(parentgroup->FullName(),".jctype"))->Unlock();
		((gadget*)FindObject(parentgroup->FullName(),".jcdelbutton"))->Unlock();
	}
	else
	{
		((gadget*)FindObject(parentgroup->FullName(),".jcname"))->Lock();
		((gadget*)FindObject(parentgroup->FullName(),".jctype"))->Lock();
		((gadget*)FindObject(parentgroup->FullName(),".jcdelbutton"))->Lock();
		((visible*)FindObject(parentgroup->FullName(),".jcopts"))->Apply(NULL);
		if (pl_group) DeleteObjectTree(pl_group->FullName());
		pl_group=NULL;
	}
}
void jc_cfg::lvhandler(gadget *g,int)
{
	if (!strcmp(g->Name(),"jclist"))
	{
		checkvalues();
		old_selected=((listview*)g)->Selected();
		sel_name=(namenode*)nlst->Find(old_selected);
		sel_plugin=(cfg_plugin*)(saveablenode*)lst->FindText((char*)*sel_name);
		showvalues();
	}
	else if (!strcmp(g->Name(),"jcaddbutton"))
	{
		char buf[50];
		listview *lv=(listview*)FindObject(parentgroup->FullName(),".jclist");
		checkvalues();

		sprintf(buf,"%s_%d%d",prefix,time(NULL),ccount++);
		sel_name=new namenode(buf);
		sel_plugin=clone(0,buf);

		nlst->AddTail(sel_name);
		lst->AddTail((workablenode*)sel_plugin);

		lv->AddAtEnd((char*)*sel_name);
		old_selected=nlst->Count();
		lv->Selected(old_selected);

		old_jctype=-1;
		((choice*)FindObject(parentgroup->FullName(),".jctype"))->Selected(0);
		showvalues();
	}
	else if (!strcmp(g->Name(),"jcdelbutton"))
	{
		listview *lv=(listview*)FindObject(parentgroup->FullName(),".jclist");
		lv->Remove(old_selected);

		if (sel_name) nlst->Remove(sel_name);
		delete sel_name;
		if (sel_plugin) lst->Remove((workablenode*)sel_plugin);
		delete sel_plugin;

		lv->Selected(0);
		old_selected=0;
	}
	else if (!strcmp(g->Name(),"jcname"))
	{
		checkvalues();
	}
	else if (!strcmp(g->Name(),"jctype"))
	{
		int tp=((choice*)FindObject(parentgroup->FullName(),".jctype"))->Selected();
		checkvalues();
		if (tp!=old_jctype)
		{
			lst->Remove((workablenode*)sel_plugin);
			sel_plugin=clone(tp,(char*)*sel_name);
			lst->AddTail((workablenode*)sel_plugin);
			showvalues();
		}
	}
	main_lockcheck();
}
void jc_cfg::close(gadget *,int)
{
	checkvalues();
	appwindow *aw=(appwindow*)FindObject(parentgroup->FullName(),".jcwindow");
	aw->Close();
	DeleteObjectTree(aw->FullName(),0);
	finish();
}
void jc_cfg::close(appwindow *aw)
{
	close(NULL,0);
}

c_cfg::c_cfg(simple_filter *p,group *po,filter *f,char **a) : jc_cfg(p,po)
{
	title="Condition of filter";
	prefix="cond";
	lst=p->getCondlst();
	nlst=&f->conds;
	alljc=a;
}
c_cfg::~c_cfg()
{
}
void c_cfg::finish(void)
{
	parent->SUFcondhandler();
}
cfg_plugin *c_cfg::clone(int n,char *nme)
{
	cfg_plugin *pl=NULL,*r=NULL;

	while (pl=(cfg_plugin*)prgplugins->nextOfType(plugins::PT_SCondition,pl))
	{
		if (!strcmp(alljc[n],pl->plugin_name()))
		{
			r=pl->clone(nme);
			break;
		}
	}
	return r;
}

j_cfg::j_cfg(simple_filter *p,group *po,order *f,char **a) : jc_cfg(p,po)
{
	title="Jobs of order";
	prefix="job";
	lst=p->getJoblst();
	nlst=&f->myorder;
	alljc=a;
}
j_cfg::~j_cfg()
{
}
void j_cfg::finish(void)
{
	parent->orderwin_SUFlvhandler();
}
cfg_plugin *j_cfg::clone(int n,char *nme)
{
	cfg_plugin *pl=NULL,*r=NULL;

	while (pl=(cfg_plugin*)prgplugins->nextOfType(plugins::PT_SJob,pl))
	{
		if (!strcmp(alljc[n],pl->plugin_name()))
		{
			r=pl->clone(nme);
			break;
		}
	}
	return r;
}
