/* 
 * This file is part of XCmail and is subject to the 
 * XCmail license agreement
 */

/* $Id: trashmgr.cxx,v 2.7 2001/10/10 17:11:58 schmitzj Exp $ */


#include "../configure.h" 

#include "../../src/mailer.h"
#include "../../src/folder.h"
#include "trashmgr.h"

trashmgr::trashmgr(void) : mainplugin("Trash manager","Jrgen Schmitz",PLF_CONFIGURABLE,"Automatically removes old mails from trash folder")
{
	cfgsetup();
}
trashmgr::~trashmgr()
{
	delete cfg;
}

void trashmgr::init(void)
{
	switch (runmode)
	{
		case 0:	// start
			checktrash();
			break;
		case 1: // quit
			break;
		case 2: // menu
			addmenu(MPMENU_GENERAL,1,"check trash folder");
			break;
	}
}
void trashmgr::finish(void)
{
	switch (runmode)
	{
		case 1: // quit
			checktrash();
			break;
	}
}
void trashmgr::handle(int id)
{
	switch (id)
	{
		case 1:
			checktrash();
			break;
	}
}

static void slidertxt(int val, char *buf, int size)
{
	if (val>0)
		sprintf(buf,"%d",val);
	else
		strcpy(buf,"none");
}
group *trashmgr::makeconfig(void)
{
	group *gp=new group;
	gp->Name("trashmgr_group");
	gp->Vert();
	 static char *itemstxt[]={"at start","after quit","by menu",NULL};
	 choice *ch=new choice;
	 ch->Name("runmode");
	 ch->Items(itemstxt);
	 ch->Selected(runmode);
	 ch->HelpText("Change will work after restart!");
	gp->Add(ch,"check trash folder");
	 slider *sl=new slider;
	 sl->Name("daylimit");
	 sl->Horiz();
	 sl->MinValue(0);
	 sl->MaxValue(100);
	 sl->Value(timelimit);
	 sl->TextCallback(slidertxt);
	gp->Add(sl,"delete after days");
	return gp;
}
void trashmgr::readconfig(group *gp)
{
	runmode=((choice*)FindObject(gp->FullName(),".runmode"))->Selected();
	timelimit=((slider*)FindObject(gp->FullName(),".daylimit"))->Value();
}
void trashmgr::readconfigfile(void)
{
	cfg->Read();
}
void trashmgr::writeconfigfile(void)
{
	cfg->Write();
}

void trashmgr::checktrash(void)
{
	vfolder *v=NULL;
	int c=1;
	while (v=folderlist()->getnextfolderoftype(v,FOLDERTYPE_TRASH))
	{
		pstatus("Reading trash folder %d...",c);
		readerwin.UpdateEvent();
		if (!v->load())
		{
			mail *ml;
			int d,m,y,h,min,sec;
			time_t t,now,tl=timelimit*(60*60*24);

			time(&now);
			pstatus("Checking for old mail in %d mails...",v->count());

			for(int i=0;i<v->count();i++)
			{
				ml=v->getmail(i+1);
				t=ml->parsesavedate(d,m,y,h,min,sec);
				if (tl && (now-t)>tl)
				{
					ml->status(ml->status() | STATUS_DELETE);
				}
			}
			pstatus("Writing trash folder %d...",c);
			v->save();
			v->unload();
			pstatus("");
			c++;
		}
		else
			pstatus("Can't load folder %s!",v->g_foldername());
	}
}
void trashmgr::cfgsetup(void)
{
	static configopts cfgitems[] = {
		{"Estart,quit,menu:runmode",(char*)&runmode},
		{"I:daylimit",(char*)&timelimit},
		{NULL, 						NULL}
	};
	cfg = new config("~/.XCmail/plugin-preferences-trashmgr");
	cfg->SetItems(cfgitems);

	runmode=1;
	timelimit=0;
}

static trashmgr t1;
