/* 
 * This file is part of XCmail and is subject to the 
 * XCmail license agreement
 */

/* $Id: address_book.cxx,v 2.44 2002/07/31 11:21:40 schmitzj Exp $ */


#include "configure.h"

#include <stdio.h> 
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <Xclasses/Xclasses.h>
#include "address_book.h"
#include "xc_snprintf.h"
#include "mailer.h"
#include "stringTokenizer.h"


Address::Address(void) : node()
{
	for(int i=0;i<FIELDSIZE;i++)
	{
		fields[i]=NULL;
	}
	fields[AddressBook::UNIQUEID]=(char*)malloc(UNIQUE_ID_LEN);
	fields[AddressBook::UNIQUEID][0]=0;
	getUniqueID(fields[AddressBook::UNIQUEID]);
	char buf[20];
	xc_snprintf(buf,sizeof(buf),"%d",time(NULL));
	fields[AddressBook::ADDDATE]=strdup(buf);
}
Address::Address(Address &a) : node()
{
	for(int i=0;i<FIELDSIZE;i++)
	{
		fields[i]=a.fields[i]?strdup(a.fields[i]):NULL;
	}
	if (fields[AddressBook::UNIQUEID]) free(fields[AddressBook::UNIQUEID]);
	fields[AddressBook::UNIQUEID]=(char*)malloc(UNIQUE_ID_LEN);
	fields[AddressBook::UNIQUEID][0]=0;
	getUniqueID(fields[AddressBook::UNIQUEID]);
}
Address::Address(Address *a) : node()
{
	for(int i=0;i<FIELDSIZE;i++)
	{
		fields[i]=a->fields[i]?strdup(a->fields[i]):NULL;
	}
	if (fields[AddressBook::UNIQUEID]) free(fields[AddressBook::UNIQUEID]);
	fields[AddressBook::UNIQUEID]=(char*)malloc(UNIQUE_ID_LEN);
	fields[AddressBook::UNIQUEID][0]=0;
	getUniqueID(fields[AddressBook::UNIQUEID]);
}

Address::~Address()
{
	for(int i=0;i<FIELDSIZE;i++)
	{
		if (fields[i]) free(fields[i]);
	}
}
char *Address::getField(int n)
{
	if (n>=0 && n<FIELDSIZE)
		return fields[n]?fields[n]:(char*)"";
	else
		return (char*)"";
}
int Address::setField(int i,char *v)
{
	if (i>=0 && i<FIELDSIZE)
	{
		if (fields[i])
			free(fields[i]);
		fields[i]=NULL;

		if (v)
			fields[i]=strdup(v);
	}
	return 0;
}

static int sortaddress(struct node *n1,struct node *n2)
{
	char buf1[500],buf2[500];
	Address *a1=(Address*)n1,
			*a2=(Address*)n2;

	buf1[0]=0;
	xc_snprintf(buf1,sizeof(buf1),"%s%s%s",
				a1->getField(AddressBook::LASTNAME),
				a1->getField(AddressBook::FIRSTNAME),
				a1->getField(AddressBook::EMAIL));

	buf2[0]=0;
	xc_snprintf(buf2,sizeof(buf2),"%s%s%s",
				a2->getField(AddressBook::LASTNAME),
				a2->getField(AddressBook::FIRSTNAME),
				a2->getField(AddressBook::EMAIL));

	return strcscmp(buf1,buf2);
}

AddressBook::AddressBook(void) : dlist(), vcard()
{
}
AddressBook::~AddressBook()
{
	clearall();
}
void AddressBook::clearall(void)
{
	Address *a;
	while (a=(Address*)RemTail())
	{
		delete a;
	}
}
static char *abook_names[]={
"LASTNAME",
"FIRSTNAME",
"ALIAS",
"EMAIL",
"NOTE",
"QUICKMENU",
"HOMEPHONE",
"WORKPHONE",
"FAX",
"URL",
"ORGANISATION",
"ADDRESS",
"CITY",
"STATE",
"ZIP",
"COUNTRY",
"UNREMOVEABLE",
"FIXEDALIAS",
"MOBIL",
"UID",
"XFACE",
"ADDDATE",
"USEDATE",
"MODDATE",
};
int AddressBook::save(FILE *f)
{
	int r=0;
	Address *a=(Address*)head;
	fprintf(f,"#V2\n");
	fprintf(f,"# %s addressbook\n",PROGRAMNAME);
	fprintf(f,"# this is a generated file, do not edit\n");
	Sort(sortaddress);

	while (a->next)
	{
		fprintf(f,"# entry %d\n",Find(a));
		fprintf(f,"ADDRESS\n{\n");
		for(int i=0;i<FIELDSIZE;i++)
		{
			if (a->getField(i) && strlen(a->getField(i))>0)
			{
				token t(a->getField(i));
				fprintf(f,".%s:%s\n",abook_names[i],t.tokenize());
//				fprintf(f,"F%02d:%s\n",i,t.tokenize());
			}
		}
		fprintf(f,"}\n");
		a=(Address*)a->next;
	}
	return r;
}
int AddressBook::load(FILE *f)
{
	int r=0;
	int oldbook=-1;
	enum RMODE {NONE,ADDRESS} mode=NONE;
	char c[500];
	Address *a=NULL;

	int had_me=0,had_me_work=0;
	while (fgets(c,sizeof(c),f))
	{
		if (oldbook==-1 && !strcmp(c,"#V2\n"))
		{
			oldbook=0;
		}
		else if (oldbook==0  && c[0]!='#')
		{
			while (strlen(c)>0 && c[strlen(c)-1]=='\n') c[strlen(c)-1]=0;

			if (!strcncmp("ADDRESS",c))
			{
				a=new Address;
				AddTail(a);
				mode=ADDRESS;
			}
			if (mode==ADDRESS && c[0]=='F' && a)
			{
				tokenizer t(c+4,"\t","\t");
				c[3]=0;
				int n=atoi(c+1);
				a->setField(n,t[0]);
				if (n==ALIAS && !strcmp(t[0],"me"))
				{
					had_me++;
					if (had_me>1)
					{
						Remove(a);
						delete a;
						a=NULL;
					}
				}
				if (n==ALIAS && !strcmp(t[0],"me_work"))
				{
					had_me_work++;
					if (had_me_work>1)
					{
						Remove(a);
						delete a;
						a=NULL;
					}
				}
			}
			if (mode==ADDRESS && c[0]=='.' && a)
			{
				char *cp=c;
				while (*cp && *cp!=':') cp++;
				tokenizer t(cp+1,"\t","\t");
				*cp=0;
				int n=-1;
				for(int i=0;i<FIELDSIZE;i++)
				{
					if (!strcscmp(c+1,abook_names[i]))
					{
						n=i;
						break;
					}
				}
				if (n>=0)
					a->setField(n,t[0]);
				if (n==ALIAS && !strcmp(t[0],"me"))
				{
					had_me++;
					if (had_me>1)
					{
						Remove(a);
						delete a;
						a=NULL;
					}
				}
				if (n==ALIAS && !strcmp(t[0],"me_work"))
				{
					had_me_work++;
					if (had_me_work>1)
					{
						Remove(a);
						delete a;
						a=NULL;
					}
				}
			}
		}
		else if (c[0]!='#')
		{
			tokenizer t(c,"\t","");
			if (t.count()>1)
			{
				Address *a=new Address;
				if (a)
				{
					a->setField(ALIAS,t[0]);
					a->setField(LASTNAME,t[1]);
					a->setField(FIRSTNAME,t[2]);
					a->setField(EMAIL,t[3]);
					a->setField(QUICKMENU,t[4]);
					a->setField(NOTE,t[5]);
					AddTail(a);
				}
				oldbook=1;
			}
		}
	}
	Sort(sortaddress);
	return r;
}
int AddressBook::new_entry(void)
{
	Address *a=new Address;
	if (a)
	{
		AddTail(a);
		return Find(a);
	}
	return 0;
}
int AddressBook::new_entry(Address *a)
{
	Address *b=new Address(a);
	AddTail(b);
	Sort(sortaddress);
	return Find(b);
}
int AddressBook::new_or_old_entry(Address *n,int nonew)
{
	Address *b=NULL;
	Address *a=(Address*)head;
	while (a->next)
	{
		if ((a->getField(EMAIL)[0] && !strcscmp(a->getField(EMAIL),n->getField(EMAIL)))
		  ||(a->getField(FIXEDALIAS)[0] && atoi(a->getField(FIXEDALIAS)) && a->getField(ALIAS)[0] 
		     && n->getField(FIXEDALIAS)[0] && atoi(n->getField(FIXEDALIAS)) && n->getField(ALIAS)[0] 
			 && !strcscmp(a->getField(ALIAS),n->getField(ALIAS))))
		{
			b=a;
			for(int i=0;i<FIELDSIZE;i++)
			{
				if (i!=AddressBook::ADDDATE)
				{
					char *f=b->getField(i);
					if (!f || !f[0])
					{
						b->setField(i,n->getField(i));
					}
				}
			}
			char buf[20];
			xc_snprintf(buf,sizeof(buf),"%d",time(NULL));
			b->setField(AddressBook::MODIFIEDDATE,buf);
			break;
		}

		a=(Address*)a->next;
	}
	if (!b && !nonew)
	{
		b=new Address(n);
		AddTail(b);
		Sort(sortaddress);
	}
	return Find(b);
}
int AddressBook::new_or_old_entry(Address *a)
{
	return new_or_old_entry(a,0);
}
int AddressBook::update_old_entry(class Address *a)
{
	return new_or_old_entry(a,1);
}

int AddressBook::erase_entry(int i)
{
	Address *a=(Address*)Find(i);
	if (a)
	{
		Remove(a);
		delete a;
		return 0;
	}
	return 1;
}
int AddressBook::setField(int i,enum FIELDS f,char *v)
{
	Address *a=(Address*)Find(i);
	if (a)
	{
		char buf[20];
		xc_snprintf(buf,sizeof(buf),"%d",time(NULL));
		a->setField(AddressBook::MODIFIEDDATE,buf);
		if (f==ALIAS)
		{
			if (v[0])
			{
				tokenizer t(v,",",", ");
				t.reduce();
				v=t.string();
				char *c=v;
				while (*c)
				{
					if (*c==' ') *c='_';
					c++;
				}
				a->setField(f,v);
			}
			else
				a->setField(f,v);
		}
		else
			a->setField(f,v);
//		Sort(sortaddress);
//		i=Find(a);
	}
	return i;
}
int AddressBook::setField(int i,enum FIELDS f,int v)
{
	char s[10];
	sprintf(s,"%d",v);
	return setField(i,f,s);
}
char *AddressBook::getField(int i,enum FIELDS f)
{
	Address *a=(Address*)Find(i);
	if (a)
		return a->getField(f);
	else
		return NULL;
}
Address *AddressBook::getEntry(int i)
{
	return (Address*)Find(i);
}
char *AddressBook::getName(int i,char *buf)
{
	buf[0]=0;
	if (getField(i,LASTNAME)[0]) strcat(buf,getField(i,LASTNAME));
	if (getField(i,FIRSTNAME)[0] && buf[0]) strcat(buf,", ");
	if (getField(i,FIRSTNAME)[0]) strcat(buf,getField(i,FIRSTNAME));
	if (!buf[0]) strcpy(buf,getField(i,ALIAS));
	return buf;
}
char *AddressBook::getEmailByAlias(char *aliasList)
{
	return getEmailByAliasRecursion(aliasList,0);
}

char *AddressBook::getEmailByAliasRecursion(char *aliasList, int depth)
{ // printf("getEmailByAliasRecursion(%s,%d)\n",aliasList,depth);
	static char *result=NULL;
	tokenizer t(aliasList,",;",",; ");

	for(int i=0;i<t.Count();i++)
	{
		token tk=t[i];
		if (!strcncmp("mailto:",t[i]))
		{
			token tk=t[i];
			t[i]=((char*)tk)+7;
		}

		Address *a=(Address*)head;
		while (a->next)
		{
			int ok=0;
			tokenizer at(a->getField(ALIAS),",",", ");
			at.reduce();

			for(int j=0;j<at.count();j++)
			{
				if (((char*)at[j])[0] && !strcscmp(at[j],t[i]))
				{
					ok=1;
					break;
				}
			}
			if (ok)
			{
				char *r=a->getField(EMAIL);
				t[i]=r;
				if (r && r[0])
				{
					tokenizer sub_t(r,",;",",; ");
					t[i]=sub_t[0];
					for(int k=1;k<sub_t.Count();k++)
					{
						r=(char*)sub_t[k];
						t.append(new token(r));
					}
				}
				break;
			}
			a=(Address*)a->next;
		}
// 		a=(Address*)head;
// 		while (a->next)
// 		{
// 			if (((char*)t[i])[0] && programconfig.smtpsenderon && programconfig.authaddress[0])
// 			{
// 				if (!strchr(t[i],'@'))
// 				{
// 					rextokenizer rt(programconfig.authaddress,"([^@]*)@([^@]*)");
// 					if (rt.count()==2)
// 					{
// 						char *buf=new char[strlen(t[i])+1+strlen(rt[1])+1];
// 						if (buf)
// 						{
// 							strcpy(buf,t[i]);
// 							strcat(buf,"@");
// 							strcat(buf,rt[1]);
// 							t[i]=buf;
// 							delete [] buf;
// 						}
// 					}
// 				}
// 			}
// 			if (programconfig.longrecipient)
// 			{
// 				Address *b=(Address*)head;
// 				while (b->next)
// 				{
// 					if (!strcscmp(t[i],b->getField(EMAIL)))
// 					{
// 						char buf[200];
// 						xc_snprintf(buf,sizeof(buf),"%s %s <%s>",b->getField(FIRSTNAME),b->getField(LASTNAME),b->getField(EMAIL));
// 						t[i]=buf;
// 						break;
// 					}
// 					b=(Address*)b->next;
// 				}
// 			}
// 			a=(Address*)a->next;
// 		}
	}
	if (result)
		delete [] result;
	result=NULL;

	t.removeempty();
	char *r=new char[strlen(t.string())+1];
	strcpy(r,t.string());
	if (depth<2)
	{
		char *r2=getEmailByAliasRecursion(r,depth+1);
		delete [] r;
		r=r2;
	}

	tokenizer t2(r,",;",",; ");
	for(int i=0;i<t2.Count();i++)
	{
		if (((char*)t2[i])[0] && programconfig.smtpsenderon && programconfig.authaddress[0])
		{
			if (!strchr(t2[i],'@'))
			{
				rextokenizer rt(programconfig.authaddress,"([^@]*)@([^@]*)");
				if (rt.count()==2)
				{
					char *buf=new char[strlen(t2[i])+1+strlen(rt[1])+1];
					if (buf)
					{
						strcpy(buf,t2[i]);
						strcat(buf,"@");
						strcat(buf,rt[1]);
						t2[i]=buf;
						delete [] buf;
					}
				}
			}
		}
		if (programconfig.longrecipient)
		{
			Address *b=(Address*)head;
			while (b->next)
			{
				if (!strcscmp(t2[i],b->getField(EMAIL)))
				{
					char buf[200];
					xc_snprintf(buf,sizeof(buf),"%s %s <%s>",b->getField(FIRSTNAME),b->getField(LASTNAME),b->getField(EMAIL));
					t2[i]=buf;
					break;
				}
				b=(Address*)b->next;
			}
		}
	}
	t2.removeempty();
	delete [] r;
	r=new char[strlen(t2.string())+1];
	strcpy(r,t2.string());
	result=r;
	return result;
}

int AddressBook::getAddressByAlias(char *alias)
{
	Address *a=getAddress(alias);
	if (a)
		return Find(a);
	else
		return 0;
}
Address *AddressBook::getAddress(char *alias)
{
	Address *ret=NULL;

	Address *a=(Address*)head;
	while (a->next)
	{
		int ok=0;
		tokenizer at(a->getField(ALIAS),",",", ");
		at.reduce();

		for(int j=0;j<at.count();j++)
		{
			if (!strcscmp(at[j],alias))
			{
				ok=1;
				break;
			}
		}
		if (ok)
		{
			ret=a;
			break;
		}
		a=(Address*)a->next;
	}
	return ret;
}
int AddressBook::getAddressByMatch(char *string,tokenizer *dest)
{
	int count=0;
	while (*string && *string==' ') string++;
//	int l=strlen(string);
//	while (l>0 && string[l-1]==' ') { string[l-1]=0; l--; }

	if (*string)
	{
		Address *a=(Address*)head;
		while (a->next)
		{
			int ok=0;
			tokenizer at(a->getField(ALIAS),",",", ");
			at.reduce();

			for(int j=0;j<at.count();j++)
			{
				if (!strcscmp(string,at[j]))
				{
					ok=1;
					break;
				}
			}
			ok=ok||!strcscmp(string,a->getField(EMAIL));
			ok=ok||!strcscmp(string,a->getField(LASTNAME));
			ok=ok||!strcscmp(string,a->getField(FIRSTNAME));
			if (a->getField(FIRSTNAME)[0] && a->getField(LASTNAME)[0] && strchr(string,' '))
			{
				char buf[200];
				xc_snprintf(buf,sizeof(buf),"%s %s",a->getField(FIRSTNAME),a->getField(LASTNAME));
				ok=ok||!strcncmp(string,buf);
			}

			if (ok)
			{
				char buf[200];
				if (a->getField(FIRSTNAME)[0] || a->getField(LASTNAME)[0])
					xc_snprintf(buf,sizeof(buf),"%s %s <%s>",a->getField(FIRSTNAME),a->getField(LASTNAME),a->getField(EMAIL));
				else
					xc_snprintf(buf,sizeof(buf),"%s",a->getField(EMAIL));

				token *tk=new token(buf);
				dest->append(tk);
				count++;
			}
			a=(Address*)a->next;
		}
		a=(Address*)head;
		while (a->next)
		{
			int ok=0;
			tokenizer at(a->getField(ALIAS),",",", ");
			at.reduce();

			for(int j=0;j<at.count();j++)
			{
				if (!strcncmp(string,at[j]) && strcscmp(string,at[j]))
				{
					ok=1;
					break;
				}
			}
			ok=ok||(strcscmp(string,a->getField(EMAIL)) && !strcncmp(string,a->getField(EMAIL)));
			ok=ok||(strcscmp(string,a->getField(LASTNAME)) && !strcncmp(string,a->getField(LASTNAME)));
			ok=ok||(strcscmp(string,a->getField(FIRSTNAME)) && !strcncmp(string,a->getField(FIRSTNAME)));

			if (ok)
			{
				char buf[200];
				if (a->getField(FIRSTNAME)[0] || a->getField(LASTNAME)[0])
					xc_snprintf(buf,sizeof(buf),"%s%s%s%s<%s>",a->getField(FIRSTNAME),a->getField(FIRSTNAME)[0]?" ":"",a->getField(LASTNAME),a->getField(LASTNAME)[0]?" ":"",a->getField(EMAIL));
				else
					xc_snprintf(buf,sizeof(buf),"%s",a->getField(EMAIL));

				token *tk=new token(buf);
				dest->append(tk);
				count++;
			}
			a=(Address*)a->next;
		}
	}
	return count;
}
int AddressBook::getEntryByEmail(char *email)
{
	Address *ret=NULL;

	Address *a=(Address*)head;
	while (a->next)
	{
		if (!strcscmp(a->getField(EMAIL),email))
		{
			ret=a;
			break;
		}
		a=(Address*)a->next;
	}
	if (ret)
		return Find(ret);
	return 0;
}


#define ADDFIELD(f,n)	if (a->getField(f)[0]) { vcard+=n; vcard+=Translate827Line(a->getField(f),programconfig.encoding8bit); vcard+="\n"; }
char *AddressBook::vcardtext(enum VCARDTYPE vct)
{
	if (vct==NOVCARD)
		return "";

	vcard.clear();

	vcard+="begin:vcard\n";
	Address *a;
	if (vct==PRIVATE)
		a=getAddress("me");
	else
		a=getAddress("me_work");
	if (a)
	{
		vcard+="n:";
		vcard+=a->getField(LASTNAME),programconfig.encoding8bit;
		vcard+=";";
		vcard+=a->getField(FIRSTNAME),programconfig.encoding8bit;
		vcard+="\n";
		ADDFIELD(HOMEPHONE,"tel;home:");
		ADDFIELD(FAX,"tel;fax:");
		ADDFIELD(WORKPHONE,"tel;work:");
		ADDFIELD(MOBIL,"tel;cell:");
		vcard+="adr:;;";
		vcard+=a->getField(ADDRESS),programconfig.encoding8bit;
		vcard+=";";
		vcard+=a->getField(CITY),programconfig.encoding8bit;
		vcard+=";";
		vcard+=a->getField(STATE),programconfig.encoding8bit;
		vcard+=";";
		vcard+=a->getField(ZIP),programconfig.encoding8bit;
		vcard+=";";
		vcard+=a->getField(COUNTRY),programconfig.encoding8bit;
		vcard+="\n";
		vcard+="version:2.1\n";
		ADDFIELD(EMAIL,"email;internet:");
		ADDFIELD(URL,"url:");
		ADDFIELD(ORGANISATION,"org:");
		vcard+="fn:";
		vcard+=a->getField(FIRSTNAME),programconfig.encoding8bit;
		vcard+=" ";
		vcard+=a->getField(LASTNAME),programconfig.encoding8bit;
		vcard+="\n";

	};
	vcard+="end:vcard\n";
	return vcard;
}

int AddressBook::mix(class AddressBook *ab)
{
	Address *a=(Address*)ab->head,*b;
	while (a->next)
	{
		int p=new_or_old_entry(a);
		b=(Address*)Find(p);
		if (b)
		{
			for(int i=0;i<FIELDSIZE;i++)
			{
				char *f=b->getField(i);
				if (!f || !f[0])
				{
					b->setField(i,a->getField(i));
				}
			}
		}
		a=(Address*)a->next;
	}

}

void makemenu(int md=0);
void add_auto_book(char *from,char *email,char *xface)
{
	if (email && email[0])
	{
		Address a;

		char *sp=strrchr(from,' ');
		char *co=strchr(from,',');
		if (co)
		{
			char *s=new char[strlen(from)+1];
			if (s)
			{
				strcpy(s,from);
				char *c=s+(co-from);
				while (c>s && *(c-1)==' ')c--;
				*c=0;
				c=s+(co-from)+1;
				while (*c && *c==' ')c++;

				a.setField(AddressBook::FIRSTNAME,c);
				a.setField(AddressBook::LASTNAME,s);
				delete [] s;
			}
		}
		else if (sp)
		{
			char *s=new char[strlen(from)+1];
			if (s)
			{
				strcpy(s,from);
				char *c=s+(sp-from);
				while (c>s && *(c-1)==' ')c--;
				*c=0;
				c=s+(sp-from)+1;
				while (*c && *c==' ')c++;

				a.setField(AddressBook::FIRSTNAME,s);
				a.setField(AddressBook::LASTNAME,c);
				delete [] s;
			}
		}
		else
		{
			a.setField(AddressBook::FIRSTNAME,"");
			a.setField(AddressBook::LASTNAME,from);
		}
		a.setField(AddressBook::EMAIL,email);
		if (xface && xface[0])
		{
			a.setField(AddressBook::XFACE,xface);
		}
		if (programconfig.generate_autoabook)
		{
			abook_auto.new_or_old_entry(&a);
			auto_addressbookupdate();
		}
		abook.update_old_entry(&a);
		makemenu(1);
	}
}
void add_auto_book(char *from)
{
	char *name=strdup(from);
	char *email=strdup(from);
	mail::parse_from_address(email);
	mail::parse_from_name(name);

	add_auto_book(name,email);

	free(email);
	free(name);
}

/********************************************************/

int AddressBook::addZmailrc(void)
{
// 	char *h=getenv("HOME");
// 	if (!h) return 0;
// 	
// 	int l=strlen(h)+12;
// 	char fname[l];
// 	xc_snprintf(fname,l,"%s/.zmailrc", h);
// 
// 	FILE *fp = fopen(fname, "r");
// 	if (!fp) return 0;
// 	
// 	char line[ADDRLENGTH];
// 	
// 	while (read_line(fp, line, ADDRLENGTH-1))
// 	{
// 		char cmd[ADDRLENGTH];
// 		
// 		if (get_token(line, 1, cmd, "\t ") && strcasecmp(cmd, "alias") == 0)
// 		{
// 			char alias[100];
// 			char *c=line;
// 			int i;
// 			
// 			while (*c && *c!=' ' && *c!='\t') c++;
// 			while (*c && (*c==' ' || *c=='\t')) c++;
// 	
// 			for (i=0; i<sizeof(alias)-1 && *c && *c!=' ' && *c!='\t'; c++)
// 				alias[i++]=*c;
// 			alias[i]=0;
// 
// 			while (*c && (*c==' ' || *c=='\t')) c++;
// 			
// 			if (strchr(c, ','))
// 			{
// 				/* alias to address list */
// 			
// 				char ulist[4000];
// 				int bra=0;
// 				
// 				for (i=0; i<sizeof(ulist)-1 && *c; c++) {
// 					if (*c=='(') bra++;
// 					if (*c!='\'' && !bra) {
// 						if (*c==',' && ulist[i-1]==' ') i--;
// 						ulist[i++]=*c;
// 					}
// 					if (*c==')') bra--;
// 				}
// 				ulist[i]=0;
// 				insert(alias, alias, NULL, ulist, "0", "Converted from Zmail");
// 			}
// 			else
// 			{
// 				/* alias to one address */
// 			
// 				char addr[200];
// 				
// 				for (i=0; i<sizeof(addr)-1 && *c && *c!=' ' && *c!='\t'; c++)
// 					addr[i++]=*c;
// 				addr[i]=0;
// 				
// 				while (*c && *c!='(') c++;
// 				if (*c=='(')
// 				{  /* Real-Name attached */
// 					char name[200];
// 					c++;
// 					for (i=0; i<sizeof(name)-1 && *c && *c!=')'; c++)
// 						name[i++]=*c;
// 					name[i]=0;
// 					insert(alias, name, NULL, addr, "0", "Converted from Zmail");
// 				}
// 				else
// 				{
// 					insert(alias, alias, NULL, addr, "0", "Converted from Zmail");
// 				}
// 			}
// 
// 		}			
// 	}
// 	return 1;
}
