/* 
 * This file is part of XCmail and is subject to the 
 * XCmail license agreement
 */

/* $Id: createmail.cxx,v 2.51 2002/02/27 13:09:07 schmitzj Exp $ */


#include "configure.h"

#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>
#include <ctype.h>
#include <time.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <dirent.h>
#include <unistd.h>
#include <sys/types.h>
#include <setjmp.h>
#include <signal.h>
#include <sys/utsname.h>

#ifdef HAVE_SYSENT_H
#include <sysent.h>
#endif

#include "createmail.h"
#include "sendmail.h"
#include "strbuf.h"
#include "address_book.h"
#include "stringTokenizer.h"
#include "strbuf.h"

static int allowMulti=1;

createmail::createmail(void)
{
	_to=NULL;
	_cc=NULL;
	_subject=NULL;
	_bcc=NULL;
	_body=NULL;
	_priority=1;
	_pgpmode=0;
	_replyto=NULL;
	if (ISSET(programconfig.replyto))
		replyto(programconfig.replyto);

	if (programconfig.vcard_enable)
	{
		if (programconfig.vcard_work_default)
			_vcardtype=AddressBook::WORK;
		else
			_vcardtype=AddressBook::PRIVATE;
	}
	else
		_vcardtype=AddressBook::NOVCARD;
	
	attacount=0;
	attas=NULL;
	_headers=NULL;
	_forwardtomail=NULL;
	_replytomail=NULL;
	aw=&readerwin;
	sobj=NULL;
}
createmail::~createmail()
{
	freestring(_to);
	freestring(_cc);
	freestring(_subject);
	freestring(_bcc);
	freestring(_body);
	freestring(_headers);
	freestring(_replyto);
	if (_forwardtomail) _forwardtomail->remreferer();
	if (_replytomail) _replytomail->remreferer();
}

void createmail::to(char *t)
{
	setemailstring(_to,t);

	if (programconfig.generate_autoabook)
	{
		tokenizer tok1(t,","," \t");
		for(int i=0;i<tok1.count();i++)
		{
			add_auto_book((char*)tok1[i]);
		}
	}
}
void createmail::cc(char *t)
{
	setemailstring(_cc,t);

	if (programconfig.generate_autoabook)
	{
		tokenizer tok1(t,","," \t");
		for(int i=0;i<tok1.count();i++)
		{
			add_auto_book((char*)tok1[i]);
		}
	}
}
void createmail::subject(char *t)
{
	setstring(_subject,t);
}
void createmail::replyto(char *t)
{
	setstring(_replyto,t);
}
void createmail::bcc(char *t)
{
	setemailstring(_bcc,t);

	if (programconfig.generate_autoabook)
	{
		tokenizer tok1(t,","," \t");
		for(int i=0;i<tok1.count();i++)
		{
			add_auto_book((char*)tok1[i]);
		}
	}
}
void createmail::body(char *t)
{
	setstring(_body,t);
}
void createmail::headers(char *t)
{
	setstring(_headers,t);
}
void createmail::priority(int p)
{
	_priority=p;
}
void createmail::pgpmode(int p)
{
	_pgpmode=p;
}
void createmail::attachments(int count,struct attachments **a)
{
	attacount=count;
	attas=a;
}
void createmail::vcardtype(enum AddressBook::VCARDTYPE vct)
{
	_vcardtype=vct;
}
void createmail::replytomail(class mail *m)
{
	if (_replytomail) _replytomail->remreferer();
	_replytomail=m;
	m->addreferer();
}
void createmail::forwardtomail(class mail *m)
{
	if (_forwardtomail) _replytomail->remreferer();
	_forwardtomail=m;
	m->addreferer();
}
void createmail::referingwindow(appwindow *a)
{
	aw=a;
}
void createmail::outputobject(class statusobj *s)
{
	sobj=s;
}
void createmail::freestring(char *&t)
{
	if (t) free(t);
	t=NULL;
}
void createmail::setstring(char *&t,char *f)
{
	freestring(t);
	t=strdup(f);
}
void createmail::setemailstring(char *&t,char *f)
{
	freestring(t);
	if (f)
	{
		tokenizer tk(f,",;",",; ");
		tk.removeempty();

		if (tk.count()>0)
		{
			for(int i=0;i<tk.count();i++)
			{
				rextokenizer rt(tk[i],"(.*[^ ]) *<(.*)>");
				if (rt.count()==2)
				{
					rt[0]=Translate827Line(rt[0],programconfig.encoding8bit);
					char *b=new char[strlen(rt[0])+strlen(rt[1])+1+1+4];
					xc_snprintf(b,strlen(rt[0])+strlen(rt[1])+1+1+4,"\"%s\" <%s>",(char*)rt[0],(char*)rt[1]);
					tk[i]=b;
					delete [] b;
				}
			}
			t=strdup(tk.list(",",",",1));
		}
		else
			t=strdup("");
	}
	else
		t=strdup("");
}

int createmail::send(void)
{
	char buf[MAXLENGTH+1];
	xcm_strbuf head,outboxhead;
	char bound[200];
	int l;
	easy_dialog ed;
	int pgp=0;

#ifdef USEPGP
	if (programconfig.usepgp)
		pgp=_pgpmode;
#endif

	gethostname(buf,sizeof(buf));
	xc_snprintf(bound,sizeof(bound),"boundary_"PROGRAMNAME"_"VERSION"_%08X_%s",time(NULL),buf);

	if (programconfig.smtpsenderon)
	{
		head+="From: ";
		xc_snprintf(buf,sizeof(buf),"%s <%s>\n",Translate827Line(programconfig.authname,programconfig.encoding8bit),programconfig.authaddress);
		head+=buf;
	}
	else
	{
		char *fromenv=getenv("MAILFROM");
		if (fromenv)
		{
			head+="From: ";
			xc_snprintf(buf,sizeof(buf),"%s\n",Translate827Line(fromenv,programconfig.encoding8bit));
			head+=buf;
		}
	}

	head+="To: ";
	head+=_to;
	head+="\n";

	if (ISSET(_cc))
	{
		head+="Cc: ";
		head+=_cc;
		head+="\n";
	}

	if (ISSET(_replyto))
	{
		head+="Reply-To: ";
		head+=Translate827Line(_replyto,programconfig.encoding8bit);
		head+="\n";
	}

	head+="Subject: ";
	head+=Translate827Line(SAVESTRING(_subject),programconfig.encoding8bit);
	head+="\n";

	if (_replytomail)
	{
		head+="In-Reply-To: ";
		head+=_replytomail->messageid();
		head+="\n";
	}

	if (ISSET(programconfig.organization))
	{
		head+="Organization: ";
		head+=Translate827Line(programconfig.organization,programconfig.encoding8bit);
		head+="\n";
	}
	if (ISSET(_bcc) && !programconfig.smtpsenderon)
	{
		head+="Bcc: ";
		head+=_bcc;
		head+="\n";
	}
	char dbuf[50];
	time_t t;
	time(&t);
	strftime(dbuf,sizeof(dbuf),"%a, %d %b %Y %H:%M:%S",localtime(&t));
	head+="Date: ";
	head+=dbuf;
	int tz=-(int_timezone/60-(int_daylight?60:0));
	xc_snprintf(dbuf,sizeof(dbuf)," %c%02d%02d ",tz<0?'-':'+',abs(tz)/60,abs(tz)%60);
	head+=dbuf;
	strftime(dbuf,sizeof(dbuf),"(%Z)",localtime(&t));
	head+=dbuf;
	head+="\n";
	if (_priority!=1)
	{
		xc_snprintf(buf,sizeof(buf),"Priority: %s\n",_priority?"non-urgent":"urgent");
		head+=buf;
	}
	if (ISSET(programconfig.xface))
	{
		head+="X-Face: ";
		head+=programconfig.xface;
		head+="\n";
	}
    if (_forwardtomail)
	{
		head+="X-ReplyTo: ";
		head+=_forwardtomail->replyto();
		head+="\n";
	}

#ifdef USEPGP
	head+="X-Mailer: "PROGRAMNAME" "VERSION" - with PGP support, "PGPVERSION" ("XC_UNAME")\n";
#else
	head+="X-Mailer: "PROGRAMNAME" "VERSION" ("XC_UNAME")\n";
#endif
	head+="X-Mailerorigin: http://www.js-home.org/XCmail/\n";

	if (ISSET(_headers))
		head+=_headers;

	if (char *dl=getenv("XCMAIL_DEVLEVEL"))
	{
		head+="X-XCmail-Developer: ";
		head+=getusername();
		if (dl[0])
		{
			head+="\nX-XCmail-Development-Level: ";
			head+=dl;
		}
		head+="\n";
	}
	head+="MIME-Version: 1.0\n";

	outboxhead+=head();
	if (pgp==1) // sign
	{
		head+="Content-Type: multipart/signed; boundary=\"";
		head+=bound;
		head+="\";\n micalg=pgp-md5; protocol=\"application/pgp-signature\"\n";
	}
	else if (pgp==2) // code
	{
		head+="Content-Type: multipart/encrypted; boundary=\"";
		head+=bound;
		head+="\";\n protocol=\"application/pgp-encrypted\"\n";
	}
	else if (pgp==3) // code & sign
	{
		head+="Content-Type: multipart/encrypted; boundary=\"";
		head+=bound;
		head+="\";\n micalg=pgp-md5; protocol=\"application/pgp-encrypted\"";
	}
	else if (attacount==0 && _vcardtype==AddressBook::NOVCARD)
	{
		head+="Content-Type: text/plain; charset=";
		head+=programconfig.encoding8bit;
		head+="\n";
	}
	else
	{
		head+="Content-Type: multipart/mixed; boundary=\"";
		head+=bound;
		head+="\"\n";
	}

	if (attacount==0)
	{
		outboxhead+="Content-Type: text/plain; charset=";
		outboxhead+=programconfig.encoding8bit;
		outboxhead+="\n";
	}
	else
	{
		outboxhead+="Content-Type: multipart/mixed; boundary=\"";
		outboxhead+=bound;
		outboxhead+="\"\n";
	}

	xcm_strbuf b;
	b+=_body;
	b+="\n";
	char *tb=Translate827Text(b(),b.length());
	
	switch (programconfig.transumlauts)
	{
		case 1: // 7bit
			head+="Content-Transfer-Encoding: 7bit\n";
			outboxhead+="Content-Transfer-Encoding: 7bit\n";
			break;
		case 2: // 8bit
			head+="Content-Transfer-Encoding: 8bit\n";
			outboxhead+="Content-Transfer-Encoding: 8bit\n";
			break;
		default: // qp
			if (tb)
			{
				head+="Content-Transfer-Encoding: quoted-printable\n";
				outboxhead+="Content-Transfer-Encoding: quoted-printable\n";
			}
			else
			{
				head+="Content-Transfer-Encoding: 7bit\n";
				outboxhead+="Content-Transfer-Encoding: 7bit\n";
			}
			break;
	}
	int r=0;
	if (tb)
	{
		r=send_mail(head(),outboxhead(),tb,bound);
		TranslateFree(tb);
	}
	else
		r=send_mail(head(),outboxhead(),b(),bound);	// 0: error, 1: sent, 2: async sending

	if (r)
	{
		if (_replytomail)
		{
			_replytomail->status(_replytomail->status() | STATUS_REPLYED);
		}
		if (_forwardtomail)
		{
			_forwardtomail->status(_forwardtomail->status() | STATUS_FORWARDED);
		}
	}
	return r;
}
int createmail::RCPT(FILE *file,char *buf)
{
	int err=0;
	tokenizer tk(buf,",;",",; ");
	tk.removeempty();

	if (tk.count()>0)
	{
		for(int i=0;i<tk.count();i++)
		{
			rextokenizer rt(tk[i],"(.*) ?<(.*)>");
			char *c=tk[i];
			if (rt.count()==2)
			{
				c=rt[1];
			}
			err=err||myfprintf(file,"%s\n",c)<0;
		}
	}
	return !err;
}
int createmail::send_mail(char *head, char *outboxhead, char *b,char *bound)
{
	int r=0;
	int err=0, n;
	char com[300],tnam[100],*c;
	FILE *file=NULL;
	int sockfd=0;
	struct sockaddr_in cli_addr,serv_addr;
	struct hostent *host_info;

	if (programconfig.smtpsenderon) // && programconfig.smtpoutspool)
	{
		char *h=getenv("HOME");
		xc_snprintf(tnam,sizeof(tnam),"%s/%s/%s",h,HOMEMAIL,POPSPOOL);
		mkdir(tnam,S_IXUSR|S_IRUSR|S_IWUSR);
		xc_snprintf(tnam,sizeof(tnam),"%s/%s/%s/%x%08x",h,HOMEMAIL,POPSPOOL,getpid(),time(NULL));
	}
	else
		xc_snprintf(tnam,sizeof(tnam),"/tmp/"PROGRAMNAME"_tmp_%x%08x",getpid(),time(NULL));

	file=fopen(tnam,"w");
	if (file)
	{
		setvbuf(file,NULL,_IOFBF,2000);

#ifdef USEPGP
		if (_pgpmode==0)
#endif
		{
			n = strlen(head);
			err=err||fwrite(head,1,n,file)!=n;
			err=err||fputc('\n',file)==EOF;
		}
// PGP Schleife:
// alles per Pipe (oder File) an PGP, dann von dort lesen und nach header 
// an Sendmail (file) plus Mime-Parts
// bei SIGN normale Mail vorher an Sendmail geben
// "pgp -eatf toID toID2 toID3..." - f bei Pipe : encrypt
// "pgp -satf toID toID2 toID3..." : sign - pass phrase
// "pgp -esatf toID toID2 toID3..." : sign&encrypt - pass phrase
// env PGPPASSFD : =0: stdin, z.B. (echo "pass phrase";cat file)|pgp ....

		if (allowMulti) appwindow::UpdateEvent();

		if (attacount==0 && _vcardtype==AddressBook::NOVCARD)
		{
#ifdef USEPGP
			if (_pgpmode>0)
			{
				err = err ||
					myfprintf(file, "X-Mailer: "PROGRAMNAME" "VERSION" - with PGP support, "PGPVERSION"\n")<0 ||
					myfprintf(file, "X-Mailerorigin: http://www.js-home.org/XCmail/\n")<0 ||
					myfprintf(file, "MIME-Version: 1.0\n")<0 ||
					myfprintf(file, "Content-Type: text/plain; charset=%s\n",programconfig.encoding8bit)<0;
					
				switch (programconfig.transumlauts)
				{
					case 1: // 7bit
						err=err||myfprintf(file,"Content-Transfer-Encoding: 7bit\n\n")<0;
						break;
					case 2: // 8bit
						err=err||myfprintf(file,"Content-Transfer-Encoding: 8bit\n\n")<0;
						break;
					default: // qp
						err=err||myfprintf(file,"Content-Transfer-Encoding: quoted-printable\n\n")<0;
						break;
				}
			}
#endif
			err=err||fwrite(b,strlen(b),1,file)!=1;
		}
		else
		{
			char sbound[300];
#ifdef USEPGP
			if (_pgpmode>0)
			{
				xc_snprintf(sbound,sizeof(sbound),"subbound_%s",bound);
				err = err ||
					myfprintf(file, "X-Mailer: "PROGRAMNAME" "VERSION" - with PGP support, "PGPVERSION"\n")<0 ||
					myfprintf(file, "X-Mailerorigin: http://www.js-home.org/XCmail/\n")<0 ||
					myfprintf(file, "MIME-Version: 1.0\n")<0 ||
					myfprintf(file, "Content-Type: multipart/mixed; boundary=\"%s\"\n\n",sbound)<0 ||
					myfprintf(file, "Content-Transfer-Encoding: quoted-printable\n\n")<0;
			}
			else
#endif
				strcpy(sbound,bound);

			struct attachments *mp;
			if (!programconfig.no_mime_preamble)
			{
				err = err ||
					myfprintf(file,"This is the preamble area of a multipart message.\n"
								   "Mail readers that understand multipart format\n"
								   "should ignore this preamble.\n\n"
								   "If you are reading this text, you might want to\n"
								   "consider changing to a mail reader that understands\n"
								   "how to properly display multipart messages.\n(like the cool "PROGRAMNAME")")<0;
			}
			err = err ||
				myfprintf(file,"\n\n--%s\n",sbound)<0 ||
				myfprintf(file,"Content-Type: text/plain\n")<0 ||
				myfprintf(file,"Content-Description: Main text\n")<0;
				
			switch (programconfig.transumlauts)
			{
				case 1: // 7bit
					err=err||myfprintf(file,"Content-Transfer-Encoding: 7bit\n\n")<0;
					break;
				case 2: // 8bit
					err=err||myfprintf(file,"Content-Transfer-Encoding: 8bit\n\n")<0;
					break;
				default: // qp
					err=err||myfprintf(file,"Content-Transfer-Encoding: quoted-printable\n\n")<0;
					break;
			}
			err=err||fwrite(b,strlen(b),1,file)!=1;
			
			for(int i=0;i<attacount;i++)
			{
				if (allowMulti) appwindow::UpdateEvent();
				
				err=err||myfprintf(file,"\n--%s\n",sbound)<0;
				mp=attas[i];
				if (!mp->body)
				{
					if (mp->filename)
					{
						FILE *afile;
						if (afile=fopen(mp->filename,"r"))
						{
							char cpbuf[1000];
							int rsize=0;

							setvbuf(afile,NULL,_IOFBF,2000);
							while (rsize=fread(cpbuf,1,sizeof(cpbuf),afile))
							{
								if (allowMulti) appwindow::UpdateEvent();
								err=err||fwrite(cpbuf,1,rsize,file)!=rsize;
							}
							fclose(afile);
						}
						else
						{
							err = err ||
								myfprintf(file, "Content-Type: text/plain\n"
											    "Content-Description: Attachment Error\n"
											    "Content-Transfer-Encoding: 7bit\n\n")<0 ||
								myfprintf(file, "can't open file %s\n",mp->filename)<0;
						}
					}
				}
				else
					err=err||fwrite(mp->body,strlen(mp->body),1,file)!=1;
			}
			if (_vcardtype!=AddressBook::NOVCARD)
			{
				err=err||myfprintf(file,"\n--%s\n",sbound)<0;
				err=err||myfprintf(file,"Content-Type: text/x-vcard; charset=us-ascii;\n"
				                        "  name=\"vCard.vcf\"\n"
									    "Content-Description: vCard\n"
						                "Content-Transfer-Encoding: quoted-printable\n\n")<0;

				char *vcard=abook.vcardtext(_vcardtype);
				char *qpvcard=Translate827Text(vcard,strlen(vcard));
				err=err||myfprintf(file,"%s\n\n",qpvcard?qpvcard:vcard)<0;
				if (qpvcard) TranslateFree(qpvcard);
			}
			err=err||myfprintf(file,"\n--%s--\n",sbound)<0;
		}
// end PGP Schleife

		err=fclose(file)!=0||err;
		
		if (!err)
		{
			FILE *f;
			char buf[MAXLENGTH+1];

			strcpy(com,tnam);
			strcat(com,".rcpt");

			if (f=fopen(com,"w"))
			{
				setvbuf(f,NULL,_IOFBF,2000);
				if (ISSET(_to)) err=err||!RCPT(f,SAVESTRING(_to));
				if (ISSET(_cc)) err=err||!RCPT(f,SAVESTRING(_cc));
				if (ISSET(_bcc)) err=err||!RCPT(f,SAVESTRING(_bcc));
				err=fclose(f)!=0||err;
				if (err)
				{
					remove(com);
					remove(tnam);
				}
			}
		}
		else
			remove(tnam);

#ifdef USEPGP
		switch (_pgpmode)
		{
			case 1: // sign
			case 2: // code
			case 3: // code & sign
				if (!err && aw && PGPcode(aw,tnam,com,head,bound,_pgpmode))
				{ // code error
					easy_dialog ed;
					ed.Text("PGP encoding/signing error!");
					ed.Options("_back to edit");
					ed.Start(aw);
					remove(com);
					remove(tnam);
					return 0;
				}
				break;
		}
#endif

		if (!err && programconfig.outbox)
		{
			vfolder *v=folderlist->getfolder_byid(programconfig.outboxname);
			if (v)
			{
				xcm_strbuf mstring;
				char buf[200];

				time_t t;
				time(&t);
				char *n=getusername();
				struct utsname nts;
				char tbuf[40];

            	int tz_hour=0;
            	int tz_min=0;
            	int tz=int_timezone;
            	char tzc[5]="+0";

            	tzc[2]='\0';

            	tz = -tz;

            	if (int_daylight)		
                	tz += 3600;

            	tz = tz/60;
            	tz_hour = tz / 60;
            	tz_min = tz % 60;

            	tz = tz_hour * 100;
            	tz += tz_min;

            	if(tz>=0)
                	tzc[0]='+';
            	else
                	tzc[0]='-';

            	if(abs(tz_hour) < 10)
            	{
                	tzc[1]='0';
                	tzc[2]='\0';
            	}
            	else  if(tz==0)
            	{
                	tzc[1]='0';
                	tzc[2]='0';
                	tzc[3]='0';
                	tzc[4]='\0';
            	}
            	else
                	tzc[1]='\0';

				uname(&nts);
				strftime(tbuf,sizeof(tbuf),"%a %b %d %H:%M:%S %Y",gmtime(&t));

				xc_snprintf(buf,sizeof(buf),"From %s@%s %s %s%d\n",n,nts.nodename,tbuf, tzc, abs(tz));
				mstring+=buf;

				if (!strstr(outboxhead, "\nFrom: ") && strncmp(outboxhead, "From: ",6)!=0)
				{
					// only insert regular "From: " line, when not already inserted (r.b.)
					xc_snprintf(buf,sizeof(buf),"From: %s@%s\n",n,nts.nodename);
					mstring+=buf;
				}

				mstring+=outboxhead;
				mstring+="X-XCmail-Status: Tr\n";
				mstring+="Status: RO\n\n";

				if (attacount==0)
				{
					char *c1=b;
					while (*c1)
					{
						if (!strncmp(c1,"\nFrom",5))
						{
							mstring+='\n';
							mstring+='>';
						}
						else
						{
							mstring+=*c1;
						}
						c1++;
					}
				}
				else
				{
					struct attachments *mp;

					xc_snprintf(buf,sizeof(buf),"\n--%s\n",bound);
					mstring+=buf;
					mstring+="Content-Type: text/plain\n";
					mstring+="Content-Description: Main text\n";

					switch (programconfig.transumlauts)
					{
						case 1: // 7bit
							mstring+="Content-Transfer-Encoding: 7bit\n";
							break;
						case 2: // 8bit
							mstring+="Content-Transfer-Encoding: 8bit\n";
							break;
						default: // qp
							mstring+="Content-Transfer-Encoding: quoted-printable\n";
							break;
					}
					mstring+="\n";
					char *c1=b;
					while (*c1)
					{
						if (!strncmp(c1,"\nFrom",5))
						{
							mstring+='\n';
							mstring+='>';
						}
						else
						{
							mstring+=*c1;
						}
						c1++;
					}

					if (programconfig.outbox_noattachments)
					{
						xc_snprintf(buf,sizeof(buf),"\n--%s\n",bound);
						mstring+=buf;

						mstring+="Content-Type: text/plain\n";
						mstring+="Content-Description: Attachments not saved\n";
						mstring+="Content-Transfer-Encoding: 7bit\n\n";

						for(int i=0;i<attacount;i++)
						{
							if (allowMulti) appwindow::UpdateEvent();
							mp=attas[i];
							xc_snprintf(buf,sizeof(buf),"Attachment %d file '%s' not saved in outbox!\n",i+1,mp->name);
							mstring+=buf;
						}
					}
					else
					{
						for(int i=0;i<attacount;i++)
						{
							if (allowMulti) appwindow::UpdateEvent();
							xc_snprintf(buf,sizeof(buf),"\n--%s\n",bound);
							mstring+=buf;
							mp=attas[i];

							if (!mp->body)
							{
								if (mp->filename)
								{
									FILE *afile;
									if (afile=fopen(mp->filename,"r"))
									{
										char cpbuf[1001];
										int rsize=0;

										setvbuf(afile,NULL,_IOFBF,2000);
										while (rsize=fread(cpbuf,1,sizeof(cpbuf)-1,afile))
										{
											if (allowMulti) appwindow::UpdateEvent();
											cpbuf[sizeof(cpbuf)-1]=0;
											cpbuf[rsize]=0;
											mstring+=cpbuf;
										}
										fclose(afile);
									}
									else
									{
										mstring+="Content-Type: text/plain\n";
										mstring+="Content-Description: Attachment Error\n";
										mstring+="Content-Transfer-Encoding: 7bit\n\n";
										xc_snprintf(buf,sizeof(buf),"can't open file %s\n",mp->filename);
										mstring+=buf;
									}
								}
							}
							else
								mstring+=mp->body;
						}
					}
					xc_snprintf(buf,sizeof(buf),"\n--%s--\n",bound)<0;
					mstring+=buf;
				} //printf("OB: %s (%s)\n",v->g_foldername(),programconfig.outboxname);
				mail *m=new mail(strdup(mstring()));
				m->addreferer();
				if (v->appendmail_default(m))
				{
					if (aw)
					{
						easy_dialog ed;
						ed.Text("Unable to append mail to outbox:\nno space left on device\n");
						ed.Options("_Ok");
						ed.Start(aw);
					}
				}
				m->remreferer();
			}
			else
			{
				if (aw)
				{
					easy_dialog ed;
					ed.Text("Non existing outbox folder specified!\n");
					ed.Options("_Ok");
					ed.Start(aw);
				}
				else
				{
					fprintf(stderr,"ERROR: Non existing outbox folder specified!\n");
				}
			}
		}
		r=!err;
		if (programconfig.smtpsenderon) // && programconfig.smtpoutspool)
		{
			if (err)
			{
				if (aw)
				{
					easy_dialog ed;
					ed.Text("Unable to spool mail:\nno space left on device\n");
					ed.Options("_Ok");
					ed.Start(aw);
				}
			}
			else
			{
				pstatus("Mail spooled.");
				if (spooled<0) spooled=0;
				spooled++;
				if (!programconfig.smtpoutspool)
				{
					sendspooled();
				}
			}
		}
		else
		{
			if (err)
			{
				if (aw)
				{
					easy_dialog ed;
					ed.Text("Unable to send mail:\nno space left on device\n");
					ed.Options("_Ok");
					ed.Start(aw);
				}
			}
			else
			{
				r=sendmail(tnam,&readerwin);
			}
		}
	}
	return r;
}
// -------------------------------------------------------------------
int sendmail(char *mailfile,appwindow *aw,class statusobj *nm)
{
	class sendmail *s=new class sendmail(mailfile,0,aw,nm);
	int r=s->send();

	if (r>0) return r;
	if (programconfig.smtpsenderon)
		return 1;
	else
		return 2;
}
