/* 
 * This file is part of XCmail and is subject to the 
 * XCmail license agreement
 */

/* $Id: 7bit_encoding.cxx,v 2.3 1999/01/07 16:09:07 beckerr Exp $ */


#include "../configure.h"

#include <ctype.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "../encoder.h"
#include "../mailer.h"

//--------------------------------------------------------------------

class encoder_7bit : public encoder {

	public:
		encoder_7bit(void) : encoder(""/*"7bit;x-7bit"*/, "Ralf Becker", EF_DECODE) {};
		~encoder_7bit() {};
		virtual char *data2mail(char *,int size,int &newsize,char *path,char *filename);
		virtual char *mail2data(char *,int size,int &newsize,char *path,char *filename);
		virtual void dispose(char *b) { free(b);}
};

static encoder_7bit enc7bit;
static char linebuffer[TRANSLATE_LINE_WIDTH];

static struct {
	unsigned char code;
	char *ascii;
} umlauts[] = {

	{ 161, "!"		},	 // isolatin-1: 241 a1 161 
	{ 162, "c"		},	 // isolatin-1: 242 a2 162 
	{ 163, "pound"	},	 // isolatin-1: 243 a3 163 
	{ 164, "o"		},	 // isolatin-1: 244 a4 164 
	{ 165, "Y"		},	 // isolatin-1: 245 a5 165 
	{ 166, "|"		},	 // isolatin-1: 246 a6 166 
	{ 167, ""		},	 // isolatin-1: 247 a7 167 
	{ 168, "-"		},	 // isolatin-1: 250 a8 168 
	{ 169, "(C)"	},	 // isolatin-1: 251 a9 169 
	{ 170, "="		},	 // isolatin-1: 252 aa 170 
	{ 171, "<<"		},	 // isolatin-1: 253 ab 171 
	{ 172, "-,"		},	 // isolatin-1: 254 ac 172 
	{ 173, "-"		},	 // isolatin-1: 255 ad 173 
	{ 174, "(R)"	},	 // isolatin-1: 256 ae 174 
	{ 175, "-"		},	 // isolatin-1: 257 af 175 
	{ 176, "o"		},	 // isolatin-1: 260 b0 176 
	{ 177, "+/-"	},	 // isolatin-1: 261 b1 177 
	{ 178, "^2"		},	 // isolatin-1: 262 b2 178 
	{ 179, "^3"		},	 // isolatin-1: 263 b3 179 
	{ 180, "'"		},	 // isolatin-1: 264 b4 180 
	{ 181, "u"		},	 // isolatin-1: 265 b5 181 
	{ 182, "P"		},	 // isolatin-1: 266 b6 182 
	{ 183, "."		},	 // isolatin-1: 267 b7 183 
	{ 184, ","		},	 // isolatin-1: 270 b8 184 
	{ 185, "i"		},	 // isolatin-1: 271 b9 185 
	{ 186, "o_"		},	 // isolatin-1: 272 ba 186 
	{ 187, ">>"		},	 // isolatin-1: 273 bb 187 
	{ 188, "1/4"	},	 // isolatin-1: 274 bc 188 
	{ 189, "1/2"	},	 // isolatin-1: 275 bd 189 
	{ 190, "3/4"	},	 // isolatin-1: 276 be 190 
	{ 191, "?"		},	 // isolatin-1: 277 bf 191 
	{ 192, "A"		},	 // isolatin-1: 300 c0 192 
	{ 193, "A"		},	 // isolatin-1: 301 c1 193 
	{ 194, "A"		},	 // isolatin-1: 302 c2 194 
	{ 195, "A"		},	 // isolatin-1: 303 c3 195 
	{ 196, "Ae"		},	 // isolatin-1: 304 c4 196 
	{ 197, "A"		},	 // isolatin-1: 305 c5 197 
	{ 198, "AE"		},	 // isolatin-1: 306 c6 198 
	{ 199, "C"		},	 // isolatin-1: 307 c7 199 
	{ 200, "E"		},	 // isolatin-1: 310 c8 200 
	{ 201, "E"		},	 // isolatin-1: 311 c9 201 
	{ 202, "E"		},	 // isolatin-1: 312 ca 202 
	{ 203, "Ee"		},	 // isolatin-1: 313 cb 203 
	{ 204, "I"		},	 // isolatin-1: 314 cc 204 
	{ 205, "I"		},	 // isolatin-1: 315 cd 205 
	{ 206, "I"		},	 // isolatin-1: 316 ce 206 
	{ 207, "I"		},	 // isolatin-1: 317 cf 207 
	{ 208, "D"		},	 // isolatin-1: 320 d0 208 
	{ 209, "N"		},	 // isolatin-1: 321 d1 209 
	{ 210, "O"		},	 // isolatin-1: 322 d2 210 
	{ 211, "O"		},	 // isolatin-1: 323 d3 211 
	{ 212, "O"		},	 // isolatin-1: 324 d4 212 
	{ 213, "O"		},	 // isolatin-1: 325 d5 213 
	{ 214, "Oe"		},	 // isolatin-1: 326 d6 214 
	{ 215, "X"		},	 // isolatin-1: 327 d7 215 
	{ 216, "O"		},	 // isolatin-1: 330 d8 216 
	{ 217, "U"		},	 // isolatin-1: 331 d9 217 
	{ 218, "U"		},	 // isolatin-1: 332 da 218 
	{ 219, "U"		},	 // isolatin-1: 333 db 219 
	{ 220, "Ue"		},	 // isolatin-1: 334 dc 220 
	{ 221, "Y"		},	 // isolatin-1: 335 dd 221 
	{ 222, "p"		},	 // isolatin-1: 336 de 222 
	{ 223, "ss"		},	 // isolatin-1: 337 df 223 
	{ 224, "a"		},	 // isolatin-1: 340 e0 224 
	{ 225, "a"		},	 // isolatin-1: 341 e1 225 
	{ 226, "a"		},	 // isolatin-1: 342 e2 226 
	{ 227, "a"		},	 // isolatin-1: 343 e3 227 
	{ 228, "ae"		},	 // isolatin-1: 344 e4 228 
	{ 229, "a"		},	 // isolatin-1: 345 e5 229 
	{ 230, "ae"		},	 // isolatin-1: 346 e6 230 
	{ 231, "C"		},	 // isolatin-1: 347 e7 231 
	{ 232, "e"		},	 // isolatin-1: 350 e8 232 
	{ 233, "e"		},	 // isolatin-1: 351 e9 233 
	{ 234, "e"		},	 // isolatin-1: 352 ea 234 
	{ 235, "e"		},	 // isolatin-1: 353 eb 235 
	{ 236, "i"		},	 // isolatin-1: 354 ec 236 
	{ 237, "i"		},	 // isolatin-1: 355 ed 237 
	{ 238, "i"		},	 // isolatin-1: 356 ee 238 
	{ 239, "i"		},	 // isolatin-1: 357 ef 239 
	{ 240, "o"		},	 // isolatin-1: 360 f0 240 
	{ 241, "n"		},	 // isolatin-1: 361 f1 241 
	{ 242, "o"		},	 // isolatin-1: 362 f2 242 
	{ 243, "o"		},	 // isolatin-1: 363 f3 243 
	{ 244, "o"		},	 // isolatin-1: 364 f4 244 
	{ 245, "o"		},	 // isolatin-1: 365 f5 245 
	{ 246, "oe"		},	 // isolatin-1: 366 f6 246 
	{ 247, "/"		},	 // isolatin-1: 367 f7 247 
	{ 248, "o"		},	 // isolatin-1: 370 f8 248 
	{ 249, "u"		},	 // isolatin-1: 371 f9 249 
	{ 250, "u"		},	 // isolatin-1: 372 fa 250 
	{ 251, "u"		},	 // isolatin-1: 373 fb 251 
	{ 252, "ue"		},	 // isolatin-1: 374 fc 252 
	{ 253, "y"		},	 // isolatin-1: 375 fd 253 
	{ 254, "p"		},	 // isolatin-1: 376 fe 254 
	{ 255, "y"		},	 // isolatin-1: 377 ff 255 
	{ 0, NULL }
};

//--------------------------------------------------------------------

static int encode728(char *inbuf, char *outbuf, int size) {

	unsigned char *cP = (unsigned char *)outbuf;
	int newsize = 0;

	for (int i = 0; i < size; i++) {

		unsigned char c = (unsigned char)inbuf[i];

		if (c == '=' 
			&& isxdigit(inbuf[i+1])
			&& isxdigit(inbuf[i+2])) {
			
			char h[3] = "00";
			
			h[0] = inbuf[++i];
			h[1] = inbuf[++i];
			
			c = strtol(h, NULL, 16);
		}
		if (c >= 32) {
			*(cP++) = c;
			newsize++;	
		}
	}
	*cP = 0;
	return newsize;
}

// 7bit decode
//
char *encoder_7bit::mail2data(char *inbuf,int size,int &newsize,char */*path*/,char */*filename*/) {
	char *outbuf, *cP;
	if (outbuf = (char *)malloc(size + 1), !outbuf)
		return NULL;
	newsize = encode728(inbuf, outbuf, size);
	cP = (char *)realloc(outbuf, newsize + 1);
	return cP ? cP : outbuf;
}

// 7bit encode
//
char *encoder_7bit::data2mail(char *inbuf, int size, int &newsize,char */*path*/,char */*filename*/) {

	unsigned char *outbuf, *cP;
	int i;

	if (programconfig.transumlauts == 2) {
		return NULL;
	}
	
	newsize = 0;

	for (i = 0; inbuf[i]; i++)
		if (inbuf[i] & 0x80)
			break;
		
	if (!inbuf[i]) return NULL;

	if (outbuf = cP = (unsigned char *)malloc(size * 5), !cP)
		return NULL;
		
	for (int j = 0; j < i; j++) {
		*(cP++) = inbuf[j];
		newsize++;	// js
	}
			
	for (; inbuf[i]; i++) {

		if (programconfig.transumlauts == 1) {
		
			
			if (inbuf[i] & 0x80) {
				
				int ul;
				
				for (ul = 0; umlauts[ul].ascii; ul++) {
					if (umlauts[ul].code == (unsigned char)inbuf[i]) {
						strcpy((char *)cP, umlauts[ul].ascii);
						newsize += strlen(umlauts[ul].ascii);
						cP += strlen(umlauts[ul].ascii);
						break;
					}
				}
				if (!umlauts[ul].ascii) {
					strcpy((char *)cP, "??");
					newsize += 2;
					cP += 2;
				}					
			} else {
				newsize++;
				*(cP++) = inbuf[i];
			}
		
		
		} else {
			
			if ((inbuf[i] & 0x80) || inbuf[i] == '=') {
				
				int v = (unsigned char)inbuf[i];
				sprintf((char *)cP, "=%02X", v);
				newsize += 3;
				cP += 3;
					
			} else {
				newsize++;
				*(cP++) = inbuf[i];
			}
		}
	}
	*cP = 0;
	cP = (unsigned char *)realloc(outbuf, newsize + 1);
	return (char *)(cP ? cP : outbuf); 
}

// String der Form: =?iso...?Q?blabla?=
// 1)                 i     k
// 2)                        ik
// 3)                          i     k   (fertig)
//		
char *Translate728Line(char *text) {

	int k;
	int found = 0;
	char iso[20] = "";

	if (!text[0]) return text;

	for (k = 0; text[k+1]; k++)
		if (text[k] == '=' && text[k+1] == '?') {
			found = 1;
			break;
		}
		
	if (!found) return text;

	int start = k++;
	int field = 0;

	while (text[k]) {

		int i = k + 1;
		while (text[++k] && text[k] != '?')
			;
		switch (++field) {
				
			case 1: // iso-String 
				iso[0] = 0;
				strncat(iso, text + i, k - i);
				break;
						
			case 2:	// ignore - immer 'Q'?
				break;
						
			case 3: // Der Text...
				for (int j = i; j < k; j++)
					if (text[j] == '_')
						text[j] = ' ';
		
				encode728(text + i, text + start, k - i);
				i = strlen(text);
				strcat(text, text + k);
				k = i - 1;
				break;

			default:
				if (text[k] && text[k + 1] == '=') { // Das wars!
					strcpy(text + i, text + k + 2);
					Translate728Line(text + i);
					return text;
				}
				break;
		}
	}
	return text;
}


char *Translate827Line(char *text, char *iso) {

	int i;

	for (i = 0; text[i]; i++)
		if (((unsigned char)text[i]) & 0x80)
			break;
		
	if (text[i]) {

		sprintf(linebuffer, "=?%s?Q?", iso);
		char *cP = strchr(linebuffer, 0);
		
		for (int j = 0; j < i; j++) {
			if (text[i] == '?' || text[i] == ' ' || text[i] == '\t') {
				int v = (unsigned char)text[i];
				sprintf(cP, "=%02X", v);
				cP += 3;
			} else
				*(cP++) = text[j];
		}
			
		for (; text[i]; i++) {
			
			if ((text[i] & 0x80) ||
				 text[i] == '?' || text[i] == ' ' || text[i] == '\t') {
				int v = (unsigned char)text[i];
				sprintf(cP, "=%02X", v);
				cP += 3;
					
			} else
				*(cP++) = text[i];
		}
		strcpy(cP, "?=");
		return linebuffer;
	}

	return text;
}

char *Translate827Text(char *text, int size) {
	int newsize;
	return enc7bit.data2mail(text, size, newsize, NULL, NULL);
}

void TranslateFree(char *text) {
	if (text && text != linebuffer) free(text);
}
