/* 
 * This file is part of XCmail and is subject to the 
 * XCmail license agreement
 */

/* $Id: base64_encoding.cxx,v 2.7 2002/08/06 08:09:20 schmitzj Exp $ */


#include "../configure.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "../encoder.h"

//--------------------------------------------------------------------

class encoder_base64 : public encoder {

	public:
		encoder_base64(void) : encoder("base64;x-base64", "Ralf Becker") {};
		~encoder_base64() {};
		virtual char *data2mail(char *,int size,int &newsize,char *path,char *filename);
		virtual char *mail2data(char *,int size,int &newsize,char *path,char *filename);
		virtual void dispose(char *b) { free(b);}
};

static encoder_base64 base64;


//--------------------------------------------------------------------

static char vec[] =
	"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";


// base64 decode
//
char *encoder_base64::mail2data(char *inbuf,int size,int &newsize,char */*path*/,char */*filename*/) {

	char nw[4], buf[128], *p;
	char *outbuf, *cP;
	int done = 0;
	
	newsize = 0;
	outbuf = cP = (char *)malloc(size);
	if (!outbuf) return NULL;

	while (!done && egets(buf, 80, &inbuf)) {
	
		int len = strlen(buf);
		if (buf[len-1]=='\n') len--;

		// Workaround fr fehlerhaftes base64 encoding
		// = falsche Umbruch!!!!
		while (len%4 && *inbuf) {
			if (*inbuf != '\n')
				buf[len++]=*(inbuf++);
			else
				inbuf++;
		}
		buf[len]=0;

		for (int i = 0; i < len; i += 4) {
			
			int j, d;
			int val = 0;
			int num = 3;
			char *c = buf + i;
			
			if (c[2] == '=')
				num = 1;
			else
			if (c[3] == '=')
				num = 2;

			for (j = 0; j <= num; j++) {
				if (!c[j] || (p = strchr(vec, c[j]), !p)) {
					fprintf(stderr, "base64: not correct base64 encoded!\n");
					newsize=0;
					dispose(outbuf);
					return NULL;
				}
				d = p - vec;
				d <<= (3 - j) * 6;
				val += d;
			}
			for (j = 2; j >= 0; j--) {
				nw[j] = val & 255;
				val >>= 8;
			}
			
			memcpy(cP, nw, num);
			newsize += num;
			cP += num;
			
			if (num < 3) done=1;
		}
	}
	cP = (char *)realloc(outbuf, newsize + 1);
	return cP ? cP : outbuf;
}

// base64 encode
//
char *encoder_base64::data2mail(char *inbuf, int size, int &newsize,char *path,char *filename)
{ // PATH & FILENAME is special:
  // PATH==-1: FILENAME is flag
  // flags:
  //   1: no linebreaks

	unsigned char *outbuf, *cP;
	unsigned char enc[4];
	int c, i, n = 0, count = 0;
	unsigned int val = 0;
	int flags=0;

	if (((int)path)==-1)
	{
		flags=(int)filename;
	}

	newsize = 0;
	outbuf = cP = (unsigned char *)malloc(size * 3 /* / 2 */);
	if (!outbuf) return NULL;

	for (; size > 0; size--)
	{
	
		c = (unsigned char)*(inbuf++);
	
		if (n++ <= 2)
		{
			val <<= 8;
			val += c;
			continue;
		}

		for (i = 0; i < 4; i++)
		{
			enc[i] = val & 63;
			val >>= 6;
		}

		for (i = 3; i >= 0; i--)
		{
			if (enc[i]<66)
			{
				*(cP++) = vec[enc[i]];
				newsize++;
			}
		}
			
		n = 1;
		count += 4;
		val = c;

		if (count >= 76 && !flags)
		{
			*(cP++) = '\n';
			newsize++;
			count = 0;
		}
	}
	
	if (n == 1)
	{
		val <<= 16;
		for (i = 0; i < 4; i++)
		{
			enc[i] = val & 63;
			val >>= 6;
		}
		enc[0] = enc[1] = 64;
	}
	
	if (n == 2)
	{
		val <<= 8;
		for (i = 0; i < 4; i++) {
			enc[i] = val & 63;
			val >>= 6;
		}
		enc[0] = 64;
	}
	
	if (n == 3)
	{
		for (i = 0; i < 4; i++)
		{
			enc[i] = val & 63;
			val >>= 6;
		}
	}
	
	if (n)
	{
		for (i = 3; i >= 0; i--)
		{
			if (enc[i]<66)
			{
				*(cP++) = vec[enc[i]];
				newsize++;
			}
		}
	}
	if (!flags)
	{
		*(cP++) = '\n';
		newsize++;
	}
	*cP = 0;
	cP = (unsigned char *)realloc(outbuf, newsize + 1);
	return (char *)(cP ? cP : outbuf); 
}

