/* 
 * This file is part of XCmail and is subject to the 
 * XCmail license agreement
 */

/* $Id: uu_encoding.cxx,v 2.3 1999/02/16 12:23:40 beckerr Exp $ */


#include "../configure.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/stat.h>
#include <unistd.h>

#include "../encoder.h"

#define MAXPATHLEN	400

#define	DEC(c)		(((c) - ' ') & 077)		/* single character decode */
#define	ENC(c) 		((c) ? ((c) & 077) + ' ': '`')

#define PUTCHAR(c)	(newsize++, *(cP++)=c)

//--------------------------------------------------------------------

class encoder_uu : public encoder {

	public:
		encoder_uu(void) : encoder("uuencode;x-uuencode", "Ralf Becker") {};
		~encoder_uu() {};

		virtual char *data2mail(char *,int size,int &newsize,char *path,char *filename);
		virtual char *mail2data(char *,int size,int &newsize,char *path,char *filename);
		virtual void dispose(char *);
};


//--------------------------------------------------------------------



// uudecode
//
char *encoder_uu::mail2data(char *inbuf,int size,int &newsize,char *path,char *filename) {

	register int n;
	unsigned char ch, *p;
	unsigned char buf[MAXPATHLEN];
	unsigned char *outbuf, *cP;
	int mode;
	
	newsize = 0;
	outbuf = cP = (unsigned char *)malloc(size);
	if (!cP) {
		fprintf(stderr, "uudecode: out of memory!\n");
		return NULL;
	}

	/* search for header line */
	do {
		if (!egets((char *)buf, MAXPATHLEN - 1, &inbuf)) {
			fprintf(stderr, "uudecode: no \"begin\" line\n");
			dispose((char *)outbuf);
			return NULL;
		}
	} while (strncmp((char *)buf, "begin ", 6));

	sscanf((char *)buf, "begin %o %s", &mode, buf); // hier war scanf statt sscanf ?!?

	/* for each input line
	 */
	for (;;) {
		p = buf;
		if (!egets((char *)buf, MAXPATHLEN - 1, &inbuf)) {
			fprintf(stderr, "uudecode: short file.\n");
			dispose((char *)outbuf);
			return NULL;
		}

		/*
		 * `n' is used to avoid writing out all the characters
		 * at the end of the file.
		 */
		if ((n = DEC(*p)) <= 0)
			break;
			
		for (++p; n > 0; p += 4, n -= 3)
			if (n >= 3) {
				ch = DEC(p[0]) << 2 | DEC(p[1]) >> 4;
				PUTCHAR(ch);
				ch = DEC(p[1]) << 4 | DEC(p[2]) >> 2;
				PUTCHAR(ch);
				ch = DEC(p[2]) << 6 | DEC(p[3]);
				PUTCHAR(ch);
			} else {
				if (n >= 1) {
					ch = DEC(p[0]) << 2 | DEC(p[1]) >> 4;
					PUTCHAR(ch);
				}
				if (n >= 2) {
					ch = DEC(p[1]) << 4 | DEC(p[2]) >> 2;
					PUTCHAR(ch);
				}
				if (n >= 3) {
					ch = DEC(p[2]) << 6 | DEC(p[3]);
					PUTCHAR(ch);
				}
			}
	}
	if (!egets((char *)buf, sizeof(buf), &inbuf) ||
		 strcmp((char *)buf, "end")) {
		fprintf(stderr, "uudecode: no \"end\" line.\n");
		dispose((char *)outbuf);
		return NULL;
	}
	cP = (unsigned char *)realloc(outbuf, newsize);
	return (char *)(cP ? cP : outbuf);
}

char *encoder_uu::data2mail(char *inbuf, int size, int &newsize,char *path,char *filename) {

	int mode=0600;
	if (path && filename)
	{
		char fname[strlen(path)+strlen(filename)+1];
		strcpy(fname,path);
		strcat(fname,filename);

		struct stat st;
		stat(fname,&st);
		mode=st.st_mode&0777;
	}

	int ch, n;
	unsigned char *p;
	unsigned char tmp[MAXPATHLEN];
	
	newsize = 0;
	unsigned char *outbuf, *cP;
	
	outbuf = cP = (unsigned char *)malloc(size * 2 + 255);
	if (!cP) {
		fprintf(stderr, "uuencode: our of memory!\n");
		return NULL;
	}
	
	sprintf((char *)tmp, "begin %03o %s\n",mode,filename);
	eputs((char *)tmp, (char **)&cP);
	newsize += strlen((char *)tmp);

	for(int i = 0; i < size; i += 45) {
		n = 45;
		if (size - i < n)
			n = size - i;
		memcpy(tmp, inbuf + i, n);

		ch = ENC(n);
		PUTCHAR(ch);
		for (p = tmp; n > 0; n -= 3, p += 3) {
			ch = *p >> 2;
			ch = ENC(ch);
			PUTCHAR(ch);
			ch = (*p << 4) & 060 | (p[1] >> 4) & 017;
			ch = ENC(ch);
			PUTCHAR(ch);
			ch = (p[1] << 2) & 074 | (p[2] >> 6) & 03;
			ch = ENC(ch);
			PUTCHAR(ch);
			ch = p[2] & 077;
			ch = ENC(ch);
			PUTCHAR(ch);
		}
		PUTCHAR('\n');
	}
	ch = ENC('\0');
	PUTCHAR(ch);
	PUTCHAR('\n');

	strcpy((char *)tmp, "end\n");
	eputs((char *)tmp, (char **)&cP);
	newsize += strlen((char *)tmp);

	cP = (unsigned char *)realloc(outbuf, newsize);
	return (char *)(cP ? cP : outbuf); 
}

void encoder_uu::dispose(char *b) {
	free(b);
}

static encoder_uu uu;
