/* 
 * This file is part of XCmail and is subject to the 
 * XCmail license agreement
 */

/* $Id: folder.cxx,v 2.70 2002/09/23 10:45:10 schmitzj Exp $ */


#include "configure.h"

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/types.h>
#include <utime.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <ctype.h>
#include <signal.h>
#include <netdb.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <fcntl.h>
#include <arpa/inet.h>
// #ifdef __IRIX__
// #define _BSD_COMPAT
// #endif
#include <sys/file.h>
#include <setjmp.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <unistd.h>
#include <Xclasses/Xclasses.h>

#include "folder.h"
#include "mailer.h"
#include "encoder.h"
#include "helper.h"
#include "xc_snprintf.h"
#include "stringTokenizer.h"
#include "extraggs.h"
#include "filter_mgr.h"
#include "folder_io_mbox.h"
#include "folder_io_gzip.h"
#include "npop3get.h"
#include "address_book.h"
#include "gfxobjects.h"

class iolist *allios=NULL;
class folders *folderlist=NULL;

static int sortcount,sortanim;
static int cmp(class mail *m1,class mail *m2);
static int cmp(struct node *n1,struct node *n2);

extern xcfx_gadget f_busymode;

int io_class::ckname(char *n)
{
	int ret=0;
	char *c=n;
	while (*c)
	{
		for(int i=0;i<illegalcount;i++)
		{
			if (*c==illegalchars[i])
			{
				ret=1;
				break;
			}
		}
		c++;
	}
	return ret;
}
void io_class::ckname(char *n,char *buf,int size)
{
	char *c=n;
	char *b=buf;
	int s=0;
	while (*c)
	{
		*b=*c;
		for(int i=0;i<illegalcount;i++)
		{
			if (*c==illegalchars[i])
			{
				if (replaceby[i])
				{
					*b=0;
					strcat(b,replaceby[i]);
					s+=strlen(replaceby[i]);
				}
				else
				{
					b--;
				}
				s--;
			}
		}
		if (s<size)
			b++;
		c++;
		s++;
	}
	*b=0;
}
void io_class::setlasterror(char *e)
{
	if (lasterror)
		free(lasterror);
	lasterror=strdup(e);
}

// -----------------------------------------------------------------------------

folder::folder(io_class *i,char *name,int ft) : allmails(), dropedmails()
{
	io=i;
	foldername=strdup(name?name:"");
	fsize=0;
	ftype=ft;
	loadc=0;
	loaded=0;
	changed=0;
	readonly=0;
	nosavesupport=0;
	noappendsupport=0;
	nodeletesupport=0;
	data=strdup("");
	lnm_io=NULL;
	lnm_count=0;
}
folder::~folder()
{
	if (lnm_io)
	{
		io->close(lnm_io);
		lnm_io=NULL;
	}
	if (data) free(data);
	struct folder_data *n;
	while (n=(folder_data*)dropedmails.RemTail())
	{
		n->m->remreferer();
		delete n;
	}
	while (n=(folder_data*)allmails.RemTail())
	{
		n->m->remreferer();
		delete n;
	}
	if (foldername) free(foldername);
	foldername=NULL;
}
void folder::chname(char *name)
{
	if (foldername) free(foldername);
	foldername=strdup(name?name:"");
}
int folder::filter(mail *m)
{
	int r=1;
	if ((ftype==FOLDERTYPE_INBOX) || (m->getvfolder() && m->getvfolder()->getftype()==FOLDERTYPE_INBOX))
	{
		if (!(m->status() & STATUS_FILTERED))
		{
			m->status(m->status() | STATUS_FILTERED);
			if (filtermaster.filterMail(m))
			{
				r=0;
				m->makesmall();
			}
		}
	}
	return r;
}
int folder::unload(void)
{
	if (--loadc<=0)
	{
		loadc=0;

		struct folder_data *n;
		while (n=(folder_data*)dropedmails.RemTail())
		{
			n->m->remreferer();
			delete n;
		}
		while (n=(folder_data*)allmails.RemTail())
		{
			n->m->remreferer();
			delete n;
		}
		loaded=0;
		changed=0;
	}
	return 0;
}
int folder::load(int nofilter)
{
	int r=1;
	if (loadc++==0)
	{
		f_busymode.On();
		io_data *f=io->open(foldername,'r');
		if (f)
		{
			class mail *m;
			r=0;
			changed=0;

			while (m=io->nextmail(f))
			{
				appendmail_i(m,nofilter);
			}
			loaded=1;
			setdata(io->getdata(f));
			io->close(f);
		}
		f_busymode.Off();
	}
	else
	{
		folder_data *n=(folder_data*)allmails.head;
		while (n->next)
		{
			folderlist->updateoutput(this,n->m,1);
			n=(folder_data*)n->next;
		}		
		r=0;
	}
	return r;
}
mail *folder::loadnextmail(void)
{
	class mail *ret=NULL;
	if (loaded)
	{
		if (lnm_count==0)
		{
			loadc++;
		}
		struct folder_data *n=(folder_data*)allmails.Find(lnm_count+1);

		while (n && n->next)
		{
			lnm_count++;
			mail *m=n->m;
			if (!(m->status()&STATUS_DELETE))
			{
				ret=m;
				break;
			}
		}
		if (!ret)
		{
			lnm_count=0;
			loadc--;
		}
	}
	else
	{
		if (!lnm_io)
		{
			lnm_io=io->open(foldername,'r');
		}
		if (lnm_io)
		{
			ret=io->nextmail(lnm_io);
			if (!ret)
			{
				io->close(lnm_io);
				lnm_io=NULL;
			}
		}
	}
	return ret;
}
void folder::loadnextmail_abort(void)
{
	if (loaded)
	{
		if (lnm_count>0)
		{
			lnm_count=0;
			loadc--;
		}
	}
	else
	{
		if (lnm_io)
		{
			io->close(lnm_io);
			lnm_io=NULL;
		}
	}
}
int folder::save(void)
{ // printf("Save folder %s (LC: %d,L:%d/C:%d)\n",foldername,loadc,loaded,changed);
	int r=1;
	if (loadc==1)
	{
		if (!loaded || !changed)
			return 0;

		f_busymode.On();
		sortcount=0;
		sortanim=0;
		allmails.Sort(cmp);

changedagain:
		r=1;
		io_data *f=io->open(foldername,'w',getdata(),this);
		if (f)
		{
			int error_ret=0;
			r=0;
			changed=0;
			struct folder_data *n=(folder_data*)allmails.head;
			time_t now,tl; //=programconfig.rec_older_days*(60*60*24);
			time(&now);

			while (n->next)
			{
				mail *m=n->m;
				if (m->status()&STATUS_DELETE)
				{
					if (programconfig.usetrash && (ftype!=FOLDERTYPE_TRASH) && !(m->getvfolder() && m->getvfolder()->getftype()==FOLDERTYPE_TRASH))
					{
						if (!(m->status()&STATUS_SAVED))
						{
							io_data *fl;
							io_class *tio;
							char *fname;
							folder *t=folderlist->getmatchingfolder(io,FOLDERTYPE_TRASH,&tio,&fname);
							if (!t)
							{
								t=folderlist->create(io,TRASHFOLDER);
								if (t)
								{
									vfolder *v=folderlist->create(FOLDERTYPE_TRASH,TRASHFOLDER);
									if (!v)
									{
										v=folderlist->getfolder(TRASHFOLDER,1);
									}
									if (v)
									{
										folderlist->add(v,t);
									}
								}
							}
							if (t)
							{
								m->status(m->status() & ~STATUS_DELETE & ~STATUS_SAVED | STATUS_READ | STATUS_XCMAILREAD & ~STATUS_XCMAILNEW & ~STATUS_XCPOP3NEW);
								m->clearsavedate();
								m->oldfolder(g_foldername());
								t->appendmail(m);
							}
							m->status(m->status()|STATUS_SAVED|STATUS_DELETE);
						}
					}
				}
				else
				{
					int d,mi,y,h,min,sec;
					int r=1;
					tl=(m->getvfolder() && m->getvfolder()->rec_own?m->getvfolder()->rec_older_days:programconfig.rec_older_days)*(60*60*24);

					if ((ftype==FOLDERTYPE_INBOX || (m->getvfolder() && m->getvfolder()->getftype()==FOLDERTYPE_INBOX)) &&
					   (
					   ((m->getvfolder() && m->getvfolder()->rec_own?m->getvfolder()->rec_always:programconfig.rec_always)) ||
					   ((m->getvfolder() && m->getvfolder()->rec_own?m->getvfolder()->rec_read:programconfig.rec_read) && m->status()&STATUS_XCMAILREAD) ||
					   ((m->getvfolder() && m->getvfolder()->rec_own?m->getvfolder()->rec_reply:programconfig.rec_reply) && m->status()&STATUS_REPLYED) ||
					   ((m->getvfolder() && m->getvfolder()->rec_own?m->getvfolder()->rec_forward:programconfig.rec_forward) && m->status()&STATUS_FORWARDED) ||
					   ((m->getvfolder() && m->getvfolder()->rec_own?m->getvfolder()->rec_older:programconfig.rec_older) && m->status()&STATUS_XCMAILREAD && (now-m->parsesavedate(d,mi,y,h,min,sec))>tl)
// 					   (programconfig.rec_always) ||
// 					   (programconfig.rec_read && m->status()&STATUS_XCMAILREAD) ||
// 					   (programconfig.rec_reply && m->status()&STATUS_REPLYED) ||
// 					   (programconfig.rec_forward && m->status()&STATUS_FORWARDED) ||
// 					   (programconfig.rec_older && m->status()&STATUS_XCMAILREAD && (now-m->parsesavedate(d,mi,y,h,min,sec))>tl)
					   ))
					{
						io_data *fl;
						io_class *tio;
						char *fname;
						folder *t;
						vfolder *v=NULL;
						if (m->getvfolder() && m->getvfolder()->rec_own && m->getvfolder()->rec_folder_id[0])
						{
							v=folderlist->getfolder_byid(m->getvfolder()->rec_folder_id);
							if (v)
							{
								m->clearsavedate();
								v->appendmail(m);
								m->status(m->status()|STATUS_SAVED|STATUS_DELETE);
							}
						}
						else
						{
							t=folderlist->getmatchingfolder(io,FOLDERTYPE_RECEIVED,&tio,&fname);
							if (!t)
							{
								t=folderlist->create(io,RECEIVEDFOLDER);
								if (t)
								{
									v=folderlist->create(FOLDERTYPE_RECEIVED,RECEIVEDFOLDER);
									if (!v)
									{
										v=folderlist->getfolder(RECEIVEDFOLDER,1);
									}
									if (v)
									{
										folderlist->add(v,t);
									}
								}
							}
							if (t)
							{
								m->clearsavedate();
								t->appendmail(m);
								m->status(m->status()|STATUS_SAVED|STATUS_DELETE);
							}
						}
					}

// 					{
// 						io_data *fl;
// 						io_class *tio;
// 						char *fname;
// 						printf("RF: %x\n",folderlist->getmatchingfolder(io,FOLDERTYPE_RECEIVED,&tio,&fname));
// 					}
				}
				r=io->writemail(f,m);
				if (r) error_ret=1;

				if ((m->status() & STATUS_SERVERDELETE) && m->pop3host() && m->pop3user())
				{
					UIDLhistory *dele_uidlhist=new UIDLhistory(m->pop3host(),m->pop3user(),REMOVEFILE,NULL);
					if (dele_uidlhist)
					{
						dele_uidlhist->Read();
						dele_uidlhist->Add(m->xuidl());
						dele_uidlhist->Write();
						delete dele_uidlhist;
					}
				}

				if (m->status() & STATUS_DELETE)
				{
					folder_data *n2=(folder_data*)n->next;
					allmails.Remove(n);
					delete n;
					remmail(m);
					n=n2;
				}
				else
					n=(folder_data*)n->next;
			}
			int ret=io->close(f);
			if (ret<0)
			{ // folder changed durring open/write - no problem here, try again
			  // OK, ugly GOTO, but the easiest - and hopefully not ofter required
				goto changedagain;
			}
			else if (ret>0)
			{ // general error!
				r=1;
			}
			else
			{
				r=0;
				changed=0;
				io_data *f=io->open(foldername,'r');
				if (f)
				{
					setdata(io->getdata(f));
					io->close(f);
				}
			}
		}
		f_busymode.Off();
	}
	else
		r=0;
	return r;
}
int folder::cknew(void)
{
	int r=1;
	io_data *f=io->open(foldername,'i');
	if (f)
	{
		r=io->cknew(f,data);
//		setdata(io->getdata(f));
		io->close(f);
	}
	return r;	
}
int folder::getnew(void)
{
	if (!loaded)
		return 0;

	int r=0;
	io_data *f=io->open(foldername,'n',getdata(),this);
	if (f)
	{
		class mail *m;

		while (m=io->nextmail(f))
		{
			appendmail_i(m);
			r++;
		}
		setdata(io->getdata(f));
		io->close(f);
	}
	return r;	
}
void folder::mix(folder *mc)
{
// add new mails from f into this

	mail *m1,*m2;
	folder_data *n1=(folder_data*)allmails.head,*n2;
	int val;

	// check for already deleted mail in folder
	while (n1->next)
	{
		m1=n1->m;
		if (!(m1->status() & STATUS_DELETE))
		{
			val=0;
			n2=(folder_data*)mc->allmails.head;
			while (n2->next && !val)
			{
				m2=n2->m;
				if (*m1==*m2)
				{
					val=1;
				}
				n2=(folder_data*)n2->next;
			}
			if (!val)
			{ // printf("Already deleted mail found\n");
				m1->status(STATUS_DELETE);
			}
		}
		n1=(folder_data*)n1->next;
	}

	// check for new mail in folder
	n1=(folder_data*)mc->allmails.head;
	while (n1->next)
	{
		m1=n1->m;
		val=0;
		n2=(folder_data*)allmails.head;
		while (n2->next && !val)
		{
			m2=n2->m;
			if (*m1==*m2)
			{
				val=1;
				if (!(m1->status() & STATUS_NEW))
					m2->status(m2->status() & ~STATUS_NEW| m1->status());
			}
			n2=(folder_data*)n2->next;
		}
		n2=(folder_data*)dropedmails.head;
		while (n2->next && !val)
		{
			m2=n2->m;
			if (*m1==*m2)
			{
				val=1;
			}
			n2=(folder_data*)n2->next;
		}
		if (!val)
		{ // new mail m1
			class mail *m=new mail(strdup(m1->themailtext()));
			appendmail_i(m);
			// printf("New mail found %s\n",m1->messageid());
		}
		n1=(folder_data*)n1->next;
	}
}
void folder::sub(folder *mc)
{
// remove all mail from current that are in mc

	mail *m1,*m2;
	int val;
	folder_data *n1=(folder_data*)allmails.head,*n2;

	while (n1->next)
	{
		m1=n1->m;
		n2=(folder_data*)mc->allmails.head;
		val=0;
		while (n2->next && !val)
		{
			m2=n2->m;
			if (*m1==*m2)
			{
				val=1;
			}
			n2=(folder_data*)n2->next;
		}
		n2=(folder_data*)n1->next;
		if (val)
		{
			allmails.Remove(n1);
			delete n1;
			m1->remreferer();
		}
		n1=n2;
	}

	n1=(folder_data*)allmails.head;
	while (n1->next)
	{
		m1=n1->m;
		n2=(folder_data*)mc->dropedmails.head;
		val=0;
		while (n2->next && !val)
		{
			m2=n2->m;
			if (*m1==*m2)
			{
				val=1;
			}
			n2=(folder_data*)n2->next;
		}
		n2=(folder_data*)n1->next;
		if (val)
		{
			allmails.Remove(n1);
			delete n1;
			m1->remreferer();
		}
		n1=n2;
	}

	n1=(folder_data*)dropedmails.head;
	while (n1->next)
	{
		m1=n1->m;
		n2=(folder_data*)mc->allmails.head;
		val=0;
		while (n2->next && !val)
		{
			m2=n2->m;
			if (*m1==*m2)
			{
				val=1;
			}
			n2=(folder_data*)n2->next;
		}
		n2=(folder_data*)n1->next;
		if (val)
		{
			allmails.Remove(n1);
			delete n1;
			m1->remreferer();
		}
		n1=n2;
	}

	n1=(folder_data*)dropedmails.head;
	while (n1->next)
	{
		m1=n1->m;
		n2=(folder_data*)mc->dropedmails.head;
		val=0;
		while (n2->next && !val)
		{
			m2=n2->m;
			if (*m1==*m2)
			{
				val=1;
			}
			n2=(folder_data*)n2->next;
		}
		n2=(folder_data*)n1->next;
		if (val)
		{
			allmails.Remove(n1);
			delete n1;
			m1->remreferer();
		}
		n1=n2;
	}
}
void folder::clear(void)
{
	if (loaded)
	{
		mail *m;
		folder_data *n1=(folder_data*)allmails.head;

		while (n1->next)
		{
			m=n1->m;
			m->deleteLocalMail();
			n1=(folder_data*)n1->next;
		}
	}
	else
	{
		io_data *f=io->open(foldername,'w',getdata(),this);
		if (f)
		{
			io->close(f);
			io_data *f=io->open(foldername,'r');
			if (f)
			{
				setdata(io->getdata(f));
				io->close(f);
			}
		}		
	}
}
int folder::appendmail_i(mail *m,int nofilter)
{
	m->addreferer();
	m->setfolder(this);
	folder_data *n=new folder_data;
	n->m=m;

	int f=1;
	if (!nofilter)
		f=filter(m);

	if (f)
	{
		allmails.AddTail(n);
		folderlist->updateoutput(this,m);
	}
	else
		dropedmails.AddTail(n);
	return 0;
}
int folder::appendmail(mail *m)
{
	int ret=1;
	m->addreferer();
	m->setfolder(this);
	int f=filter(m);
	if (loaded || loadc>0)
	{
		folder_data *n=new folder_data;
		n->m=m;

		if (f)
		{
			allmails.AddTail(n);
			folderlist->updateoutput(this,m);
		}
		else
			dropedmails.AddTail(n);

		if (f)
		{
	 		io_data *fl=io->open(foldername,'a');
 			if (fl)
 			{
 				if (!io->writemail(fl,m))
					ret=0;
	 			io->close(fl);

				fl=io->open(foldername,'r');
				if (fl)
				{
					setdata(io->getdata(fl));
					io->close(fl);
				}
	 		}
		}
	}
	else
	{
		if (f)
		{
			io_data *fl=io->open(foldername,'a');
			if (fl)
			{
				if (!io->writemail(fl,m))
					ret=0;
				io->close(fl);
			}
		}
		m->remreferer();
	}
	return ret;
}
int folder::remmail(mail *m)
{
	folderlist->rmoutput(this,m);
	m->remreferer();
	return 0;
}
void folder::setdata(char *d)
{
	if (data)
		free(data);
	data=strdup(d?d:"");
}
int folder::count(void)
{
	return allmails.Count();
}
int folder::size(void)
{
	int r=0;
	io_data *f=io->open(foldername,'i');
	if (f)
	{
		r=(int)io->getFolderInfo(f,IOCD_SIZE);
		io->close(f);
	}
	return r;	

}
class mail *folder::getmail(int n)
{
	if (n>=1 && n<=allmails.Count())
	{
		struct folder_data *m=(folder_data*)allmails.Find(n);
		if (m)
		{
			return m->m;
		}
	}
	return NULL;
}

// -----------------------------------------------------------------------------

vfolder::vfolder(char *name,dlist *vlst,int ft,char *s) : allmails()
{
	ftype=ft;
	allfolders=vlst;
	hidden=0;
	readonly=0;
	nosavesupport=0;
	noappendsupport=0;
	nodeletesupport=0;
	foldername=strdup(name);
	subdir=strdup(s?s:"");
	loadc=0;
	rec_own=0;
	rec_always=0;
	rec_read=0;
	rec_reply=0;
	rec_forward=0;
	rec_older=0;
	rec_older_days=7;
	rec_folder_id[0]=0;

	getUniqueID(unique_id);
}
vfolder::~vfolder()
{
	folder_data *n;
	while (n=(folder_data*)allmails.RemTail())
	{
		n->m->remreferer();
		delete n;
	}
	free(foldername);
	free(subdir);
}
void vfolder::chname(char *name)
{
	if (foldername) free(foldername);
	foldername=strdup(name?name:"");
}
char *normalize_subject(mail *in, char *out,int maxsize, int addre=1)
{
	char *p,*s;

	p=s=in->subject();
	if (strcncmp("Re:", p) == 0 || strcncmp("AW:", p) == 0)
	{
		p+=3;
		while (*p && !isgraph(*p)) p++;
		if (addre)
		{
			xc_snprintf(out,maxsize, "%s\x01", p);
			strncat(out, s, maxsize<(p-s)?maxsize:(p-s));
		}
		else
			xc_strncpy(out,p,maxsize);
	}
	else
	{
		xc_strncpy(out, s,maxsize);
	}
	return out;
}
static char *anim[]={"-","\\\\","|","/"};
int cmp(class mail *m1,class mail *m2)
{// sort by 0:unsorted, 1:sender, 2: recipient, 2: subject, 3: send date, 4: received date, 5: new first received date, 6: size, 7: priority
	int r=-1;
	int sortmode = programconfig.currentsortmode;
	int circulate = 0;

	if (sortcount++>1000)
	{
		pstatus("Sorting mail... %s",anim[sortanim]);
		sortcount=0;
		if (++sortanim>3) sortanim=0;
	}

	while(sortmode && circulate<4)
	{
		switch (sortmode)
		{
			case 1: // sender
				r=strcscmp(m1->from(),m2->from());
				sortmode = r ? 0 : 4;  // next is "send date"
				break;
			case 2: // recipient
				r=strcscmp(m1->to(),m2->to());
				sortmode = r ? 0 : 4;  // next is "send date"
				break;
			case 3: // subject
				{
					char buf1[200], buf2[200];
					r=strcscmp(normalize_subject(m1, buf1,sizeof(buf1)),normalize_subject(m2, buf2,sizeof(buf2)));
					sortmode = r ? 0 : 4;  // next is "send date"
				}
				break;
			case 4: // send date
				{
					int d,m,y,h,min,sec;
					r=m1->parsefromdate(d,m,y,h,min,sec)
					 -m2->parsefromdate(d,m,y,h,min,sec);
					sortmode = r ? 0 : 1; // next is "sender"
				}
				break;
			case 5: // rev send date
				{
					int d,m,y,h,min,sec;
					r=-( m1->parsefromdate(d,m,y,h,min,sec)
					    -m2->parsefromdate(d,m,y,h,min,sec));
					sortmode = r ? 0 : 1; // next is "sender"
				}
				break;
			case 6: // newfirstrevsenddate
				{
					int d,m,y,h,min,sec;
					char buf1[100],buf2[100];
					r=m1->parsefromdate(d,m,y,h,min,sec)
					 -m2->parsefromdate(d,m,y,h,min,sec);

					if ((m1->status() & (STATUS_OLD|STATUS_NEW|STATUS_XCMAILNEW|STATUS_XCPOP3NEW))
					  &&(m2->status() & (STATUS_OLD|STATUS_NEW|STATUS_XCMAILNEW|STATUS_XCPOP3NEW)))
						r=r;
					else if (m1->status() & (STATUS_OLD|STATUS_NEW|STATUS_XCMAILNEW|STATUS_XCPOP3NEW))
						r=-1;
					else if (m2->status() & (STATUS_OLD|STATUS_NEW|STATUS_XCMAILNEW|STATUS_XCPOP3NEW))
						r=1;
					else
						r=-r;
					sortmode = r ? 0 : 1; // next is "sender"
				}
				break;
			case 7:
				r=strlen(m2->themailtext())-strlen(m1->themailtext());
				sortmode = r ? 0 : 1; // next is "sender"
				break;
            case 8:
            	r=(m2->priority()-m1->priority());
				sortmode = r ? 0 : 1; // next is "sender"
				break;
		}
		circulate++;
	}
	return r;
}
int cmp(struct node *n1,struct node *n2)
{
	return cmp(((folder_data*)n1)->m,((folder_data*)n2)->m);
}
void vfolder::sort(void)
{
	if (programconfig.currentsortmode)
	{
		sortcount=0;
		sortanim=0;
		allmails.Sort(cmp);
	}
}
int vfolder::count(void)
{
	return allmails.Count();
}
int vfolder::size(void)
{
	int r=0;
	struct vfolder_data *n=(vfolder_data*)allfolders->head;

	while (n->next)
	{
		r+=n->f->size();
		n=(vfolder_data*)n->next;
	}
	return r;
}

class mail *vfolder::getmail(int n)
{
	if (n>=1 && n<=allmails.Count())
	{
		struct folder_data *m=(folder_data*)allmails.Find(n);
		if (m)
		{
			return m->m;
		}
	}
	return NULL;
}
int vfolder::getpos(mail *m)
{
	struct folder_data *n=(folder_data*)allmails.head;

	while (n->next)
	{
		if (n->m==m)
		{
			return allmails.Find(n);
		}
		n=(folder_data*)n->next;
	}
	return 0;
}
int vfolder::search(int start,char *type,char *pattern,short ncase,short substring,short reg)
{
	int n=0,r=0,dummy;
// beckerr
	if (!strcmp(type,"sender")) n=1;
	else if (!strcmp(type,"sender_email")) n=2;
	else if (!strcmp(type,"subject")) n=3;
	else if (!strcmp(type,"word")) n=4;
	else if (!strcmp(type,"older")) n=5;
	else if (!strcmp(type,"newer")) n=6;
	else if (!strcmp(type,"recipient")) n=7;
	else if (!strcmp(type,"carbon_copy")) n=8;

	if (!n) return 0;

	int (*cmp)(char*,char*)=strcncmp;

	if (reg) cmp=strregcmp;
	else if (ncase && substring) cmp=strscncmp;
	else if (!ncase && substring) cmp=strsnncmp;
	else if (ncase && !substring) cmp=strcncmp;
	else if (!ncase && !substring) cmp=strnncmp;

	for(int i=start;i<=allmails.Count()&&!r;i++)
	{
		struct folder_data *m=(folder_data*)allmails.Find(i);
		if (m)
		{
			switch(n)
			{
				case 1: // sender
					r=!cmp(pattern,m->m->from());
					break;
				case 2: // sender email
					r=!cmp(pattern,m->m->from_email());
					break;
				case 3: // subject
					r=!cmp(pattern,m->m->subject());
					break;
				case 4: // word
					r=!cmp(pattern,m->m->mailbody(dummy,0));
					break;
				case 5: // older
					break;
				case 6: // newer
					break;
				case 7: // recipient
					r=!cmp(pattern,m->m->to());
					break;					
				case 8: // cc
					r=!cmp(pattern,m->m->cc());			
					break;	
			}
			if (r) return i;
		}
	}
	return 0;
}
int vfolder::load(void)
{
	int r=0;
	loadc++;
	struct vfolder_data *n=(vfolder_data*)allfolders->head;
	sort_marker_loading=1;
	while (n->next)
	{
		sort_marker_counter=0;
		r=n->f->load()|r;
		sort_marker_loading=0; // only the first - all others always need to sort into the mlv
		n=(vfolder_data*)n->next;
	}
	sort_marker_loading=0;
	folders::saveprefs(this);
	return r;
}
int vfolder::save(void)
{
	int r=0;
	struct vfolder_data *n=(vfolder_data*)allfolders->head;

	while (n->next)
	{
		r=n->f->save()|r;
		n=(vfolder_data*)n->next;
	}
	return r;
}
int vfolder::unload(void)
{ // printf("vfolder::unload - %d\n",loadc-1);
	int r=0;
	struct vfolder_data *n=(vfolder_data*)allfolders->head;

	while (n->next)
	{
		r=n->f->unload()|r;
		n=(vfolder_data*)n->next;
	}

	if (--loadc<=0)
	{
		loadc=0;
		folder_data *n2;
		while (n2=(folder_data*)allmails.RemTail())
		{
			n2->m->remreferer();
			delete n2;
		}
	}
	return r;
}
int vfolder::cknew(void)
{
	int r=1;
	struct vfolder_data *n=(vfolder_data*)allfolders->head;

	while (n->next)
	{
		r=r&n->f->cknew();
		n=(vfolder_data*)n->next;
	}
	return r;
}
int vfolder::getnew(void)
{
	int r=0;
	struct vfolder_data *n=(vfolder_data*)allfolders->head;

	while (n->next)
	{
		r=n->f->getnew()+r;
		n=(vfolder_data*)n->next;
	}
	folders::saveprefs(this);
	return r;
}
void vfolder::clear(void)
{
	struct vfolder_data *n=(vfolder_data*)allfolders->head;

	while (n->next)
	{
		n->f->clear();
		n=(vfolder_data*)n->next;
	}
	folders::saveprefs(this);
}
int vfolder::appendmail_default(class mail *m)
{
	int r=0;
	struct vfolder_data *n=(vfolder_data*)allfolders->head;
	m->setvfolder(this);
	if (n)
		r=n->f->appendmail(m);
	return r;
}
int vfolder::appendmail(class mail *m,multilistview *mlv,int oldmail)
{ // printf("vfolder::appendmail(%x,%x,%d)\n",m,mlv,oldmail);
	int r=0;
	int done=0;
	int i=1;
	m->setvfolder(this);
	if (oldmail)
	{
		if (mlv)
		{
			for(int j=0;i<mlv->Count();j++)
			{
				if ((mail*)mlv->GetData(j+1)==m)
				{
					done=1;
					break;
				}
			}

			if (!done)
			{
				folder_data *n2=(folder_data*)allmails.head;
				while (n2->next)
				{
					if (cmp(m,n2->m)<0)
					{
						mlv->AddBefore(i,m->lvline(),(void*)m);
						done=1;
						break;	
					}
					i++;
					n2=(folder_data*)n2->next;
				}
				if (!done)
				{
					mlv->AddAtEnd(m->lvline(),(void*)m);
				}
			}
		}
	}
	else
	{
		folder_data *n1=new folder_data;
		n1->m=m;
		m->addreferer();

		folder_data *n2=(folder_data*)allmails.head;
		while (n2->next)
		{
			if (cmp(n1->m,n2->m)<0)
			{
				allmails.Insert(n1, n2);
				if (mlv)
				{
					mlv->AddBefore(i,m->lvline(),(void*)m);
					if (sort_marker_loading)
					{
						if (i<(mlv->Count()>>1))
						{
							sort_marker_counter++;
							if (sort_marker_counter>10)
							{
								if (m->getfolder())
								{
									m->getfolder()->change();
								}
							}
						}
					}
				}
				done=1;
				break;	
			}
			i++;
			n2=(folder_data*)n2->next;
		}
		if (!done)
		{
			allmails.AddTail(n1);
			if (mlv)
			{
				mlv->AddAtEnd(m->lvline(),(void*)m);
			}
		}
	}
	return r;
}
int vfolder::rmmail(class mail *m,multilistview *mlv)
{
	if (mlv)
	{
		for(int i=0;i<mlv->Count();i++)
		{
			if ((mail*)mlv->GetData(i+1)==m)
			{
				mlv->Remove(i+1);
				break;
			}
		}
	}
	folder_data *n2=(folder_data*)allmails.head;
	while (n2->next)
	{
		if (n2->m==m)
		{
			allmails.Remove(n2);
			delete n2;
			m->remreferer();
			break;
		}
		n2=(folder_data*)n2->next;
	}
	return 0;
}
int vfolder::del(void)
{
	folderlist->remove(this);
	return 0;
}
int vfolder::add(class folder *f)
{
	struct vfolder_data *n=new vfolder_data;
	n->f=f;
	allfolders->AddTail(n);
	return 0;
}
int vfolder::remove(class folder *f)
{
	int r=1;
	struct vfolder_data *n=(vfolder_data*)allfolders->head;
	while (n->next)
	{
		if (n->f==f)
		{
			allfolders->Remove(n);
			delete n;
			r=0;
			break;
		}
		n=(vfolder_data*)n->next;
	}
	return r;
}
int vfolder::writefolder(class folder *f)
{
	int r=1;
	struct vfolder_data *n=(vfolder_data*)allfolders->head;
	while (n->next)
	{
		if (n->f==f)
		{
			allfolders->Remove(n);
			allfolders->AddHead(n);
			r=0;
			break;
		}
		n=(vfolder_data*)n->next;
	}
	return r;
}
int vfolder::g_size(void)
{
	return size();
}

// -----------------------------------------------------------------------------

folders_functions::folders_functions(void (*fn)(void)) : node()
{
	func=fn;
}
folders_functions::folders_functions(class rfunc_class *fn) : node()
{
	rfunc=fn;
	func=NULL;
}
folders_functions::folders_functions(folders_functions *fo) : node()
{
	func=NULL;
	if (fo->func)
		func=fo->func;
	else
		rfunc=fo->rfunc;
}
folders_functions::~folders_functions()
{ }
void folders_functions::call(enum CALLTYPE m=FOLDERS)
{
	if (func)
		func();
	else if (m==FOLDERS)
		rfunc->updatefolders();
	else
		rfunc->updatecurrent();
}
Bool folders_functions::operator==(void (*fn)(void))
{
	return fn==func;
}
Bool folders_functions::operator==(class rfunc_class *fn)
{
	return fn==rfunc;
}

// -----------------------------------------------------------------------------

folders::folders(void) : allvfolders(), allfolders(), registeredfolderfunctions(), registeredcurrentfunctions()
{
	currentvf=NULL;
	for(int i=0;i<ALLFOLDERSSTR;i++)
		allfolders_str[i]=NULL;
	cknewcounter=1;

	spoolc_folderfunction=0;
	spool_folderfunction_need=0;
	spoolc_currentfunction=0;
	spool_currentfunction_need=0;
}
folders::folders(folders *of) : allvfolders(), allfolders(), registeredfolderfunctions(), registeredcurrentfunctions()
{
	currentvf=of->currentvf;
	for(int i=0;i<ALLFOLDERSSTR;i++)
		allfolders_str[i]=NULL;
	cknewcounter=1;

	spoolc_folderfunction=0;
	spool_folderfunction_need=0;
	spoolc_currentfunction=0;
	spool_currentfunction_need=0;

	folders_functions *n1;
	folders_functions *n3=(folders_functions*)of->registeredcurrentfunctions.head;
	while (n3->next)
	{
		n1=new folders_functions(n3);
		registeredcurrentfunctions.AddTail(n1);
		n3=(folders_functions*)n3->next;
	}
	n3=(folders_functions*)of->registeredfolderfunctions.head;
	while (n3->next)
	{
		n1=new folders_functions(n3);
		registeredfolderfunctions.AddTail(n1);
		n3=(folders_functions*)n3->next;
	}
	signalfolderregisteredfunctions();
	signalcurrentregisteredfunctions();
}
folders::~folders()
{
	folders_vdata *n2;
	while (n2=(folders_vdata*)allvfolders.RemTail())
	{
		delete n2->f;
		delete n2;
	}
	for(int i=0;i<ALLFOLDERSSTR;i++)
	{
		if (allfolders_str[i])
		{
			char **a=allfolders_str[i];
			while (*a)
			{
				free(*a);
				a++;
			}
			delete [] allfolders_str[i];
		}
	}
	folders_data *n1;
	while (n1=(folders_data*)allfolders.RemTail())
	{
		delete n1->f;
		delete n1;
	}
	folders_functions *n3;
	while (n3=(folders_functions*)registeredcurrentfunctions.RemTail())
	{
		delete n3;
	}
	while (n3=(folders_functions*)registeredfolderfunctions.RemTail())
	{
		delete n3;
	}
}
void folders::addfolder(dlist *subfolders,char *name,char *data)
{
// name:
// io_name:foldername

	tokenizer tk(name,":");

	io_class *io=allios->FindIO((char*)tk[0]);
	if (io)
	{
		folders_data *n1=NULL;
		do
		{
			n1=(folders_data*)(n1?allfolders.FindText((char*)tk[1],n1):allfolders.FindText((char*)tk[1]));
			if (n1)
			{
				if (n1->f->getio()==io)
				{
					vfolder_data *v=new vfolder_data;
					v->f=n1->f;
					n1->f->setdata(data);
					subfolders->AddTail(v);
					break;
				}
			}
			else
			{ // maybe better remove this folder ?
				folder *f=new folder(io,(char*)tk[1]);
				folders_data *n2=new folders_data((char*)tk[1],f);
				allfolders.AddTail(n2);
					vfolder_data *v=new vfolder_data;
				v->f=n2->f;
				f->setdata(data);
				subfolders->AddTail(v);
				break;
			}
		}
		while (n1);
	}
}

static int  folder_version=0;
static char folder_files[801];
static char folder_data[801];
static int  folder_type;
static int  folder_hidden;
static char folder_subdir[20];
static char folder_id[20];
static int  rec_own;
static int  rec_always;
static int  rec_read;
static int  rec_reply;
static int  rec_forward;
static int  rec_older;
static int  rec_older_days;
static char rec_folder_id[20];

static configopts folder_prefs[]={
{"I:version",(char*)&folder_version},
{"T800:file",folder_files},
{"T20:subdir",folder_subdir},
{"Enormal,inbox,trash,received,outbox:type",(char*)&folder_type},
{"B:hidden",(char*)&folder_hidden},
{"T800:statdata",folder_data},
{"T20:unique_id",folder_id},
{"B:rec_own",(char*)&rec_own},
{"B:rec_always",(char*)&rec_always},
{"B:rec_read",(char*)&rec_read},
{"B:rec_reply",(char*)&rec_reply},
{"B:rec_forward",(char*)&rec_forward},
{"B:rec_older",(char*)&rec_older},
{"I:rec_older_days",(char*)&rec_older_days},
{"T20:rec_folder_id",rec_folder_id},
{NULL,NULL}};

void folders::saveprefs(class vfolder *v)
{
	char dir[300],wdir[300];
	strcpy(dir,getenv("HOME"));
	strcat(dir,"/"HOMEMAIL"/.");
	if (v->g_subdir() && v->g_subdir()[0])
	{
		strcat(dir,"D");
		strcat(dir,v->g_subdir());
		strcat(dir,"D_");
	}
	strcat(dir,v->g_foldername());
	strcat(dir,".prefs");

	config cfg_file(dir);
	folder_type=v->getftype();
	folder_data[0]=0;
	folder_files[0]=0;
	folder_hidden=v->hidden;
	strcpy(folder_id,v->g_id());
	strcpy(folder_subdir,v->g_subdir());
	folder_version=2;

	tokenizer tk(NULL,",");
	tokenizer tk2(NULL,",");

	struct vfolder_data *n=(vfolder_data*)v->allfolders->head;
	while (n->next)
	{
		xc_snprintf(wdir,sizeof(wdir),"%s:%s",n->f->getio()->io_name(),n->f->g_foldername());
		tk.append(new token(wdir));
		tk2.append(new token(n->f->getdata()));
		n=(vfolder_data*)n->next;
	}
//printf("tk1:\n");
//tk.debugOutput();
//printf("tk2:\n");
//tk2.debugOutput();
	strcpy(folder_files,tk.string());
	strcpy(folder_data,tk2.string());

	cfg_file.SetItems(folder_prefs);
	cfg_file.Write();
}
void folders::saveprefs(void)
{
	char dir[300],wdir[300],ndir[300];
	DIR *dfd;
	struct dirent *dptr;
	struct stat sbuf;

	strcpy(dir,getenv("HOME"));
	strcat(dir,"/"HOMEMAIL"/");
	if (dfd=opendir(dir))
	{
		while (dptr=readdir(dfd))
		{
			if (dptr->d_name[0]=='.')
			{
				int l=strlen(dptr->d_name);
				strcpy(wdir,dir);
				strcat(wdir,dptr->d_name);
				stat(wdir,&sbuf);
				if (S_ISREG(sbuf.st_mode) && !S_ISDIR(sbuf.st_mode))
				{
					if (l>6 && 
							 dptr->d_name[l-6]=='.' &&
							 dptr->d_name[l-5]=='p' &&
							 dptr->d_name[l-4]=='r' &&
							 dptr->d_name[l-3]=='e' &&
							 dptr->d_name[l-2]=='f' &&
							 dptr->d_name[l-1]=='s')
					{
						strcpy(ndir,wdir);
						strcat(ndir,".old");
						::rename(wdir,ndir);
					}
				}
			}
		}
		closedir(dfd);
	}

	folders_vdata *n=(folders_vdata*)allvfolders.head;
	while (n->next)
	{
		saveprefs(n->f);
		n=(folders_vdata*)n->next;
	}

	if (dfd=opendir(dir))
	{
		while (dptr=readdir(dfd))
		{
			if (dptr->d_name[0]=='.')
			{
				int l=strlen(dptr->d_name);
				strcpy(wdir,dir);
				strcat(wdir,dptr->d_name);
				stat(wdir,&sbuf);
				if (S_ISREG(sbuf.st_mode) && !S_ISDIR(sbuf.st_mode))
				{
					if (l>10 && 
							 dptr->d_name[l-10]=='.' &&
							 dptr->d_name[l-9]=='p' &&
							 dptr->d_name[l-8]=='r' &&
							 dptr->d_name[l-7]=='e' &&
							 dptr->d_name[l-6]=='f' &&
							 dptr->d_name[l-5]=='s' &&
							 dptr->d_name[l-4]=='.' &&
							 dptr->d_name[l-3]=='o' &&
							 dptr->d_name[l-2]=='l' &&
							 dptr->d_name[l-1]=='d')
					{
						::remove(wdir);
					}
				}
			}
		}
		closedir(dfd);
	}
}
static int vcmp(struct node *n1,struct node *n2)
{
	if (((folders_vdata*)n1)->f->readonly!=((folders_vdata*)n2)->f->readonly)
		return ((folders_vdata*)n1)->f->readonly?-1:1;
	else if (((folders_vdata*)n1)->f->getftype()!=((folders_vdata*)n2)->f->getftype()
	       && !strcmp(((folders_vdata*)n1)->f->g_subdir(),((folders_vdata*)n2)->f->g_subdir()))
	{
		int r=0;
		if (((folders_vdata*)n1)->f->getftype()==FOLDERTYPE_INBOX)
			r=-1;
		else if (((folders_vdata*)n2)->f->getftype()==FOLDERTYPE_INBOX)
			r=1;
		else if (((folders_vdata*)n1)->f->getftype()==FOLDERTYPE_OUTBOX)
			r=-1;
		else if (((folders_vdata*)n2)->f->getftype()==FOLDERTYPE_OUTBOX)
			r=1;
		else if (((folders_vdata*)n1)->f->getftype()==FOLDERTYPE_TRASH)
			r=-1;
		else if (((folders_vdata*)n2)->f->getftype()==FOLDERTYPE_TRASH)
			r=1;
		else if (((folders_vdata*)n1)->f->getftype()==FOLDERTYPE_RECEIVED)
			r=-1;
		else if (((folders_vdata*)n2)->f->getftype()==FOLDERTYPE_RECEIVED)
			r=1;
		else if (((folders_vdata*)n1)->f->getftype()==FOLDERTYPE_UNKNOWN)
			r=-1;
		else if (((folders_vdata*)n2)->f->getftype()==FOLDERTYPE_UNKNOWN)
			r=1;
		return r;
	}
	else if (((folders_vdata*)n1)->f->g_subdir()[0] && !((folders_vdata*)n2)->f->g_subdir()[0])
		return 1;
	else if (((folders_vdata*)n2)->f->g_subdir()[0] && !((folders_vdata*)n1)->f->g_subdir()[0])
		return -1;
	else
		return strcscmp(((folders_vdata*)n1)->filename,((folders_vdata*)n2)->filename);
}
static int fcmp(struct node *n1,struct node *n2)
{
	return strcscmp(((folders_data*)n1)->f->g_foldername(),((folders_data*)n2)->f->g_foldername());
}
int folders::init(void)
{
// 1. read all folders from known io's
// 2. read all vfolders from MAIL directory (*.prefs)
// 2a. put all folders in their vfolders
// 3. add for all folders (foldername) without vfolder (foldername) default vfolder

	if (allvfolders.Count()>0)
		return 0;

// TODO: check if folder/vfolder exists, then add/don't add
// ??????

	int c=0;
	char dir[300],wdir[300];
	DIR *dfd;
	struct dirent *dptr;
	struct stat sbuf;

	strcpy(dir,getenv("HOME"));
	strcat(dir,"/"HOMEMAIL"/");
	if (dfd=opendir(dir))
	{
		closedir(dfd);
	}
	else
	{
		easy_dialog ed;
		ed.Text("You have no {FONT_FIXED}'"HOMEMAIL"'{/FONT} directory in your home\n"
			    "directory. This is required by some features of "PROGRAMNAME".\n\n"
			    "Should this directory now be created (you may become\n"
			    "difficulties if you didn't) ?");
		ed.Options("_Make it so|_Not yet!");
		if (ed.Start(readerwin)==1)
		{
			mkdir(dir,S_IXUSR|S_IRUSR|S_IWUSR);
		}
		else
		{
			easy_dialog ed2;
			ed2.Text("Very well, I won't create it.\nBut, you may run into difficulties later.");
			ed2.Options("_Create it|_OK, start without it");
			if (ed2.Start(readerwin)==1)
			{
				mkdir(dir,S_IXUSR|S_IRUSR|S_IWUSR);
			}
		}
	}


// 1.
	io_class *n=(io_class*)allios->head;
	while (n->next)
	{
		if (n->can_list_folders)
		{
			folder **allf=n->getfolders();
			if (allf)
			{
				folder *fn;
				while (fn=*allf)
				{
					folders_data *n1=new folders_data(fn->g_foldername(),fn);
					allfolders.AddTail(n1);
					allf++;
				}
			}
		}
		n=(io_class*)n->next;
	}

// 2.

	if (dfd=opendir(dir))
	{
		while (dptr=readdir(dfd))
		{
			if (dptr->d_name[0]=='.')
			{
				int l=strlen(dptr->d_name);
				strcpy(wdir,dir);
				strcat(wdir,dptr->d_name);
				stat(wdir,&sbuf);
				if (S_ISREG(sbuf.st_mode) && !S_ISDIR(sbuf.st_mode))
				{
					if (l>6 && 
							 dptr->d_name[l-6]=='.' &&
							 dptr->d_name[l-5]=='p' &&
							 dptr->d_name[l-4]=='r' &&
							 dptr->d_name[l-3]=='e' &&
							 dptr->d_name[l-2]=='f' &&
							 dptr->d_name[l-1]=='s')
					{
						config cfg_file(wdir);
						char *nm=dptr->d_name+1; l--;
						char *foldername=strdup(nm);
						foldername[l-6]=0;
						rextokenizer rtk(foldername,"D(.+)D_(.+)");
						if (rtk.count()>0)
							strcpy(foldername,rtk[1]);

						folder_type=0;
						folder_data[0]=0;
						folder_version=0;
						
						folders_data *n1;
						if (n1=(folders_data*)allfolders.FindText(foldername))
						{
							folder_type=n1->f->getftype();
						}
						
						strcpy(folder_files,"file:");
						strcat(folder_files,foldername);
						folder_hidden=0;
						folder_subdir[0]=0;
						folder_id[0]=0;
						rec_own=0;
						rec_always=0;
						rec_read=0;
						rec_reply=0;
						rec_forward=0;
						rec_older=0;
						rec_older_days=7;
						rec_folder_id[0]=0;

						cfg_file.SetItems(folder_prefs);
						cfg_file.Read();
						if (folder_version>=2)
						{
							folders_vdata *n2=new folders_vdata(foldername,folder_subdir);
// 2a.
							tokenizer tk(folder_files,",");
							tokenizer tk2(folder_data,",");
							for(int i=0;i<tk.count();i++)
							{
								addfolder(&n2->subfolders,(char*)tk[i],(i+1<=tk2.count()?(char*)tk2[i]:NULL));
							}
							vfolder *f=new vfolder(foldername,&n2->subfolders,folder_type,folder_subdir);
							f->hidden=folder_hidden;
							f->rec_own=rec_own;
							f->rec_always=rec_always;
							f->rec_read=rec_read;
							f->rec_reply=rec_reply;
							f->rec_forward=rec_forward;
							f->rec_older=rec_older;
							f->rec_older_days=rec_older_days;
							strcpy(f->rec_folder_id,rec_folder_id);

							if (folder_id[0])
								f->s_id(folder_id);
							if (n2->subfolders.Count()>0)
							{
								vfolder_data *v=(vfolder_data*)n2->subfolders.head;
								f->noappendsupport=v->f->noappendsupport;
								f->nodeletesupport=v->f->nodeletesupport;
								f->readonly=v->f->readonly;
							}
							n2->f=f;
							allvfolders.AddTail(n2);
						}
						free(foldername);
					}
				}
			}
		}
		closedir(dfd);
	}

// 3.

	folders_data *n1=(folders_data*)allfolders.head;
	folders_vdata *n2;
	while (n1->next)
	{
		n2=(folders_vdata*)allvfolders.FindText(n1->text);
		if (!n2)
		{
			char *sd=n1->f->getio()->io_name();
			int ft=n1->f->getftype();
			if (!strcmp(n1->f->getio()->io_name(),"file"))
			{ // for old users
				if (!strcscmp(n1->text,"Outbox"))
				{
					sd="";
					ft=FOLDERTYPE_OUTBOX;
				}
				else if (!strcscmp(n1->text,"Trash"))
				{
					sd="";
					ft=FOLDERTYPE_TRASH;
				}
				else if (!strcscmp(n1->text,"Received"))
				{
					sd="";
					ft=FOLDERTYPE_RECEIVED;
				}
				else if (!strcscmp(n1->text,"POP3MAILBOX"))
				{
					sd="";
					ft=FOLDERTYPE_OUTBOX;
				}
			}
			else if (!strcmp(n1->f->getio()->io_name(),"INBOX"))
			{ // for old users
				sd="";
				ft=FOLDERTYPE_INBOX;
			}
			n2=new folders_vdata(n1->text,sd);
			vfolder_data *v=new vfolder_data;
			v->f=n1->f;
			n2->subfolders.AddTail(v);

			vfolder *f=new vfolder(n1->text,&n2->subfolders,ft,sd);
			n2->f=f;
			allvfolders.AddTail(n2);
		}
		n1=(folders_data*)n1->next;
	}
	allvfolders.Sort(vcmp);
	allfolders.Sort(fcmp);
	return 0;
}
char **folders::readlist(int mode,int withhidden)
{
// sort vfolder list
	int md=mode+(withhidden?RL_ALLFOLDERS:0);
	if (allfolders_str[md])
	{
		char **a=allfolders_str[md];
		while (*a)
		{
			free(*a);
			a++;
		}
		delete [] allfolders_str[md];
	}
	allfolders_str[md]=new char*[allvfolders.Count()+2];
	int j=0;
	folders_vdata *n=(folders_vdata*)allvfolders.head;
	char **a=allfolders_str[md];

	while (n->next)
	{
		switch (mode)
		{
			case RL_VISIBLES:
				if (!n->f->hidden || withhidden)
				{
					a[j]=strdup(n->filename);
					j++;
				}
				break;
			case RL_WRITEABLES:
				if ((!n->f->hidden || withhidden) && !n->f->readonly && !n->f->noappendsupport)
				{
					a[j]=strdup(n->filename);
					j++;
				}
				break;
			case RL_INBOXES:
				if ((!n->f->hidden || withhidden) && !n->f->readonly && n->f->getftype()==FOLDERTYPE_INBOX)
				{
					a[j]=strdup(n->filename);
					j++;
				}
				break;
			case RL_OUTBOXES:
				if ((!n->f->hidden || withhidden) && !n->f->readonly && n->f->getftype()==FOLDERTYPE_OUTBOX)
				{
					a[j]=strdup(n->filename);
					j++;
				}
				break;
			case RL_TRASHES:
				if ((!n->f->hidden || withhidden) && n->f->getftype()==FOLDERTYPE_TRASH)
				{
					a[j]=strdup(n->filename);
					j++;
				}
				break;
			case RL_ALL:
			default:
				a[j]=strdup(n->filename);
				j++;
				break;
		}
		n=(folders_vdata*)n->next;
	}
	if (j==0)
	{
		a[j++]=strdup("---");
	}
	a[j]=NULL;
	return allfolders_str[md];
}
tv_tree *folders::readlist_tree(int mode,int withhidden)
{
	char *txt="";
	switch (mode)
	{
		case RL_VISIBLES:
			txt="Visible folders";
			break;
		case RL_WRITEABLES:
			txt="Writable folders";
			break;
		case RL_INBOXES:
			txt="Inbox folders";
			break;
		case RL_OUTBOXES:
			txt="Outbox folders";
			break;
		case RL_TRASHES:
			txt="Trash folders";
			break;
		case RL_ALL:
		default:  
			txt="All folders";
			break;
	}
	tv_tree *t=new tv_tree(txt),*t2=NULL,*t3=NULL;
	char **a=readlist(mode,withhidden);
	int i=0;
	char *tname=NULL;
	while (a[i])
	{
		if (a[i][0]!='>')
		{
			t3=new tv_tree(a[i]);
			t3->user=i;
			t->AddTail(t3);
		}
		else
		{
			char *tmp=strdup(a[i]);
			char *c1=tmp+1,*c2=c1;
			while (*c2 && *c2!='/') c2++;
			if (*c2)
			{
				*c2=0;
				c2++;
			}
			else
			{
				free(tmp);
				i++;
				continue;
			}
			if (!tname || strcmp(c1,tname))
			{
				if (tname)
					free(tname);
				tname=strdup(c1);
				t2=new tv_tree(tname);
				t->AddTail(t2);
			}
			if (t2)
			{
				t3=new tv_tree(c2);
				t3->user=i;
				t2->AddTail(t3);
			}
			free(tmp);
		}
		i++;
	}
	if (tname)
		free(tname);
	return t;
}
void folders::free_tree(tv_tree *t)
{
	if (t)
	{
		t->RemoveAll(RAF_DELETE);
		delete t;
	}
}
vfolder *folders::getfolder(int i,int mode,int withhidden)
{
	int j=0;
	folders_vdata *n=(folders_vdata*)allvfolders.head;
	while (n->next)
	{
		switch (mode)
		{
			case RL_VISIBLES:
				if (!n->f->hidden || withhidden)
				{
					if (j==i)
						return n->f;
					j++;
				}
				break;
			case RL_WRITEABLES:
				if ((!n->f->hidden || withhidden) && !n->f->readonly && !n->f->noappendsupport)
				{
					if (j==i)
						return n->f;
					j++;
				}
				break;
			case RL_INBOXES:
				if ((!n->f->hidden || withhidden) && !n->f->readonly && n->f->getftype()==FOLDERTYPE_INBOX)
				{
					if (j==i)
						return n->f;
					j++;
				}
				break;
			case RL_OUTBOXES:
				if ((!n->f->hidden || withhidden) && !n->f->readonly && n->f->getftype()==FOLDERTYPE_OUTBOX)
				{
					if (j==i)
						return n->f;
					j++;
				}
				break;
			case RL_TRASHES:
				if ((!n->f->hidden || withhidden) && n->f->getftype()==FOLDERTYPE_TRASH)
				{
					if (j==i)
						return n->f;
					j++;
				}
				break;
			case RL_ALL:
			default:
				if (j==i)
					return n->f;
				j++;
				break;
		}
		n=(folders_vdata*)n->next;
	}
	return NULL;
}
int folders::getfolder(class vfolder *v,int mode,int withhidden)
{
	int j=0;
	folders_vdata *n=(folders_vdata*)allvfolders.head;
	while (n->next)
	{
		switch (mode)
		{
			case RL_VISIBLES:
				if (!n->f->hidden || withhidden)
				{
					if (n->f==v)
						return j;
					j++;
				}
				break;
			case RL_WRITEABLES:
				if ((!n->f->hidden || withhidden) && !n->f->readonly && !n->f->noappendsupport)
				{
					if (n->f==v)
						return j;
					j++;
				}
				break;
			case RL_INBOXES:
				if ((!n->f->hidden || withhidden) && !n->f->readonly && n->f->getftype()==FOLDERTYPE_INBOX)
				{
					if (n->f==v)
						return j;
					j++;
				}
				break;
			case RL_OUTBOXES:
				if ((!n->f->hidden || withhidden) && !n->f->readonly && n->f->getftype()==FOLDERTYPE_OUTBOX)
				{
					if (n->f==v)
						return j;
					j++;
				}
				break;
			case RL_TRASHES:
				if ((!n->f->hidden || withhidden) && n->f->getftype()==FOLDERTYPE_TRASH)
				{
					if (n->f==v)
						return j;
					j++;
				}
				break;
			case RL_ALL:
			default:
				if (n->f==v)
					return j;
				j++;
				break;
		}
		n=(folders_vdata*)n->next;
	}
	return 0;
}
class vfolder *folders::getfolder(char *na,int nosubdir)
{
	int j=0;
	folders_vdata *n=(folders_vdata*)allvfolders.head;
	while (n->next)
	{
		if (nosubdir)
		{
			if (!strcmp(n->f->g_foldername(),na))
				return n->f;
		}
		else
		{
			if (!strcmp(n->filename,na))
				return n->f;
		}
		n=(folders_vdata*)n->next;
	}
	return NULL;
}
class vfolder *folders::getfolder(char *na,char *subdir)
{
	int j=0;
	folders_vdata *n=(folders_vdata*)allvfolders.head;
	while (n->next)
	{
		if (!strcmp(n->f->g_foldername(),na) && !strcmp(n->f->g_subdir(),subdir))
			return n->f;
		n=(folders_vdata*)n->next;
	}
	return NULL;
}
class vfolder *folders::getfolder(vfolder *pre,char *na)
{
	int j=0;
	folders_vdata *n=(folders_vdata*)allvfolders.head;
	if (pre)
	{
		while (n->next)
		{
			if (n->f==pre)
			{
				n=(folders_vdata*)n->next;
				break;
			}
			n=(folders_vdata*)n->next;
		}
	}
	while (n->next)
	{
		if (!strcmp(n->f->g_foldername(),na))
			return n->f;
		n=(folders_vdata*)n->next;
	}
	return NULL;
}
class vfolder *folders::getfolder_byid(char *id)
{
	int j=0;
	folders_vdata *n=(folders_vdata*)allvfolders.head;
	while (n->next)
	{
		if (!strcmp(n->f->g_id(),id))
			return n->f;
		n=(folders_vdata*)n->next;
	}
	return NULL;
}
class vfolder *folders::getnextfolderoftype(vfolder *pre,int type)
{
	int j=0;
	folders_vdata *n=(folders_vdata*)allvfolders.head;
	if (pre)
	{
		while (n->next)
		{
			if (n->f==pre)
			{
				n=(folders_vdata*)n->next;
				break;
			}
			n=(folders_vdata*)n->next;
		}
	}
	while (n->next)
	{
		if (n->f->getftype()==type)
			return n->f;
		n=(folders_vdata*)n->next;
	}
	return NULL;
}
class folder *folders::getfolder(io_class *io,char *na)
{
	folders_data *n=(folders_data*)allfolders.head;
	while (n->next)
	{
		if (n->f->getio()==io && !strcmp(n->f->g_foldername(),na))
			return n->f;
		n=(folders_data*)n->next;
	}
	return NULL;
}

int folders::add(vfolder *v)
{
	folders_vdata *n2=new folders_vdata(v->g_foldername(),v->g_subdir());
	n2->f=v;
	allvfolders.AddTail(n2);

	allvfolders.Sort(vcmp);
	signalfolderregisteredfunctions();
	return 0;
}
int folders::remove(vfolder *v)
{
	int r=1;

	folders_vdata *n=(folders_vdata*)allvfolders.head;
	while (n->next)
	{
		if (n->f==v)
		{
			allvfolders.Remove(n);
			delete v;
			delete n;
			r=0;
			break;
		}
		n=(folders_vdata*)n->next;
	}
	if (!r)
	{
		allvfolders.Sort(vcmp);
		signalfolderregisteredfunctions();
	}
	return r;
}
class vfolder *folders::create(int t,char *name,char *dir)
{
	folders_vdata *n2=new folders_vdata(name,dir);
	vfolder *v=new vfolder(name,&n2->subfolders,t,dir);
	n2->f=v;
	allvfolders.AddTail(n2);

	allvfolders.Sort(vcmp);
	signalfolderregisteredfunctions();
	return v;
}
int folders::rename(vfolder *v,char *name)
{
	int r=1;

	v->chname(name);
	folders_vdata *n=(folders_vdata*)allvfolders.head;
	while (n->next)
	{
		if (n->f==v)
		{
			n->chname(v->g_foldername(),v->g_subdir());
			break;
		}
		n=(folders_vdata*)n->next;
	}
	allvfolders.Sort(vcmp);
	signalfolderregisteredfunctions();
	r=0;
	return r;
}
int folders::add(vfolder *v,folder *f)
{
	vfolder_data *n=(vfolder_data*)v->allfolders->head;
	while (n->next)
	{
		if (n->f==f)
		{
			return 1;
		}
		n=(vfolder_data*)n->next;
	}

	vfolder_data *n1=new vfolder_data;
	n1->f=f;
	v->allfolders->AddTail(n1);
	return 0;
}
int folders::remove(vfolder *v,folder *f)
{
	vfolder_data *n=(vfolder_data*)v->allfolders->head,*n2;
	while (n->next)
	{
		n2=(vfolder_data*)n->next;
		if (n->f==f)
		{
			v->allfolders->Remove(n);
			delete n;
		}
		n=n2;
	}
	return 0;
}
int folders::add(folder *f)
{
	folders_data *n1=new folders_data(f->g_foldername(),f);
	allfolders.AddTail(n1);

	allfolders.Sort(fcmp);
	signalfolderregisteredfunctions();
}
int folders::remove(folder *v)
{
	int r=1;

	folders_data *n=(folders_data*)allfolders.head;
	while (n->next)
	{
		if (n->f==v)
		{
			folders_vdata *n2=(folders_vdata*)allvfolders.head;
			while (n2->next)
			{
				remove(n2->f,v);
				n2=(folders_vdata*)n2->next;
			}
			
			allfolders.Remove(n);
			v->getio()->removefolder(v->g_foldername());
			delete v;
			delete n;
			r=0;
			break;
		}
		n=(folders_data*)n->next;
	}
	if (!r)
	{
		allfolders.Sort(fcmp);
		signalfolderregisteredfunctions();
	}
	return r;
}
class folder *folders::create(io_class *i,char *name)
{
	folder *f=NULL;
	int l=strlen(name)*2;
	char *buf=new char[l+1];
	ckfoldername(i,name,buf,l);

	if (!i->createfolder(buf))
	{
		f=new folder(i,buf);

		folders_data *n1=new folders_data(f->g_foldername(),f);
		allfolders.AddTail(n1);

		allfolders.Sort(fcmp);
		signalfolderregisteredfunctions();
	}
	delete [] buf;
	return f;
}
int folders::rename(folder *v,char *name)
{
	int r=1;
	io_class *io=v->getio();
	int l=strlen(name)*2;
	char *buf=new char[l+1];

	ckfoldername(io,name,buf,l);

	if (!io->renamefolder(v->g_foldername(),buf))
	{
		v->chname(buf);

		folders_data *n=(folders_data*)allfolders.head;
		while (n->next)
		{
			if (n->f==v)
			{
				n->chname(v->g_foldername());
				break;
			}
			n=(folders_data*)n->next;
		}

		allfolders.Sort(fcmp);
		signalfolderregisteredfunctions();
		r=0;
	}
	delete [] buf;
	return r;
}

void folders::change_vfolder_data_hidden(vfolder *v,int h)
{
	v->hidden=h;
	signalfolderregisteredfunctions();
}
void folders::change_vfolder_data_subdir(vfolder *v,char *sdir)
{
	if (!getfolder(v->g_foldername(),sdir))
	{
		folders_vdata *n=(folders_vdata*)allvfolders.head;
		while (n->next)
		{
			if (n->f==v)
			{
				n->chname(v->g_foldername(),sdir);
			}
			n=(folders_vdata*)n->next;
		}
		free(v->subdir);
		v->subdir=strdup(sdir);
		allvfolders.Sort(vcmp);
		signalfolderregisteredfunctions();
	}
}
void folders::change_vfolder_data_ftype(vfolder *v,int ft)
{
	v->ftype=ft;
	allvfolders.Sort(vcmp);
	signalfolderregisteredfunctions();
}

int folders::ckfoldername(io_class *i,char *name)
{
	return i->ckname(name);
}
void folders::ckfoldername(io_class *i,char *name,char *toname,int size)
{
	i->ckname(name,toname,size);
}

void folders::add(io_class *n)
{
	int r=0;
	allios->AddTail(n);
	if (n->can_list_folders)
	{
		folder **allf=n->getfolders();
		if (allf)
		{
			folder *fn;
			while (fn=*allf)
			{
				folders_data *n1=new folders_data(fn->g_foldername(),fn);
				allfolders.AddTail(n1);

				char *sd=n->io_name();
				int ft=n1->f->getftype();

				folders_vdata *n2=new folders_vdata(n1->text,sd);
				vfolder_data *v=new vfolder_data;
				v->f=n1->f;
				n2->subfolders.AddTail(v);

				vfolder *f=new vfolder(n1->text,&n2->subfolders,ft,sd);
				n2->f=f;
				allvfolders.AddTail(n2);

				r=1;
				allf++;
			}
		}
	}
	if (r)
	{
		allvfolders.Sort(vcmp);
		allfolders.Sort(fcmp);
	}
}
int folders::remove(io_class *io)
{
	int r=1;

	folders_data *n=(folders_data*)allfolders.head,*nn;
	while (n->next)
	{
		nn=(folders_data*)n->next;
		if (n->f->getio()==io)
		{
			folders_vdata *n2=(folders_vdata*)allvfolders.head;
			while (n2->next)
			{
				remove(n2->f,n->f);
				n2=(folders_vdata*)n2->next;
			}
			
			allfolders.Remove(n);
//			n->f->getio()->removefolder(v->g_foldername());
			delete n->f;
			delete n;
			r=0;
		}
		n=nn;
	}
	if (!r)
	{
		allfolders.Sort(fcmp);
		signalfolderregisteredfunctions();
	}
	return r;
}

void folders::registerfolderfunction(void (*rf)(void))
{
	folders_functions *n3=new folders_functions(rf);
	registeredfolderfunctions.AddTail(n3);
}
void folders::registerfolderfunction(class rfunc_class *rf)
{
	folders_functions *n3=new folders_functions(rf);
	registeredfolderfunctions.AddTail(n3);
}
void folders::unregisterfolderfunction(void (*rf)(void))
{
	folders_functions *n3=(folders_functions*)registeredfolderfunctions.head;
	while (n3->next)
	{
		if (*n3==rf)
		{
			registeredfolderfunctions.Remove(n3);
			delete n3;
			break;
		}
		n3=(folders_functions*)n3->next;
	}
}
void folders::unregisterfolderfunction(class rfunc_class *rf)
{
	folders_functions *n3=(folders_functions*)registeredfolderfunctions.head;
	while (n3->next)
	{
		if (*n3==rf)
		{
			registeredfolderfunctions.Remove(n3);
			delete n3;
			break;
		}
		n3=(folders_functions*)n3->next;
	}
}

void folders::spool_folderfunction(void)
{
	spoolc_folderfunction++;
}
void folders::unspool_folderfunction(void)
{
	spoolc_folderfunction--;
	if (spoolc_folderfunction==0 && spool_folderfunction_need)
		signalfolderregisteredfunctions();
	spool_folderfunction_need=0;
}
void folders::signalfolderregisteredfunctions(void)
{
	if (spoolc_folderfunction>0)
	{
		spool_folderfunction_need=1;
		return;
	}
	folders_functions *n3=(folders_functions*)registeredfolderfunctions.head;
	while (n3->next)
	{
		n3->call(folders_functions::FOLDERS);
		n3=(folders_functions*)n3->next;
	}
}

void folders::registercurrentfunction(void (*rf)(void))
{
	folders_functions *n3=new folders_functions(rf);
	registeredcurrentfunctions.AddTail(n3);
}
void folders::registercurrentfunction(class rfunc_class *rf)
{
	folders_functions *n3=new folders_functions(rf);
	registeredcurrentfunctions.AddTail(n3);
}
void folders::unregistercurrentfunction(void (*rf)(void))
{
	folders_functions *n3=(folders_functions*)registeredcurrentfunctions.head;
	while (n3->next)
	{
		if (*n3==rf)
		{
			registeredcurrentfunctions.Remove(n3);
			delete n3;
			break;
		}
		n3=(folders_functions*)n3->next;
	}
}
void folders::unregistercurrentfunction(class rfunc_class *rf)
{
	folders_functions *n3=(folders_functions*)registeredcurrentfunctions.head;
	while (n3->next)
	{
		if (*n3==rf)
		{
			registeredcurrentfunctions.Remove(n3);
			delete n3;
			break;
		}
		n3=(folders_functions*)n3->next;
	}
}
void folders::spool_currentfunction(void)
{
	spoolc_currentfunction++;
}
void folders::unspool_currentfunction(void)
{
	spoolc_currentfunction--;
	if (spoolc_currentfunction==0 && spool_currentfunction_need)
		signalcurrentregisteredfunctions();
	spool_currentfunction_need=0;
}
void folders::signalcurrentregisteredfunctions(void)
{
	if (spoolc_currentfunction>0)
	{
		spool_currentfunction_need=1;
		return;
	}
	folders_functions *n3=(folders_functions*)registeredcurrentfunctions.head;
	while (n3->next)
	{
		n3->call(folders_functions::CURRENT);
		n3=(folders_functions*)n3->next;
	}
}

extern Xclasses subwindows;
extern multilistview mailslv;
extern output foldername;
extern menu mn;
extern gfx_folder gx_folder;
extern gfx_readonly gx_readonly;
extern gfx_trash gx_trash;
extern gfx_received gx_received;
extern gfx_inbox gx_inbox;
extern gfx_outbox gx_outbox;

void folders::currentfolder(vfolder *v)
{
	if (programconfig.closeOnChangeFolder)
		DeleteObjectTree(subwindows.FullName(),DOT_NOTBASE);
	readerwin.Update();

	if (currentvf)
	{
		pstatus("Writing mailfolder...");
		currentvf->save();
	}

	if (currentvf!=v)
	{
		mailslv.RemoveAll();
		if (currentvf)
			currentvf->unload();
		currentvf=v;

		mn.Selected(2,1,programconfig.sortmode);
		programconfig.currentsortmode=mn.Selected(2,1);

		pstatus("Reading mailfolder...");
		if (currentvf)
		{
			foldername.Output(currentvf->g_foldername());
		}
		else
		{
			foldername.Output("");
		}

		readerwin.UpdateEvent();
		if (currentvf)
		{
			int c=currentvf->load();
			pstatus("Read %d mails.",currentvf->count());
		}
		else
			pstatus("");

		signalcurrentregisteredfunctions();
	}
	else
	{
		pstatus("Updating mailfolder...");

		readerwin.UpdateEvent();
		if (currentvf)
		{
			int n=currentvf->getnew();
 			pstatus("Got %d new mail.",n);
		}
		else
			pstatus("");
	}

	// more do to ?
}
vfolder *folders::currentfolder(void)
{
	return currentvf;
}
void folders::updateoutput(folder *f,mail *m,int oldmail)
{
// search f in allvfolders -> found: vfolder->appendmail()

	folders_vdata *n1=(folders_vdata*)allvfolders.head;
	while (n1->next)
	{
		vfolder_data *n2=(vfolder_data*)n1->subfolders.head;
		while (n2->next)
		{
			if (n2->f==f && n1->f==currentvf)
			{
				n1->f->appendmail(m,&mailslv,oldmail);
				appwindow::UpdateEvent();
			}
			else if (n2->f==f)
			{
				n1->f->appendmail(m,NULL,oldmail);
			}
			n2=(vfolder_data*)n2->next;
		}
		n1=(folders_vdata*)n1->next;
	}
}
void folders::rmoutput(class folder *v,class mail *m)
{
	folders_vdata *n1=(folders_vdata*)allvfolders.head;
	while (n1->next)
	{
		vfolder_data *n2=(vfolder_data*)n1->subfolders.head;
		while (n2->next)
		{
			if (n2->f==v && n1->f==currentvf)
			{
				n1->f->rmmail(m,&mailslv);
			}
			else if (n2->f==v)
			{
				n1->f->rmmail(m,NULL);
			}
			n2=(vfolder_data*)n2->next;
		}
		n1=(folders_vdata*)n1->next;
	}
}
folder *folders::getmatchingfolder(io_class *io,int ftype,io_class **tio,char **fname)
{
// search in all vfolders for ftype, check folders in it for same IO
// or same IO name or same IO type or IO type "file"

	folders_vdata *v1=(folders_vdata*)allvfolders.head;
	while (v1->next)
	{
		if (v1->f->getftype()==ftype)
		{
			vfolder_data *v2=(vfolder_data*)v1->subfolders.head;
			while (v2->next)
			{
				if (v2->f->getio()==io)
				{
					*tio=io;
					*fname=v2->f->g_foldername();
					return v2->f;
				}
				v2=(vfolder_data*)v2->next;
			}
		}
		v1=(folders_vdata*)v1->next;
	}

	v1=(folders_vdata*)allvfolders.head;
	while (v1->next)
	{
		if (v1->f->getftype()==ftype)
		{
			vfolder_data *v2=(vfolder_data*)v1->subfolders.head;
			while (v2->next)
			{
				if (!strcmp(v2->f->getio()->io_name(),io->io_name()))
				{
					*tio=io;
					*fname=v2->f->g_foldername();
					return v2->f;
				}
				v2=(vfolder_data*)v2->next;
			}
		}
		v1=(folders_vdata*)v1->next;
	}
	v1=(folders_vdata*)allvfolders.head;
	while (v1->next)
	{
		if (v1->f->getftype()==ftype)
		{
			vfolder_data *v2=(vfolder_data*)v1->subfolders.head;
			while (v2->next)
			{
				if (!strcmp(v2->f->getio()->io_type(),io->io_type()))
				{
					*tio=io;
					*fname=v2->f->g_foldername();
					return v2->f;
				}
				v2=(vfolder_data*)v2->next;
			}
		}
		v1=(folders_vdata*)v1->next;
	}
	v1=(folders_vdata*)allvfolders.head;
	while (v1->next)
	{
		if (v1->f->getftype()==ftype)
		{
			vfolder_data *v2=(vfolder_data*)v1->subfolders.head;
			while (v2->next)
			{
				if (!strcmp(v2->f->getio()->io_type(),"file"))
				{
					*tio=io;
					*fname=v2->f->g_foldername();
					return v2->f;
				}
				v2=(vfolder_data*)v2->next;
			}
		}
		v1=(folders_vdata*)v1->next;
	}


// 	folders_data *n1=(folders_data*)allfolders.head;
// 	while (n1->next)
// 	{
// 		if (n1->f->getio()==io && n1->f->getftype()==ftype)
// 		{
// 			*tio=io;
// 			*fname=n1->filename;
// 			return n1->f;
// 		}
// 		n1=(folders_data*)n1->next;
// 	}
// 	n1=(folders_data*)allfolders.head;
// 	while (n1->next)
// 	{
// 		if (!strcmp(n1->f->getio()->io_name(),io->io_name()) && n1->f->getftype()==ftype)
// 		{
// 			*tio=io;
// 			*fname=n1->filename;
// 			return n1->f;
// 		}
// 		n1=(folders_data*)n1->next;
// 	}
// 	n1=(folders_data*)allfolders.head;
// 	while (n1->next)
// 	{
// 		if (!strcmp(n1->f->getio()->io_type(),io->io_type()) && n1->f->getftype()==ftype)
// 		{
// 			*tio=io;
// 			*fname=n1->filename;
// 			return n1->f;
// 		}
// 		n1=(folders_data*)n1->next;
// 	}
// 	n1=(folders_data*)allfolders.head;
// 	while (n1->next)
// 	{
// 		if (!strcmp(n1->f->getio()->io_type(),"file") && n1->f->getftype()==ftype)
// 		{
// 			*tio=io;
// 			*fname=n1->filename;
// 			return n1->f;
// 		}
// 		n1=(folders_data*)n1->next;
// 	}
	return NULL;
}
int folders::cknew(int all)
{
	int i=0,ret=0;
	folders_vdata *n1=(folders_vdata*)allvfolders.Find(cknewcounter);

	if (!n1)
	{
		n1=(folders_vdata*)allvfolders.head;
		cknewcounter=1;
	}
	while (n1->next && (i<10 || all))
	{
		if (!n1->f->hidden)
		{
			if (!n1->f->cknew())
			{
				ret|=1;
				if (n1->f==currentvf)
				{
					int n=currentvf->getnew();
		 			pstatus("Got %d new mail.",n);
					ret|=2;
				}
				if (n1->f->getftype()==FOLDERTYPE_INBOX)
				{
					ret|=4;
				}
			}
			i++;
		}
		cknewcounter++;
		n1=(folders_vdata*)n1->next;
	}
	return ret;
}

vfolder *folders::chooseFolder(class appwindow *wi,int flag,char *title)
{
	vfolder *v=NULL;
	appwindow *aw=new appwindow;
	int x,y,w,h;

	wi->Position(x,y);
	wi->Size(w,h);

	aw->ParentClass(wi);
	aw->Name("folderchoosewindow");

	aw->InitSize(x,y,150);
	if (!aw->Init())
	{
		delete aw;
		pstatus("Error in opening window!");
		return NULL;
	}
	aw->Adjust(x,y,w,h);
	aw->Title(title);
	aw->IconTitle(title);
	aw->TransientFor(wi);
	aw->SetIcon(f_xicon1.pixmap(),f_xicon1.shadowmap());

	spacegroup *bg=new spacegroup;
	bg->Name("fcgroup");
	bg->Border(10);
	bg->YDistance(10);
	bg->Vert();

	text *txt=new text;
	 txt->Text(title);
	bg->Add(txt);
	treeview *tv=new treeview;
	 tv_tree *vflst=readlist_tree(flag);
	 vflst->Open();
	 tv->SetRoot(vflst);
	bg->Add(tv);

	fbutton *b=new fbutton;
	 b->Text(" _Ok ");
	 b->Callback((gadget*)aw,(CALLBACK)&appwindow::Close);
	bg->Add(b);

	aw->SetGroup(bg);
	aw->Callback();
	aw->Create();
	aw->BlockExcept();

	int r=aw->NextEvent();

	tv_tree *sel=tv->Selected();
	if (sel && sel->user)
	{
		v=getfolder((int)sel->user,flag);
	}

	aw->UnblockExcept();
	aw->Close();
	DeleteObjectTree(aw->FullName());
	free_tree(vflst);
	return v;
}

// -----------------------------------------------------------------------------

void rfunc_class::updatefolders(void)
{
}
void rfunc_class::updatecurrent(void)
{
}

// -----------------------------------------------------------------------------

// This will create the initial io's.
// Dynamically added io's shouldn't be here but should use
// folder::add
// This will require some more tests and a new
// card in the folder manager to handle new io's
// eg: subdirectories in ~/Mail, maildir types, other
//     gzfiles, IMAP servers and more
void io_list_create(void)
{
	allios=new iolist;

	char dir[300];

	strcpy(dir,getenv("HOME"));
	strcat(dir,"/"HOMEMAIL"/");
	allios->AddTail(new io_mbox(dir));

	strcpy(dir,getenv("HOME"));
	strcat(dir,"/"FOLDERARCHIV"/");
	allios->AddTail(new io_gzip(dir));

	dir[0]=0;
	char *mf=getenv("MAIL");
	if (mf)
	{
		strcpy(dir,mf);
	}
	else
	{
		char *uname=getusername();
		if (uname)
		{
			strcpy(dir,MAILFOLDER);
			strcat(dir,"/");
			strcat(dir,uname);
		}
	}
	allios->AddTail(new io_mbox(dir,"INBOX"));
// if we allow multiple mbox or gzboxes we have to use better names
// for them than "file" or "gzfile" (2nd parameter of io_XXX() constructor
}
