/* 
 * This file is part of XCmail and is subject to the 
 * XCmail license agreement
 */

/* $Id: folder.h,v 2.35 2002/09/23 08:38:45 schmitzj Exp $ */


#ifndef FOLDER_H
#define FOLDER_H
#include "config.h"
#include <Xclasses/Xclasses.h>
#include <string.h>
//#include "mailer.h"

class io_id_data
{
public:
	virtual ~io_id_data() {};
};
class io_data
{
public:
	virtual ~io_data() {};
};

#define IOCD_SIZE			0x00000001
#define IOCD_DATE			0x00000002
#define IOCD_FREESTORAGE	0x00000003
#define IOCD_START_OF_OWN	0x80000001

class io_class : public node
{
	char *lasterror;
public:
	unsigned int	can_create_folders:1,
					can_delete_folders:1,
					can_rename_folders:1,
					can_list_folders:1;

	int illegalcount;
	char *illegalchars;		// char[]
	char **replaceby;		// string[]

	io_class(void) : node()
	{
		can_create_folders=can_delete_folders=can_rename_folders=can_list_folders=0;
		lasterror=strdup("");
	};
	virtual ~io_class() { if (lasterror) free(lasterror); };

	virtual char *io_name(void)=0;	// eg. mbox_Mail, mbox_spool (short for prefs-file)
	virtual char *io_type(void)=0;	// eg. file, imap
	virtual char *io_username(void)=0;	// eg. ~/Mail, hostname+user

	virtual io_data *open(char *foldername,char mode,char *data=NULL,class folder *f=NULL)=0;
		// modes: 'r'ead, 'w'rite(over) [data&folder], 'a'ppend, 'n'ewread [data&folder], 'i'nfo - doesn't lock nor open for reading mail
	virtual int close(io_data *)=0;
	virtual int fgets(io_data *,char *,int)=0;

	virtual class folder **getfolders(void)=0;

	virtual class mail *nextmail(io_data *)=0;
	virtual int writemail(io_data *,class mail *)=0;
	virtual int firstmail(io_data *)=0;
	virtual int lastmail(io_data *)=0;
	virtual int lock(char *foldername)=0;
	virtual int unlock(char *foldername)=0;
// 0: ok, 1: error

	virtual void *getFolderInfo(io_data *,int datatype)=0;
// NULL: datatype not implemented or error

	virtual char *getdata(io_data *)=0;
	virtual int cknew(io_data *,char *data)=0;

	virtual int createfolder(char *foldername)=0;
	virtual int removefolder(char *foldername)=0;
	virtual int renamefolder(char *foldername,char *newfoldername)=0;
// 0: ok, 1: error

	int ckname(char *);	// 0: OK
	void ckname(char *,char *,int);	// 0: OK

	char *getlasterror(void) { return lasterror; };
	void setlasterror(char *);
};

class iolist : public dlist
{
public:
	iolist(void) : dlist() {};
	~iolist() {};
	io_class *FindIO(char *ioname)
	{
		io_class *n=(io_class*)head;
		while (n->next)
		{
			if (!strcmp(ioname,n->io_name()))
				return n;
			n=(io_class*)n->next;
		}
		return NULL;
	};
};

#define FOLDERTYPE_UNKNOWN		0x00000000
#define FOLDERTYPE_INBOX		0x00000001
#define FOLDERTYPE_TRASH		0x00000002
#define FOLDERTYPE_RECEIVED		0x00000003
#define FOLDERTYPE_OUTBOX		0x00000004

class folder_data : public node
{
public:
	class mail *m;
};
class vfolder_data : public node
{
public:
	class folder *f;
};
class folders_data : public node
{
public:
	folders_data(char *fn,class folder *v) : node() { filename=strdup(fn); f=v; text=filename; };
	~folders_data() { free(filename); };
	void chname(char *fn) { free(filename); filename=strdup(fn); text=filename; };
	char *filename;
	class folder *f;
};
class folders_vdata : public node
{
public:
	char *filename;
	class vfolder *f;
	dlist subfolders;

	folders_vdata(char *f,char *d=NULL) : subfolders() 
	{
		if (!d)
			d="";
		filename=(char*)malloc(strlen(f)+strlen(d)+3);
		if (d[0])
			sprintf(filename,">%s/%s",d,f);
		else
			strcpy(filename,f);
		text=strdup(f);
	};
	~folders_vdata()
	{
		struct vfolder_data *n;
		while (n=(vfolder_data*)subfolders.RemTail())
		{
			delete n;
		}
		free(text);
		free(filename);
	};
	void chname(char *f,char *d=NULL)
	{
		free(text);
		free(filename);
		if (!d)
			d="";
		filename=(char*)malloc(strlen(f)+strlen(d)+3);
		if (d[0])
			sprintf(filename,">%s/%s",d,f);
		else
			strcpy(filename,f);
		text=strdup(f);
	};
};

class rfunc_class
{
public:
	virtual void updatefolders(void);
	virtual void updatecurrent(void);
};
class folders_functions : public node
{
	void (*func)(void);
	class rfunc_class *rfunc;
public:
	enum CALLTYPE {FOLDERS,CURRENT};
	folders_functions(void (*fn)(void));
	folders_functions(class rfunc_class *fn);
	folders_functions(folders_functions *fo);
	~folders_functions();
	void call(enum CALLTYPE m=FOLDERS);
	Bool operator==(void (*fn)(void));
	Bool operator==(class rfunc_class *fn);
};

class folder
{
friend class folder_manager;
	dlist allmails;
	dlist dropedmails;

	io_class *io;
	char *foldername;
	int fsize;
	int ftype;
	 // problem: vfolders have ftype, folders may have different types,
	 // solution could be: use bit field in folders for all types 
	 // but: which is returned by getftype() - only 2 times used!
	 // however: translation vfolder::ftype->folder:ftype is required and make things more confusing!
	 // result: should be done later
	char *data;
	int loadc;
	unsigned int loaded:1,
				 changed:1;

	int lnm_count;
	io_data *lnm_io;
	int filter(class mail *);

	int appendmail_i(mail *m,int nofilter=0);
public:
	folder(io_class *,char *name,int ftype=FOLDERTYPE_UNKNOWN);
	virtual ~folder();

	unsigned int readonly:1,
				 nosavesupport:1,
	             noappendsupport:1,
				 nodeletesupport:1;

	void chname(char *name);
	void change(void) { changed=1; };
	char *g_foldername(void) {  return foldername; };
// 0: ok, 1: error
	int load(int nofilter=0);
	int save(void);
	int unload(void);

	int cknew(void);		// 0: new, 1: nonew
	int getnew(void);		// new mails count
	int appendmail(class mail *); 	// calls folders->updateoutput(), 0: ok, 1: error
	int remmail(class mail *);		// only on loaded folders
	class mail *loadnextmail(void);
	void loadnextmail_abort(void);

	void mix(folder *);
	void sub(folder *);

	void clear(void);		// clears folder (removes all mail)

	io_class *getio(void) { return io; };
	int       getftype(void) { return ftype; };

	void setdata(char *d);
	char *getdata(void) { return data?data:(char*)""; };

	int count(void);
	int size(void);
	class mail *getmail(int);
};


class vfolder
{
friend class folder_manager;
friend class folders;
	char *foldername;
	char *subdir;
	int ftype;
	dlist allmails;
	int loadc;

	int sort_marker_loading;
	int	sort_marker_counter;

	char unique_id[UNIQUE_ID_LEN];
public:
	dlist *allfolders;
	unsigned int hidden:1,
				 nosavesupport:1,
	             noappendsupport:1,
				 nodeletesupport:1,
			     readonly:1;
	unsigned int rec_own:1,
				 rec_always:1,
				 rec_read:1,
				 rec_reply:1,
				 rec_forward:1,
				 rec_older:1;
	int rec_older_days;
	char rec_folder_id[UNIQUE_ID_LEN];

	vfolder(char *,dlist *folderlst,int ftype=0,char *subdir="");
	~vfolder();
	void chname(char *name);
	void s_id(char *id) { strncpy(unique_id,id,sizeof(unique_id)); };

	int load(void);
	int save(void);
	int unload(void);

	int cknew(void); // 0: new mail
	int getnew(void); // new mails count
	int appendmail_default(class mail *); // folder->appendmail(), 0: ok, 1: error
	int appendmail(class mail *,multilistview *mlv=NULL,int oldmail=0); // recalc allmails list
	int rmmail(class mail *m,multilistview *mlv);

	int del(void);
	int add(class folder *);
	int remove(class folder *);
	int writefolder(class folder *);	// set default write folder, already added!!

	void clear(void);		// clears all folders (removes all mail) vfolder

	char *g_foldername(void) { return foldername; };
	char *g_subdir(void) { return subdir; };
	int   g_size(void);
	char *g_id(void) { return unique_id; };

	int count(void);
	int size(void);
	void sort(void);
	
	class mail *getmail(int);
	int getpos(class mail *);

	int getftype(void) { return ftype; };

	int search(int start,char *type,char *pattern,short ncase,short substring,short reg);

	void id(char *id) { s_id(id); };
	char *id(void) { return g_id(); };
};

#define RL_ALL			0
#define RL_VISIBLES		1
#define RL_WRITEABLES	2
#define RL_INBOXES		3
#define RL_OUTBOXES		4
#define RL_TRASHES		5

// max RL_*
#define RL_ALLFOLDERS	6

#define ALLFOLDERSSTR	(RL_ALLFOLDERS*2)
class folders
{
friend class folder_manager;
	dlist allvfolders;
	dlist allfolders;
	dlist registeredfolderfunctions;
	dlist registeredcurrentfunctions;

	int spool_folderfunction_need:1,
	    spool_currentfunction_need:1;

	int spoolc_folderfunction;
	int spoolc_currentfunction;

	class vfolder *currentvf;
	char **allfolders_str[ALLFOLDERSSTR];
	int cknewcounter;

	void addfolder(dlist *subfolders,char *name,char *data);
	void signalfolderregisteredfunctions(void);
	void signalcurrentregisteredfunctions(void);

public:
	folders(void);
	folders(folders *);
	~folders();
	int init(void);
	static void saveprefs(class vfolder *v);
	void saveprefs(void);

	char **readlist(int mode=RL_ALL,int withhidden=0);
	class tv_tree *readlist_tree(int mode,int withhidden=0);
	void free_tree(class tv_tree *t);
	class vfolder *getfolder(int,int mode=RL_ALL,int withhidden=0);
	int getfolder(class vfolder *,int mode=RL_ALL,int withhidden=0);
	class vfolder *getfolder(char *,int nosubdir=0);
	class vfolder *getfolder(char *na,char *subdir);
	class vfolder *getfolder(class vfolder *,char *na);
	class vfolder *getfolder_byid(char *id);
	class vfolder *getnextfolderoftype(class vfolder *pre,int type);

	class folder *getfolder(io_class *,char *);

	int add(class vfolder *);	// 0: Ok
	int remove(class vfolder *);
	int rename(class vfolder *,char *name);
	class vfolder *create(int type,char *name,char *dir=NULL);

	int add(vfolder *,folder *);
	int remove(vfolder *,folder *);

	int add(folder *);
	int remove(folder *);
	class folder *create(io_class *,char *name);
	int rename(folder *,char *name);

	void change_vfolder_data_hidden(vfolder *v,int);
	void change_vfolder_data_subdir(vfolder *v,char *);
	void change_vfolder_data_ftype(vfolder *v,int);

	void add(io_class *);
	int remove(io_class *);

	void spool_folderfunction(void);
	void unspool_folderfunction(void);
	void registerfolderfunction(void (*)(void));
	void registerfolderfunction(class rfunc_class *);
	void unregisterfolderfunction(void (*)(void));
	void unregisterfolderfunction(class rfunc_class *);

	void spool_currentfunction(void);
	void unspool_currentfunction(void);
	void registercurrentfunction(void (*)(void));
	void registercurrentfunction(class rfunc_class *);
	void unregistercurrentfunction(void (*)(void));
	void unregistercurrentfunction(class rfunc_class *);

	void currentfolder(class vfolder *);
	class vfolder *currentfolder(void);

	void updateoutput(class folder *v,class mail *m,int oldmail=0); // search, if folder is in currentvfolder -> LV output update
	void rmoutput(class folder *v,class mail *m); // search, if folder is in currentvfolder -> LV output update

	class folder *getmatchingfolder(io_class *,int ftype,io_class **,char **);

	int cknew(int all=0); // 0: no new, bit1: new, 2: new in current, 4: new in inbox

	int ckfoldername(io_class *,char *name);
	void ckfoldername(io_class *,char *name,char *toname,int size);

	vfolder *chooseFolder(class appwindow *,int flag,char *title);
};

void io_list_create(void);

extern class iolist *allios;
extern class folders *folderlist;

#endif
