/* 
 * This file is part of XCmail and is subject to the 
 * XCmail license agreement
 */

/* $Id: folder_io_gzip.cxx,v 2.18 2002/06/27 09:26:22 schmitzj Exp $ */


#include "configure.h"

#include <Xclasses/Xclasses.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <dirent.h>
#include <unistd.h>
#include <stdarg.h>
#include <signal.h>
#include <unistd.h>
#include <errno.h>
#include <ctype.h>
#include <fcntl.h>

#include "mailer.h"
#include "folder.h"
#include "xc_snprintf.h"
#include "folder_io_gzip.h"
#include "stringTokenizer.h"

class io_id_gdata : public io_id_data
{
public:
	int		seek_mail;
	int		seek_body;
	int		size;

	io_id_gdata(int m,int b,int s)
	{ seek_mail=m; seek_body=b; size=s; };
};
class io_gdata : public io_data
{
public:
	char *filename;
	char *filename_new;
	char *lockname;
	FILE *file;
	int  wtpipe[2];
	time_t fsize,fdate;

	char *rdfilename;

	char *txt;
	int size;
	int used;
	int error;
	char mode;
	int gpid;

	class folder *fol;

	io_gdata(char _mode,char *dir,char *fn)
	{
		mode=_mode;
		int l=strlen(dir);
		if (fn) l+=strlen(fn)+1;

		filename=(char*)malloc(l+1);
		strcpy(filename,dir);
		if (fn)
		{
			strcat(filename,"/");
			strcat(filename,fn);
		}
		
		lockname=(char*)malloc(strlen(filename)+6);
		strcpy(lockname,filename);
		strcat(lockname,".lock");

		filename_new=(char*)malloc(strlen(filename)+5);
		strcpy(filename_new,filename);
		strcat(filename_new,".new");

		file=NULL;
		txt=NULL;
		size=0;
		used=0;
		fsize=0;
		fdate=0;
		error=0;
		fol=NULL;
		wtpipe[0]=wtpipe[1]=-1;

		char buf[400];
		xc_snprintf(buf,sizeof(buf),"gzip -cd '%s' 2>/dev/null",filename);
		rdfilename=strdup(buf);

		gpid=0;
	}
	~io_gdata()
	{
		free(rdfilename);
		if (fol) delete fol;
		free(lockname);
		free(filename);
	}
};
io_gzip::io_gzip(char *_dir, char *_dirname) : io_class()
{
	dir=strdup(_dir);
	dirname=(_dirname?strdup(_dirname):NULL);
	can_create_folders=can_delete_folders=can_rename_folders=can_list_folders=1;

	illegalcount=1;
	illegalchars=new char[1];
	replaceby=new char*[1];
	illegalchars[0]='/';
	replaceby[0]="_";
}
io_gzip::~io_gzip()
{
	if (dirname) free(dirname);
	free(dir);
}

char *io_gzip::io_name(void)
{ //printf("Dirname: %s\n",dirname);
	if (dirname)
		return dirname;
	else
		return "gzfile";
}
char *io_gzip::io_type(void)
{
	return "gzfile";
}
char *io_gzip::io_username(void)
{
	return dir;
}
void io_gzip::data2int(char *data,time_t &fsize,time_t &fdate)
{
	tokenizer tk(data,"/");
	if (tk.count()>0) fsize=atoi(tk[0]);
	if (tk.count()>1) fdate=atoi(tk[1]);
}
char *io_gzip::int2data(time_t fsize,time_t fdate)
{
	static char buf[50];
	xc_snprintf(buf,sizeof(buf),"%d/%d",fsize,fdate);
	return buf;
}

io_data *io_gzip::open(char *foldername,char mode,char *data,class folder *fol)
{
// modes: 'r'ead, 'w'rite(over), 'a'ppend, 'n'ewread
// write problem:
//  new mails?

	io_gdata *r=new io_gdata(mode,dir,foldername);

	signal(SIGPIPE, SIG_IGN);

//printf("open %s (%s/%s)\n",foldername,r->filename,r->filename_new);

	switch (mode)
	{
		case 'r':
			if (!lock(r->lockname))
			{
				r->file=popen(r->rdfilename,"r");
				if (r->file)
				{
					struct stat st;
					stat(r->filename,&st);
					r->fsize=st.st_size;
					r->fdate=st.st_mtime;
					setvbuf(r->file,NULL,_IOFBF,20000);
				}
				else
				{
					unlock(r->lockname);
					delete r;
					r=NULL;
				}
			}
			else
			{
				delete r;
				r=NULL;
			}
			break;
		case 'i':	// like read, but not locked and not opened
			r->file=NULL;
			{
				struct stat st;
				if (!stat(r->filename,&st))
				{
					r->fsize=st.st_size;
					r->fdate=st.st_mtime;
				}
				else
				{
					delete r;
					r=NULL;
				}
			}
			break;
		case 'w':
			if (!lock(r->lockname))
			{
				int fl=creat(r->filename_new,S_IRUSR|S_IWUSR);
				if (fl>=0)
				{
					pipe(r->wtpipe);
					fcntl(ConnectionNumber(((appwindow*)FindObject(".mainwindow"))->display()),F_SETFD,1);
					int gpid=fork();
					if (gpid==0)
					{ // gzipper
						signal(SIGPIPE, SIG_DFL);
						::close(r->wtpipe[1]);
						::close(0);
						dup(r->wtpipe[0]);
						::close(1);
						dup(fl);
						execlp("gzip","gzip","-c",NULL);
						::close(fl);
						::close(r->wtpipe[0]);
						::close(0);
						::close(1);
						::close(2);
						_exit(127);
					}
					else if (gpid==-1)
					{ // error
						::close(fl);
						::close(r->wtpipe[1]);
						::close(r->wtpipe[0]);
						remove(r->filename_new);
						unlock(r->lockname);
						delete r;
						r=NULL;
					}
					else
					{ // all fine, send data to the pipe
						::close(fl);
						::close(r->wtpipe[0]);
						r->gpid=gpid;
	
						struct stat st;
						time_t fsize=0,fdate=0;
						stat(r->filename,&st);
						fsize=r->fsize=st.st_size;
						fdate=r->fdate=st.st_mtime;
						if (data)
							data2int(data,fsize,fdate);
						r->file=fdopen(r->wtpipe[1],"a");
						if (fsize!=st.st_size || fdate!=st.st_mtime)
						{ // changed
//							printf("Folder changed! Now we have to do lots of work!\n");
							// prfen:
							// - neue Mails
							// - Mails bereits gelscht
							// - hier gelschte Mails lschen
							// - Status aktulisieren
							// 
							folder *f=new folder(this,foldername);
							unlock(r->lockname);
							do
							{
								f->load(1);
							}
							while (lock(r->lockname) && (f->unload(),1));
							fol->mix(f);
							delete f;
						}
					}
				}
				else
				{ // can't create new file
					unlock(r->lockname);
					delete r;
					r=NULL;
				}
			}
			else
			{
				delete r;
				r=NULL;
			}
			break;
		case 'a':
			if (!lock(r->lockname))
			{
				int fl=::open(r->filename,O_CREAT|O_APPEND|O_WRONLY,S_IRUSR|S_IWUSR);
				if (fl>=0)
				{
					pipe(r->wtpipe);
					fcntl(ConnectionNumber(((appwindow*)FindObject(".mainwindow"))->display()),F_SETFD,1);
					int gpid=fork();
					if (gpid==0)
					{ // gzipper
						signal(SIGPIPE, SIG_DFL);
						::close(r->wtpipe[1]);
						::close(0);
						dup(r->wtpipe[0]);
						::close(1);
						dup(fl);
						execlp("gzip","gzip","-c",NULL);
						::close(fl);
						::close(r->wtpipe[0]);
						::close(0);
						::close(1);
						::close(2);
						_exit(127);
					}
					else if (gpid==-1)
					{ // error
						::close(fl);
						::close(r->wtpipe[1]);
						::close(r->wtpipe[0]);
						unlock(r->lockname);
						delete r;
						r=NULL;
					}
					else
					{ // all fine, send data to the pipe
						::close(fl);
						::close(r->wtpipe[0]);
						r->gpid=gpid;

						struct stat st;
						time_t fsize=0,fdate=0;
						stat(r->filename,&st);
						r->fsize=st.st_size;
						r->fdate=st.st_mtime;
						r->file=fdopen(r->wtpipe[1],"a");
					}
				}
				else
				{ // can't open file
					unlock(r->lockname);
					delete r;
					r=NULL;
				}
			}
			else
			{
				delete r;
				r=NULL;
			}
			break;
		case 'n':
			if (fol && !lock(r->lockname))
			{
				r->file=popen(r->rdfilename,"r");
				if (r->file)
				{
					struct stat st;
					stat(r->filename,&st);
					r->fsize=st.st_size;
					r->fdate=st.st_mtime;
					setvbuf(r->file,NULL,_IOFBF,20000);

					folder *f=new folder(this,foldername);
					unlock(r->lockname);
					do
					{
						f->load(1);
					}
					while (lock(r->lockname) && (f->unload(),1));
					f->sub(fol);
					r->fol=f;
				}
				else
				{
					unlock(r->lockname);
					delete r;
					r=NULL;
				}
			}
			else
			{
				delete r;
				r=NULL;
			}
			break;
		default:
			delete r;
			r=NULL;
			break;
	}

	return r;
}
int io_gzip::close(io_data *f)
{
	io_gdata *r=(io_gdata*)f;
	int ret=0;
	if (r->mode=='r' || r->mode=='n')
	{
		r->error=pclose(((io_gdata*)r)->file)||r->error;
		ret=r->error;
	}
	else if (r->mode=='w')
	{
		r->error=fclose(((io_gdata*)r)->file)||r->error;
		ret=r->error;
// so maybe we have to wait until gzip has finished here ??

		waitpid(r->gpid,NULL,WUNTRACED);

		struct stat st;
		stat(r->filename,&st);
		if (r->fsize!=st.st_size || r->fdate!=st.st_mtime)
		{ // changed again
			ret=-1;
			remove(r->filename_new);
		}
		else
		{
			if (!r->error)
			{
				remove(r->filename);
				rename(r->filename_new,r->filename);
			}
			else
			{
				easy_dialog ed;
				ed.Text("Warning:\nCould not write folder, leaving it unchanged!");
				ed.Options("_Ok");
				ed.Start(readerwin);
				remove(r->filename_new);
			}
		}
		chmod(r->filename,S_IRUSR|S_IWUSR);
	}
	else if (r->mode=='a')
	{
		r->error=::close(((io_gdata*)r)->wtpipe[1])||r->error;
		ret=r->error;
// so maybe we have to wait until gzip has finished here ??

		waitpid(r->gpid,NULL,WUNTRACED);
	}
	if (r->mode!='a')
		unlock(r->lockname);
	delete r;
	return ret;
}
int io_gzip::fgets(io_data *f,char *buf,int size)
{ // TODO
	return 0;
}
class mail *io_gzip::nextmail(io_data *f)
{
	io_gdata *r=(io_gdata*)f;

	if (r->mode=='r')
	{
		char line[200];
		int mls=1,ln;
		while (::fgets(line,sizeof(line),r->file)>0)
		{
			appwindow::UpdateEvent();
			ln=strlen(line);
			if (r->txt && r->txt[0] && !strncmp("From ",line,5) && ln>5)
			{
//
// A valid from line will be in the following format:
//
//  From <user> <weekday> <month> <day> <hr:min:sec>
//      [TZ1 [TZ2]] <year> [remote from sitelist]
//
				int isRealFrom=0;
				tokenizer strt(line," ");
				if (strt.count()>=6)
				{
					if (((char*)strt[1])[0]&&
						isalnum(((char*)strt[2])[0]) &&
						isalnum(((char*)strt[3])[0]) &&
						isalnum(((char*)strt[4])[0]) &&
						isdigit(((char*)strt[5])[0]))

					{
						isRealFrom=1;
					}
				}

				if (isRealFrom)
				{
					class mail *m=new mail(r->txt,new io_id_gdata(0,0,0));
					r->txt=NULL; r->used=0; r->size=0;

					if (!r->txt)
					{
						r->txt=(char*)malloc(1000);
						*r->txt=0;
						r->size=1000;
						r->used=0;
					}
					if (ln+r->used>=r->size)
					{
						int ns=1000;
						if (ln*2>ns) ns=ln*2;
						r->txt=(char*)realloc(r->txt,r->size+ns);
						r->size+=ns;
					}
					strcat(&r->txt[r->used],line);
					r->used+=ln;
					return m;
				}
			}
			if (!r->txt)
			{
				r->txt=(char*)malloc(1000);
				*r->txt=0;
				r->size=1000;
				r->used=0;
			}
			if (ln+r->used>=r->size)
			{
				int ns=1000;
				if (ln*2>ns) ns=ln*2;
				r->txt=(char*)realloc(r->txt,r->size+ns);
				r->size+=ns;
			}
			strcat(&r->txt[r->used],line);
			r->used+=ln;
		}
		if (r->txt && r->used)
		{
			class mail *m=new mail(r->txt,new io_id_gdata(0,0,0));
			r->txt=NULL; r->used=0; r->size=0;
			return m;
		}
	}
	else if (r->mode=='n' && r->fol)
	{
		if (r->used<r->fol->count())
		{
			return r->fol->getmail(++r->used);
		}
	}
	return NULL;
}
int io_gzip::writemail(io_data *f,class mail *m)
{
	io_gdata *r=(io_gdata*)f;
	if (!(m->status() & STATUS_DELETE))
	{
		r->error=(m->write(r->file)?0:1)||r->error;
	}
	return r->error;
}
int io_gzip::firstmail(io_data *f)
{
	io_gdata *r=(io_gdata*)f;
// TODO

	return r->error;
}
int io_gzip::lastmail(io_data *f)
{
	io_gdata *r=(io_gdata*)f;
// TODO

	return r->error;
}
int io_gzip::dolock(char *lockname)
{
	if (symlink("lock",lockname))
	{
		if (errno==EEXIST)
		{
			struct stat sbuf;
			stat(lockname,&sbuf);
			if (time(NULL)-sbuf.st_mtime>600)
			{
				remove(lockname);
				return dolock(lockname);
			}
			return 1;
		}
		return 1;
	}
	return 0;
}

int io_gzip::lock(char *lockname)
{
	int i=0,r=1;
	while ((r=dolock(lockname))&&i++<10)
	{
		pstatus("Locking folder failed, attempt %d",i);
		sleep(1);
	}
	return r;
}
int io_gzip::unlock(char *lockname)
{
	unlink(lockname);
}
int io_gzip::cknew(io_data *f,char *data)
{
	io_gdata *r=(io_gdata*)f;
	int ret=0;
	time_t fsize,fdate;

	data2int(data,fsize,fdate);
	if (fsize!=r->fsize || fdate!=r->fdate)
		ret=0;
	else
		ret=1;
//printf("Check %s: %d/%d - %d/%d => %d\n",r->filename,r->fsize,fsize,r->fdate,fdate,ret);
	return ret;
}
char *io_gzip::getdata(io_data *f)
{
	io_gdata *r=(io_gdata*)f;
	struct stat st;
	stat(r->filename,&st);
	r->fsize=st.st_size;
	r->fdate=st.st_mtime;
	return int2data(r->fsize,r->fdate);
}
void *io_gzip::getFolderInfo(io_data *f,int datatype)
{
	io_gdata *r=(io_gdata*)f;
	void *ret=NULL;
	switch (datatype)
	{
		case IOCD_SIZE:
			ret=(void*)r->fsize;
			break;
		case IOCD_DATE:
			ret=(void*)r->fdate;
			break;
	}
	return ret;
}

class folder **io_gzip::getfolders(void)
{
	int c=0;
	char wdir[300];
	DIR *dfd;
	struct dirent *dptr;
	struct stat sbuf;
	static folder *allfolders[MAXFOLDERS];

	if (dfd=opendir(dir))
	{
		while ((dptr=readdir(dfd)) && c<MAXFOLDERS)
		{
			if (dptr->d_name[0]!='.')
			{
				int l=strlen(dptr->d_name);
				strcpy(wdir,dir);
				strcat(wdir,dptr->d_name);
				stat(wdir,&sbuf);
				if (S_ISREG(sbuf.st_mode) && !S_ISDIR(sbuf.st_mode))
				{
					if (l>5 && 
							 dptr->d_name[l-5]=='.' &&
							 dptr->d_name[l-4]=='l' &&
							 dptr->d_name[l-3]=='o' &&
							 dptr->d_name[l-2]=='c' &&
							 dptr->d_name[l-1]=='k')
					{
						if (time(NULL)-sbuf.st_mtime>600)
						{ printf("Old LOCK: %d\n",time(NULL)-sbuf.st_mtime);
							remove(wdir);
						}
					}
					else 
// 					     if (l>3 && 
// 							 dptr->d_name[l-3]=='.' &&
// 							 dptr->d_name[l-2]=='g' &&
// 							 dptr->d_name[l-1]=='z')
					{
						int ftype=0;
						if (!strcncmp("Outbox",dptr->d_name))
							ftype=FOLDERTYPE_OUTBOX;
						else if (!strcncmp(RECEIVEDFOLDER,dptr->d_name))
							ftype=FOLDERTYPE_RECEIVED;
						else if (!strcncmp(TRASHFOLDER,dptr->d_name))
							ftype=FOLDERTYPE_TRASH;
						else if (!strcncmp(POP3ID,dptr->d_name))
							ftype=FOLDERTYPE_INBOX;
							
						folder *f=new folder(this,dptr->d_name,ftype);
						allfolders[c]=f;
						c++;
					}
				}
			}
		}
		closedir(dfd);
	}
	else
	{
		if (stat(dir,&sbuf))
		{ // create ARCHIV dir
			mkdir(dir,S_IXUSR|S_IRUSR|S_IWUSR);
		}
	}
	allfolders[c]=NULL;
	return allfolders;
}

int io_gzip::createfolder(char *foldername)
{
	int ret=1;
	io_gdata *r=new io_gdata('w',dir,foldername);

	r->file=fopen(r->filename,"a");
	if (r->file)
	{
		fclose(r->file);
		ret=0;
	}
	else
	{
		setlasterror(strerror(errno));
	}
	delete r;

	return ret;
}
int io_gzip::removefolder(char *foldername)
{
	int ret=1;
	io_gdata *r=new io_gdata('w',dir,foldername);

	if (unlink(r->filename))
	{
		setlasterror(strerror(errno));
	}
	else
		ret=0;
	delete r;

	return ret;
}
int io_gzip::renamefolder(char *foldername,char *newfoldername)
{
	int ret=1;
	io_gdata *r1=new io_gdata('w',dir,foldername);
	io_gdata *r2=new io_gdata('w',dir,newfoldername);

	if (rename(r1->filename,r2->filename))
	{
		setlasterror(strerror(errno));
	}
	else
		ret=0;
	delete r2;
	delete r1;

	return ret;
}

