/* 
 * This file is part of XCmail and is subject to the 
 * XCmail license agreement
 */

/* $Id: folder_io_mbox.cxx,v 2.18 2002/06/27 09:26:22 schmitzj Exp $ */


#include "configure.h"

#include <Xclasses/Xclasses.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <unistd.h>
#include <stdarg.h>
#include <signal.h>
#include <unistd.h>
#include <errno.h>
#include <ctype.h>

#include "mailer.h"
#include "folder.h"
#include "xc_snprintf.h"
#include "folder_io_mbox.h"
#include "stringTokenizer.h"

class io_id_mdata : public io_id_data
{
public:
	int		seek_mail;
	int		seek_body;
	int		size;

	io_id_mdata(int m,int b,int s)
	{ seek_mail=m; seek_body=b; size=s; };
};
class io_mdata : public io_data
{
public:
	char *filename;
	char *filename_new;
	char *lockname;
	FILE *file;
	time_t fsize,fdate;

	char *txt;
	int size;
	int used;
	int error;
	char mode;
	
	class folder *fol;

	io_mdata(char _mode,char *dir,char *fn)
	{
		mode=_mode;
		int l=strlen(dir);
		if (fn) l+=strlen(fn)+1;

		filename=(char*)malloc(l+1);
		strcpy(filename,dir);
		if (fn)
		{
			strcat(filename,"/");
			strcat(filename,fn);
		}
		
		lockname=(char*)malloc(strlen(filename)+6);
		strcpy(lockname,filename);
		strcat(lockname,".lock");

		filename_new=(char*)malloc(strlen(filename)+5);
		strcpy(filename_new,filename);
		strcat(filename_new,".new");

		file=NULL;
		txt=NULL;
		size=0;
		used=0;
		fsize=0;
		fdate=0;
		error=0;
		fol=NULL;
	}
	~io_mdata()
	{
		if (fol) delete fol;
		free(lockname);
		free(filename);
	}
};
io_mbox::io_mbox(char *_dir, char *_dirname) : io_class()
{
	dir=strdup(_dir);
	dirname=(_dirname?strdup(_dirname):NULL);
	isfile=0;
	can_create_folders=can_delete_folders=can_rename_folders=can_list_folders=1;

	illegalcount=1;
	illegalchars=new char[1];
	replaceby=new char*[1];
	illegalchars[0]='/';
	replaceby[0]="_";
}
io_mbox::~io_mbox()
{
	if (dirname) free(dirname);
	free(dir);
}

char *io_mbox::io_name(void)
{ //printf("Dirname: %s\n",dirname);
	if (dirname)
		return dirname;
	else
		return "file";
}
char *io_mbox::io_type(void)
{
	return "file";
}
char *io_mbox::io_username(void)
{
	if (dirname)
		return dirname;
	else
		return dir;
}
void io_mbox::data2int(char *data,time_t &fsize,time_t &fdate)
{
	tokenizer tk(data,"/");
	if (tk.count()>0) fsize=atoi(tk[0]);
	if (tk.count()>1) fdate=atoi(tk[1]);
}
char *io_mbox::int2data(time_t fsize,time_t fdate)
{
	static char buf[50];
	xc_snprintf(buf,sizeof(buf),"%d/%d",fsize,fdate);
	return buf;
}

io_data *io_mbox::open(char *foldername,char mode,char *data,class folder *fol)
{
// modes: 'r'ead, 'w'rite(over), 'a'ppend, 'n'ewread, 'i'nfo
// write problem:
//  new mails?

	io_mdata *r=new io_mdata(mode,dir,isfile?NULL:foldername);

//printf("open %s (%s/%s)\n",foldername,r->filename,r->filename_new);

	switch (mode)
	{
		case 'r':
			if (!lock(r->lockname))
			{
				r->file=fopen(r->filename,"r");
				if (r->file)
				{
					struct stat st;
					stat(r->filename,&st);
					r->fsize=st.st_size;
					r->fdate=st.st_mtime;
					setvbuf(r->file,NULL,_IOFBF,20000);
				}
				else
				{
					unlock(r->lockname);
					delete r;
					r=NULL;
				}
			}
			else
			{
				delete r;
				r=NULL;
			}
			break;
		case 'i':	// like read, but not locked and not opened
			r->file=NULL;
			{
				struct stat st;
				if (!stat(r->filename,&st))
				{
					r->fsize=st.st_size;
					r->fdate=st.st_mtime;
				}
				else
				{
					delete r;
					r=NULL;
				}
			}
			break;
		case 'w':
			if (!lock(r->lockname))
			{
				r->file=fopen(r->filename_new,"w");
				if (r->file)
				{
					struct stat st;
					time_t fsize=0,fdate=0;
					stat(r->filename,&st);
					fsize=r->fsize=st.st_size;
					fdate=r->fdate=st.st_mtime;
					setvbuf(r->file,NULL,_IOFBF,20000);
					if (data)
						data2int(data,fsize,fdate);
					if (fsize!=st.st_size || fdate!=st.st_mtime)
					{ // changed
//						printf("Folder changed! Now we have to do lots of work!\n");
						// prfen:
						// - neue Mails
						// - Mails bereits gelscht
						// - hier gelschte Mails lschen
						// - Status aktulisieren
						// 
						folder *f=new folder(this,foldername);
						unlock(r->lockname);
						do
						{
							f->load(1);
						}
						while (lock(r->lockname) && (f->unload(),1));
						fol->mix(f);
						delete f;
					}
				}
				else
				{
					unlock(r->lockname);
					delete r;
					r=NULL;
				}
			}
			else
			{
				delete r;
				r=NULL;
			}
			break;
		case 'a':
			if (!lock(r->lockname))
			{
				r->file=fopen(r->filename,"a");
				if (r->file)
				{
					struct stat st;
					time_t fsize=0,fdate=0;
					stat(r->filename,&st);
					r->fsize=st.st_size;
					r->fdate=st.st_mtime;
					setvbuf(r->file,NULL,_IOFBF,20000);
				}
				else
				{
					unlock(r->lockname);
					delete r;
					r=NULL;
				}
			}
			else
			{
				delete r;
				r=NULL;
			}
			break;
		case 'n':
			if (fol && !lock(r->lockname))
			{
				r->file=fopen(r->filename,"r");
				if (r->file)
				{
					struct stat st;
					stat(r->filename,&st);
					r->fsize=st.st_size;
					r->fdate=st.st_mtime;

					folder *f=new folder(this,foldername);
					unlock(r->lockname);
					do
					{
						f->load(1);
					}
					while (lock(r->lockname) && (f->unload(),1));
					f->sub(fol);
					r->fol=f;
				}
				else
				{
					unlock(r->lockname);
					delete r;
					r=NULL;
				}
			}
			else
			{
				delete r;
				r=NULL;
			}
			break;
		default:
			delete r;
			r=NULL;
			break;
	}

	return r;
}
int io_mbox::close(io_data *f)
{
	io_mdata *r=(io_mdata*)f;
	int ret=0;
	if (r->file)
		r->error=fclose(r->file)||r->error;
	ret=r->error;
	if (r->mode=='w')
	{
		struct stat st;
		stat(r->filename,&st);
		if (r->fsize!=st.st_size || r->fdate!=st.st_mtime)
		{ // changed again
			ret=-1;
			remove(r->filename_new);
		}
		else
		{
			if (!r->error)
			{
				remove(r->filename);
				rename(r->filename_new,r->filename);
			}
			else
			{
				easy_dialog ed;
				ed.Text("Warning:\nCould not write folder, leaving it unchanged!");
				ed.Options("_Ok");
				ed.Start(readerwin);
				remove(r->filename_new);
			}
		}
		chmod(r->filename,S_IRUSR|S_IWUSR);
	}
	if (r->mode!='i')
		unlock(r->lockname);
	delete r;
	return ret;
}
int io_mbox::fgets(io_data *f,char *buf,int size)
{ // TODO
	return 0;
}
class mail *io_mbox::nextmail(io_data *f)
{
	io_mdata *r=(io_mdata*)f;

	if (r->mode=='r')
	{
		char line[200];
		int mls=1,ln;
		while (::fgets(line,sizeof(line),r->file)>0)
		{
			appwindow::UpdateEvent();
			ln=strlen(line);
			if (r->txt && r->txt[0] && !strncmp("From ",line,5) && ln>5)
			{
//
// A valid from line will be in the following format:
//
//  From <user> <weekday> <month> <day> <hr:min:sec>
//      [TZ1 [TZ2]] <year> [remote from sitelist]
//
				int isRealFrom=0;
				tokenizer strt(line," ");
				if (strt.count()>=6)
				{
					if (((char*)strt[1])[0]&&
						isalnum(((char*)strt[2])[0]) &&
						isalnum(((char*)strt[3])[0]) &&
						isalnum(((char*)strt[4])[0]) &&
						isdigit(((char*)strt[5])[0]))

					{
						isRealFrom=1;
					}
				}

				if (isRealFrom)
				{
					class mail *m=new mail(r->txt,new io_id_mdata(0,0,0));
					r->txt=NULL; r->used=0; r->size=0;

					if (!r->txt)
					{
						r->txt=(char*)malloc(1000);
						*r->txt=0;
						r->size=1000;
						r->used=0;
					}
					if (ln+r->used>=r->size)
					{
						int ns=1000;
						if (ln*2>ns) ns=ln*2;
						r->txt=(char*)realloc(r->txt,r->size+ns);
						r->size+=ns;
					}
					strcat(&r->txt[r->used],line);
					r->used+=ln;
					return m;
				}
			}
			if (!r->txt)
			{
				r->txt=(char*)malloc(1000);
				*r->txt=0;
				r->size=1000;
				r->used=0;
			}
			if (ln+r->used>=r->size)
			{
				int ns=1000;
				if (ln*2>ns) ns=ln*2;
				r->txt=(char*)realloc(r->txt,r->size+ns);
				r->size+=ns;
			}
			strcat(&r->txt[r->used],line);
			r->used+=ln;
		}
		if (r->txt && r->used)
		{
			class mail *m=new mail(r->txt,new io_id_mdata(0,0,0));
			r->txt=NULL; r->used=0; r->size=0;
			return m;
		}
	}
	else if (r->mode=='n' && r->fol)
	{
		if (r->used<r->fol->count())
		{
			return r->fol->getmail(++r->used);
		}
	}
	return NULL;
}
int io_mbox::writemail(io_data *f,class mail *m)
{
	io_mdata *r=(io_mdata*)f;
	if (!(m->status() & STATUS_DELETE))
	{
		r->error=(m->write(r->file)?0:1)||r->error;
	}
	return r->error;
}
int io_mbox::firstmail(io_data *f)
{
	io_mdata *r=(io_mdata*)f;
// TODO

	return r->error;
}
int io_mbox::lastmail(io_data *f)
{
	io_mdata *r=(io_mdata*)f;
// TODO

	return r->error;
}
int io_mbox::dolock(char *lockname)
{ // is this working correctly ???? Not sure! Someone is telling "lock successfull" and will remove the lock.....
	int r=0;
	if (r=symlink("lock",lockname))
	{ // printf("not locked (%x) %s\n",this,lockname);
		if (errno==EEXIST)
		{
			struct stat sbuf;
			stat(lockname,&sbuf);
			if (time(NULL)-sbuf.st_mtime>600)
			{
				remove(lockname);
				return dolock(lockname);
			}
			return 1;
		}
		return 1;
	}
	return 0;
}

int io_mbox::lock(char *lockname)
{
	int i=0,r=1;
	while ((r=dolock(lockname))&&i++<10)
	{
		pstatus("Locking folder failed, attempt %d",i);
		sleep(1);
	}
	return r;
}
int io_mbox::unlock(char *lockname)
{
	unlink(lockname);
// printf("unlocked (%x) %s\n",this,lockname);
}
int io_mbox::cknew(io_data *f,char *data)
{
	io_mdata *r=(io_mdata*)f;
	int ret=0;
	time_t fsize,fdate;

	data2int(data,fsize,fdate);
	if (fsize!=r->fsize || fdate!=r->fdate)
		ret=0;
	else
		ret=1;
//printf("Check %s: %d/%d - %d/%d => %d\n",r->filename,r->fsize,fsize,r->fdate,fdate,ret);
	return ret;
}
char *io_mbox::getdata(io_data *f)
{
	io_mdata *r=(io_mdata*)f;
	struct stat st;
	stat(r->filename,&st);
	r->fsize=st.st_size;
	r->fdate=st.st_mtime;
	return int2data(r->fsize,r->fdate);
}
void *io_mbox::getFolderInfo(io_data *f,int datatype)
{
	io_mdata *r=(io_mdata*)f;
	void *ret=NULL;
	switch (datatype)
	{
		case IOCD_SIZE:
			ret=(void*)r->fsize;
			break;
		case IOCD_DATE:
			ret=(void*)r->fdate;
			break;
	}
	return ret;
}

class folder **io_mbox::getfolders(void)
{
	int c=0;
	char wdir[300];
	DIR *dfd;
	struct dirent *dptr;
	struct stat sbuf;
	static folder *allfolders[MAXFOLDERS];

	if (dfd=opendir(dir))
	{
		while ((dptr=readdir(dfd)) && c<MAXFOLDERS)
		{
			if (dptr->d_name[0]!='.')
			{
				int l=strlen(dptr->d_name);
				strcpy(wdir,dir);
				strcat(wdir,dptr->d_name);
				stat(wdir,&sbuf);
				if (S_ISREG(sbuf.st_mode) && !S_ISDIR(sbuf.st_mode))
				{
					if (l>5 && 
							 dptr->d_name[l-5]=='.' &&
							 dptr->d_name[l-4]=='l' &&
							 dptr->d_name[l-3]=='o' &&
							 dptr->d_name[l-2]=='c' &&
							 dptr->d_name[l-1]=='k')
					{
						if (time(NULL)-sbuf.st_mtime>600)
						{
							remove(wdir);
						}
					}
					else
					{
						int ftype=0;
						if (!strcscmp(dptr->d_name,"Outbox"))
							ftype=FOLDERTYPE_OUTBOX;
						else if (!strcmp(dptr->d_name,RECEIVEDFOLDER))
							ftype=FOLDERTYPE_RECEIVED;
						else if (!strcscmp(dptr->d_name,TRASHFOLDER))
							ftype=FOLDERTYPE_TRASH;
						else if (!strcscmp(dptr->d_name,POP3ID))
							ftype=FOLDERTYPE_INBOX;
							
						folder *f=new folder(this,dptr->d_name,ftype);
						allfolders[c]=f;
						c++;
					}
				}
			}
		}
		closedir(dfd);
	}
	else
	{
		if (!stat(dir,&sbuf) && (S_ISREG(sbuf.st_mode) && !S_ISDIR(sbuf.st_mode)))
		{ // file (= mini folder)
			folder *f=new folder(this,dirname,FOLDERTYPE_INBOX);
			allfolders[c]=f;
			c++;
			isfile=1;
			can_create_folders=can_delete_folders=can_rename_folders=0;
		}
	}
	allfolders[c]=NULL;
	return allfolders;
}

int io_mbox::createfolder(char *foldername)
{
	if (isfile)
	{
		setlasterror("Not possible to create folders in filebox!");
		return 1;
	}
	int ret=1;
	io_mdata *r=new io_mdata('w',dir,isfile?NULL:foldername);

	r->file=fopen(r->filename,"a");
	if (r->file)
	{
		fclose(r->file);
		ret=0;
	}
	else
	{
		setlasterror(strerror(errno));
	}
	delete r;

	return ret;
}
int io_mbox::removefolder(char *foldername)
{
	if (isfile)
	{
		setlasterror("Not possible to remove folders in filebox!");
		return 1;
	}
	int ret=1;
	io_mdata *r=new io_mdata('w',dir,isfile?NULL:foldername);

	if (unlink(r->filename))
	{
		setlasterror(strerror(errno));
	}
	else
		ret=0;
	delete r;

	return ret;
}
int io_mbox::renamefolder(char *foldername,char *newfoldername)
{
	if (isfile)
	{
		setlasterror("Not possible to remove folders in filebox!");
		return 1;
	}
	int ret=1;
	io_mdata *r1=new io_mdata('w',dir,isfile?NULL:foldername);
	io_mdata *r2=new io_mdata('w',dir,isfile?NULL:newfoldername);

	if (rename(r1->filename,r2->filename))
	{
		setlasterror(strerror(errno));
	}
	else
		ret=0;
	delete r2;
	delete r1;

	return ret;
}

