/* 
 * This file is part of XCmail and is subject to the 
 * XCmail license agreement
 */

/* $Id: folder_manager.cxx,v 2.22 2002/09/23 09:37:13 schmitzj Exp $ */


#include "configure.h"
#include "folder_manager.h"

#include "mailer.h"
#include "folder.h"
#include "gfxobjects.h"
#include "extraggs.h"

static void changeHandler(void)
{
	folder_manager *fa;

	if (fa=(folder_manager*)FindObject(".folder_manager"))
	{
		fa->show_folder_list();
	}	
}
static void changeCFHandler(void)
{
	folder_manager *fa;

	if (fa=(folder_manager*)FindObject(".folder_manager"))
	{
		fa->check_folder_list();
	}	
}
gfx_folder gx_folder;
gfx_readonly gx_readonly;
gfx_trash gx_trash;
gfx_received gx_received;
gfx_inbox gx_inbox;
gfx_outbox gx_outbox;
static int lvhook(struct hook_data *hd)
{
	int w=hd->height-4;
	vfolder *v=(vfolder*)hd->itemdata;

	if (hd->flags&HD_SELECTED)
		XSetForeground(hd->list->display(),hd->drawable->gg_gc(),hd->drawable->col_selected());
	else if (hd->flags&HD_MARKED)
		XSetForeground(hd->list->display(),hd->drawable->gg_gc(),hd->drawable->col_markbackground());
	else
		XSetForeground(hd->list->display(),hd->drawable->gg_gc(),hd->drawable->col_shinebackground());
	XFillRectangle(hd->list->display(),hd->drawable->gg_win(),hd->drawable->gg_gc(),
						hd->x+1,hd->y,w-2,hd->height);

	switch (v->getftype())
	{
		case FOLDERTYPE_INBOX:
			gx_inbox.draw_normal(hd->drawable,hd->x,hd->y,w,w);
			break;
		case FOLDERTYPE_TRASH:
			gx_trash.draw_normal(hd->drawable,hd->x,hd->y,w,w);
			break;
		case FOLDERTYPE_RECEIVED:
			gx_received.draw_normal(hd->drawable,hd->x,hd->y,w,w);
			break;
		case FOLDERTYPE_OUTBOX:
			gx_outbox.draw_normal(hd->drawable,hd->x,hd->y,w,w);
			break;
		default:
			if (v->readonly)
				gx_readonly.draw_normal(hd->drawable,hd->x,hd->y,w,w);
			else
				gx_folder.draw_normal(hd->drawable,hd->x,hd->y,w,w);
			break;
	}

	return -(w+4);
}
folder_manager::folder_manager(void) : Xclasses("folder_manager_init")
{
	folder_manager *fa;

	ParentClass(readerwin);

	if (fa=(folder_manager*)FindObject(".folder_manager"))
	{
		fa->raise();
		delete this;
		return;
	}
	Name("folder_manager");

	old_vfolder_selected=NULL;
	delaywindow=NULL;
	abort_pressed=0;
	makewin();
}

folder_manager::~folder_manager()
{
	DeleteObjectTree(FullName(),DOT_NOTBASE);
}
void folder_manager::raise(void)
{
	appwindow *aw=(appwindow*)FindObject(FullName(),"window");
	if (aw) aw->Raise();
}
void folder_manager::quit(gadget *g,int i)
{
	folderlist->unregistercurrentfunction(changeCFHandler);
	folderlist->unregisterfolderfunction(changeHandler);
	listview *lv=(listview*)FindObject(FullName(),".vfolders.vfolderlist");
	lv->Selected(0);
	folder_editor(lv,0);
	folderlist->saveprefs();

	((appwindow*)FindObject(FullName(),"window"))->Close();
	delete this;
}
void folder_manager::menuhandler(gadget *g,int i)
{
	switch (((menu*)g)->Selection())
	{
		case 0x01000001:	// Create folder
			{
			}
			break;
	}
}
void folder_manager::makewin(void)
{
	int x,y,w,h;
	appwindow *aw=new appwindow;

	readerwin.Position(x,y);
	readerwin.Size(w,h);

	pstatus("Open folder manager window");

	aw->InitSize(x,y,10);
	if (!aw->Init())
	{
		delete this;
		pstatus("Error in opening window!");
		return;
	}
	aw->Adjust(x,y,w,h,ADJUST_CENTER|ADJUST_MIDDLE);
	aw->Title(PROGRAMNAME" mail folder manager");
	aw->IconTitle("Folder manager");
	aw->SetIcon(f_xicon1.pixmap(),f_xicon1.shadowmap());
//	aw->TransientFor(readerwin);
	aw->ParentClass(this);
	aw->Name("window");
	aw->Callback(this,(CLOSECALLBACK)&folder_manager::quit);
	aw->Group(readerwin);

	fbutton *fb1;
	bordergroup *bg,*bg2;
	group *g1,*g2;
	choice *ch;
	listview *lv;
	input *ip;
	checkbox *cb;
	text *txt;
	static char *chdummy[]={"OOOOOOOOO","OOOOOOOOO",NULL};
	spacegroup *sg1=new spacegroup;
	sg1->Border(4);
	sg1->Name("maingroup");
	sg1->Vert();
	 card *cd=new card;
	 cd->Name("thecard");
	  bg=new bordergroup;
	  bg->Name("controllergroup");
	  bg->Horiz();
	  bg->Center();
	  bg->XDistance(4);
	   g1=new group;
	   g1->Name("source");
	   g1->Vert();
	    ch=new choice;
	    ch->Name("io_list");
		ch->Items(chdummy);
		ch->Callback(this,(CALLBACK)&folder_manager::folder_controller);
		ch->HelpText("folder IO: a place where folders\nmay be stored");
	   g1->Add(ch);
	    lv=new listview;
		lv->Name("folderlist");
		lv->ShowSelected();
		lv->MultiSelect();		// not easy to handle!
		lv->Callback(this,(CALLBACK)&folder_manager::folder_controller);
		lv->HelpText("all folders of this IO");
	   g1->Add(lv);
	    g2=new group;
		g2->Name(NULL);
		g2->Horiz();
		 fb1=new fbutton;
		 fb1->Name("new");
		 fb1->Text("_New...");
		 fb1->Callback(this,(CALLBACK)&folder_manager::folder_controller);
		g2->Add(fb1);
		 fb1=new fbutton;
		 fb1->Name("rename");
		 fb1->Text("_Rename...");
		 fb1->Callback(this,(CALLBACK)&folder_manager::folder_controller);
		 fb1->Lock();
		g2->Add(fb1);
		 fb1=new fbutton;
		 fb1->Name("delete");
		 fb1->Text("_Delete...");
		 fb1->Callback(this,(CALLBACK)&folder_manager::folder_controller);
		 fb1->Lock();
		g2->Add(fb1);
	   g1->Add(g2);
	  bg->Add(g1);
	   g1=new group;
	   g1->Name("work");
	   g1->Vert();
	   g1->UseSmallest();
	   g1->YDistance(5);
	   g1->Weight(10);
	    txt=new text;
		txt->Center();
		txt->Text("Folder ops");
	   g1->Add(txt);
	    fb1=new fbutton;
	    fb1->Name("copy");
		fb1->Text(" _Copy ");
		fb1->Callback(this,(CALLBACK)&folder_manager::folder_controller);
		fb1->HelpText("copies the folder");
		fb1->Lock();
	   g1->Add(fb1);
	    fb1=new fbutton;
	    fb1->Name("move");
		fb1->Text(" _Move ");
		fb1->Callback(this,(CALLBACK)&folder_manager::folder_controller);
		fb1->HelpText("moves the folder (copy & delete original)");
		fb1->Lock();
	   g1->Add(fb1);
	    fb1=new fbutton;
	    fb1->Name("movemail");
		fb1->Text(" _Move mails");
		fb1->Callback(this,(CALLBACK)&folder_manager::folder_controller);
		fb1->HelpText("copies the mail and clears the original folder");
		fb1->Lock();
	   g1->Add(fb1);
	   g1->Add(new ruler);
	    txt=new text;
		txt->Center();
		txt->Text("In vfolders");
	   g1->Add(txt);
	    lv=new listview;
		lv->Name("invfolders");
		lv->ReadOnly();
		lv->Font(FONT_SMALL);
		lv->HelpText("the selected folder belongs to\nthis vfolders");
	   g1->Add(lv);
	  bg->Add(g1);
	   g1=new group;
	   g1->Name("destination");
	   g1->Vert();
	    ch=new choice;
	    ch->Name("io_list");
		ch->Items(chdummy);
		ch->Callback(this,(CALLBACK)&folder_manager::folder_controller);
		ch->HelpText("folder IO: a place where folders\nmay be stored");
	   g1->Add(ch);
	    lv=new listview;
		lv->Name("folderlist");
		lv->ReadOnly();
		lv->HelpText("all folders of this IO");
	   g1->Add(lv);
	  bg->Add(g1);
	 cd->Add("Folder controller",bg);

	  bg=new bordergroup;
	  bg->Name("editorgroup");
	  bg->Horiz();
	  bg->Center();
	  bg->XDistance(4);
	   g1=new group;
	   g1->Name("vfolders");
	   g1->Vert();
	    txt=new text;
		txt->Center();
		txt->Text("Virtual folders");
	   g1->Add(txt);
	    lv=new listview;
		lv->Name("vfolderlist");
		lv->ShowSelected();
//		lv->MultiSelect();		// not easy to handle!
		lv->Callback(this,(CALLBACK)&folder_manager::folder_editor);
		lv->SetHook(lvhook);
		lv->HelpText("your vfolders");
	   g1->Add(lv);
	    g2=new group;
		g2->Name(NULL);
		g2->Horiz();
		 fb1=new fbutton;
		 fb1->Name("new");
		 fb1->Text("_New...");
		 fb1->Callback(this,(CALLBACK)&folder_manager::folder_editor);
		 if (!programconfig.vfolder_manager)
		 	fb1->Lock();
		g2->Add(fb1);
		 fb1=new fbutton;
		 fb1->Name("rename");
		 fb1->Text("_Rename...");
		 fb1->Callback(this,(CALLBACK)&folder_manager::folder_editor);
		 fb1->Lock();
		g2->Add(fb1);
		 fb1=new fbutton;
		 fb1->Name("delete");
		 fb1->Text("_Delete...");
		 fb1->Callback(this,(CALLBACK)&folder_manager::folder_editor);
		 fb1->Lock();
		g2->Add(fb1);
	   g1->Add(g2);
	  bg->Add(g1);
	   g1=new group;
	   g1->Name("folders");
	   g1->Vert();
	    txt=new text;
		txt->Center();
		txt->Text("Virtual folder settings");
	   g1->Add(txt);
	    static char *vfoldertypes[]={"folder","inbox","trash","received","outbox",NULL};
	    ch=new choice;
	    ch->Name("vfoldertype");
		ch->Items(vfoldertypes);
		ch->Callback(this,(CALLBACK)&folder_manager::folder_editor);
		ch->HelpText("every vfolder may have a type and by this\nhas a different meaning\neg inboxes are checked for new mail,\nin outboxes the recipient is shown not the sender");
		ch->Lock();
	   g1->Add(ch,"VFolder _type");
	    ip=new input;
	    ip->Name("subdir");
		ip->MaxChars(MAXLENGTH);
		ip->Callback(this,(CALLBACK)&folder_manager::folder_editor);
		ip->Text("_Subdir");
		ip->HelpText("a vfolder may be in a (virtual) subdirectory");
		ip->Lock();
	   g1->Add(ip);
	    cb=new checkbox;
	    cb->Name("hidden");
		cb->Callback(this,(CALLBACK)&folder_manager::folder_editor);
		cb->Text("_Hidden");
		cb->HelpText("a hidden vfolder is not visible in the main window\nin the folder list");
		cb->Lock();
	   g1->Add(cb,"");

	    cb=new checkbox;
	    cb->Name("rec_own");
		cb->Callback(this,(CALLBACK)&folder_manager::folder_editor);
		cb->Text("own received settin_gs");
		cb->HelpText("each vfolder may have its own move into received folder\nsettings - if this is disabled the global settings from\ngeneral prefs window are used");
		cb->Lock();
	   g1->Add(cb,"");

	    bg2=new bordergroup;
		bg2->Name(NULL);
		bg2->Title("Move mail into received folder");
		bg2->Horiz();
		bg2->UseSmallest();
		 g2=new group;
		 g2->Vert();
		 g2->Name(NULL);
	      cb=new checkbox;
	      cb->Name("rec_always");
		  cb->Callback(this,(CALLBACK)&folder_manager::folder_editor);
		  cb->Text("al_ways");
		  cb->Lock();
		 g2->Add(cb,"");
	      cb=new checkbox;
	      cb->Name("rec_read");
		  cb->Callback(this,(CALLBACK)&folder_manager::folder_editor);
		  cb->Text("if _read");
		  cb->Lock();
		 g2->Add(cb,"");
	      cb=new checkbox;
	      cb->Name("rec_reply");
		  cb->Callback(this,(CALLBACK)&folder_manager::folder_editor);
		  cb->Text("if re_plyed");
		  cb->Lock();
		 g2->Add(cb,"");
		bg2->Add(g2);
		bg2->Add(new ruler);
		 g2=new group;
		 g2->Vert();
		 g2->Name(NULL);
		 g2->UseSmallest();
	      cb=new checkbox;
	      cb->Name("rec_forward");
		  cb->Callback(this,(CALLBACK)&folder_manager::folder_editor);
		  cb->Text("if _forwarded");
		  cb->Lock();
		 g2->Add(cb,"");
	      cb=new checkbox;
	      cb->Name("rec_older");
		  cb->Callback(this,(CALLBACK)&folder_manager::folder_editor);
		  cb->Text("if _older than days");
		  cb->Lock();
		 g2->Add(cb,"");
 		  ip=new input;
 		  ip->Name("rec_older_days");
 		  ip->Text("Da_ys");
 		  ip->MaxChars(10);
		  ip->IntegerMode();
		  ip->Lock();
    	 g2->Add(ip,NULL);
		bg2->Add(g2);
	   g1->Add(bg2,"");

	   g1->Add(new ruler);
	    txt=new text;
		txt->Center();
		txt->Text("Folders of vfolder");
	   g1->Add(txt);
	    lv=new listview;
		lv->Name("folderlist");
		lv->ShowSelected();
		lv->Callback(this,(CALLBACK)&folder_manager::folder_editor);
		lv->HelpText("the folders of a vfolder");
		lv->Lock();
	   g1->Add(lv);
	    g2=new group;
		g2->Name(NULL);
		g2->Vert();
		 fb1=new fbutton;
		 fb1->Name("default_folder");
		 fb1->Text("_Set as write default");
		 fb1->Callback(this,(CALLBACK)&folder_manager::folder_editor);
		 fb1->HelpText("the default folder is used if you\nwrite (add) mail into a vfolder\nbecause a folder stores the mail, not the vfolder");
		 fb1->Lock();
		g2->Add(fb1);
		 fb1=new fbutton;
		 fb1->Name("remove_folder");
		 fb1->Text("Remove from _vfolder");
		 fb1->Callback(this,(CALLBACK)&folder_manager::folder_editor);
		 fb1->Lock();
		g2->Add(fb1);
	   g1->Add(g2);
	  bg->Add(g1);
	  bg->Add(new ruler);
	   g1=new group;
	   g1->Name("allfolders");
	   g1->Vert();
	    txt=new text;
		txt->Center();
		txt->Text("Available folders");
	   g1->Add(txt);
	    lv=new listview;
		lv->Name("allfolderlist");
		lv->ShowSelected();
		lv->Callback(this,(CALLBACK)&folder_manager::folder_editor);
		lv->HelpText("all your real folders of all IOs");
		if (!programconfig.vfolder_manager)
			lv->Lock();
	   g1->Add(lv);
	    g2=new group;
		g2->Name(NULL);
		g2->Horiz();
		 fb1=new fbutton;
		 fb1->Name("add_into_vfolder");
		 fb1->Text("_Add into vfolder");
		 fb1->Callback(this,(CALLBACK)&folder_manager::folder_editor);
		 fb1->Lock();
		g2->Add(fb1);
	   g1->Add(g2);
	  bg->Add(g1);
	 cd->Add("Folder editor",bg);

	  bg=new bordergroup;
	  bg->Name("editorgroup");
	  bg->Horiz();
	  bg->Center();
	  bg->XDistance(4);
	   html_gadget *helptxt=new html_gadget;
	   helptxt->Text("<body>The folder management is divided in 2 areas:
<ul><li>the folder controller
<li>the folder editor
</ul>
<p>
The <b>folder controllers</b> allows to create, rename, delete, move and copy
real folders from one IO to another. IOs are the different places for
folders - like your home directory (~/Mail), the folder archive (~/Mail/ARCHIVE).<p>
Each IO normally allows only unique folder names - ie they are stored in files. But 
different IOs may have the same folder names.
<p>
<b>To move mail from a folder (eg received) into the folder archiv do this:</b>
<ol>
<li>select your mail directory in the left popup list (..../Mail/)
<li>select the ARCHIV directory in the right popup list (..../Mail/ARCHIV/)
<li>select the folder you want to archiv in the left listbox (eg received)
<li>press one of the buttons 'Copy' to copy the folder, 'Move' to copy the folder and remove the source or 'Move mail' to copy the folder and clear the source (the best choice here)
</ol>
<p>
The <b>folder editor</b> handles the so called 'virtual folders' or
vfolders. To use this you have to enable the 'advanced vfolder concept'
in the main preferences.
<p>
Every real folder  belongs to one or more virtual folders
and every vertiual folder has one or more real folders in it.<p>
The idea is to have only a handfull visible folders in the mail window (this only 
shows the virtual folders) but they belong to different IOs - eg. all your
POP3 servers may write their mails in different real folders but you will see 
all mails in one vfolder - but you can still access the real folders of each 
POP3 server if you also create a virtual folders for this.<p>

An image that might help you:
<pre>
  +--------+             +--------+            +--------+
  |vfolder1|             |vfolder2|      +-----|vfolder3|
  +--------+             +--------+     /      +--------+
       |                      |        /           |  
  +---------+     +--------------------+    +-------------+
  |~/Mail/f1|     |/var/spool/mail/user|    |.../ARCHIV/f2|
  +---------+     +--------------------+    +-------------+
</pre>
<dl>
<dd>vfolder1 uses ~/Mail/f1,
<dd>vfolder2 uses /var/spool/mail/user and
<dd>vfolder3 uses /var/spool/mail/user AND .../ARCHIV/f2.
</dl>
<p>
</body>");
	  bg->Add(helptxt);
	 cd->Add("Information/Short help",bg);

	sg1->Add(cd);
	sg1->Add(new ruler);
	 g1=new group;
	 g1->Horiz();
	 g1->Name(NULL);
	 g1->DialogMode();
	  fb1=new fbutton;
	  fb1->Name("close");
	  fb1->Text("Close");
	  fb1->Callback(this,(CALLBACK)&folder_manager::quit);
	  fb1->ApplyXKey(XK_Escape);
	 g1->Add(fb1);
	sg1->Add(g1);

	aw->SetGroup(sg1);

//	menu *mn=new menu;
//	mn->Callback(this,(CALLBACK)&folder_archiv::menuhandler);
//	mn->AddMenu("Folder");
//	mn->Add("Create new folder...","n",1);
//	aw->SetMenu(mn);
	aw->Create();
	show_io_list();
	folderlist->registerfolderfunction(changeHandler);
	folderlist->registercurrentfunction(changeCFHandler);

	ch=(choice*)FindObject(FullName(),".folders.vfoldertype");
	ch->SetResource(".folder.graphic",gx_folder.FullName());
	ch->SetResource(".inbox.graphic",gx_inbox.FullName());
	ch->SetResource(".outbox.graphic",gx_outbox.FullName());
	ch->SetResource(".trash.graphic",gx_trash.FullName());
	ch->SetResource(".received.graphic",gx_received.FullName());
}

void folder_manager::show_io_list(void)
{
	static char **ch1_arr=NULL;
	static char **ch2_arr=NULL;

	choice *ch1=(choice*)FindObject(".controllergroup.source.io_list");
	choice *ch2=(choice*)FindObject(".controllergroup.destination.io_list");

	if (ch1_arr)
		delete [] ch1_arr;
	if (ch2_arr)
		delete [] ch2_arr;

	ch1_arr=new char*[allios->Count()+1];
	ch2_arr=new char*[allios->Count()+1];

	int i1=0,i2=0;
	io_class *n=(io_class*)allios->head;
	while (n->next)
	{
		if (n->can_list_folders)
		{
			ch1_arr[i1++]=n->io_username();
			if (n->can_create_folders)
			{
				ch2_arr[i2++]=n->io_username();
			}
		}
		n=(io_class*)n->next;
	}
	ch1_arr[i1]=NULL;
	ch2_arr[i2]=NULL;

	ch1->Items(ch1_arr);
	ch2->Items(ch2_arr);
	show_folder_list();
}
void folder_manager::show_folder_list(void)
{
	char buf[200];
	int top;
	choice *ch1=(choice*)FindObject(".controllergroup.source.io_list");
	choice *ch2=(choice*)FindObject(".controllergroup.destination.io_list");
	folder_controller(ch1,0);
	folder_controller(ch2,0);

	listview *lv=(listview*)FindObject(".editorgroup.allfolders.allfolderlist");
	top=lv->Top();
	lv->RemoveAll();
	folders_data *f=(folders_data*)folderlist->allfolders.head;
	while (f->next)
	{
		xc_snprintf(buf,sizeof(buf),"%s (%s)",f->f->g_foldername(),f->f->getio()->io_username());
		lv->AddAtEnd(buf,f->f);
		f=(folders_data*)f->next;
	}
	lv->Top(top);
	folder_editor(lv,0);

	lv=(listview*)FindObject(FullName(),".vfolders.vfolderlist");
	top=lv->Top();
	lv->RemoveAll();
	folders_vdata *v=(folders_vdata*)folderlist->allvfolders.head;
	while (v->next)
	{
		xc_snprintf(buf,sizeof(buf),"%s/%s",v->f->g_subdir(),v->f->g_foldername());
		lv->AddAtEnd(buf,v->f);
		if (v->f==old_vfolder_selected)
		{
			lv->Selected(lv->Count());
		}
		v=(folders_vdata*)v->next;
	}
	old_vfolder_selected=(vfolder*)lv->GetData(lv->Selected());
	lv->Top(top);
	folder_editor(lv,0);
	check_folder_list();
}
void folder_manager::check_folder_list(void)
{
	listview *lv=(listview*)FindObject(FullName(),".vfolders.vfolderlist");
	folder_editor(lv,0);

	lv=(listview*)FindObject(FullName(),".controllergroup.source.folderlist");
	folder_controller(lv,0);
}
io_class *folder_manager::getiofromlist(int source)
{
	io_class *n=(io_class*)allios->head,*io=NULL;
	choice *ch;
	if (source)
		ch=(choice*)FindObject(".controllergroup.source.io_list");
	else
		ch=(choice*)FindObject(".controllergroup.destination.io_list");
	int i=0;
	while (n->next)
	{
		if (n->can_list_folders)
		{
			if (source || n->can_create_folders)
			{
				if (i++==ch->Selected())
				{
					io=n;
					break;
				}
			}
		}
		n=(io_class*)n->next;
	}
	return io;
}
void folder_manager::folder_controller(gadget *g,int)
{
	Xclasses *p=g->ParentClass();
	while (p && (!p->Name() || !p->Name()[0]))
	{
		p=p->ParentClass();
	}
	int source_mode=!strcmp(p->Name(),"source");
	listview *lv=NULL;

	if (!strcmp(g->Name(),"io_list"))
	{
		lv=(listview*)FindObject(p->FullName(),"folderlist");
		lv->RemoveAll();

		io_class *io=getiofromlist(source_mode);

		folders_data *f=(folders_data*)folderlist->allfolders.head;
		while (f->next)
		{
			if (f->f->getio()==io)
				lv->AddAtEnd(f->f->g_foldername(),f->f);
			f=(folders_data*)f->next;
		}
		if (source_mode)
		{
			if (io && io->can_create_folders)
				((fbutton*)FindObject(p->FullName(),".new"))->Unlock();
			else
				((fbutton*)FindObject(p->FullName(),".new"))->Lock();
		}
		folder_controller(lv,0);
	}
	else if (!strcmp(g->Name(),"folderlist"))
	{
		int not_delete_folder=0;
		lv=(listview*)FindObject(FullName(),".controllergroup.source.folderlist");

		vfolder *v=folderlist->currentfolder();
		if (v)
		{
 			folder *f=(folder*)lv->GetData(lv->Selected());
			vfolder_data *n=(vfolder_data*)v->allfolders->head;
			while (n->next)
			{
				if (n->f==f)
				{
					not_delete_folder=1;
					break;
				}
				n=(vfolder_data*)n->next;
			}
		}

		if (source_mode)
		{
			io_class *io1=getiofromlist(1);

			((fbutton*)FindObject(p->FullName(),".rename"))->Lock();
			((fbutton*)FindObject(p->FullName(),".delete"))->Lock();

			if (lv->Selected()>0)
			{
				if (io1 && io1->can_rename_folders)
					((fbutton*)FindObject(p->FullName(),".rename"))->Unlock();
				if (io1 && io1->can_delete_folders && !not_delete_folder)
					((fbutton*)FindObject(p->FullName(),".delete"))->Unlock();

				listview *lv2=(listview*)FindObject(FullName(),".controllergroup.work.invfolders");
				lv2->RemoveAll();
	 			folder *f=(folder*)lv->GetData(lv->Selected());
				folders_vdata *n1=(folders_vdata*)folderlist->allvfolders.head;
				while (n1->next)
				{
					vfolder_data *n2=(vfolder_data*)n1->f->allfolders->head;
					while (n2->next)
					{
						if (n2->f==f)
						{
							char buf[200];
							xc_snprintf(buf,sizeof(buf),"%s/%s",n1->f->g_subdir(),n1->f->g_foldername());
							lv2->AddAtEnd(buf);
							break;
						}
						n2=(vfolder_data*)n2->next;
					}
					n1=(folders_vdata*)n1->next;
				}
			}
			else
				((listview*)FindObject(FullName(),".controllergroup.work.invfolders"))->RemoveAll();
		}
		io_class *io1=getiofromlist(1);
		io_class *io2=getiofromlist(0);

		if (lv->Selected()==0 || io1==io2)
		{
			((fbutton*)FindObject(FullName(),".controllergroup.work.copy"))->Lock();
			((fbutton*)FindObject(FullName(),".controllergroup.work.move"))->Lock();
			((fbutton*)FindObject(FullName(),".controllergroup.work.movemail"))->Lock();
		}
		else
		{
			((fbutton*)FindObject(FullName(),".controllergroup.work.copy"))->Unlock();
			if (not_delete_folder)
				((fbutton*)FindObject(FullName(),".controllergroup.work.move"))->Lock();
			else
				((fbutton*)FindObject(FullName(),".controllergroup.work.move"))->Unlock();
			((fbutton*)FindObject(FullName(),".controllergroup.work.movemail"))->Unlock();
		}
	}
	else if (!strcmp(g->Name(),"new"))
	{
		askString(g,"Create new folder","New folder",&folder_manager::folder_new);
	}
	else if (!strcmp(g->Name(),"rename"))
	{
		lv=(listview*)FindObject(FullName(),".controllergroup.source.folderlist");
		askString(g,"Rename folder",((folder*)lv->GetData(lv->Selected()))->g_foldername(),&folder_manager::folder_rename);
	}
	else if (!strcmp(g->Name(),"delete"))
	{
		lv=(listview*)FindObject(FullName(),".controllergroup.source.folderlist");

		folderlist->spool_folderfunction();
		delete_all=0;
		delete_abort=0;
		lv_forMarked(lv,this,(LVWORKCAST)&folder_manager::folder_delete);
		folderlist->unspool_folderfunction();
	}
	else if (!strcmp(g->Name(),"copy"))
	{
		lv=(listview*)FindObject(FullName(),".controllergroup.source.folderlist");
		folderlist->spool_folderfunction();
		lv_forMarked(lv,this,(LVWORKCAST)&folder_manager::folder_copy);
		folderlist->unspool_folderfunction();
	}
	else if (!strcmp(g->Name(),"move"))
	{
		lv=(listview*)FindObject(FullName(),".controllergroup.source.folderlist");
		folderlist->spool_folderfunction();
		lv_forMarked(lv,this,(LVWORKCAST)&folder_manager::folder_move);
		folderlist->unspool_folderfunction();
	}
	else if (!strcmp(g->Name(),"movemail"))
	{
		lv=(listview*)FindObject(FullName(),".controllergroup.source.folderlist");
		folderlist->spool_folderfunction();
		lv_forMarked(lv,this,(LVWORKCAST)&folder_manager::folder_movemails);
		folderlist->unspool_folderfunction();
	}
}

void folder_manager::delayWindow_cb(gadget *g,int)
{
	abort_pressed=1;
}
void folder_manager::openDelayWindow(char *t)
{
	int x,y,w,h;
	appwindow *aw=(appwindow*)FindObject(FullName(),".window");
	delaywindow=new appwindow;

	abort_pressed=0;

	aw->Position(x,y);
	aw->Size(w,h);

	delaywindow->InitSize(x,y,150);
	if (!delaywindow->Init())
	{
		delete delaywindow;
		delaywindow=NULL;
		return;
	}
	delaywindow->Adjust(x,y,w,h,ADJUST_CENTER|ADJUST_MIDDLE);
	delaywindow->Title(PROGRAMNAME" mail folder manager working");
	delaywindow->IconTitle("Folder manager");
	delaywindow->SetIcon(f_xicon1.pixmap(),f_xicon1.shadowmap());
	delaywindow->TransientFor(aw);
	delaywindow->ParentClass(this);
	delaywindow->Name("delaywindow");
	delaywindow->Callback(this,(CLOSECALLBACK)&folder_manager::delayWindow_cb);
	delaywindow->Group(readerwin);

	group *g=new group;
	 g->Name("maingroup");
	 g->UseSmallest();
	 g->Center();
	 g->Horiz();
	  text *txt=new text;
	  txt->Weight(60);
	  txt->Text(t);
	 g->Add(txt);
	  xcfx_gadget *fx=new xcfx_gadget;
	  fx->On();
	 g->Add(fx);
	  fbutton *fb=new fbutton;
	  fb->Text("Abort");
	  fb->Weight(10);
	  fb->Callback(this,(CALLBACK)&folder_manager::delayWindow_cb);
	 g->Add(fb);
	delaywindow->SetGroup(g);
	delaywindow->Create();
	aw->Block();
	appwindow::UpdateEvent();
}
void folder_manager::closeDelayWindow(void)
{
	if (delaywindow)
	{
		appwindow *aw=(appwindow*)FindObject(FullName(),".window");
		aw->Unblock();
		delaywindow->Close();
		DeleteObjectTree(delaywindow->FullName());
		delaywindow=NULL;
	}
}

int folder_manager::folder_del_v_of_f(folder *f)
{ // del folder and its vfolder
	int r=1;
	int nothing_found=1;
	vfolder *v2=NULL;
	while (v2=folderlist->getfolder(v2,f->g_foldername()))
	{
		// if v2 only contains f -> remove v2
		if (v2 && v2->allfolders->Count()==1 && ((vfolder_data*)v2->allfolders->head)->f==f)
		{
			nothing_found=0;
			if (!folderlist->remove(f))
			{
				folderlist->remove(v2);
				r=0;
			}
			else
			{
				io_class *io=getiofromlist(0);
				Alert((appwindow*)FindObject(FullName(),".window"),"Can't remove folder '%s':\n%s",f->g_foldername(),io->getlasterror());
			}
			break;
		}
	}
	if (nothing_found)
	{
		if (!folderlist->remove(f))
		{
			r=0;
		}
		else
		{
			io_class *io=getiofromlist(0);
			Alert((appwindow*)FindObject(FullName(),".window"),"Can't remove folder '%s':\n%s",f->g_foldername(),io->getlasterror());
		}
	}
	return r;
}
folder *folder_manager::folder_new2(char *n)
{
	folder *r=NULL;
	io_class *io=getiofromlist(0);
	int l=strlen(n)*2;
	char *buf=new char[l+1];
	folderlist->ckfoldername(io,n,buf,l);

//	printf("create new folder %s (%s) in %s\n",n,buf,io->io_username());

	if (r=folderlist->getfolder(io,buf))
	{ // folder exists
		if (Alert((appwindow*)FindObject(FullName(),".window"),"Folder '%s' exists.|Append|Cancel",n)>1)
			r=NULL;
	}
	else
	{
		folder *f=folderlist->create(io,buf);
		if (f)
		{ // vfolder exists? add else create
			vfolder *v=folderlist->getfolder(n,io->io_name());
			if (!v)
				v=folderlist->create(0,n,io->io_name());
			if (v)
				v->add(f);
			folderlist->saveprefs();
			r=f;
		}
		else
		{ // can't create folder, io->getlasterror()
			Alert((appwindow*)FindObject(FullName(),".window"),"Error! Can't create folder '%s':\n%s",n,io->getlasterror());
			r=NULL;
		}
	}
	return r;
}
void folder_manager::folder_copy(void *d)
{
	folder *f=(folder*)d;
	folder *f2=NULL;
	io_class *io=getiofromlist(0);

	if (io)
	{
		if (f->getio()!=io)
		{
//			printf("copy folder %s to %s\n",f->g_foldername(),io->io_username());

			if (f2=folder_new2(f->g_foldername()))
			{
				mail *m;
				openDelayWindow("Copying mails of folder...");
				while ((m=f->loadnextmail()) && !abort_pressed)
				{ //printf("%x\n",m);
					m->addreferer();
					f2->appendmail(new mail(m));
					m->remreferer();
				}
				f->loadnextmail_abort();
				closeDelayWindow();
			}
		}
	}
}
void folder_manager::folder_move(void *d)
{
	folder *f=(folder*)d;
	folder *f2=NULL;
	io_class *io=getiofromlist(0);

	if (io)
	{
		if (f->getio()!=io)
		{
//			printf("move folder %s to %s\n",f->g_foldername(),io->io_username());

			if (f2=folder_new2(f->g_foldername()))
			{
				mail *m;
				int err=0;
				openDelayWindow("Moving mails of folder...");
				while ((m=f->loadnextmail()) && !abort_pressed)
				{
					m->addreferer();
					err=f2->appendmail(new mail(m))||err;
					m->remreferer();
				}
				f->loadnextmail_abort();

				if (!abort_pressed)
				{
					if (!err)
					{
						folder_del_v_of_f(f);
					}
					else
					{
						Alert((appwindow*)FindObject(FullName(),".window"),"Couldn't write all mail to\nfolder:'%s'\n",f2->g_foldername());
					}
				}
				closeDelayWindow();
			}
		}
	}
}
void folder_manager::folder_movemails(void *d)
{
	folder *f=(folder*)d;
	folder *f2=NULL;
	io_class *io=getiofromlist(0);

	if (io)
	{
		if (f->getio()!=io)
		{
//			printf("move mails of folder %s to %s\n",f->g_foldername(),io->io_username());

			if (f2=folder_new2(f->g_foldername()))
			{
				mail *m;
				int err=0;
				openDelayWindow("Moving mails...");
				while ((m=f->loadnextmail()) && !abort_pressed)
				{
					m->addreferer();
					if (!f2->appendmail(new mail(m)))
						m->deleteLocalMail();
					else
						err=1;
					m->remreferer();
				}
				f->loadnextmail_abort();
				if (!abort_pressed)
				{
					if (!err)
					{
						f->clear();
					}
					else
					{
						Alert((appwindow*)FindObject(FullName(),".window"),"Couldn't write all mail to\nfolder:'%s'\n",f2->g_foldername());
					}

				}
				closeDelayWindow();
			}
		}
	}
}

int folder_manager::folder_new(char *n)
{
	int r=0;
	io_class *io=getiofromlist(1);
	int l=strlen(n)*2;
	char *buf=new char[l+1];
	folderlist->ckfoldername(io,n,buf,l);

//	printf("create new folder %s (%s) in %s\n",n,buf,io->io_username());

	if (folderlist->getfolder(io,buf))
	{ // folder exists
		Alert((appwindow*)FindObject(FullName(),".window"),"Folder '%s' exists.",n);
		r=1;
	}
	else
	{
		folder *f=folderlist->create(io,buf);
		if (f)
		{ // vfolder exists? add else create
			vfolder *v=folderlist->getfolder(n,0);
			if (v)
			{
				vfolder *v=folderlist->getfolder(n,io->io_name());
				if (!v)
					v=folderlist->create(0,n,io->io_name());
			}
			else
				v=folderlist->create(0,n);
			if (v)
				v->add(f);
			folderlist->saveprefs();
		}
		else
		{ // can't create folder, io->getlasterror()
			Alert((appwindow*)FindObject(FullName(),".window"),"Error! Can't create folder '%s':\n%s",n,io->getlasterror());
			r=1;
		}
	}
	return r;
}
int folder_manager::folder_rename(char *n)
{
	int r=1;
	listview *lv=(listview*)FindObject(FullName(),".controllergroup.source.folderlist");
	folder *f=(folder*)lv->GetData(lv->Selected());
//	printf("rename folder %s to %s\n",f->g_foldername(),n);

	io_class *io=getiofromlist(1);
	int l=strlen(n)*2;
	char *buf=new char[l+1];
	folderlist->ckfoldername(io,n,buf,l);

	folder *f2=folderlist->getfolder(io,buf);
	if (f2 && f2!=f)
	{ // folder exists
		Alert((appwindow*)FindObject(FullName(),".window"),"Can't rename folder '%s'.\nFolder '%s' exists.",f->g_foldername(),n);
	}
	else if (!f2)
	{
		vfolder *v1,*v2=NULL;
		int nothing_found=1;
		while (v2=folderlist->getfolder(v2,f->g_foldername()))
		{
			v1=folderlist->getfolder(n,v2?v2->g_subdir():(char*)"");

			// if v2 only contains f -> rename v2 (if no vfolder with the new name exists)

			if (v2 && v2->allfolders->Count()==1 && ((vfolder_data*)v2->allfolders->head)->f==f)
			{
				nothing_found=0;
				if (!v1)
				{
					if (!folderlist->rename(f,buf))
					{
						folderlist->rename(v2,n);
						r=0;
					}
					else
					{
						Alert((appwindow*)FindObject(FullName(),".window"),"Can't rename folder '%s':\n%s",f->g_foldername(),io->getlasterror());
					}
					break;
				}
				else
				{ // vfolder exists
					Alert((appwindow*)FindObject(FullName(),".window"),"There is a vfolder '%s',\nso can't rename.",n);
					break;
				}
			}
		}
		if (nothing_found)
		{
			if (folderlist->rename(f,buf))
				Alert((appwindow*)FindObject(FullName(),".window"),"Can't rename folder '%s':\n%s",f->g_foldername(),io->getlasterror());
			else
				r=0;
		}
	}
	return r;
}
int folder_manager::folder_delete(void *d)
{
	int r=1;
	folder *f=(folder*)d;
	int m=0;

	if (!delete_abort && (delete_all || (m=Alert((appwindow*)FindObject(FullName(),".window"),"Really delete folder '%s'\nand all its content?|_Yes|_All|A_bort|_No",f->g_foldername()))<=2))
	{ //printf("delete folder %s\n",f->g_foldername());
		folder_del_v_of_f(f);
	}
	switch (m)
	{
		case 2:
			delete_all=1;
			break;
		case 3:
			delete_abort=1;
			break;
	}
	return r;
}

void folder_manager::folder_editor(gadget *g,int)
{
	if (!strcmp(g->Name(),"vfolderlist"))
	{
		int not_delete_folder=0;
		listview *lv=(listview*)FindObject(FullName(),".vfolders.vfolderlist");

		if (((vfolder*)lv->GetData(lv->Selected()))==folderlist->currentfolder())
		{
			not_delete_folder=1;
		}

		if (lv->Selected()>0)
		{
			if (programconfig.vfolder_manager)
				((gadget*)FindObject(FullName(),".vfolders.rename"))->Unlock();
			((gadget*)FindObject(FullName(),".folders.vfoldertype"))->Unlock();
			if (not_delete_folder)
			{
				((gadget*)FindObject(FullName(),".vfolders.delete"))->Lock();
				((gadget*)FindObject(FullName(),".folders.folderlist"))->Lock();
				((gadget*)FindObject(FullName(),".folders.hidden"))->Lock();
				((gadget*)FindObject(FullName(),".folders.subdir"))->Lock();
				((listview*)FindObject(FullName(),".folders.folderlist"))->Selected(0);
			}
			else
			{
				if (programconfig.vfolder_manager)
				{
					((gadget*)FindObject(FullName(),".vfolders.delete"))->Unlock();
					((gadget*)FindObject(FullName(),".folders.folderlist"))->Unlock();
				}
				else
					((listview*)FindObject(FullName(),".folders.folderlist"))->Selected(0);

				((gadget*)FindObject(FullName(),".folders.hidden"))->Unlock();
				((gadget*)FindObject(FullName(),".folders.subdir"))->Unlock();
			}
		}
		else
		{
			((gadget*)FindObject(FullName(),".vfolders.rename"))->Lock();
			((gadget*)FindObject(FullName(),".folders.vfoldertype"))->Lock();
			((gadget*)FindObject(FullName(),".vfolders.delete"))->Lock();
			((gadget*)FindObject(FullName(),".folders.folderlist"))->Lock();
			((gadget*)FindObject(FullName(),".folders.hidden"))->Lock();
			((gadget*)FindObject(FullName(),".folders.rec_own"))->Lock();
			((gadget*)FindObject(FullName(),".folders.rec_always"))->Lock();
			((gadget*)FindObject(FullName(),".folders.rec_read"))->Lock();
			((gadget*)FindObject(FullName(),".folders.rec_reply"))->Lock();
			((gadget*)FindObject(FullName(),".folders.rec_forward"))->Lock();
			((gadget*)FindObject(FullName(),".folders.rec_older"))->Lock();
			((gadget*)FindObject(FullName(),".folders.rec_older_days"))->Lock();
			((gadget*)FindObject(FullName(),".folders.subdir"))->Lock();
			((listview*)FindObject(FullName(),".folders.folderlist"))->Selected(0);
		}
		listview *lv2=(listview*)FindObject(FullName(),".allfolders.allfolderlist");
		if (!programconfig.vfolder_manager)
		{
			lv2->Selected(0);
			lv2->Lock();
			((gadget*)FindObject(FullName(),".vfolders.new"))->Lock();
		}
		else
		{
			lv2->Unlock();
			((gadget*)FindObject(FullName(),".vfolders.new"))->Unlock();
		}
		folder_editor(lv2,0);

		if (old_vfolder_selected && old_vfolder_selected!=(vfolder*)lv->GetData(lv->Selected()))
		{
			vfolder *v=old_vfolder_selected;
			char buf[MAXLENGTH];
			old_vfolder_selected=NULL;

			((input*)FindObject(FullName(),".folders.subdir"))->Input(buf);

			folderlist->spool_folderfunction();
			folderlist->change_vfolder_data_hidden(v,((checkbox*)FindObject(FullName(),".folders.hidden"))->State()==PRESSED?1:0);
			folderlist->change_vfolder_data_ftype(v,((choice*)FindObject(FullName(),".folders.vfoldertype"))->Selected());
			folderlist->change_vfolder_data_subdir(v,buf);

			if (v->getftype()==FOLDERTYPE_INBOX)
			{
				v->rec_own=((checkbox*)FindObject(FullName(),".folders.rec_own"))->State()==PRESSED?1:0;
				v->rec_always=((checkbox*)FindObject(FullName(),".folders.rec_always"))->State()==PRESSED?1:0;
				v->rec_read=((checkbox*)FindObject(FullName(),".folders.rec_read"))->State()==PRESSED?1:0;
				v->rec_reply=((checkbox*)FindObject(FullName(),".folders.rec_reply"))->State()==PRESSED?1:0;
				v->rec_forward=((checkbox*)FindObject(FullName(),".folders.rec_forward"))->State()==PRESSED?1:0;
				v->rec_older=((checkbox*)FindObject(FullName(),".folders.rec_older"))->State()==PRESSED?1:0;
				((input*)FindObject(FullName(),".folders.rec_older_days"))->Input(buf);
				v->rec_older_days=atoi(buf);
			}
			else
			{
				v->rec_own=0;
				v->rec_always=0;
				v->rec_read=0;
				v->rec_reply=0;
				v->rec_forward=0;
				v->rec_older=0;
				v->rec_older_days=7;
			}
			if (lv->Selected()>0)
			{
				old_vfolder_selected=(vfolder*)lv->GetData(lv->Selected());
			}
			folderlist->unspool_folderfunction();
		}
		if (lv->Selected()>0)
		{
			char buf[200];
			vfolder *v=(vfolder*)lv->GetData(lv->Selected());
			((choice*)FindObject(FullName(),".folders.vfoldertype"))->Selected(v->getftype());
			((input*)FindObject(FullName(),".folders.subdir"))->Default(v->g_subdir());
			((checkbox*)FindObject(FullName(),".folders.hidden"))->State(v->hidden?PRESSED:RELEASED);

			((checkbox*)FindObject(FullName(),".folders.rec_own"))->State(v->rec_own?PRESSED:RELEASED);
			((checkbox*)FindObject(FullName(),".folders.rec_always"))->State(v->rec_always?PRESSED:RELEASED);
			((checkbox*)FindObject(FullName(),".folders.rec_read"))->State(v->rec_read?PRESSED:RELEASED);
			((checkbox*)FindObject(FullName(),".folders.rec_reply"))->State(v->rec_reply?PRESSED:RELEASED);
			((checkbox*)FindObject(FullName(),".folders.rec_forward"))->State(v->rec_forward?PRESSED:RELEASED);
			((checkbox*)FindObject(FullName(),".folders.rec_older"))->State(v->rec_older?PRESSED:RELEASED);
			xc_snprintf(buf,sizeof(buf),"%d",v->rec_older_days);
			((input*)FindObject(FullName(),".folders.rec_older_days"))->Default(buf);

			if (v->getftype()==FOLDERTYPE_INBOX)
			{
				((gadget*)FindObject(FullName(),".folders.rec_own"))->Unlock();
				folder_editor((gadget*)FindObject(FullName(),".folders.rec_own"),0);
			}
			else
			{
				((gadget*)FindObject(FullName(),".folders.rec_own"))->Lock();
				((gadget*)FindObject(FullName(),".folders.rec_always"))->Lock();
				((gadget*)FindObject(FullName(),".folders.rec_read"))->Lock();
				((gadget*)FindObject(FullName(),".folders.rec_reply"))->Lock();
				((gadget*)FindObject(FullName(),".folders.rec_forward"))->Lock();
				((gadget*)FindObject(FullName(),".folders.rec_older"))->Lock();
				((gadget*)FindObject(FullName(),".folders.rec_older_days"))->Lock();
			}

			listview *lv2=(listview*)FindObject(FullName(),".folders.folderlist");
			int top=lv2->Top();
			int sel=lv2->Selected();
			lv2->RemoveAll();

			vfolder_data *n=(vfolder_data*)v->allfolders->head;
			while (n->next)
			{
				xc_snprintf(buf,sizeof(buf),"%s (%s)",n->f->g_foldername(),n->f->getio()->io_username());
				lv2->AddAtEnd(buf,n->f);
				n=(vfolder_data*)n->next;
			}

			lv2->Top(top);
			lv2->Selected(sel);
			folder_editor(lv2,0);

			old_vfolder_selected=v;
		}
		else
			old_vfolder_selected=NULL;
	}
	else if (!strcmp(g->Name(),"new"))
	{
		askString(g,"Create new vfolder","New vfolder",&folder_manager::vfolder_new);
	}
	else if (!strcmp(g->Name(),"rename"))
	{
		listview *lv=(listview*)FindObject(FullName(),".vfolders.vfolderlist");
		askString(g,"Rename vfolder",((vfolder*)lv->GetData(lv->Selected()))->g_foldername(),&folder_manager::vfolder_rename);
	}
	else if (!strcmp(g->Name(),"delete"))
	{
		listview *lv=(listview*)FindObject(FullName(),".vfolders.vfolderlist");
		old_vfolder_selected=NULL;

		folderlist->spool_folderfunction();
		delete_all=0;
		delete_abort=0;
		lv_forMarked(lv,this,(LVWORKCAST)&folder_manager::vfolder_delete);
		folderlist->unspool_folderfunction();
	}
	else if (!strcmp(g->Name(),"folderlist"))
	{
		listview *lv=(listview*)FindObject(FullName(),".folders.folderlist");
		if (lv->Selected()>0)
		{
			((gadget*)FindObject(FullName(),".folders.remove_folder"))->Unlock();
		}
		else
		{
			((gadget*)FindObject(FullName(),".folders.remove_folder"))->Lock();
		}
		if (lv->Selected()>1)
		{
			((gadget*)FindObject(FullName(),".folders.default_folder"))->Unlock();
		}
		else
		{
			((gadget*)FindObject(FullName(),".folders.default_folder"))->Lock();
		}
	}
	else if (!strcmp(g->Name(),"default_folder"))
	{
		listview *lv=(listview*)FindObject(FullName(),".folders.folderlist");
		vfolder *v=old_vfolder_selected;
		folder *f=(folder*)lv->GetData(lv->Selected());
		vfolder_data *n=(vfolder_data*)v->allfolders->head;
		while (n->next)
		{
			if (n->f==f)
			{
				v->allfolders->Remove(n);
				v->allfolders->AddHead(n);
				break;
			}
			n=(vfolder_data*)n->next;
		}
		listview *lv2=(listview*)FindObject(FullName(),".vfolders.vfolderlist");
		folder_editor(lv2,0);
		lv->Selected(1);
		lv->Top(1);
		folder_editor(lv,0);
	}
	else if (!strcmp(g->Name(),"remove_folder"))
	{
		listview *lv=(listview*)FindObject(FullName(),".folders.folderlist");
		vfolder *v=old_vfolder_selected;
		folder *f=(folder*)lv->GetData(lv->Selected());
		int top=lv->Top();

		folderlist->remove(v,f);

		lv=(listview*)FindObject(FullName(),".vfolders.vfolderlist");
		folder_editor(lv,0);

		lv=(listview*)FindObject(FullName(),".folders.folderlist");
		lv->Selected(0);
		lv->Top(top);
		folder_editor(lv,0);
	}
	else if (!strcmp(g->Name(),"vfoldertype"))
	{
		vfolder *v=old_vfolder_selected;
		folderlist->change_vfolder_data_ftype(v,((choice*)FindObject(FullName(),".folders.vfoldertype"))->Selected());

		if (v->getftype()==FOLDERTYPE_INBOX)
		{
			((gadget*)FindObject(FullName(),".folders.rec_own"))->Unlock();
			folder_editor((gadget*)FindObject(FullName(),".folders.rec_own"),0);
		}
		else
		{
			((gadget*)FindObject(FullName(),".folders.rec_own"))->Lock();
			((gadget*)FindObject(FullName(),".folders.rec_always"))->Lock();
			((gadget*)FindObject(FullName(),".folders.rec_read"))->Lock();
			((gadget*)FindObject(FullName(),".folders.rec_reply"))->Lock();
			((gadget*)FindObject(FullName(),".folders.rec_forward"))->Lock();
			((gadget*)FindObject(FullName(),".folders.rec_older"))->Lock();
			((gadget*)FindObject(FullName(),".folders.rec_older_days"))->Lock();
		}
	}
	else if (!strcmp(g->Name(),"subdir"))
	{
		vfolder *v=old_vfolder_selected;
		char buf[MAXLENGTH];
		((input*)FindObject(FullName(),".folders.subdir"))->Input(buf);
		folderlist->change_vfolder_data_subdir(v,buf);
		((input*)FindObject(FullName(),".folders.subdir"))->Default(v->g_subdir());
	}
	else if (!strcmp(g->Name(),"hidden"))
	{
		vfolder *v=old_vfolder_selected;
		folderlist->change_vfolder_data_hidden(v,((checkbox*)FindObject(FullName(),".folders.hidden"))->State()==PRESSED?1:0);
	}
	else if (!strcmp(g->Name(),"rec_own"))
	{
		vfolder *v=old_vfolder_selected;
		if (((checkbox*)FindObject(FullName(),".folders.rec_own"))->State()==PRESSED)
		{
			if (v) v->rec_own=1;
			((gadget*)FindObject(FullName(),".folders.rec_always"))->Unlock();
			((gadget*)FindObject(FullName(),".folders.rec_read"))->Unlock();
			((gadget*)FindObject(FullName(),".folders.rec_reply"))->Unlock();
			((gadget*)FindObject(FullName(),".folders.rec_forward"))->Unlock();
			((gadget*)FindObject(FullName(),".folders.rec_older"))->Unlock();
			folder_editor((gadget*)FindObject(FullName(),".folders.rec_older"),0);
		}
		else
		{
			if (v) v->rec_own=0;
			((gadget*)FindObject(FullName(),".folders.rec_always"))->Lock();
			((gadget*)FindObject(FullName(),".folders.rec_read"))->Lock();
			((gadget*)FindObject(FullName(),".folders.rec_reply"))->Lock();
			((gadget*)FindObject(FullName(),".folders.rec_forward"))->Lock();
			((gadget*)FindObject(FullName(),".folders.rec_older"))->Lock();
			((gadget*)FindObject(FullName(),".folders.rec_older_days"))->Lock();
		}
	}
	else if (!strcmp(g->Name(),"rec_always"))
	{
		vfolder *v=old_vfolder_selected;
		if (((checkbox*)g)->State()==PRESSED)
			v->rec_always=1;
		else
			v->rec_always=0;
	}
	else if (!strcmp(g->Name(),"rec_read"))
	{
		vfolder *v=old_vfolder_selected;
		if (((checkbox*)g)->State()==PRESSED)
			v->rec_read=1;
		else
			v->rec_read=0;
	}
	else if (!strcmp(g->Name(),"rec_reply"))
	{
		vfolder *v=old_vfolder_selected;
		if (((checkbox*)g)->State()==PRESSED)
			v->rec_reply=1;
		else
			v->rec_reply=0;
	}
	else if (!strcmp(g->Name(),"rec_forward"))
	{
		vfolder *v=old_vfolder_selected;
		if (((checkbox*)g)->State()==PRESSED)
			v->rec_forward=1;
		else
			v->rec_forward=0;
	}
	else if (!strcmp(g->Name(),"rec_older"))
	{
		vfolder *v=old_vfolder_selected;
		if (((checkbox*)g)->State()==PRESSED)
		{
			if (v) v->rec_older=1;
			((gadget*)FindObject(FullName(),".folders.rec_older_days"))->Unlock();
		}
		else
		{
			if (v) v->rec_older=0;
			((gadget*)FindObject(FullName(),".folders.rec_older_days"))->Lock();
		}
	}
	else if (!strcmp(g->Name(),"allfolderlist"))
	{
		int not_delete_folder=0;
		listview *lv=(listview*)FindObject(FullName(),".allfolders.allfolderlist");
		listview *lv2=(listview*)FindObject(FullName(),".vfolders.vfolderlist");

		if (((vfolder*)lv2->GetData(lv2->Selected()))==folderlist->currentfolder())
		{
			not_delete_folder=1;
		}

		if (lv->Selected()>0 && lv2->Selected()>0 && !not_delete_folder)
		{
			((gadget*)FindObject(FullName(),".allfolders.add_into_vfolder"))->Unlock();
			if (g->DoubleClick())
				folder_editor(((gadget*)FindObject(FullName(),".allfolders.add_into_vfolder")),0);
		}
		else
			((gadget*)FindObject(FullName(),".allfolders.add_into_vfolder"))->Lock();
	}
	else if (!strcmp(g->Name(),"add_into_vfolder"))
	{
		listview *lv=(listview*)FindObject(FullName(),".allfolders.allfolderlist");
		folder *f=(folder*)lv->GetData(lv->Selected());
		vfolder *v=old_vfolder_selected;

		folderlist->add(v,f);

		lv->Selected(0);
		folder_editor(lv,0);

		lv=(listview*)FindObject(FullName(),".vfolders.vfolderlist");
		folder_editor(lv,0);

		lv=(listview*)FindObject(FullName(),".folders.folderlist");
		lv->Selected(lv->Count());
		lv->Top(lv->Count());
		folder_editor(lv,0);
	}
}

int folder_manager::vfolder_new(char *name)
{
//	printf("create new vfolder %s\n",name);
	int r=1;

	if (folderlist->getfolder(name))
	{ // vfolder exists
		Alert((appwindow*)FindObject(FullName(),".window"),"Vfolder '%s' exists.",name);
	}
	else
	{
		vfolder *v=folderlist->create(FOLDERTYPE_UNKNOWN,name,"");
		if (v)
		{ 
			folderlist->saveprefs();
			r=0;
		}
		else
		{ // can't create vfolder
			Alert((appwindow*)FindObject(FullName(),".window"),"Error! Can't create vfolder '%s'!\n",name);
		}
	}
	return r;
}
int folder_manager::vfolder_rename(char *name)
{
	int r=1;
	listview *lv=(listview*)FindObject(FullName(),".vfolders.vfolderlist");
	vfolder *v=(vfolder*)lv->GetData(lv->Selected());

//	printf("rename vfolder %s/%s to %s/%s\n",v->g_subdir(),v->g_foldername(),v->g_subdir(),name);

	vfolder *v2=folderlist->getfolder(name,v->g_subdir());
	if (v2 && v2!=v)
	{ // vfolder exists
		Alert((appwindow*)FindObject(FullName(),".window"),"Can't rename vfolder '%s'.\nVfolder '%s' exists.",v->g_foldername(),name);
	}
	else if (!v2)
	{
		if (!folderlist->rename(v,name))
		{
			folderlist->saveprefs();
			r=0;
		}
		else
		{
			Alert((appwindow*)FindObject(FullName(),".window"),"Can't rename folder '%s'!",v->g_foldername());
		}
	}
	return r;
}
int folder_manager::vfolder_delete(void *f)
{
	vfolder *v=(vfolder*)f;
	int m=0;
	int r=1;

//	if (!delete_abort && 
//		(delete_all || 
//		 (m=Alert((appwindow*)FindObject(FullName(),".window"),"Really delete vfolder\n'%s'?|_Yes|_All|A_bort|_No",v->g_foldername()))<=2))

	if (Alert((appwindow*)FindObject(FullName(),".window"),"Really delete vfolder\n'%s'?|_Yes|_No",v->g_foldername())==1)
	{ // printf("delete vfolder %s\n",v->g_foldername());
		if (folderlist->remove(v))
		{
			Alert((appwindow*)FindObject(FullName(),".window"),"Can't remove vfolder '%s'!",v->g_foldername());
		}
		else
		{
			folderlist->saveprefs();
			r=0;
		}
	}
// 	switch (m)
// 	{
// 		case 2:
// 			delete_all=1;
// 			break;
// 		case 3:
// 			delete_abort=1;
// 			break;
// 	}

	return r;
}


void folder_manager::askString(gadget *g,char *title,char *oldstring,int (folder_manager::*call)(char *))
{
	appwindow *a=new appwindow;
	int x,y;

	GetPositionOnRoot(g->display(),g->gg_win(),x,y);
	a->InitSize(x,y,90);
	if (!a->Init())
	{
		delete a;
		return;
	}
	askCall=call;
	a->TransientFor((appwindow*)FindObject(FullName(),".window"));
	a->Name("askstringwindow");
	a->ParentClass(this);
	a->Title(title);
	a->IconTitle(title);

	spacegroup *sg1=new spacegroup;
	group *g1=new group;
	input *ip=new input;
	fbutton *fb1=new fbutton,*fb2=new fbutton;

	sg1->Border(4);
	sg1->Vert();
	sg1->Name(NULL);
	 ip->Name("name");
	 ip->Text(title);
	 ip->MaxChars(100);
	 ip->Default(oldstring);
	 ip->Callback(this,(CALLBACK)&folder_manager::askStringWork);
	sg1->Add(ip);
	sg1->Add(new ruler);
	 g1->Horiz();
	 g1->Name(NULL);
	 g1->DialogMode();
	  fb1->Name("okbutton");
	  fb1->Text("_Ok");
	  fb1->Callback(this,(CALLBACK)&folder_manager::askStringWork);
	 g1->Add(fb1);
	  fb2->Name("cancelbutton");
	  fb2->Text("_Cancel");
	  fb2->Callback(this,(CALLBACK)&folder_manager::closeaskString);
	  fb2->ApplyXKey(XK_Escape);
	 g1->Add(fb2);
	sg1->Add(g1);
	a->Callback(this,(CLOSECALLBACK)&folder_manager::closeaskStringAW);
	a->SetGroup(sg1);
	a->Create();
	ip->ActivateKey();
	((appwindow*)FindObject(FullName(),".window"))->Block();
}
void folder_manager::closeaskString(gadget *g,int i)
{
	appwindow *a=(appwindow*)FindObject(FullName(),".askstringwindow");
	a->Close();
	DeleteObjectTree(a->FullName());
	((appwindow*)FindObject(FullName(),".window"))->Unblock();
}
void folder_manager::closeaskStringAW(appwindow *a)
{
	a->Close();
	DeleteObjectTree(a->FullName());
	((appwindow*)FindObject(FullName(),".window"))->Unblock();
}
void folder_manager::askStringWork(gadget *g,int i)
{
	char buf[110];
	((input*)FindObject(FullName(),".askstringwindow.name"))->Input(buf);
	closeaskString(NULL,0);
	(this->*askCall)(buf);
}


// 				autoeasy_dialog *aed=new autoeasy_dialog;
// 				aed->Text("Sorry, this function is not yet implemented!");
// 				aed->Options("I'll wait|I'll help|I won't come back|Ok");
// 				aed->Start(readerwin);
