/* 
 * This file is part of XCmail and is subject to the 
 * XCmail license agreement
 */

/* $Id: enriched.cxx,v 2.2 1999/01/07 16:09:09 beckerr Exp $ */


#include "../configure.h"

#include <Xclasses/Xclasses.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/wait.h>

#include "enriched.h"
#include "urltools.h"
#include "chartable.h"
#include "../mailer.h"
#include "../helper.h"

#define TAGCMDLEN 10
#define TAGPARLEN 200


//--------------------------------------------------------------------
// enriched

static CharTable chartable;

enriched::enriched(void) : classUrlList(),
							helper(	"enriched",
									"Ralf Becker",
									"text/enriched",
									HL_PDMENU|HL_SETUP,
									"This helper shows enriched text in ASCII.") {
	text = NULL;
	size = 0;
	bufP = NULL;
	urlHelper = NULL;
	cfgsetup();
}

enriched::enriched(enriched *p, char *i, int s) : classUrlList(), helper(p) {
	text = i;
	size = s;
	bufP = NULL;
	cfg=NULL;
}

enriched::~enriched() {
	if (cfg) delete cfg;
	if (bufP) delete[] bufP;
	if (urlHelper) delete urlHelper;
}

class helper *enriched::clone(char *t, int insize) {
	return new enriched(this, t, insize);
}

char *enriched::go(int &newsize) {

	if (bufP) return bufP;

	long i;
	long col = 0;
	char *cP;
	char tagpar[TAGPARLEN+1];
	char tagcmd[TAGCMDLEN+1];
	int  ignore = 0;

	bufP = cP = new char[size+1];
	if (!cP) return cP;
	
	for (i = 0; i < size; i++) {

		unsigned char c = (unsigned char)text[i];

		// Zeichenumsetzungen der Form &...
		// z.b.: &#123 bzw &uuml;
		//
		if (c == '&') {

			c = (unsigned char)text[++i];

			if (c == '#') {

				long j = i;
				char num[5];

				while (text[++j] && isdigit(text[j]))
					;
				num[0] = 0;
				strncat(num, text + i + 1, j - i);
				c = atoi(num);
				i = j;
				
				if (text[j] == ';')
					i--;

			} else {

				long j = i;
				char name[10];

				c = 0;

				while(text[++j]  && text[j] != ';' && isgraph(text[j]))
					;

				name[0] = 0;
				strncat(name, text + i, j - i);

				int tc = chartable.Translate(name);
				if (tc) {
					c = tc;
					i = j;
				} else {
					c = '&';
					i--;
				}
			}
		} else
		//
		// enriched-Tags parsen
		//
		if (c == '<') {

			int t;

			c = 0;
			t = 0;

			// Tag Befehl ermitteln = ersten String im Tag
			//
			while (text[++i] && text[i] != '>' && isgraph(text[i])) 
				if (t < TAGCMDLEN)
					tagcmd[t++] = tolower(text[i]);
			tagcmd[t] = 0;

			// ggf. Tag Parameter ermitten = Rest des Tags
			//
			t = 0;
			if (text[i] != '>') {
				while (text[++i] && text[i] != '>')
					if (t < TAGPARLEN)
						tagpar[t++] = text[i];
			}
			tagpar[t]=0;

			//
			// Die verschiedenen Tags....
			//
			int endtag = (tagcmd[0] == '/');
			if (endtag) strcpy(tagcmd, tagcmd +1);
		
			if (strcmp("param", tagcmd) == 0) {
				if (endtag)	ignore--;
				else		ignore++;
			} else
				c = 0;

		} else
		if (c == '\n' && text[i+1] == c) {
			while (text[i+1] == c) {
				*(cP++) = c;
				i++;
			}
			c = 0;
			col = 0;
		} else
		if (c == '\n' && text[i+1] != c)
			c = ' ';
		
		if (!c || ignore)
			continue;

		//
		// Zeichen anfuegen...
		//
		*(cP++) = c;
		if (c == '\n') {
			col = 0;
		} else
		if (++col > linelength) {
			col = 0;
			for (char *rP = cP - 1; rP > bufP && *rP != '\n'; rP--) {
				col++;
				if (*rP == ' ') {
					*rP = '\n';
					break;
				}
			}
		}
	}
	*cP = 0;
	newsize = cP - bufP + 1;

	urlHelper = new url(&_url, bufP, newsize);
	if (urlHelper) {
		int nnsize = newsize;
		char *tP = urlHelper->go(nnsize);
		if (tP) {
			if (tP != bufP) {
				delete[] bufP;
				bufP = NULL;
			}
			newsize = nnsize;
			return tP;
		}
	}
	return bufP;
}

int enriched::makemenu(class menu *m, int mn) {
	if (urlHelper)
		return urlHelper->makemenu(m, mn);
	return 0;
}

void enriched::menuhandler(class menu *m,int mid) {
	if (urlHelper)
		urlHelper->menuhandler(m, mid);
}
group *enriched::makeconfig(void) {

	group *g = new group;
	g->YDistance(4);
	g->Vert();
		slider *sl=new slider;
		sl->Name("linelength");
		sl->MinValue(40);
		sl->MaxValue(150);
		sl->Value(linelength);
		sl->Horiz();
		sl->HelpText("Linelength, default is 75");
	g->Add(sl,"_Max line length");
	
	return g;
}

void enriched::readconfig(group *) {
	linelength = ((slider*)FindObject(".linelength"))->Value();
}

void enriched::readconfigfile(void) {
	cfg->Read();
}
void enriched::writeconfigfile(void) {
	cfg->Write();
}

void enriched::cfgsetup(void) {

	static configopts cfgitems[] = {
		{"I:enriched-linelength",	(char*)&linelength},
		{NULL, 						NULL}
	};
	cfg = new config("~/.XCmail/helper-preferences-enriched");
	cfg->SetItems(cfgitems);

	linelength = 75;
}

//--------------------------------------------------------------------

char enriched::linelength;

//--------------------------------------------------------------------

static enriched _enriched;
