/* 
 * This file is part of XCmail and is subject to the 
 * XCmail license agreement
 */

/* $Id: html.cxx,v 2.3 2002/03/22 10:35:37 schmitzj Exp $ */


#include "../configure.h"

#include <Xclasses/Xclasses.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/wait.h>

#include "html.h"
#include "urltools.h"
#include "chartable.h"
#include "../mailer.h"
#include "../helper.h"

#define TAGCMDLEN 20
#define TAGPARLEN 200

#define TMPFILE "/tmp/xcmhtml%p.html"

//--------------------------------------------------------------------
// html

static CharTable chartable;

html::html(void) : classUrlList(),
					helper(	"html",
							"Ralf Becker",
							"text/html;text/x-html",
							HL_PDMENU|HL_SETUP,
							"This helper shows HTML in ASCII or within "
							"an external browser or test editor.") {
	text = NULL;
	size = 0;
	pid = 0;
	bufP = NULL;
	baseUrl  = NULL;
	menu1 = menu2 = menu3 = 0;
	sprintf(filebuf, "file:"TMPFILE, (void *)this);
	filepath = filebuf + 5;
	cfgsetup();					// js
}

html::html(html *p, char *i, int s) : classUrlList(), helper(p) {
	text = i;
	size = s;
	pid = 0;
	bufP = NULL;
	baseUrl = NULL;
	menu1 = menu2 = menu3 = 0;
	sprintf(filebuf, "file:"TMPFILE, (void *)this);
	filepath = filebuf + 5;
	cfg=NULL;		// js
}

html::~html() {
	char buf[255];
	int i;
	if (cfg) delete cfg;		// js
	if (bufP) delete[] bufP;
	if (baseUrl) delete[] baseUrl;
	if (pid) wait(&i);
	sprintf(buf, "rm -f %s", filepath);
	system(buf);
}

class helper *html::clone(char *t, int insize) {
	return new html(this, t, insize);
}

char *html::go(int &newsize) {

	if (bufP) return bufP;

	FILE *file;
	long len;
	long i;
	long col = 0;
	char *cP;
	char tagpar[TAGPARLEN+1];
	char tagcmd[TAGCMDLEN+1];
	int  ignore = 0;
	int	 indent = 0;
	int  listing = 0;
	int  blockquote = 0;
	int  pre = 0;

	UrlForgetAll();	// ggf. URLs vergessen

	bufP = cP = new char[size+1];
	if (!cP) return cP;

	// Daten merken (fuer Viewer-Aufruf)
	//
	if (file = fopen(filepath, "w"), file) {
		fwrite(text, size, 1, file);
		fclose(file);
	}
	
	for (i = 0; i < size; i++) {

		unsigned char c = (unsigned char)text[i];

		// Zeichenumsetzungen der Form &...
		// z.b.: &#123 bzw &uuml;
		//
		if (c == '&') {

			c = (unsigned char)text[++i];

			if (c == '#') {

				long j = i;
				char num[5];

				while (text[++j] && isdigit(text[j]))
					;
				num[0] = 0;
				strncat(num, text + i + 1, j - i);
				c = atoi(num);
				i = j;
				
				if (text[j] == ';')
					i--;

			} else {

				long j = i;
				char name[10];

				c = 0;

				while(text[++j]  && text[j] != ';' && isgraph(text[j]) && j-i < 9)
					;

				name[0] = 0;
				strncat(name, text + i, j - i);

				int tc = chartable.Translate(name);
				if (tc) {
					c = tc;
					i = j;
				} else {
					c = '&';
					i--;
				}
			}

		} else
		//
		// HTML-Tags parsen
		//
		if (c == '<') {

			int t;

			c = 0;
			t = 0;

			// Tag Befehl ermitteln = ersten String im Tag
			//
			while (text[++i] && text[i] != '>' && isgraph(text[i])) 
				if (t < TAGCMDLEN)
					tagcmd[t++] = tolower(text[i]);
			tagcmd[t] = 0;

			// ggf. Tag Parameter ermitten = Rest des Tags
			//
			t = 0;
			if (text[i] != '>') {
				while (text[++i] && text[i] != '>')
					if (t < TAGPARLEN)
						tagpar[t++] = text[i];
			}
			tagpar[t]=0;

			//
			// Die verschiedenen Tags....
			//
			int endtag = (tagcmd[0] == '/');
			if (endtag) strcpy(tagcmd, tagcmd +1);

			// <h?> & </h?> -> \n
			//	
			if (tagcmd[0] == 'h' &&
				tagcmd[1] >= '1' && tagcmd[1] <= '9' &&
				tagcmd[2] == 0) {
				c = '\n';
			} else			
			if (strcmp("listing", tagcmd) == 0) {
				listing = !endtag;
				c = '\n';
			} else			
			if (strcmp("blockquote", tagcmd) == 0) {
				blockquote = !endtag;
				if (blockquote) {
					strcpy(cP, "\n\n> ");
					cP += 4;
					col = 2;
				} else {
					strcpy(cP, "\n\n");
					cP += 2;
					col = 0;
				}
			} else			
			if (strcmp("pre", tagcmd) == 0) {
				pre = !endtag;
				c = '\n';
			} else			
			// <li> -> \n
			//
			if (strcmp("li", tagcmd) == 0) {
				if (cP == bufP) {
					strcpy(cP, " - ");
					cP += 3;
					col = 3;
				} else {
					strcpy(cP, "\n - ");
					cP += 4;
					col = 4;
				}
			} else
			// <td> -> \n
			//
			if (strcmp("td", tagcmd) == 0) {
				c = ' ';				
			} else
			// <br> -> \n
			// <tr> -> \n
			//
			if (strcmp("br", tagcmd) == 0 ||
				strcmp("tr", tagcmd) == 0) {
				if (cP != bufP) c = '\n';
				if (blockquote) {
					strcpy(cP, "\n> ");
					col = 2;
					cP += 3;
					c = 0;
				}
			} else
			//
			// <p> -> \n
			//
			if (strcmp("p", tagcmd) == 0) {
				if (cP != bufP) {
					if (blockquote) {
						strcpy(cP, "\n>\n> ");
						cP += 5;
						col = 2;
					} else {
						strcpy(cP, "\n\n");
						cP += 2;
						col = 0;
					}
				}
			} else
			//
			// <hr> -> \n----\n
			//
			if (strcmp("hr", tagcmd) == 0) {
				if (cP == bufP) {
					strcpy(cP, "---\n");
					cP += 4;
				} else {
					strcpy(cP, "\n---\n");
					cP += 5;
				}
				col = 0;
			} else
			//
			// <script ...> -> WIRD BIS </script> IGNORIERT 
			//
			if (strcmp("script", tagcmd) == 0) {
				if (endtag) ignore--;
				else		ignore++;
			} else
			//
			// <head> -> nix
			//
			if (strcmp("head", tagcmd) == 0) {
				if (endtag)	ignore--;
				else		ignore++;
			} else
			if (strcmp("body", tagcmd) == 0) {
				ignore = endtag;
			} else
			//
			// <base .... url=....> -> ermitten der BASE-URL  (nur die erste wird genommen)
			//
			if (strcmp("base", tagcmd) == 0 && !baseUrl) {

				for (int k = 0; tagpar[k];) {
				
					if (strcncmp("href=", tagpar + k) == 0) {
						k += 5;
						int ek = k;
					
						// URL in "...."
						//
						if (tagpar[k] == '"') {
							k++;
							while (tagpar[++ek] && tagpar[ek] != '"')
								;
						} else
						//
						// URL in '....'
						//
						if (tagpar[k] == '\'') {
							k++;
							while (tagpar[++ek] && tagpar[ek] != '\'')
								;
						} else {
						//
						// URL direkt nach dem =
						//
							while (tagpar[++ek] && isgraph(tagpar[ek]))
								;
						}
						
						//
						// ggf. BASE-URL merken
						//
						if (k != ek) {
						
							baseUrl = new char[ek - k + 2];
							
							if (baseUrl) {
								baseUrl[0] = 0;
								strncat(baseUrl, tagpar + k, ek - k);
							}
						}
					} else
						k++;
				}
			} else
			//
			// <a ... href=...> -> URL merken
			//
			if (strcmp("a", tagcmd) == 0) {

				for (int k = 0; tagpar[k];) {
				
					if (strcncmp("href=", tagpar + k) == 0) {
						k += 5;
						int ek = k;
						int absolut = 0;
						
						//
						// URL in "..."
						//
						if (tagpar[k] == '"') {
							k++;
							while (tagpar[++ek] && tagpar[ek] != '"')
								if (tagpar[ek] == ':')
									absolut = 1;
						} else
						//
						// URL in '...'
						//
						if (tagpar[k] == '\'') {
							k++;
							while (tagpar[++ek] && tagpar[ek] != '\'')
								if (tagpar[ek] == ':')
									absolut = 1;
						} else {
						//
						// URL nach dem =
						//
							while (tagpar[++ek] && isgraph(tagpar[ek]))
								if (tagpar[ek] == ':')
									absolut = 1;
						}
						if (k != ek) {
						
							//
							// merken der absolutne URL
							//
							if (absolut) {
								UrlInsert(tagpar + k, ek-k);
							} else
							//
							// relative URL -> wird an BASE-URL angehangen
							//
							if (baseUrl) {
								
								char s[strlen(baseUrl) + ek - k + 6];
								
 								if (s) {
 									strcpy(s, baseUrl);
 									if (s[strlen(s)-1] != '/')
 										if (strrchr(s, '/'))
 											strrchr(s, '/')[1] = 0;
 									if (tagpar[k] == '/') k++;
 									strncat(s, tagpar + k, ek - k);
 									UrlInsert(s);
 								}
							}
						}
					} else
						k++;
				}
			
			} else
				c = ' ';

		} else
		//
		// DOS-Umbruch -> ' '
		//
		if (c == '\r' && text[i + 1] == '\n') {
			i++;
			if (listing || pre)
				c = '\n';
			else c = ' ';
		} else
		//
		// UNIX, MAC und TAB -> ' '
		//
		if (c == '\n' || c == '\r') {
			if (listing || pre)
				c = '\n';
			else c = ' ';
		} else
		if (c == '\t') {
			c = ' ';
		}

		//
		// Zeichen anfuegen...
		//
		if (c && ignore == 0) {

			int ok=0;
			if (c != ' ')
				ok = 1;
			else
			if (cP > bufP && (isgraph(cP[-1]) || (unsigned char)cP[-1] >= 127))
				ok = 1;

			if (ok) {
			
				*(cP++) = c;
				if (c == '\n') {
					col = 0;
				} else {
					col++;
					if (col > linelength) {
						col = 0;
						for (char *rP = cP - 1; rP > bufP && *rP != '\n'; rP--) {
							col++;
							if (*rP == ' ') {
								*rP = '\n';
								break;
							}
						}
					}
				}
			}
		}
	}
	*cP = 0;
	newsize = cP - bufP + 1;
	
	for (cP = bufP; *cP; cP++)
		if ((unsigned char)*cP == NBSP) *cP = ' ';
		
	return bufP;
}

int html::makemenu(class menu *m, int mn) {

	int mailto = 0;
	int url = baseUrl != NULL;
	
	for (int i = 0 ; i < UrlCount() && (mailto == 0 || url == 0); i++) {
		if (strcncmp("mailto:", UrlByIndex(i)) == 0)
			mailto = 1;
		else
			url = 1;
	}
	
	if (mailto) {
		m->AddMenu("Mailto");
		menu1 = mn++;

		for (int i = 0 ; i < UrlCount(); i++)
			if (strcncmp("mailto:", UrlByIndex(i)) == 0) {
				char *c = strchr(UrlByIndex(i), ':');
				if (c) m->Add(c+1, i+2);
			}
	}
	
	if (url) {

		m->AddMenu("Browse URL");
		menu2 = mn++;
	
		if (baseUrl) {
			m->Add(baseUrl, 1);
			if (UrlCount())
				m->AddSub("Inlined URLs");
		}
		for (int i = 0 ; i < UrlCount(); i++)
			if (strcncmp("mailto:", UrlByIndex(i)) != 0)
				m->Add(UrlByIndex(i), i+2);
		
		if (baseUrl && UrlCount())
			m->EndSub();
	}
	
	m->AddMenu("Viewer");
	menu3 = mn++;
	m->Add("Web-Browser...", 1);
	m->Add("HTML-Editor...", 2);

	return 1;
}

void html::NetscapeOpen(class menu *m, char *url) {

	int i;

	if (pid) wait(&i);

	fcntl(ConnectionNumber(m->display()),F_SETFD,1);

	pid = fork();
	
	if (pid == 0) {
	
		char buffer[256];

		sprintf(buffer, browser_open, url);
		if (system(buffer) != 0) {
			sprintf(buffer, browser_start, url);
			strcat(buffer, " &");
			system(buffer);
		}
		_exit(127);
	}
}

void html::OtherOpen(class menu *m, char *cmd) {

	int i;

	if (pid) wait(&i);

	fcntl(ConnectionNumber(m->display()),F_SETFD,1);

	if (pid = fork(), pid == 0) {
		char buffer[256];
		sprintf(buffer, cmd,  filepath);
		strcat(buffer, " &");
		system(buffer);
		_exit(127);
	}
}

void html::menuhandler(class menu *m,int mid) {

	if (m->Menu() == menu3) {

		switch(mid) {

			case 1:
				NetscapeOpen(m, filebuf);
				break;
			case 2:
				OtherOpen(m, editor);
				break;
			default:
				break;
		}
		return;
	}

	if (m->Menu() != menu1 && m->Menu() != menu2)
		return;

	char *url = mid == 1 ? baseUrl : UrlByIndex(mid-2);

	if (strcncmp("mailto:", url) == 0) {

		newmail *nm = new newmail;
    	pstatus("Open create window");

		char *c = strchr(url, ':');
		if (c) url = c + 1;
		nm->addTo(url);

		input *txt=((input *)FindObject(nm, ".subject"));
		if (txt)
			txt->ActivateKey();

	} else
		NetscapeOpen(m, url);
}


group *html::makeconfig(void) {

	fileinput *fip;
	group *g = new group;
	
	g->YDistance(4);
	g->Vert();
		fip = new fileinput;
		fip->Name("html_browser_open");
		fip->ModeCommand();
		fip->SetFile(browser_open);
		fip->HelpText("Comand to open a given file in a running HTML Browser\ndefault is \"netscape -remote 'OpenUrl(%s)'\"");
	g->Add(fip,"_Open browser");
		fip = new fileinput;
	 	fip->Name("html_browser_start");
	 	fip->ModeCommand();
	 	fip->SetFile(browser_start);
		fip->HelpText("Comand to start an new HTML Browser with a given file\ndefault is \"netscape %s\"");
	g->Add(fip,"_Start browser");
		fip = new fileinput;
		fip->Name("html_editor");
		fip->ModeCommand();
		fip->SetFile(editor);
		fip->HelpText("Comand to open a given file with a HTML/file editor\ndefault is \"xterm -e vi %s\"");
	g->Add(fip,"_HTML editor");
		slider *sl=new slider;
		sl->Name("html_linelength");
		sl->MinValue(40);
		sl->MaxValue(150);
		sl->Value(linelength);
		sl->Horiz();
		sl->HelpText("Linelength, default is 75");
	g->Add(sl,"_Max line length");
	
	return g;
}

void html::readconfig(group *g) {
	((fileinput*)FindObject(g->FullName(), ".html_browser_open"))->GetFile(browser_open, sizeof(browser_open));
	((fileinput*)FindObject(g->FullName(), ".html_browser_start"))->GetFile(browser_start, sizeof(browser_start));
	((fileinput*)FindObject(g->FullName(), ".html_editor"))->GetFile(editor, sizeof(editor));
	linelength = ((slider*)FindObject(".html_linelength"))->Value();
}

void html::readconfigfile(void) {
	cfg->Read();
}
void html::writeconfigfile(void) {
	cfg->Write();
}

void html::cfgsetup(void) {

	static configopts cfgitems[] = {
		{"S80:html-browser-open",	browser_open},
		{"S80:html-browser-start",	browser_start},
		{"S80:html-editor",			editor},
		{"I:html-linelength",		(char*)&linelength},
		{NULL, 						NULL}
	};
	cfg = new config("~/.XCmail/helper-preferences-html");
	cfg->SetItems(cfgitems);

	strcpy(editor, "xterm -e vi '%s'");
	strcpy(browser_open, "netscape -remote 'OpenUrl(%s)'");
	strcpy(browser_start, "netscape '%s'");
	linelength = 75;
}

//--------------------------------------------------------------------

char html::browser_open[128];
char html::browser_start[128];
char html::editor[128];
char html::linelength;

//--------------------------------------------------------------------

static html _html;
