/* 
 * This file is part of XCmail and is subject to the 
 * XCmail license agreement
 */

/* $Id: pop3size.cxx,v 2.2 2002/09/09 13:26:29 schmitzj Exp $ */


#include "../configure.h"

#include <Xclasses/Xclasses.h>
#include <string.h>

#include "pop3size.h"
#include "../mailer.h"
#include "../helper.h"
#include "../stringTokenizer.h"
#include "../npop3get.h"

pop3size::pop3size(void) : callable(), helper("pop3size","Jrgen Schmitz","text/x-xcmail-pop3size",HL_GADGETS,
	"Handles big POP3 mails"
	)
{
	text=NULL;
	len=0;
	uidl=NULL;
	host=NULL;
	hip=NULL;
	user=NULL;
	size=0;
	maxsize=0;
	gp=NULL;
}

pop3size::pop3size(pop3size *p, char *i, int s) : callable(), helper(p)
{
	text=i;
	len=s;
	uidl=NULL;
	host=NULL;
	hip=NULL;
	user=NULL;
	size=0;
	maxsize=0;
	gp=NULL;

	tokenizer lines(text,"\n","");
	for(int i=0;i<lines.count();i++)
	{
		tokenizer t((char*)lines[i],":","");
		if (!strcscmp((char*)t[0],"UIDL"))
			uidl=strdup((char*)t[1]);
		else if (!strcscmp((char*)t[0],"HOST"))
			host=strdup((char*)t[1]);
		else if (!strcscmp((char*)t[0],"HIP"))
			hip=strdup((char*)t[1]);
		else if (!strcscmp((char*)t[0],"USER"))
			user=strdup((char*)t[1]);
		else if (!strcscmp((char*)t[0],"SIZE"))
			size=atoi((char*)t[1]);
		else if (!strcscmp((char*)t[0],"MAXSIZE"))
			maxsize=atoi((char*)t[1]);
	}
}

pop3size::~pop3size()
{
	if (uidl) free(uidl);
	if (hip) free(hip);
	if (user) free(user);
}

class helper *pop3size::clone(char *t, int insize)
{
	return new pop3size(this, t, insize);
}
void pop3size::cb_add(gadget *g,int)	// retrive
{
	UIDLhistory *retr_uidlhist=new UIDLhistory(host,user,RETRIEVEFILE);
	retr_uidlhist->Read();
	g->Lock();
	((gadget*)FindObject(gp->FullName(),".rem_mail"))->Unlock();
	retr_uidlhist->Add(uidl);
	retr_uidlhist->Write();
	((class mail*)mail)->deleteLocalMail();
	updateoutput((class mail*)mail);
	delete retr_uidlhist;
}
void pop3size::cb_del(gadget *g,int)	// del on server
{
	UIDLhistory *dele_uidlhist=new UIDLhistory(host,user,REMOVEFILE);
	dele_uidlhist->Read();
	g->Lock();
	dele_uidlhist->Add(uidl);
	dele_uidlhist->Write();
	((class mail*)mail)->deleteServerMail();
	updateoutput((class mail*)mail);
	delete dele_uidlhist;
}
void pop3size::cb_rem(gadget *g,int)	// don't retrive (rem from list)
{
	UIDLhistory *retr_uidlhist=new UIDLhistory(host,user,RETRIEVEFILE);
	retr_uidlhist->Read();
	g->Lock();
	((gadget*)FindObject(gp->FullName(),".add_mail"))->Unlock();
	retr_uidlhist->Remove(uidl);
	retr_uidlhist->Write();
	delete retr_uidlhist;
}

int pop3size::makegadget(visible *v)
{
	UIDLhistory *dele_uidlhist=new UIDLhistory(host,user,REMOVEFILE);
	UIDLhistory *retr_uidlhist=new UIDLhistory(host,user,RETRIEVEFILE);

	dele_uidlhist->Read();
	retr_uidlhist->Read();

	char buf[20];
	gp=new group;
	gp->Name("pop3size_group");
	gp->Vert();
	gp->Add(new ruler);
	 Xclasses_text *txt=new Xclasses_text;
	 txt->Font(FONT_BIG);
	 txt->Text("This mail was bigger than the specified maxsize. To retrive it\n"
	           "durring next POP3 connection press the button below.\n");
	gp->Add(txt,"");
	 output *op=new output;
	 op->Text("Host");
	 op->Output(host);
	gp->Add(op);
	 op=new output;
	 op->Text("User");
	 op->Output(user);
	gp->Add(op);
	 op=new output;
	 op->Text("Size");
	 xc_snprintf(buf,sizeof(buf),"%d",size);
	 op->Output(buf);
	gp->Add(op);
	 op=new output;
	 op->Text("Maxsize");
	 xc_snprintf(buf,sizeof(buf),"%d",maxsize);
	 op->Output(buf);
	gp->Add(op);
	gp->Add(new ruler);
	 group *g1=new group;
	 g1->Name(NULL);
	 g1->Horiz();
	 g1->DialogMode();
	  button *b=new button;
	  b->Name("add_mail");
	  b->Text("retrive");
	  b->Callback(this,(CALLBACK)&pop3size::cb_add);
	  if (retr_uidlhist->Scan(uidl)) b->Lock();
	 g1->Add(b);
	  b=new button;
	  b->Name("del_mail");
	  b->Text("remove on server");
	  b->Callback(this,(CALLBACK)&pop3size::cb_del);
	  if (dele_uidlhist->Scan(uidl)) b->Lock();
	 g1->Add(b);
	  b=new button;
	  b->Name("rem_mail");
	  b->Text("don't retrive");
	  b->Callback(this,(CALLBACK)&pop3size::cb_rem);
	  if (!retr_uidlhist->Scan(uidl)) b->Lock();
	 g1->Add(b);
	gp->Add(g1);
	v->Apply(gp);

	delete dele_uidlhist;
	delete retr_uidlhist;

	return 1;
}

char *pop3size::go(int &newsize)
{
	return text;
}

//--------------------------------------------------------------------

pop3size _pop3size;
