/* 
 * This file is part of XCmail and is subject to the 
 * XCmail license agreement
 */

/* $Id: url.cxx,v 2.4 2002/03/22 10:35:37 schmitzj Exp $ */


#include "../configure.h"

#include <Xclasses/Xclasses.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/wait.h>

#include "url.h"
#include "urltools.h"
#include "../mailer.h"
#include "../helper.h"

#define TAGCMDLEN		10
#define TAGPARLEN		200

#define URLDELIMITER	"<>[](){}\"' \t"


//--------------------------------------------------------------------
// url


url::url(void) : classUrlList(), helper("url","Ralf Becker","text/*",HL_PDMENU|HL_SETUP,
	"Extracts URLs to a pull down menu. Usefull for all text mails without special helpers."
	) {
	text = NULL;
	size = 0;
	pid = 0;
	done = 0;
	menu1 = menu2 = 0;
	cfgsetup();					// js
}

url::url(url *p, char *i, int s) : classUrlList(), helper(p) {
	text = i;
	size = s;
	pid = 0;
	done = 0;
	menu1 = menu2 = 0;
	cfg=NULL;		// js
}

url::~url() {
	int i;
	if (cfg) delete cfg;		// js
	if (pid) wait(&i);
}

class helper *url::clone(char *t, int insize) {
	return new url(this, t, insize);
}

char *url::go(int &newsize) {

	struct {
		char* pattern;
		int   found;
	} lookup[] = {
		{ "@",			0}, // immer 0
		{ "mailto:",	0},
		{ "http:",		0},
		{ "https:",		0},
		{ "ftp:",		0},
		{ "gopher:",	0},
		{ "file:",		0},
		{ "telnet:",	0},
		{ "whais:",		0},
		{ "news:",		0},
		{ "about:",		0},
		{ NULL } };

	char urldelim[] = URLDELIMITER;

	UrlForgetAll();	// ggf. URLs vergessen
		
	for (int i = 0; i == 0 || text[i-1] ; i++) {
	
		char c = text[i];
	
		for (int j = 0; lookup[j].pattern; j++) {

			if (!isgraph(c) || strchr(urldelim, c)) {
				lookup[j].found = 0;
			} else
			if (c == lookup[j].pattern[lookup[j].found]) {

				if (lookup[j].pattern[++lookup[j].found] == 0) {

					char *cP, *eP;
					
					eP = text + i;

					while (isgraph(*eP) && !strchr(urldelim, *eP))
						eP++;
					
					if (j == 0) { 			// Mailadresse gefunden!
						
						cP = text + i;
						
						while (cP > text &&
							(isalnum(cP[-1]) || strchr("._", cP[-1])))
							cP--;
							
						if (eP-cP > 3) {	// mailto: anhaengen und speichern
							char buf[eP-cP + strlen("mailto:") + 1];
							strcpy(buf, "mailto:");
							strncat(buf, cP, eP-cP);
							UrlInsert(buf);
						}
					} else {
						cP = text + i - strlen(lookup[j].pattern) + 1;

						if (eP-cP > strlen(lookup[j].pattern))
							UrlInsert(cP, eP-cP);
					}

					for (int k = 0; lookup[k].pattern; k++)
						lookup[k].found = 0;
						
					i = eP-text-1;
				}
			} else
				lookup[j].found = 0;
		}
	}
	newsize = size;
	done = 1;	
	return text;
}

int url::makemenu(class menu *m, int mn) {

	int mailto = 0;
	int url = 0;
	
	for (int i = 0 ; i < UrlCount() && (mailto == 0 || url == 0); i++) {
		if (strcncmp("mailto:", UrlByIndex(i)) == 0)
			mailto = 1;
		else
			url = 1;
	}
	
	if (mailto) {
	
		m->AddMenu("Mailto");
		menu1 = mn++;

		for (int i = 0 ; i < UrlCount(); i++)
			if (strcncmp("mailto:", UrlByIndex(i)) == 0) {
				char *c = strchr(UrlByIndex(i), ':');
				if (c) m->Add(c+1, i+1);
			}
	}
	
	if (url) {

		m->AddMenu("Browse URL");
		menu2 = mn++;
	
		for (int i = 0 ; i < UrlCount(); i++)
			if (strcncmp("mailto:", UrlByIndex(i)) != 0)
				m->Add(UrlByIndex(i), i+1);
	}

		
	return 1;
}

void url::menuhandler(class menu *m, int mid) {

	if (m->Menu() != menu1 && m->Menu() != menu2)
		return;

	char *url = UrlByIndexCleaned(mid-1);
	if (!url) return;
	
	if (strcncmp("mailto:", url) == 0) {

		newmail *nm = new newmail;
		input *txt;

    	pstatus("Open create window");

		nm->addTo(url+7);
		txt=((input *)FindObject(nm, ".subject"));
		if (txt)
			txt->ActivateKey();

	} else {
	
		fcntl(ConnectionNumber(m->display()),F_SETFD,1);

		pid = fork();
	
		if (pid == 0) {
	
			char buffer[256];

			sprintf(buffer, browser_open, url);
			if (system(buffer) != 0) {
				sprintf(buffer, browser_start, url);
				strcat(buffer, " &");
				system(buffer);
			}
			_exit(127);
		}
	}
	delete [] url;
}

group *url::makeconfig(void) {

	fileinput *fip;
	group *g = new group;
	
	g->YDistance(4);
	g->Vert();
		fip = new fileinput;
		fip->Name("url_browser_open");
		fip->ModeCommand();
		fip->SetFile(browser_open);
		fip->HelpText("Comand to open a given file in a running HTML Browser\ndefault is \"netscape -remote 'OpenUrl(%s)'\"");
	g->Add(fip,"_Open browser");
		fip = new fileinput;
	 	fip->Name("url_browser_start");
	 	fip->ModeCommand();
	 	fip->SetFile(browser_start);
		fip->HelpText("Comand to start an new HTML Browser with a given file\ndefault is \"netscape %s\"");
	g->Add(fip,"_Start browser");
	return g;
}

void url::readconfig(group *g) {
	((fileinput*)FindObject(g->FullName(), ".url_browser_open"))->GetFile(browser_open, sizeof(browser_open));
	((fileinput*)FindObject(g->FullName(), ".url_browser_start"))->GetFile(browser_start, sizeof(browser_start));
}

void url::readconfigfile(void) {
	cfg->Read();
}
void url::writeconfigfile(void) {
	cfg->Write();
}

void url::cfgsetup(void) {

	static configopts cfgitems[] = {
		{"S80:url-browser-open",	browser_open},
		{"S80:url-browser-start",	browser_start},
		{NULL, 						NULL}
	};
	cfg = new config("~/.XCmail/helper-preferences-url");
	cfg->SetItems(cfgitems);

	strcpy(browser_open, "netscape -remote 'OpenUrl(%s)'");
	strcpy(browser_start, "netscape '%s'");
}

//--------------------------------------------------------------------

char url::browser_open[128];
char url::browser_start[128];

//--------------------------------------------------------------------

url _url;
