/* 
 * This file is part of XCmail and is subject to the 
 * XCmail license agreement
 */

/* $Id: vcard.cxx,v 1.13 2002/01/07 16:20:53 schmitzj Exp $ */


#include "../configure.h"

#include <Xclasses/Xclasses.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/wait.h>

#include "vcard.h"
#include "../mailer.h"
#include "../helper.h"
#include "../stringTokenizer.h"

vcard::vcard(void) : callable(), helper("vcard","Jrgen Schmitz","text/x-vcard",HL_GADGETS,
	"Extracts vcards to addressbook."
	)
{
	text = NULL;
	size = 0;
	addr=NULL;
	mygp=NULL;
}

vcard::vcard(vcard *p, char *i, int s) : callable(), helper(p)
{
	text = i;
	size = s;
	addr=NULL;
	mygp=NULL;
}

vcard::~vcard()
{
	delete addr;
	if (mygp)
		DeleteObjectTree(mygp->FullName());
}

class helper *vcard::clone(char *t, int insize)
{
	return new vcard(this, t, insize);
}

static char *ignstr(char *s, char *p)
{
	if (strcncmp(p,s)==0)
	{
		return s+strlen(p);
	}
	return s;
}
void vcard::parse(void)
{
	int is_vcard=0;
	delete addr;
	addr=new Address;

	tokenizer tline(text,"\r\n"," \t");
	xcm_strbuf note;

	for (int i=0; i<tline.count(); i++)
	{
		char *tdef0=tline[i],*tdef1=tline[i];
		while (*tdef1 && *tdef1!=':') tdef1++;
		*tdef1=0;
		tdef1++;
		tokenizer tvar(tdef0,";"," \t");
		tokenizer tval(tdef1,";"," \t");

		if (strcncmp("begin",tvar[0])==0 && strcncmp("vcard",tval[0])==0)
		{
			is_vcard=1;
		}
		else if (strcncmp("end",tvar[0])==0 && strcncmp("vcard",tval[0])==0)
		{
			is_vcard=0;
		}
		
		if (!is_vcard) continue;

		if (!strcscmp("tel",tvar[0]))
		{
			if (!strcscmp("fax",tvar[1]))
			{
				addr->setField(AddressBook::FAX,tval[0]);
			}
			else if (!strcscmp("home",tvar[1]))
			{
				addr->setField(AddressBook::HOMEPHONE,tval[0]);
			}
			else if (!strcscmp("work",tvar[1]))
			{
				addr->setField(AddressBook::WORKPHONE,tval[0]);
			}
			else if (!strcscmp("pager",tvar[1]))
			{
				note+="Pager: ";
				note+=tval[0];
				note+="\n";
			}
			else if (!strcscmp("cell",tvar[1]))
			{
				addr->setField(AddressBook::MOBIL,tval[0]);
			}
		}
		else if (!strcscmp("url",tvar[0]))
		{
			addr->setField(AddressBook::URL,tval[0]);
		}
		else if (!strcscmp("n",tvar[0]))
		{
			addr->setField(AddressBook::LASTNAME,Translate728Line(tval[0]));
			addr->setField(AddressBook::FIRSTNAME,Translate728Line(tval[1]));
			addr->setField(AddressBook::ALIAS,tval.list("_"));
		}
		else if (!strcscmp("org",tvar[0]))
		{
			addr->setField(AddressBook::ORGANISATION,tval[0]);
			note+="Department: ";
			note+=tval[1];
			note+="\n";
		}
		else if (!strcscmp("adr",tvar[0]))
		{
			addr->setField(AddressBook::ADDRESS,tval[2]);
			addr->setField(AddressBook::CITY,tval[3]);
			addr->setField(AddressBook::STATE,tval[4]);
			addr->setField(AddressBook::ZIP,tval[5]);
			addr->setField(AddressBook::COUNTRY,tval[6]);
		}
		else if (!strcscmp("email",tvar[0]))
		{
			if (!strcmp("internet",tvar[1]))
			{
				addr->setField(AddressBook::EMAIL,tval[0]);
			}
		}
		else if (!strcscmp("title",tvar[0]))
		{
			note+="Title: ";
			note+=tval[0];
			note+="\n";
		}
		else if (!strcscmp("note",tvar[0]))
		{
			note+="Note: ";
			note+=tval[0];
			note+="\n";
		}
	}
	addr->setField(AddressBook::NOTE,note);
	if (mail)
	{
		addr->setField(AddressBook::XFACE,((class mail*)mail)->xface());
	}
}
void vcard::addbook(gadget *,int)
{
	addressbookupdate(addr);
}

int vcard::makegadget(visible *v)
{
	parse();
	group *gp=new group;
	gp->Name("vcard_group");
	gp->Vert();
	gp->Add(new ruler);
	 group *g1=new group;
	 g1->Horiz();
	  group *g2=new group;
	  g2->Vert();
	   output *op=new output;
	   op->Text("Name");
	   op->Output(addr->getField(AddressBook::LASTNAME));
	  g2->Add(op);
	   op=new output;
	   op->Text("First name");
	   op->Output(addr->getField(AddressBook::FIRSTNAME));
	  g2->Add(op);
	   op=new output;
	   op->Text("EMail");
	   op->Output(addr->getField(AddressBook::EMAIL));
	  g2->Add(op);
	   op=new output;
	   op->Text("URL");
	   op->Output(addr->getField(AddressBook::URL));
	  g2->Add(op);
	   op=new output;
	   op->Text("Organisation");
	   op->Output(addr->getField(AddressBook::ORGANISATION));
	  g2->Add(op);
	 g1->Add(g2);
	  g2=new group;
	  g2->Vert();
	   op=new output;
	   op->Text("Home");
	   op->Output(addr->getField(AddressBook::HOMEPHONE));
	  g2->Add(op);
	   op=new output;
	   op->Text("Cellular");
	   op->Output(addr->getField(AddressBook::MOBIL));
	  g2->Add(op);
	   op=new output;
	   op->Text("Work");
	   op->Output(addr->getField(AddressBook::WORKPHONE));
	  g2->Add(op);
	   op=new output;
	   op->Text("Fax");
	   op->Output(addr->getField(AddressBook::FAX));
	  g2->Add(op);
	   op=new output;
	   op->Text("Address");
	   op->Output(addr->getField(AddressBook::ADDRESS));
	  g2->Add(op);
	   op=new output;
	   op->Text("City");
	   op->Output(addr->getField(AddressBook::CITY));
	  g2->Add(op);
	 g1->Add(g2);
	gp->Add(g1);
	gp->Add(new ruler);
	 g1=new group;
	 g1->Horiz();
	 g1->DialogMode();
	  fbutton *fb=new fbutton;
	  fb->Text("add to addressbook");
	  fb->Callback(this,(CALLBACK)&vcard::addbook);
	 g1->Add(fb);
	gp->Add(g1);
	v->Apply(gp);
	mygp=gp;
	return 1;
}

char *vcard::go(int &newsize)
{
	return text;
}

//--------------------------------------------------------------------

vcard _vcard;
