/* 
 * This file is part of XCmail and is subject to the 
 * XCmail license agreement
 */

/* $Id: mail_body.cxx,v 2.57 2002/05/13 13:00:15 schmitzj Exp $ */


#include "configure.h"

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <ctype.h>
#if TIME_WITH_SYS_TIME
# include <sys/time.h>
# include <time.h>
#else
# if HAVE_SYS_TIME_H
#  include <sys/time.h>
# else
#  include <time.h>
# endif
#endif
#include <utime.h>

#include "mailer.h"
#include "encoder.h"
#include "helper.h"
#include "xc_snprintf.h"
#include "stringTokenizer.h"
#include "extraggs.h"
#include "folder.h"
#include "npop3get.h"

char *normalize_subject(mail *in, char *out,int maxsize, int addre=1);

#define SAVEPTR(x)	(x?x:"")

static int strlcmp(char *t1, char *t2)
{
	return strcncmp(t1,t2);
}

mail::mail(char *txt,class io_id_data *id,int pt)
{
	init(txt,id,pt);
}
mail::mail(mail *m)
{
	init(strdup(m->mailtext),NULL,m->partial);

	parent=m->parent;
	vparent=m->vparent;
	haschanged=m->haschanged;
	status(m->status());
}
void mail::init(char *txt,class io_id_data *id,int pt)
{
	int done=0;
	txt=(char*)realloc(txt,strlen(txt)+1);
	char *c=txt;
	parent=NULL;
	vparent=NULL;
	mailtext=txt;
	haschanged=0;
	partial=pt;

	_id=id;

	smallmode=0;

	fromptr=NULL;
	fromtext=NULL;
	toptr=NULL;
	totext=NULL;
	subjectptr=NULL;
	subjecttext=NULL;
	returnptr=NULL;
	returntext=NULL;
	replytoptr=NULL;
	replytotext=NULL;
	statusptr=NULL;
	statustext=NULL;
	xstatusptr=NULL;
	xstatustext=NULL;
	fromemailtext=NULL;
	dateptr=NULL;
	datetext=NULL;
	ccptr=NULL;
	cctext=NULL;
	mimeversionptr=NULL;
	mimeversiontext=NULL;
	contenttypeptr=NULL;
	contenttypetext=NULL;
	contentencodingptr=NULL;
	contentencodingtext=NULL;
	messageidptr=NULL;
	messageidtext=NULL;
	bodyptr=NULL;
	bodytext=NULL;
	xuidlptr=NULL;
	xuidltext=NULL;
	xfaceptr=NULL;
	xfacetext=NULL;
	savedateptr=NULL;
	savedatetext=NULL;
	priorityptr=NULL;
	prioritytext=NULL;
	oldfolderptr=NULL;
	oldfoldertext=NULL;
	pop3cfgptr=NULL;
	pop3cfgtext=NULL;
	pop3hostptr=NULL;
	pop3hosttext=NULL;
	pop3userptr=NULL;
	pop3usertext=NULL;

	sortPriority=0;
	visualBellValue=0;

	statusval=0;
	lvlinetext[0]=NULL;
	lvlinetext[1]=NULL;
	lvlinetext[2]=NULL;
	lvlinetext[3]=NULL;
	headertext=NULL;
	tagtext=NULL;

	pgpmode=0;
	subm=NULL;
	mail_t=0;
	mail_day=0;
	mail_month=0;
	mail_year=0;
	mail_hour=0;
	mail_min=0;
	mail_sec=0;

	priorityValue=-1;

	totalmulti=0;
	referers=0;

	while (*c && !done)
	{
		if (!fromptr && !strlcmp("From:",c))
		{
			fromptr=c+5;
		}
		else if (!toptr && !strlcmp("To:",c))
		{
			toptr=c+3;
		}
		else if (!ccptr && (!strlcmp("CC:",c) || !strlcmp("Cc:",c)))
		{
			ccptr=c+3;
		}
		else if (!statusptr && !strlcmp("Status:",c))
		{
			statusptr=c;
		}
		else if (!statusptr && !strlcmp("X-"PROGRAMNAME"-Status:",c))
		{
			xstatusptr=c;
		}
		else if (!returnptr && !strlcmp("Return-Path:",c))
		{
			returnptr=c+12;
		}
		else if (!replytoptr && !strlcmp("Reply-To:",c))
		{
			replytoptr=c+9;
		}
		else if (!subjectptr && !strlcmp("Subject:",c))
		{
			subjectptr=c+8;
		}
		else if (!dateptr && !strlcmp("Date:",c))
		{
			dateptr=c+5;
		}
		else if (!xfaceptr && !strlcmp("X-Face:",c))
		{
			xfaceptr=c+7;
		}
		else if (!savedateptr && !strlcmp("X-Savedate:",c))
		{
			savedateptr=c+11;
		}
		else if (!pop3cfgptr && !strlcmp("X-POPCfg:",c))
		{
			pop3cfgptr=c+9;
		}
		else if (!pop3hostptr && !strlcmp("X-POPHost:",c))
		{
			pop3hostptr=c+10;
		}
		else if (!pop3userptr && !strlcmp("X-POPUser:",c))
		{
			pop3userptr=c+10;
		}
		else if (!oldfolderptr && !strlcmp("X-Oldfolder2:",c))
		{
			oldfolderptr=c+13;
		}
		else if (!sortPriority && !strlcmp("X-"PROGRAMNAME"-Sortaccouting:",c))
		{
			sortPriority=atoi(c+strlen("X-"PROGRAMNAME"-Sortaccouting:"));
		}
		else if (!visualBellValue && !strlcmp("X-"PROGRAMNAME"-VisualBell:",c))
		{
			visualBellValue=atoi(c+strlen("X-"PROGRAMNAME"-VisualBell:"));
		}
		else if (!contenttypeptr && !strlcmp("Priority:",c))
		{
			priorityptr=c+9;
		}
		else if (!mimeversionptr && !strlcmp("Mime-Version:",c))
		{
			mimeversionptr=c+13;
		}
		else if (!contenttypeptr && !strlcmp("Content-Type:",c))
		{
			contenttypeptr=c+13;
		}
		else if (!contentencodingptr && !strlcmp("Content-Transfer-Encoding:",c))
		{
			contentencodingptr=c+26;
		}
		else if (!messageidptr && !strlcmp("Message-Id:",c))
		{
			messageidptr=c+11;
		}
		else if (!xuidlptr && !strlcmp("X-UIDL:",c))
		{
			xuidlptr=c+7;
		}
		else if (*c=='\n')
		{ // empty line
			bodyptr=c+1;
			done=1;
		}
		while (*c && *c!='\n')
			c++;
		if (*c=='\n') c++;
	}
	if (!strcscmp(mime1(),"multipart"))
	{ // Multimail
		if (!strcscmp(mime2(),"encrypted"))
			pgpmode=1;
		else if (!strcscmp(mime2(),"signed"))
			pgpmode=2;
		char *parter=multipart();
		char b1[strlen(parter)+5],b2[strlen(parter)+7];
		strcpy(b1,"--");
		strcat(b1,parter);
		strcpy(b2,b1);
		strcat(b2,"--\n");
		strcat(b1,"\n");

		while (*c)
		{
			if (!strlcmp(b1,c))
			{ // mime start
				if (totalmulti>0)
				{
					multioffsets[totalmulti-1].end=c-mailtext;
					subtypes(totalmulti);
				}
				if (totalmulti<MAXMIMEPARTS-1)
				{
					totalmulti++;
					multioffsets[totalmulti-1].start=c-mailtext+strlen(b1);
					multioffsets[totalmulti-1].end=0;
					multioffsets[totalmulti-1].text=0;
					multioffsets[totalmulti-1].app=0;
					multioffsets[totalmulti-1].save=0;
					multioffsets[totalmulti-1].freeflag1=0;
					multioffsets[totalmulti-1].freeflag2=0;
					multioffsets[totalmulti-1].freeflag3=0;
					multioffsets[totalmulti-1].freeflag4=0;
					multioffsets[totalmulti-1].nametext=NULL;
					multioffsets[totalmulti-1].nameptr=NULL;
					multioffsets[totalmulti-1].contenttypetext=NULL;
					multioffsets[totalmulti-1].contenttypeptr=NULL;
					multioffsets[totalmulti-1].bodyptr=NULL;
					multioffsets[totalmulti-1].bodytext=NULL;
					multioffsets[totalmulti-1].allmailtext=NULL;
					multioffsets[totalmulti-1].contentencodingptr=contentencodingptr;
					multioffsets[totalmulti-1].contentencodingtext=NULL;
				}
				c+=strlen(b1);
			}
			else if (!strlcmp(b2,c))
			{ // mime final
				if (totalmulti>0)
				{
					multioffsets[totalmulti-1].end=c-mailtext;
					subtypes(totalmulti);
				}
				c+=strlen(b2);
				break;
			}
			else
			{
				while (*c && *c!='\n')
					c++;
				while (*c=='\n') c++;
			}
		}
		if (totalmulti>0 && !multioffsets[totalmulti-1].end)
		{
			multioffsets[totalmulti-1].end=c-mailtext;
			subtypes(totalmulti);
		}
	}
}
mail::~mail()
{
	if (referers>0) fprintf(stderr,"Warning: A mail object that is marked as still required is freed\n");
	if (subm) subm->remreferer();
	subm=NULL;
	delete _id;
	for(int i=totalmulti;i>0;i--)
	{
		if (multioffsets[i-1].bodytext) free(multioffsets[i-1].bodytext);
		if (multioffsets[i-1].contenttypetext) free(multioffsets[i-1].contenttypetext);
		if (multioffsets[i-1].nametext) free(multioffsets[i-1].nametext);
		if (multioffsets[i-1].allmailtext) free(multioffsets[i-1].allmailtext);
		if (multioffsets[i-1].contentencodingtext) free(multioffsets[i-1].contentencodingtext);
	}
 	if (fromtext) free(fromtext);
 	if (totext) free(totext);
 	if (cctext) free(cctext);
 	if (subjecttext) free(subjecttext);
 	if (statustext) free(statustext);
 	if (xstatustext) free(xstatustext);
 	if (returntext) free(returntext);
 	if (replytotext) free(replytotext);
	if (lvlinetext[3]) free(lvlinetext[3]);
 	if (lvlinetext[2]) free(lvlinetext[2]);
 	if (lvlinetext[1]) free(lvlinetext[1]);
 	if (lvlinetext[0]) free(lvlinetext[0]);
 	if (datetext) free(datetext);
 	if (mimeversiontext) free(mimeversiontext);
 	if (contentencodingtext) free(contentencodingtext);
 	if (contenttypetext) free(contenttypetext);
 	if (messageidtext) free(messageidtext);
 	if (headertext) free(headertext);
 	if (tagtext) free(tagtext);
 	if (bodytext) free(bodytext);
	if (xuidltext) free(xuidltext);
 	if (xfacetext) free(xfacetext);
	if (savedatetext) free(savedatetext);
	if (prioritytext) free(prioritytext);

	if (oldfoldertext) free(oldfoldertext);

	if (pop3cfgtext) free(pop3cfgtext);
	if (pop3hosttext) free(pop3hosttext);
	if (pop3usertext) free(pop3usertext);

	if (mailtext) free(mailtext);
}

void mail::addreferer(void)
{
	referers++; //printf("Refs++: %d\n",referers);
}
void mail::remreferer(void)
{
	referers--; //printf("Refs--: %d\n",referers);
	if (!referers) delete this;
}
void mail::setbody(char *b)
{
	if (!partial)
		return;
// todo

}

static int readline(char *c)
{
	int l=0;
	while (*c && *c!='\n')
	{
		l++;
		c++;
	}
	if (*c && (*(c+1)==' ' || *(c+1)=='\t'))
	{ // add next line
		l+=readline(c+1)+1;
	}
	return l;
}
static char *copyline(char *t)
{ // flag: ignore "
	char *c=NULL,*c2;
	if (t)
	{
		while (*t && *t==' ') t++;
		if (c=(char*)malloc(readline(t)+2))
		{
			strncpy(c,t,readline(t));
			*(c+readline(t))=0;
			c2=c;
			while (*c2)
			{
				if (*c2=='\n')
				{
					strcpy(c2,c2+1);
				}
				else
				{
					c2++;
				}
			}
			c2=c;
			while (*c2==' ' || *c2=='\t')
			{
				c2++;
			}
			if (c2!=c)
				strcpy(c,c2);
		}
	}
	else
	{
		c=(char*)malloc(10);
		*c=0;
	}
	return c;
}
// char *copyline(char *t,int flag=0)
// { // flag: ignore "
// 	char *c=NULL,*c2;
// 	if (t)
// 	{
// 		if (c=(char*)malloc(readline(t)+2))
// 		{
// 			int quote=0;
// 			strncpy(c,t,readline(t));
// 			*(c+readline(t))=0;
// 			while ((!flag && *c=='"') || *c==' ' || *c=='\t')
// 			{
// 				if (*c=='"') quote=1;
// 				strcpy(c,c+1);
// 			}
// 			c2=c;
// 			while (*c2)
// 			{
// 				if (*c2=='\n')
// 				{
// 					strcpy(c2,c2+1);
// 					if (!flag && *(c2-1)=='"' && quote)
// 					{
// 						strcpy(c2-1,c2);
// 					}
// 					quote=0;
// 					while ((!flag && *c2=='"') || *c2==' ' || *c2=='\t')
// 					{
// 						if (*c=='"') quote=1;
// 						strcpy(c2,c2+1);
// 					}
// 				}
// 				else
// 				{
// 					c2++;
// 				}
// 			}
// 			if (!flag && c[strlen(c)-1]=='"')
// 			{
// 				c[strlen(c)-1]=0;
// 			}
// 		}
// 	}
// 	else
// 	{
// 		c=(char*)malloc(10);
// 		*c=0;
// 	}
// 	return c;
// }
static void rmQuotes(char *t)
{
	char *c=t;
	while (*c)
	{
		if (*c=='"')
		{
			strcpy(c,c+1);
		}
		else
			c++;
	}
}

void mail::setfolder(folder *f)
{
	parent=f;
}
void mail::setvfolder(vfolder *f)
{
	vparent=f;
}
int mail::write(FILE *mailf)
{
	if (smallmode) return 0;

	int totalbytes=0;
	int err=0, n;
	char st[90];
	
	if (!err)
	{
		// write unknown data
		char *c=mailtext;
		
		while (*c)
		{
			if (!strlcmp("Status:",c)
			  ||!strlcmp("X-"PROGRAMNAME"-Status:",c)
			  ||!strlcmp("X-Savedate:",c)
			  ||!strlcmp("X-Oldfolder:",c)
			  ||!strlcmp("X-Oldfolder2:",c)
			  ||!strlcmp("X-"PROGRAMNAME"-Sortaccouting:",c)
			  ||!strlcmp("X-"PROGRAMNAME"-VisualBell:",c))
			{ // headers to update - don't write here, later written
			}
			else if (*c=='\n')
			{ // empty line
				break;
			}
			else
			{ // write until end of line
				int l=0;
				while (*c && *c!='\n')
				{
					err=err||fputc(*c,mailf)==EOF;
					c++; l++; totalbytes++;
				}
				if (l)
				{
					err=err||fputc('\n',mailf)==EOF;
					totalbytes++;
				}
			}
			while (*c && *c!='\n')
				c++;
			if (*c=='\n') c++;
		}
	}

	if (!err)
	{
		if (!savedateptr)
		{
			if (savedatetext) delete [] savedatetext;
			time_t t;
			time(&t);
			savedatetext=new char[50];
			strftime(savedatetext,50,"%a, %d %b %Y %H:%M:%S +0000",gmtime(&t));
		}
		n=xc_snprintf(st,sizeof(st), "X-Savedate: %s\n",savedatetext?savedatetext:copyline(savedateptr));
		err=fwrite(st,1,n,mailf)!=n;
		totalbytes+=n;
	}
	if (!err)
	{
		if (oldfolder() && oldfolder()[0])
		{
			n=xc_snprintf(st,sizeof(st), "X-Oldfolder2: %s\n",oldfoldertext);
			err=fwrite(st,1,n,mailf)!=n;
			totalbytes+=n;
		}
	}
	if (!err)
	{
		if (sortPriority!=0)
		{
			n=xc_snprintf(st,sizeof(st), "X-"PROGRAMNAME"-Sortaccouting: %d\n",sortPriority);
			err=fwrite(st,1,n,mailf)!=n;
			totalbytes+=n;
		}
	}
	if (!err)
	{
		if (visualBellValue>0)
		{
			n=xc_snprintf(st,sizeof(st), "X-"PROGRAMNAME"-VisualBell: %d\n",visualBellValue);
			err=fwrite(st,1,n,mailf)!=n;
			totalbytes+=n;
		}
	}

	if (!err)
	{
		strcpy(st, "X-"PROGRAMNAME"-Status: ");
		if (status() & STATUS_FORWARDED)
			strcat(st,"F");
		if (status() & STATUS_REPLYED)
			strcat(st,"R");
		if (status() & STATUS_SAVED)
			strcat(st,"S");
		if (status() & STATUS_FILTERED)
			strcat(st,"T");
		if (status() & STATUS_XCMAILREAD)
			strcat(st,"r");
		if (status() & STATUS_XCPOP3NEW)
			strcat(st,"P");
		if (status() & STATUS_POPMAIL)
			strcat(st,"p");

		if (st[strlen(st)-1] == ' ')
			strcat(st, "-");
		n=strlen(strcat(st, "\n"));
		err=fwrite(st,1,n,mailf)!=n;
		totalbytes+=n;
	}
	if (!err)
	{
		n=xc_snprintf(st,sizeof(st), "Status: %s\n\n", (status()&STATUS_READ)?"RO":"O");
		err=fwrite(st,1,n,mailf)!=n;
		totalbytes+=n;
	}

	if (!err && bodyptr)
	{
		n=strlen(bodyptr);
		err=fwrite(bodyptr,1,n,mailf)!=n;
		totalbytes+=n;
			
		if (!err && bodyptr[n-1]!='\n') {
			err=fputc('\n',mailf)==EOF;
			totalbytes++;
		}
	}
	
	err=err||fflush(mailf)!=0;
	
	return err ? 0 : totalbytes;
}

void mail::freemem(void)
{
	if (smallmode) return;

	for(int i=totalmulti;i>0;i--)
	{
		if (multioffsets[i-1].bodytext) free(multioffsets[i-1].bodytext);
		multioffsets[i-1].bodytext=NULL;
		if (multioffsets[i-1].contenttypetext) free(multioffsets[i-1].contenttypetext);
		multioffsets[i-1].contenttypetext=NULL;
		if (multioffsets[i-1].nametext) free(multioffsets[i-1].nametext);
		multioffsets[i-1].nametext=NULL;
		if (multioffsets[i-1].allmailtext) free(multioffsets[i-1].allmailtext);
		multioffsets[i-1].allmailtext=NULL;
		if (multioffsets[totalmulti-1].contentencodingtext) free(multioffsets[totalmulti-1].contentencodingtext);
		multioffsets[totalmulti-1].contentencodingtext=NULL;
	}
 	if (totext) free(totext);
 	totext=NULL;
 	if (cctext) free(cctext);
 	cctext=NULL;
 	if (returntext) free(returntext);
 	returntext=NULL;
 	if (replytotext) free(replytotext);
 	replytotext=NULL;
 	if (datetext) free(datetext);
 	datetext=NULL;
 	if (mimeversiontext) free(mimeversiontext);
 	mimeversiontext=NULL;
 	if (contentencodingtext) free(contentencodingtext);
	contentencodingtext=NULL;
 	if (contenttypetext) free(contenttypetext);
 	contenttypetext=NULL;
 	if (messageidtext) free(messageidtext);
 	messageidtext=NULL;
 	if (headertext) free(headertext);
 	headertext=NULL;
 	if (bodytext) free(bodytext);
 	bodytext=NULL;
 	if (tagtext) free(tagtext);
 	tagtext=NULL;
 	if (xfacetext) free(xfacetext);
 	xfacetext=NULL;
	if (savedatetext) free(savedatetext);
	savedatetext=NULL;
	if (prioritytext) free(prioritytext);
	prioritytext=NULL;
	if (oldfoldertext) free(oldfoldertext);
	oldfoldertext=NULL;
	if (pop3cfgtext) free(pop3cfgtext);
	pop3cfgtext=NULL;
	if (pop3hosttext) free(pop3hosttext);
	pop3hosttext=NULL;
	if (pop3usertext) free(pop3usertext);
	pop3usertext=NULL;
}
void mail::makesmall(void)
{
	freemem();

	messageid();
	date();
	from();
	subject();
	to();

	if (subm) subm->remreferer();
	subm=NULL;

	if (lvlinetext[3]) free(lvlinetext[3]);
	lvlinetext[3]=NULL;
 	if (lvlinetext[2]) free(lvlinetext[2]);
	lvlinetext[2]=NULL;
 	if (lvlinetext[1]) free(lvlinetext[1]);
	lvlinetext[1]=NULL;
 	if (lvlinetext[0]) free(lvlinetext[0]);
	lvlinetext[0]=NULL;

	if (mailtext) free(mailtext);
	mailtext=NULL;

	smallmode=1;
}

Bool mail::operator==(mail &m2)
{
	return !strcmp(messageid(),m2.messageid())
	 && !strcmp(date(),m2.date())
	 && !strcmp(from(),m2.from())
	 && !strcmp(subject(),m2.subject())
	 && !strcmp(to(),m2.to());
}

#define LVPOS_DATE		1
#define LVPOS_FROM		2
#define LVPOS_SUBJECT	3

char **mail::lvline(void)
{
	if (!lvlinetext[0] || *lvlinetext[0]==0)
	{
		if (!lvlinetext[0]) lvlinetext[0]=(char*)malloc(3);
		if (!lvlinetext[LVPOS_DATE])    lvlinetext[LVPOS_DATE]=(char*)malloc(15);
		if (!lvlinetext[LVPOS_SUBJECT]) lvlinetext[LVPOS_SUBJECT]=(char*)malloc(strlen(subject())+5);

		if (lvlinetext[0] && lvlinetext[LVPOS_DATE] && lvlinetext[LVPOS_SUBJECT])
		{
			lvlinetext[0][0]=' ';
			lvlinetext[0][1]=' ';

			if (status() & STATUS_FORWARDED)
				lvlinetext[0][0]='F';
			if (status() & STATUS_REPLYED)
				lvlinetext[0][1]='R';
			if (status() & STATUS_SAVED)
				lvlinetext[0][1]='S';
 			if (status() & STATUS_XCMAILNEW && programconfig.mailnewforXCmail)
 				lvlinetext[0][0]='n';
 			if (status() & STATUS_XCPOP3NEW && programconfig.mailnewfromPop3)
 				lvlinetext[0][0]='n';
			if (status() & STATUS_NEW)
				lvlinetext[0][0]='N';
			if (status() & STATUS_OLD)
				lvlinetext[0][0]='O';
			if (status() & STATUS_SERVERDELETE)
				lvlinetext[0][0]='d';
			if (status() & STATUS_DELETE)
				lvlinetext[0][0]='D';
			lvlinetext[0][2]=0;

			{
				static char *mon[]={"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"};
				static char *wdy[]={"Sun","Mon","Tue","Wed","Thu","Fri","Sat"};
				int d=1,m=1,y=1900,h=0,min=0,sec;
				time_t mt, t = time(NULL);
				
				mt = parsefromdate(d,m,y,h,min,sec);

				if (difftime(t, mt) < t % 86400)
				{
					// ...von diesem Tag -> Uhrzeit
					//
					sprintf(lvlinetext[LVPOS_DATE],"%2d:%02d",h,min);					
				}
				else if (difftime(t-(t%86400), mt) < 604800)
				{
					// ... nicht aelter als 1 Woche -> Wochentag, Monatstag
					//
					struct tm *ts = localtime(&mt);
					sprintf(lvlinetext[LVPOS_DATE],"%s %2d",wdy[ts->tm_wday],d);
				}
				else if (difftime(t, mt) < 31557168)
				{
					// ...von diesem Jahr -> Monat + Tag				
					//
					sprintf(lvlinetext[LVPOS_DATE],"%2d %s",d,mon[m-1]);
				}
				else
				{
					// sonst -> Jahr
					//
//					sprintf(lvlinetext[LVPOS_DATE],"%s %4d",mon[m-1],y);
					sprintf(lvlinetext[LVPOS_DATE],"%d/%4d",m,y);
				}
			}

			if (!lvlinetext[LVPOS_FROM])
			{
				if (vparent && vparent->getftype()==FOLDERTYPE_OUTBOX)
					lvlinetext[LVPOS_FROM]=strdup(displayto());
				else
				{
					lvlinetext[LVPOS_FROM]=strdup(displayfrom());
				}
			}
			strcpy(lvlinetext[LVPOS_SUBJECT]," ");
			strcat(lvlinetext[LVPOS_SUBJECT],subject());
		}
	}
	return lvlinetext;
}

void mail::parse_from_name(char *_fromtext)
{
	tokenizer tok1(_fromtext,"<"," >\t");

	if (tok1.count()>1)	// xxxxx <yyyyy>
	{
		strcpy(_fromtext,tok1[0]);
	}
	else
	{
		tokenizer tok2(_fromtext,"("," )\t");
		
		if (tok2.count()>1)
		{
			strcpy(_fromtext,tok2[1]);
		}
		else
		{
			strcpy(_fromtext,tok2[0]);			
		}
	}
	Translate728Line(_fromtext);
}
void mail::parse_from_address(char *_fromemailtext)
{
	tokenizer tok1(_fromemailtext,"<"," >\t");

	if (tok1.count()>1)	// xxxxx <yyyyy>
	{
		strcpy(_fromemailtext,tok1[1]);
	}
	else
	{
		tokenizer tok2(_fromemailtext,"("," )\t");
		strcpy(_fromemailtext,tok2[0]);
	}
}
char *mail::from(void)
{
    if (!fromtext)
    {
        fromtext=copyline(fromptr);
		parse_from_name(fromtext);
    }
    return fromtext;
}
char *mail::from_email(void)
{
	if (!fromemailtext)
	{
		fromemailtext=copyline(fromptr);
		parse_from_address(fromemailtext);
	}
	return fromemailtext;
}
char *mail::to(void)
{
	if (!totext)
	{
		totext=copyline(toptr);
		Translate728Line(totext);
		rmQuotes(totext);
	}
	return totext;
}
char *mail::cc(void)
{
	if (!cctext)
	{
		cctext=copyline(ccptr);
		Translate728Line(cctext);
		rmQuotes(cctext);
	}
	return cctext;
}
char *mail::subject(void)
{
	if (!subjecttext)
	{
		subjecttext=copyline(subjectptr);
		Translate728Line(subjecttext);
	}
	return subjecttext;
}
char *mail::displayfrom(void)
{ // if address in addressbook show nick name - addresslist!
	if (from()[0])
		return from();
	else
		return from_email();
}
char *mail::displayto(void)
{ // if address in addressbook show nick name - addresslist!
	return to();
}
char *mail::displaycc(void)
{ // if address in addressbook show nick name - addresslist!
	return cc();
}
char *mail::returnto(void)
{
	if (!returntext)
	{
		if (returnptr)
			returntext=copyline(returnptr);
		else
		{
			returntext=strdup(from_email());
		}
		char *c=returntext;
		while (*c && *c!='<')
		{
			c++;
		}
		if (*c=='<')
		{
			char *c2=returntext;
			c++;
			while (*c && *c!='>')
			{
				*c2=*c;
				if (*c!=' ')
					c2++;
				c++;
			}
			*c2=0;
		}
	}
	return returntext;
}
char *mail::replyto(void)
{
	if (!replytotext)
	{
		if (replytoptr)
			replytotext=copyline(replytoptr);
		else
		{
			replytotext=strdup(from_email());
		}
		char *c=replytotext;
		parse_from_address(replytotext);
	}
	return replytotext;
}
char *mail::date(void)
{
	if (!datetext)
	{
		datetext=copyline(dateptr);
	}
	return datetext;
}
time_t mail::parsefromdate(int &d,int &m,int &y,int &h,int &min,int &sec)
{
	if (!mail_day)
	{
		mail_t=parseDate(date(),mail_day,mail_month,mail_year,mail_hour,mail_min,mail_sec);
	}

	d=mail_day;
	m=mail_month;
	y=mail_year;
	h=mail_hour;
	min=mail_min;
	sec=mail_sec;

	return mail_t;
}
void mail::oldfolder(char *o)
{
	if (oldfoldertext) free(oldfoldertext);
	oldfoldertext=strdup(o);
}
char *mail::oldfolder(void)
{
	if (!oldfoldertext)
		oldfoldertext=copyline(oldfolderptr);
	if (oldfoldertext[0]=='"')
		strcpy(oldfoldertext,oldfoldertext+1);
	if (oldfoldertext[strlen(oldfoldertext)-1]=='"')
		oldfoldertext[strlen(oldfoldertext)-1]=0;
	return oldfoldertext?oldfoldertext:(char*)"";
}
void mail::clearsavedate(void)
{
	if (savedatetext)
		delete [] savedatetext;
	savedatetext=NULL;
	savedateptr=NULL;
}
time_t mail::parsesavedate(int &d,int &m,int &y,int &h,int &min,int &sec)
{
	if (!savedatetext)
	{
		if (savedateptr)
			savedatetext=copyline(savedateptr);
		else
		{
			time_t t;
			time(&t);
			savedatetext=new char[50];
			strftime(savedatetext,50,"%a, %d %b %Y %H:%M:%S +0000",gmtime(&t));
		}
	}

	return parseDate(savedatetext,d,m,y,h,min,sec);
}
char *mail::messageid(void)
{
	if (!messageidtext)
	{
		messageidtext=copyline(messageidptr);
	}
	return messageidtext;
}
char *mail::header(void)
{
	if (!headertext)
	{
		char *c=statusptr;
		if (!c) c=bodyptr;

		headertext=(char*)malloc(c-mailtext+1);
		memcpy(headertext,mailtext,c-mailtext);
		headertext[c-mailtext]=0;
	}
	return headertext;
}
char *mail::mimeversion(void)
{
	if (!mimeversiontext)
	{
		mimeversiontext=copyline(mimeversionptr);
	}
	return mimeversiontext;
}
char *mail::contenttype(void)
{
	if (!contenttypetext)
	{
		if (contenttypeptr)
			contenttypetext=copyline(contenttypeptr);
		else
			contenttypetext=strdup("text/plain");
	}
	return contenttypetext;
}
char *mail::mime1(void)
{
	if (subm) return subm->mime1();

	static char m[100];
	char *c1=m,*c2=contenttype();
	*c1=0;
	while (*c2 && *c2!='/')
	{
		*c1=*c2;
		c1++;
		c2++;
	}
	*c1=0;
	return m;
}
char *mail::mime2(void)
{
	if (subm) return subm->mime2();

	static char m[100];
	char *c1=m,*c2=contenttype();
	*c1=0;
	while (*c2 && *c2!='/')
	{
		c2++;
	}
	if (*c2) c2++;
	while (*c2 && *c2!=';' && *c2!=' ')
	{
		*c1=*c2;
		c1++;
		c2++;
	}
	*c1=0;
	return m;
}
char *mail::multipart(void)
{
	static char m[300];
	char *c2=contenttype();
	int qt=0;

	m[0]=0;
	while (*c2 && *c2!=';')
	{
		c2++;
	}
	while (*c2)
	{
		if (!strlcmp("boundary=",c2))
		{
			strcpy(m,c2+9);
			break;
		}
		c2++;
	}
	if (m[0]=='"')
	{
		strcpy(m,&m[1]);
		qt=1;
	}
	c2=m;
	while (*c2)
	{
		if (qt && *c2=='"')
		{
			*c2=0;
			break;
		}
		if (!qt && *c2==';')
		{
			*c2=0;
			break;
		}
		c2++;
	}
	return m;
}
char *mail::submultipart(int p)
{
	static char m[300];
	char *c2=subcontenttype(p);
	int qt=0;

	m[0]=0;
	while (*c2 && *c2!=';')
	{
		c2++;
	}
	while (*c2)
	{
		if (!strlcmp("boundary=",c2))
		{
			strcpy(m,c2+9);
			break;
		}
		c2++;
	}
	if (m[0]=='"')
	{
		strcpy(m,&m[1]);
		qt=1;
	}
	c2=m;
	while (*c2)
	{
		if (qt && *c2=='"')
		{
			*c2=0;
			break;
		}
		if (!qt && *c2==';')
		{
			*c2=0;
			break;
		}
		c2++;
	}
	return m;
}
char *mail::xface(void)
{
	if (!xfacetext)
	{
		xfacetext=copyline(xfaceptr);
	}
	return xfacetext;
}

int mail::status(void)
{
	if (!statusval)
	{
		int s=STATUS_NEW;
		if (programconfig.mailnewforXCmail)
			s|=STATUS_XCMAILNEW;
		if (!statustext)
		{
			if (statusptr)
				statustext=copyline(statusptr+7);
			else
				statustext=copyline(NULL);			
		}
		int i,l=strlen(statustext);
		for(i=0;i<l;i++)
		{
			if (statustext[i]=='O')
			{
				s=(s&~STATUS_NEW)|STATUS_OLD;
			}
			if (statustext[i]=='R')
			{
				s=(s&~STATUS_NEW)|STATUS_READ|STATUS_XCMAILREAD;
			}
		}
		if (s&STATUS_READ) s=s&~STATUS_OLD;

		if (!xstatustext)
		{
			if (xstatusptr)
				xstatustext=copyline(xstatusptr+strlen("X-"PROGRAMNAME"-Status:"));
			else
				xstatustext=copyline(NULL);			
		}
		l=strlen(xstatustext);
		for(i=0;i<l;i++)
		{
			if (xstatustext[i]=='F') s|=STATUS_FORWARDED;
			if (xstatustext[i]=='R') s|=STATUS_REPLYED;
			if (xstatustext[i]=='S') s|=STATUS_SAVED;
			if (xstatustext[i]=='T') s|=STATUS_FILTERED;
			if (xstatustext[i]=='r') s=(s&~STATUS_XCMAILNEW)|STATUS_XCMAILREAD;
			if (xstatustext[i]=='P') s=(s&~STATUS_XCMAILNEW)|STATUS_XCPOP3NEW;
			if (xstatustext[i]=='p') s|=STATUS_POPMAIL;
		}
		if (s & STATUS_XCMAILNEW && !programconfig.mailnewforXCmail)
			s=(s&~STATUS_XCMAILNEW);
		if (s & STATUS_XCPOP3NEW && !programconfig.mailnewfromPop3)
			s=(s&~STATUS_XCPOP3NEW);

		statusval=s;
		if ((s & STATUS_NEW) && parent)
		{
			haschanged=1;
			parent->change();
		}

	}
	return statusval;
}
void mail::status(int s)
{
	if (lvlinetext[0]) *lvlinetext[0]=0;
	if (statusval!=s && parent)
	{
		haschanged=1;
		parent->change();
	}
	statusval=s;
	updateoutput(this);
}
char *mail::mailbody(int &size,int p)
{
	if (subm) return subm->mailbody(size,p);

	if (!p || totalmulti==0)
	{
		if (!bodytext)
		{
			size=strlen(SAVEPTR(bodyptr));
			if (bodytext=(char*)malloc(size+1))
			{
				strcpy(bodytext,SAVEPTR(bodyptr));
			}
		}
		size=strlen(SAVEPTR(bodytext));
		return bodytext;
	}
	if (p>totalmulti)
	{
		if (!bodytext)
		{
			if (bodytext=(char*)malloc(size+1))
			{
				size=strlen(SAVEPTR(bodyptr));
				strcpy(bodytext,SAVEPTR(bodyptr));
			}
		}
		size=strlen(SAVEPTR(bodytext));
		return bodytext;
	}

	if (!multioffsets[p-1].bodytext)
	{
		char *c1,*c2;
		int i;
		multioffsets[p-1].bodytext=(char*)malloc(multioffsets[p-1].end-multioffsets[p-1].start+1);
		c1=multioffsets[p-1].bodytext;
		c2=multioffsets[p-1].bodyptr;
		i=multioffsets[p-1].bodyptr-mailtext;

		while (i<multioffsets[p-1].end)
		{
			*c1=*c2;
			i++; c1++; c2++;
		}
		*c1=0;

		multioffsets[p-1].size=strlen(multioffsets[p-1].bodytext);
	}
	size=multioffsets[p-1].size;
	return multioffsets[p-1].bodytext;
}

char *mail::contentencoding(int p)
{
	if (subm) return subm->contentencoding(p);

	if (!p || totalmulti==0 || p>totalmulti)
	{
		if (!contentencodingtext) contentencodingtext=copyline(contentencodingptr);
		return contentencodingtext;
	}
	if (!multioffsets[p-1].contentencodingtext) multioffsets[p-1].contentencodingtext=copyline(multioffsets[p-1].contentencodingptr);
	return multioffsets[p-1].contentencodingtext;
}

char *mail::thesubmailtext(int p)
{
	if (subm) return subm->thesubmailtext(p);

	if (!multioffsets[p-1].allmailtext)
	{
		char *c1,*c2;
		int i;
		multioffsets[p-1].allmailtext=(char*)malloc(multioffsets[p-1].end-multioffsets[p-1].start+1);
		c1=multioffsets[p-1].allmailtext;
		c2=mailtext+multioffsets[p-1].start;
		i=multioffsets[p-1].start;

		while (i<multioffsets[p-1].end)
		{
			*c1=*c2;
			i++; c1++; c2++;
		}
		*c1=0;
	}
	return multioffsets[p-1].allmailtext;
}
int mail::totalparts(void)
{
	if (pgpmode)
	{
		if (subm)
			return -(subm->totalparts()?subm->totalparts():1);
		else
			return -totalmulti;
	}
	else
		return totalmulti;
}

char *mail::partname(int p)
{
	if (subm) return subm->partname(p);

	static char n[200];
	if (p==0 || totalmulti<p)
	{
		strcpy(n,"Text");
	}
	else
	{
		if (!multioffsets[p-1].nametext)
		{ // mehr....
			if (multioffsets[p-1].nameptr)
			{
				char *c=copyline(multioffsets[p-1].nameptr);
				xc_snprintf(n,sizeof(n),"%d %s",p,c,submime1(p),submime2(p));
				multioffsets[p-1].nametext=strdup(n);
			}
			else
			{
				xc_snprintf(n,sizeof(n),"Part %d",p,submime1(p),submime2(p));
				multioffsets[p-1].nametext=strdup(n);
			}
		}
		strcpy(n,multioffsets[p-1].nametext);
	}
	return n;
}

void mail::subtypes(int p)
{
	int done=0;
	char *c=multioffsets[p-1].start+mailtext;
	while (*c && !done)
	{
		if (!strlcmp("Content-Type:",c))
		{
			multioffsets[p-1].contenttypeptr=c+14;
		}
		else if (!strlcmp("Content-Description:",c))
		{
			multioffsets[p-1].nameptr=c+20;
		}
		else if (!strlcmp("Content-Transfer-Encoding:",c))
		{
			multioffsets[p-1].contentencodingptr=c+26;
		}
		else if (*c=='\n')
		{ // empty line
			multioffsets[p-1].bodyptr=c+1;
			done=1;
		}
		while (*c && *c!='\n')
			c++;
		if (*c=='\n') c++;
	}
	if (!strcscmp(submime1(p),"multipart"))
	{
		char *c=multioffsets[p-1].bodyptr;
		char *parter=submultipart(p);
		int stotalmulti=0;
		char b1[strlen(parter)+5],b2[strlen(parter)+7];
		strcpy(b1,"--");
		strcat(b1,parter);
		strcpy(b2,b1);
		strcat(b2,"--\n");
		strcat(b1,"\n");

		if (p==totalmulti)
		{
			if (multioffsets[p-1].bodytext) free(multioffsets[p-1].bodytext);
			if (multioffsets[p-1].contenttypetext) free(multioffsets[p-1].contenttypetext);
			if (multioffsets[p-1].nametext) free(multioffsets[p-1].nametext);
			if (multioffsets[p-1].allmailtext) free(multioffsets[p-1].allmailtext);
			if (multioffsets[p-1].contentencodingtext) free(multioffsets[p-1].contentencodingtext);
			totalmulti--;
		}

		char *endc=multioffsets[p-1].end+mailtext;

		while (*c && c<endc)
		{
			if (!strlcmp(b1,c))
			{ // mime start
				if (stotalmulti>0)
				{
					multioffsets[totalmulti-1].end=c-mailtext;
					subtypes(totalmulti);
				}
				if (totalmulti<MAXMIMEPARTS-1)
				{
					totalmulti++;
					stotalmulti++;
					multioffsets[totalmulti-1].start=c-mailtext+strlen(b1);
					multioffsets[totalmulti-1].end=0;
					multioffsets[totalmulti-1].text=0;
					multioffsets[totalmulti-1].app=0;
					multioffsets[totalmulti-1].save=0;
					multioffsets[totalmulti-1].freeflag1=0;
					multioffsets[totalmulti-1].freeflag2=0;
					multioffsets[totalmulti-1].freeflag3=0;
					multioffsets[totalmulti-1].freeflag4=0;
					multioffsets[totalmulti-1].nametext=NULL;
					multioffsets[totalmulti-1].nameptr=NULL;
					multioffsets[totalmulti-1].contenttypetext=NULL;
					multioffsets[totalmulti-1].contenttypeptr=NULL;
					multioffsets[totalmulti-1].bodyptr=NULL;
					multioffsets[totalmulti-1].bodytext=NULL;
					multioffsets[totalmulti-1].allmailtext=NULL;
					multioffsets[totalmulti-1].contentencodingptr=contentencodingptr;
					multioffsets[totalmulti-1].contentencodingtext=NULL;
				}
				c+=strlen(b1);
			}
			else if (!strlcmp(b2,c))
			{ // mime final
				if (stotalmulti>0)
				{
					multioffsets[totalmulti-1].end=c-mailtext;
					subtypes(totalmulti);
				}
				c+=strlen(b2);
				break;
			}
			else
			{
				while (*c && *c!='\n')
					c++;
				while (*c=='\n') c++;
			}
		}
		if (stotalmulti>0 && !multioffsets[totalmulti-1].end)
		{
			multioffsets[totalmulti-1].end=c-mailtext;
			subtypes(totalmulti);
		}
	}
	else if (!strcscmp(submime1(p),"message") && !strcscmp(submime2(p),"rfc822"))
	{
		int stotalmulti=0;

		if (p==totalmulti)
		{
			if (multioffsets[p-1].bodytext) free(multioffsets[p-1].bodytext);
			if (multioffsets[p-1].contenttypetext) free(multioffsets[p-1].contenttypetext);
			if (multioffsets[p-1].nametext) free(multioffsets[p-1].nametext);
			if (multioffsets[p-1].allmailtext) free(multioffsets[p-1].allmailtext);
			if (multioffsets[p-1].contentencodingtext) free(multioffsets[p-1].contentencodingtext);
			totalmulti--;
		}

		char *endc=multioffsets[p-1].end+mailtext;

//		while (*c && c<endc)
//		{
//			if (*c=='\n' && *(c+1)=='\n')
//				break;
//			c++;
//		}
		if (*c)
		{
			if (stotalmulti>0)
			{
				multioffsets[totalmulti-1].end=c-mailtext;
				subtypes(totalmulti);
			}
			if (totalmulti<MAXMIMEPARTS-1)
			{
				totalmulti++;
				stotalmulti++;
				multioffsets[totalmulti-1].start=c-mailtext;
				multioffsets[totalmulti-1].end=0;
				multioffsets[totalmulti-1].text=0;
				multioffsets[totalmulti-1].app=0;
				multioffsets[totalmulti-1].save=0;
				multioffsets[totalmulti-1].freeflag1=0;
				multioffsets[totalmulti-1].freeflag2=0;
				multioffsets[totalmulti-1].freeflag3=0;
				multioffsets[totalmulti-1].freeflag4=0;
				multioffsets[totalmulti-1].nametext=NULL;
				multioffsets[totalmulti-1].nameptr=NULL;
				multioffsets[totalmulti-1].contenttypetext=NULL;
				multioffsets[totalmulti-1].contenttypeptr=NULL;
				multioffsets[totalmulti-1].bodyptr=NULL;
				multioffsets[totalmulti-1].bodytext=NULL;
				multioffsets[totalmulti-1].allmailtext=NULL;
				multioffsets[totalmulti-1].contentencodingptr=contentencodingptr;
				multioffsets[totalmulti-1].contentencodingtext=NULL;
			}
		}
		c=endc;
		if (stotalmulti>0 && !multioffsets[totalmulti-1].end)
		{
			multioffsets[totalmulti-1].end=c-mailtext;
			subtypes(totalmulti);
		}
	}
}

char *mail::subcontenttype(int p)
{
	if (subm) return subm->subcontenttype(p);

	if (!multioffsets[p-1].contenttypetext)
	{
		multioffsets[p-1].contenttypetext=copyline(multioffsets[p-1].contenttypeptr);
	}
	return multioffsets[p-1].contenttypetext;
}
char *mail::submime1(int p)
{
	if (subm) return subm->submime1(p);

	if (p==0 || totalmulti<p) return mime1();

	static char m[100];
	char *c1=m,*c2=subcontenttype(p);
	*c1=0;
	while (*c2 && *c2!='/')
	{
		*c1=*c2;
		c1++;
		c2++;
	}
	*c1=0;
	return m;
}
char *mail::submime2(int p)
{
	if (subm) return subm->submime2(p);

	if (p==0 || totalmulti<p) return mime2();

	static char m[100];
	char *c1=m,*c2=subcontenttype(p);
	*c1=0;
	while (*c2 && *c2!='/')
	{
		c2++;
	}
	if (*c2) c2++;
	while (*c2 && *c2!=';' && *c2!=' ')
	{
		*c1=*c2;
		c1++;
		c2++;
	}
	*c1=0;
	return m;
}
void mail::setfreeflag(int p,int flag,int val)
{
	if (subm)
	{
		subm->setfreeflag(p,flag,val);
		return;
	}

	if (p>0 && p<=totalmulti)
	{
		switch (flag)
		{
			case 1:
				multioffsets[p-1].freeflag1=val;
				break;
			case 2:
				multioffsets[p-1].freeflag2=val;
				break;
			case 3:
				multioffsets[p-1].freeflag3=val;
				break;
			case 4:
				multioffsets[p-1].freeflag4=val;
				break;
		}
	}
}
int mail::getfreeflag(int p,int flag)
{
	if (subm) return subm->getfreeflag(p,flag);

	if (p>0 && p<=totalmulti)
	{
		switch (flag)
		{
			case 1:
				return multioffsets[p-1].freeflag1;
				break;
			case 2:
				return multioffsets[p-1].freeflag2;
				break;
			case 3:
				return multioffsets[p-1].freeflag3;
				break;
			case 4:
				return multioffsets[p-1].freeflag4;
				break;
		}
	}
	return 0;
}

char *mail::getTag(char *tag,int p)
{
	if (subm) return subm->getTag(tag,p);

	int done=0,quote=0;
	char *c,*ptr=NULL;

	if (p>totalmulti) p=totalmulti;

	if (p)
		c=multioffsets[p-1].start+mailtext;
	else
		c=mailtext;
	
	while (*c && !done)
	{
		if (!strlcmp(tag,c))
		{
			ptr=c+strlen(tag);
		}
		else if (*c=='\n')
		{ // empty line
			done=1;
		}
		while (*c && *c!='\n')
			c++;
		if (*c=='\n') c++;
	}
	if (tagtext) free(tagtext);
	tagtext=copyline(ptr);
	c=tagtext;
	while (*c && *c==' ')
	{
		strcpy(c,c+1);
	}
// 	while (*c && (*c!=';' || quote))
// 	{
// 		if (*c=='"') quote=!quote;
// 		c++;
// 	}
// 	*c=0;
	return tagtext;
}
char *mail::getContentTag(char *tag,int p)
{
	if (subm) return subm->getContentTag(tag,p);

	int done=0,quote=0;
	char *c,*ptr=NULL;

	if (p>totalmulti) p=totalmulti;

	if (p)
		c=subcontenttype(p);
	else
		c=contenttype();

	while (*c && !done)
	{
		if (*c==';')
		{
			c++;
			while (*c==' ' || *c=='\t' || *c=='\n') c++;
			if (!strlcmp(tag,c))
			{
				ptr=c+strlen(tag);
				done=1;
			}
		}
		c++;
	}
	if (tagtext) free(tagtext);
	tagtext=copyline(ptr);
	c=tagtext;
	while (*c && *c==' ')
	{
		strcpy(c,c+1);
	}
	while (*c && (*c!=';' || quote) && quote<2)
	{
		if (*c=='"')
		{
			quote++;
			if (quote==1)
			{
				strcpy(c,c+1);
				c--;
			}
			else if (quote==2)
			{
				*c=0;
			}
		}
		c++;
	}
	*c=0;
	return tagtext;
}
char *mail::xuidl(void)
{
	if (!xuidltext)
	{
		xuidltext=copyline(xuidlptr);
	}
	return xuidltext;
}
char *mail::pop3cfg(void)
{
	if (!pop3cfgtext)
	{
		pop3cfgtext=copyline(pop3cfgptr);
	}
	return pop3cfgtext;
}
char *mail::pop3host(void)
{
	if (!pop3hosttext)
	{
		pop3hosttext=copyline(pop3hostptr);
	}
	return pop3hosttext;
}
char *mail::pop3user(void)
{
	if (!pop3usertext)
	{
		pop3usertext=copyline(pop3userptr);
	}
	return pop3usertext;
}

char *mail::formtext(char *format)
{
	static char buf[1000];
	buf[0]=0;

	char *c1,*c2;
	c1=buf;
	c2=format;

	while (*c2)
	{
		if (*c2=='%')
		{
			int wordno=0;
			char cc[200]="";

			c2++;
			if (isdigit(*c2))
			{
				wordno=atoi(c2);
				while (isdigit(*c2)) c2++;
			}
			switch (*c2)
			{
				case '%':
					xc_strncpy(cc,"%",sizeof(cc));
					break;
				case 's': // subject
					xc_strncpy(cc,subject(),sizeof(cc));
					break;
				case 'S': // normalized subject
					normalize_subject(this,cc,sizeof(cc),0);
					break;
				case 'n': // sender
					xc_strncpy(cc,from(),sizeof(cc));
					break;
				case 'e': // sender address
					xc_strncpy(cc,replyto(),sizeof(cc));
					break;
				case 'd': // date
					xc_strncpy(cc,date(),sizeof(cc));
					break;
			}
			char *ccc=cc;
			if (wordno>0)
			{
				while (*ccc && wordno>0)
				{
					if (*ccc==' ' || ccc==cc)
					{
						wordno--;
						if (wordno==0)
						{ // copy
							while (*ccc==' ') ccc++;
							for(;*ccc && *ccc!=' ';ccc++,c1++)
							{
								if (c1<buf+sizeof(buf)-1)
									*c1=*ccc;
							}
							break;
						}
					}
					ccc++;
				}
			}
			else
			{
				for(;*ccc;ccc++,c1++)
				{
					if (c1<buf+sizeof(buf)-1)
						*c1=*ccc;
				}
			}
		}
		else if (*c2=='\\')
		{
			c2++;
			switch (*c2)
			{
				case 'n': // new line
					if (c1<buf+sizeof(buf)-1)
						*c1='\n';
					c1++;
					break;
				case '\\':
					if (c1<buf+sizeof(buf)-1)
						*c1='\\';
					c1++;
					break;
			}
		}
		else
		{
			if (c1<buf+sizeof(buf)-1)
				*c1=*c2;
			c1++;
		}
		c2++;
	}
	if (c1<buf+sizeof(buf)-1)
		*c1='\n';
	c1++;
	if (c1<buf+sizeof(buf)-1)
		*c1=0;
	else
		buf[sizeof(buf)-1]=0;
	return buf;
}
int mail::priority(void)
{
	if (priorityValue==-1)
	{
		if (!prioritytext)
		{
			if (priorityptr)
	        	prioritytext=copyline(priorityptr);
            else
            	prioritytext="normal";
        }

		if (!strcscmp(prioritytext,"normal"))
			priorityValue=1;
		else if (!strcscmp(prioritytext,"urgent"))
			priorityValue=0;
		else if (!strcscmp(prioritytext,"non-urgent"))
			priorityValue=2;
		else
			priorityValue=1;
	}
	return priorityValue;
}

int mail::sortAcct(void)
{
	return sortPriority;
}
void mail::sortAcct(int a)
{
	if (sortPriority!=a && parent)
	{
		haschanged=1;
		parent->change();
	}
	sortPriority=a;
}
unsigned int mail::visualBell(void)
{
	return visualBellValue;
}
void mail::visualBell(unsigned int a)
{
	if (visualBellValue!=a && parent)
	{
		haschanged=1;
		parent->change();
	}
	visualBellValue=a;
	updateoutput(this);
}

// -----------------------------------------------------------------------

int mail::PGPdecode(class appwindow *aw)
// return 0: password error
//        1: decoded
//        2: signatur error
//        3: signatur ok
{
// pgpmode 0: no PGP
//         1: coded  -> subm
//         2: signed
//         3: already decode
//         4: already sign checked

	int r=0;

	switch (pgpmode)
	{
		case 0:
			r=1;
			break;
		case 1: // part 1: head, part 2: coded text
			{
				char *pw=getPW(aw);
				if (pw)
				{
					int sign,s;
					char *t=::PGPdecode(aw,mailbody(s,2),pw,sign);
					if (t)
					{
						r=1;
						pgpmode=3;
						if (subm) subm->remreferer();
						subm=new mail(t);
						subm->addreferer();
					}
				}
			}
			break;
		case 2: // part 1: text, part 2: signature
			{
				int s;
				if (PGPcheckSign(aw,thesubmailtext(1),mailbody(s,2)))
				{
					r=1;
					pgpmode=4;
				}
				else
				{
					easy_dialog ed;
					ed.Text("Warning:\nSignature don't match with text\nor signature is unknown!");
					ed.Options("_Ok");
					ed.Start(aw);
					r=1;
					pgpmode=4;
				}
				if (subm) subm->remreferer();
				subm=new mail(strdup(thesubmailtext(1)));
				subm->addreferer();
			}
			break;
		case 3: // decode ok
			r=1;
			break;
		case 4: // signature ok
			r=3;
			break;
	}
	return r;
}
void mail::removeparts(void)
{
	freemem();
	if (referers<=2 && totalmulti>1 && !pgpmode)
	{
		for(int i=1;i<totalmulti;i++)
		{
			if (multioffsets[i].bodytext) free(multioffsets[i].bodytext);
			multioffsets[i].bodytext=NULL;
			if (multioffsets[i].contenttypetext) free(multioffsets[i].contenttypetext);
			multioffsets[i].contenttypetext=NULL;
			if (multioffsets[i].nametext) free(multioffsets[i].nametext);
			multioffsets[i].nametext=NULL;
			if (multioffsets[i].allmailtext) free(multioffsets[i].allmailtext);
			multioffsets[i].allmailtext=NULL;
			if (multioffsets[i].contentencodingtext) free(multioffsets[i].contentencodingtext);
			multioffsets[i].contentencodingtext=NULL;
		}
		mailtext[multioffsets[1].start]=0;
		char *c=&mailtext[multioffsets[1].start]-1;
		while (*c && *c=='\n') c--;
		c++;
		*c='-'; c++;
		*c='-'; c++;
		*c='\n'; c++;
		*c='\n'; c++;
		*c=0;
		totalmulti=1;
		if (parent)
		{
			haschanged=1;
			parent->change();
		}
		updateoutput(this);
	}
}
void mail::deleteMail(void)
{
	status(status()|STATUS_DELETE);
	{
		Pop3User *u=NULL;
		if (pop3host() && pop3user() && (u=findPop3User(pop3host(),pop3user())))
		{
			if (u->popdeleteonserver) status(status()|STATUS_SERVERDELETE);
		}
		else
		{
			char *c=pop3cfg();
			if (c)
			{
				u=findPop3User(c);
				if (u)
				{
					if (u->popdeleteonserver) status(status()|STATUS_SERVERDELETE);
				}
			}
		}
	}
}
void mail::deleteMail(deletebutton *b)
{
	switch (b->Selected())
	{
		case 0: // default
			status(status()|STATUS_DELETE);
			{
				Pop3User *u=NULL;
				if (pop3host() && pop3user() && (u=findPop3User(pop3host(),pop3user())))
				{
					if (u->popdeleteonserver) status(status()|STATUS_SERVERDELETE);
				}
				else
				{
					char *c=pop3cfg();
					if (c)
					{
						u=findPop3User(c);
						if (u)
						{
							if (u->popdeleteonserver) status(status()|STATUS_SERVERDELETE);
						}
					}
				}
			}
			break;
		case 1: // local
			status(status()|STATUS_DELETE);
			break;
		case 2: // server
			status(status()|STATUS_SERVERDELETE);
			break;
		case 3: // both
			status(status()|STATUS_DELETE);
			status(status()|STATUS_SERVERDELETE);
			break;
	}
	status(status() & ~(STATUS_XCMAILNEW|STATUS_XCPOP3NEW));
}
void mail::deleteLocalMail(void)
{
	status(status()|STATUS_DELETE);
	status(status() & ~(STATUS_XCMAILNEW|STATUS_XCPOP3NEW));
}
void mail::deleteServerMail(void)
{
	status(status()|STATUS_SERVERDELETE);
	status(status() & ~(STATUS_XCMAILNEW|STATUS_XCPOP3NEW));
}
void mail::undeleteMail(void)
{
	status(status()&~STATUS_DELETE);
	status(status()&~STATUS_SERVERDELETE);
	status(status() & ~(STATUS_XCMAILNEW|STATUS_XCPOP3NEW));
}
void mail::undeleteLocalMail(void)
{
	status(status()&~STATUS_DELETE);
}
void mail::undeleteServerMail(void)
{
	status(status()&~STATUS_SERVERDELETE);
}
void mail::readMail(void)
{
	status(status() | STATUS_READ | STATUS_XCMAILREAD);
	status(status() & ~(STATUS_OLD|STATUS_NEW|STATUS_XCMAILNEW|STATUS_XCPOP3NEW));
}
void mail::unreadMail(void)
{
	status(status() | STATUS_NEW);
	status(status() & ~(STATUS_OLD|STATUS_READ|STATUS_XCMAILREAD|STATUS_XCMAILNEW|STATUS_XCPOP3NEW));
}
