/* 
 * This file is part of XCmail and is subject to the 
 * XCmail license agreement
 */

/* $Id: mail_parser.h,v 2.2 1999/01/07 16:08:58 beckerr Exp $ */


#ifndef MAIL_PARSER_H
#define MAIL_PARSER_H

// -----------------------------------------------------
//
// classUrlList - Verwalten einer URL-Liste
//
// -----------------------------------------------------
//
class classUrlList {

	private:
		struct URL {
			URL *next;
			char *value;
			URL(int i) {
				value = new char[i + 1];
				value[0] = 0;
			};
			~URL() { delete[] value; }
		};

		URL   *urlP;
		URL   **urlPP;
		int   urlCount;

	public:
		classUrlList () {
			urlP = NULL;
			urlPP = &urlP;
			urlCount = 0;
		};
		~classUrlList() {
			UrlForgetAll();
		};

		void  UrlForgetAll(void);					// Alle URL vergesseb
		void  UrlAppend(char *ut, int len = -1);	// URL mit Laenge len anfuegen
		char* UrlByIndex(int i);					// URL mit Index i ermitteln
		int   UrlCount(void) { return urlCount; };	// Anzahl der URL ermitteln

};


// -----------------------------------------------------
//
// classParsedHtml - Parsen eine HTML-Files (in place)
//
// -----------------------------------------------------
//
class classParsedHtml : public classUrlList {

	private:
		char *textP;
		char *baseUrl;

	public:
		classParsedHtml(char *text = NULL) {
			textP = text ? Parse(text) : text;
			baseUrl = NULL;
		};
		
		~classParsedHtml() {
			if (baseUrl) delete[] baseUrl;
		};

		char* Parse(char *text);					// Text parsen (in place!)
		char* Text(void) { return textP; };			// Text ermitteln
};


// -----------------------------------------------------
//
// classParsedText - parsen eine ISO-Textes (in place)
//
// -----------------------------------------------------
//
class classParsedText : public classUrlList {

	private:
		char *textP;
		
	private:
		void  ParseTextForUrls(void);
	
	public:
		classParsedText()  { textP = NULL; };
		~classParsedText() { };
		char* Text(void)   { return textP; };		// letzten Text ermitten
		
		char* ParseText(char *text, char *iso);		// Text parsen (message body)
		char* ParseLine(char *text);				// Zeile parse (From, To, usw.)
};

// -----------------------------------------------------
//
// classTranslateOutgoing - ISO-Text erzeugen
//
// -----------------------------------------------------
//
class classTranslateOutgoing {

	private:
		char lineP[TRANSLATE_LINE_WIDTH];
		
	public:
		classTranslateOutgoing() 	{ };
		~classTranslateOutgoing() 	{ };
		void  Free(char *t) 		{ if (t)free(t); };

		char* TranslateText(char *text, char *iso);
		char* TranslateLine(char *text, char *iso);
};

#endif
