/* 
 * This file is part of XCmail and is subject to the 
 * XCmail license agreement
 */

/* $Id: mime_settings.cxx,v 2.6 2002/02/11 09:11:20 schmitzj Exp $ */


#include "configure.h"

#include <Xclasses/Xclasses.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <time.h>
#include <sys/utsname.h>
#include <dirent.h>
#include <errno.h>

#include "mailer.h"
#include "encoder.h"
#include "helper.h"
#include "pluginsmgr.h"
#include "filter_mgr.h"
#include "stringTokenizer.h"


//--------------------------------------------------------------------
// helpers/MIME prefs

static int isDisabled(char *plName,int global)
{
	char pbuf[100];
	tokenizer t(programconfig.disabledPlugins,",");
	int r=RELEASED;

	if (global)
		strcpy(pbuf,"GLOBAL::");
	else
		strcpy(pbuf,"USER::");
	strncat(pbuf,plName,sizeof(pbuf));

	if (t.find(pbuf))
		r=PRESSED;
	return r;
}
static void setDisabled(char *plName,int m,int global)
{
	char pbuf[100];
	int ch=0;
	tokenizer t(programconfig.disabledPlugins,",");
	t.removeempty();

	if (global)
		strcpy(pbuf,"GLOBAL::");
	else
		strcpy(pbuf,"USER::");
	strncat(pbuf,plName,sizeof(pbuf));

	if (m==PRESSED)
	{
		if (!t.find(pbuf))
		{
			t.append(new token(pbuf));
			ch=1;
		}
	}
	else
	{
		token *tk=t.find(pbuf);
		if (tk)
		{
			t.remove(tk);
			ch=1;
		}
	}
	if (ch)
	{
		programconfig.disabledPlugins[0]=0;
		strncat(programconfig.disabledPlugins,t.string(),sizeof(programconfig.disabledPlugins));
	}
}

class tv_mime : public tv_tree
{
public:
	tv_mime(char *n) : tv_tree(n)
	{
	};
	~tv_mime()
	{
		RemoveAll(RAF_DELETE);
	};
	virtual void setupGUI(gadget *base)
	{
		((output*)FindObject(".plugins_mgr.name"))->Output("");
		((output*)FindObject(".plugins_mgr.type"))->Output("");
		((output*)FindObject(".plugins_mgr.author"))->Output("");
		((output*)FindObject(".plugins_mgr.filename"))->Output("");
		((checkbox*)FindObject(".plugins_mgr.disable"))->Lock();
		((autotextbox*)FindObject(".plugins_mgr.descr"))->ReplaceText("");
	};
};
static tv_mime *tvt1=NULL;

class tv_load : public tv_mime
{
	char *name;
	int global;
public:
	tv_load(char *n,int _global) : tv_mime(n)
	{
		name=strdup(n);
		global=_global;
	};
	~tv_load()
	{
		free(name);
	}
	virtual void setupGUI(gadget *base)
	{
		((output*)FindObject(".plugins_mgr.name"))->Output(name);
		((output*)FindObject(".plugins_mgr.type"))->Output("");
		((output*)FindObject(".plugins_mgr.author"))->Output("");
		((output*)FindObject(".plugins_mgr.filename"))->Output("");
		((checkbox*)FindObject(".plugins_mgr.disable"))->Unlock();
		((checkbox*)FindObject(".plugins_mgr.disable"))->State(isDisabled(name,global));
		((autotextbox*)FindObject(".plugins_mgr.descr"))->ReplaceText("");
	};
	void setDisabled(gadget *base)
	{
		::setDisabled(name,((checkbox*)FindObject(".plugins_mgr.disable"))->State(),global);
	}
};
class tv_dir : public tv_mime
{
	int scanned;
	int global;
	char *dir;
public:
	tv_dir(char *name,char *d,int _global) : tv_mime(name)
	{
		SetHasSub();
		dir=strdup(d);
		scanned=0;
		global=_global;
	};
	~tv_dir()
	{
		free(dir);
	};
	void Open(void)
	{
		if (!scanned)
		{
			scanned=1;

			DIR *d;
			struct dirent *de;
			int l;
			if (d=opendir(dir))
			{
				while (de=readdir(d))
				{
					l=strlen(de->d_name);
					if (l>2 && de->d_name[l-1]=='o' && de->d_name[l-2]=='.')
						AddTail(new tv_load(de->d_name,global));
				}
			}
			closedir(d);
			if (!sub.Count()) SetHasNoSub();
		}
		tv_tree::Open();
	};
	void Close(void)
	{
		tv_tree::Close();
	};
};
class tv_helper : public tv_mime
{
	struct help_struct *help;
public:
	tv_helper(struct help_struct *h) : tv_mime(h->name)
	{
		help=h;
	};
	virtual void setupGUI(gadget *base)
	{
		((output*)FindObject(".plugins_mgr.name"))->Output(help->name);
		((output*)FindObject(".plugins_mgr.type"))->Output("helper");
		((output*)FindObject(".plugins_mgr.author"))->Output(help->help->helper_author());
		((output*)FindObject(".plugins_mgr.filename"))->Output(help->help->helper_filename());
		((checkbox*)FindObject(".plugins_mgr.disable"))->Lock();

	    if (help->help->helper_information())
			((autotextbox*)FindObject(".plugins_mgr.descr"))->ReplaceText(help->help->helper_information());
		else
			((autotextbox*)FindObject(".plugins_mgr.descr"))->ReplaceText("");
	};
};
class tv_encoder : public tv_mime
{
	struct encode *enc;
public:
	tv_encoder(struct encode *e) : tv_mime(e->name)
	{
		enc=e;
	};
	virtual void setupGUI(gadget *base)
	{
		((output*)FindObject(".plugins_mgr.name"))->Output(enc->name);
		((output*)FindObject(".plugins_mgr.type"))->Output("encoder");
		((output*)FindObject(".plugins_mgr.author"))->Output(enc->enc->encoder_author());
		((output*)FindObject(".plugins_mgr.filename"))->Output(enc->enc->encoder_filename());
		((checkbox*)FindObject(".plugins_mgr.disable"))->Lock();

	    if (enc->enc->encoder_information())
			((autotextbox*)FindObject(".plugins_mgr.descr"))->ReplaceText(enc->enc->encoder_information());
		else
			((autotextbox*)FindObject(".plugins_mgr.descr"))->ReplaceText("");
	};
};
class tv_plugin : public tv_mime
{
	plugins *pl;
public:
	tv_plugin(plugins *p) : tv_mime(p->plugin_name())
	{
		pl=p;
	};
	virtual void setupGUI(gadget *base)
	{
		((output*)FindObject(".plugins_mgr.name"))->Output(pl->plugin_name());
		((output*)FindObject(".plugins_mgr.type"))->Output(prgplugins->typeName(pl));
		((output*)FindObject(".plugins_mgr.author"))->Output(pl->plugin_author());
		((output*)FindObject(".plugins_mgr.filename"))->Output(pl->plugin_filename());
		((checkbox*)FindObject(".plugins_mgr.disable"))->Lock();

	    if (pl->plugin_information())
			((autotextbox*)FindObject(".plugins_mgr.descr"))->ReplaceText(pl->plugin_information());
		else
			((autotextbox*)FindObject(".plugins_mgr.descr"))->ReplaceText("");
	};
};
class tv_loaded : public tv_mime
{
	tv_tree	*helper;
	tv_tree	*encoder;
	tv_tree	*plugin;
public:
	tv_loaded(char *name) : tv_mime(name)
	{
		helper=new tv_mime("helpers");
		struct help_struct *hn=(help_struct*)prghelper->head;
		while (hn->base.next)
		{
			helper->AddTail(new tv_helper(hn));
			hn=(help_struct*)hn->base.next;
		}
		encoder=new tv_mime("encoders");
		struct encode *en=(encode*)prgencode->head;
		while (en->base.next)
		{
			encoder->AddTail(new tv_encoder(en));
			en=(encode*)en->base.next;
		}
		
		plugin=new tv_mime("general plugins");
		plugins *pn=NULL;
		while (pn=prgplugins->next(pn))
		{
			plugin->AddTail(new tv_plugin(pn));
		}

		AddTail(helper);
		AddTail(encoder);
		AddTail(plugin);
	};
	~tv_loaded()
	{
		RemoveAll(RAF_DELETE);
	};
};

// close button
static void MIMEprefs_close(void)
{
	treeview *tv=(treeview*)FindObject(".plugins_mgr.pluginsdir");
	tv->SetRoot(NULL);
	tvt1->RemoveAll(RAF_DELETE);
	delete tvt1;
	tvt1=NULL;
	appwindow *aw=(appwindow*)FindObject(".mainwindow.mimeprefs");
	aw->Close();
	DeleteObjectTree(aw->FullName(),0);
}
// use button
static void MIMEprefs_use(void)
{
	prghelper->readconfig();
	prgplugins->readconfig();
	MIMEprefs_close();
	writeconfig();
}
static void MIMEprefs_tv(gadget *g,int)
{
	tv_mime *tvm=(tv_mime*)((treeview*)g)->Selected();
	if (tvm)
		tvm->setupGUI((gadget*)FindObject(".plugins_mgr"));
	else
	{
		tvm=new tv_mime("");
		tvm->setupGUI((gadget*)FindObject(".plugins_mgr"));
		delete tvm;
	}
}
static void MIMEprefs_load(gadget *g,int)
{
	tv_load *tvm=(tv_load*)((treeview*)FindObject(".plugins_mgr.pluginsdir"))->Selected();
	if (tvm)
	{ // fehlt noch
		tvm->setDisabled((gadget*)FindObject(".plugins_mgr"));
	}
}

// helpers
static void MIMEprefs_helpers_list(void)
{
	listview *lv=(listview*)FindObject(".configcard.helpers.mimes");
	struct cap *c;
	char buf[MIMELEN*2];

	lv->RemoveAll();
	prgcap.sort();
	c=(struct cap*)prgcap.head;
	while (c->base.next)
	{
		xc_snprintf(buf,sizeof(buf),"%s/%s",c->mime1,c->mime2);
		lv->AddAtEnd(buf,c);
		c=(struct cap*)c->base.next;
	}
}
static void MIMEprefs_helpers_show(void)
{
	listview *lv=(listview*)FindObject(".configcard.helpers.mimes");
	struct cap *cp=(struct cap*)lv->GetData(lv->Selected());

	((input*)FindObject(".configcard.helpers.mime1"))->Default(cp->mime1);
	((input*)FindObject(".configcard.helpers.mime2"))->Default(cp->mime2);
	((choice*)FindObject(".configcard.helpers.command"))->Selected(cp->handlemail);

	if (cp->handlemail==SAVE || cp->handlemail==SHOW)
	{
		((fileinput*)FindObject(".configcard.helpers.shadow_input.start"))->Lock();
		((choice*)FindObject(".configcard.helpers.shadow_choice.hstart"))->Lock();
	}
	else if (cp->handlemail==PREVIEW)
	{
		((fileinput*)FindObject(".configcard.helpers.shadow_input.start"))->Lock();
		((choice*)FindObject(".configcard.helpers.shadow_choice.hstart"))->Unlock();
		((group*)FindObject(".configcard.helpers.shadow_choice"))->Show();

		((choice*)FindObject(".configcard.helpers.shadow_choice.hstart"))->Selected(prghelper->Count());
		for (int s=0;s<prghelper->Count();s++)
		{
			if (!strcscmp(cp->start,prghelper->helperarray()[s]))
			{
				((choice*)FindObject(".configcard.helpers.shadow_choice.hstart"))->Selected(s);
				s=prghelper->Count();
			}
		}
	}
	else if (cp->handlemail==COMMAND)
	{
		((fileinput*)FindObject(".configcard.helpers.shadow_input.start"))->Unlock();
		((choice*)FindObject(".configcard.helpers.shadow_choice.hstart"))->Lock();
		((group*)FindObject(".configcard.helpers.shadow_input"))->Show();
		((fileinput*)FindObject(".configcard.helpers.shadow_input.start"))->SetFile(cp->start);
	}
}
static void MIMEprefs_helpers_read(void)
{
	char m1[MIMELEN],m2[MIMELEN],cm[COMMANDLEN]="";
	((input*)FindObject(".configcard.helpers.mime1"))->Input(m1);
	((input*)FindObject(".configcard.helpers.mime2"))->Input(m2);

	switch (((choice*)FindObject(".configcard.helpers.command"))->Selected())
	{
		case 0: // Show
		case 1: // Save
			((fileinput*)FindObject(".configcard.helpers.shadow_input.start"))->Lock();
			((choice*)FindObject(".configcard.helpers.shadow_choice.hstart"))->Lock();
			break;
		case 2: // Helper
			((fileinput*)FindObject(".configcard.helpers.shadow_input.start"))->Lock();
			((choice*)FindObject(".configcard.helpers.shadow_choice.hstart"))->Unlock();
			((group*)FindObject(".configcard.helpers.shadow_choice"))->Show();
			strcpy(cm,prghelper->helperarray()[((choice*)FindObject(".configcard.helpers.shadow_choice.hstart"))->Selected()]);
			break;
		case 3: // Command
			((fileinput*)FindObject(".configcard.helpers.shadow_input.start"))->Unlock();
			((choice*)FindObject(".configcard.helpers.shadow_choice.hstart"))->Lock();
			((group*)FindObject(".configcard.helpers.shadow_input"))->Show();
			((fileinput*)FindObject(".configcard.helpers.shadow_input.start"))->GetFile(cm,sizeof(cm));
			break;
	}
	if (strlen(m1) && strlen(m1))
		prgcap.add(m1,m2,(HANDLEMAIL)((choice*)FindObject(".configcard.helpers.command"))->Selected(),cm);
}
static void MIMEprefs_helpers(gadget *g,int i)
{
	if (!strcmp(g->Name(),"mimes"))
	{
		MIMEprefs_helpers_read();
		MIMEprefs_helpers_show();
		MIMEprefs_helpers_list();
	}
	else if (!strcmp(g->Name(),"mime1") || !strcmp(g->Name(),"mime2") || !strcmp(g->Name(),"add") || !strcmp(g->Name(),"start") || !strcmp(g->Name(),"command") || !strcmp(g->Name(),"hstart"))
	{
		MIMEprefs_helpers_read();
		MIMEprefs_helpers_list();
	}
	else if (!strcmp(g->Name(),"del"))
	{
		char m1[MIMELEN],m2[MIMELEN];
		((input*)FindObject(".configcard.helpers.mime1"))->Input(m1);
		((input*)FindObject(".configcard.helpers.mime2"))->Input(m2);
		if (strlen(m1) && strlen(m2))
			prgcap.del(m1,m2);
		MIMEprefs_helpers_list();
		((input*)FindObject(".configcard.helpers.mime1"))->Default("");
		((input*)FindObject(".configcard.helpers.mime2"))->Default("");
	}
}

// MIME settings window
void generalMIMEprefs(void)
{
	if (FindObject(".mainwindow.mimeprefs")) return;

	char buf[400],buf2[420];
	int x,y;
	appwindow *aw=new appwindow;
	readerwin.Position(x,y);
	aw->ParentClass(readerwin);
	aw->Name("mimeprefs");

	pstatus("Open plugin & MIME preferences window");

	aw->InitSize(x,y,10);
	if (!aw->Init())
	{
		delete aw;
		pstatus("Error in opening window!");
		return;
	}
	aw->Title("Plugin & MIME preferences");
	aw->IconTitle("Plugin & MIME preferences");
	aw->Callback((void (*)(appwindow *))MIMEprefs_use);
	aw->Group(readerwin);
	aw->TransientFor(readerwin.window());

	group *maingp=new group;
	maingp->Name("cfggroup");
	maingp->Horiz();

	spacegroup *gp=new spacegroup;
	gp->Vert();
	gp->Name("maingroup");
	gp->YDistance(8);
	gp->Border(4);
	gp->Add(maingp);
	
	group *bgp=new group;
	bgp->Horiz();
	bgp->DialogMode();
	 fbutton *b=new fbutton;
	 b->Text(" _Close ");
	 b->Callback((void (*)(gadget*,int))MIMEprefs_use);
// 	bgp->Add(b);
//  	 b=new fbutton;
//  	 b->Text(" _Cancel ");
//  	 b->Callback((void (*)(gadget*,int))MIMEprefs_close);
	 b->ApplyXKey(XK_Escape);
 	bgp->Add(b);
	gp->Add(bgp);

	card *ccard=new card;
	ccard->Name("configcard");
	maingp->Add(ccard);

// 	g1=new bordergroup;
// 	g1->Name("encoding");
// 	g1->Vert();
// 	g1->YDistance(4);
// 	ccard->Add("Encoding",g1);
// 	 text *dummy=new text;
// 	 dummy->Name("dummy1");
// 	 dummy->FixedText();
// 	 dummy->NoUnderline();
// 	 dummy->AutoFormat();
// 	 dummy->Text("Nothing yet\n\nEncode settings: internal, external, nothing");
// 	 g1->Add(dummy);	 


	bordergroup *g1=new bordergroup;
	g1->Name("helpers");
	g1->Horiz();
	g1->XDistance(4);
	g1->Center();
	ccard->Add("MIME Helpers",g1);

	rubbergroup *rg=new rubbergroup;
	g1->Add(rg);
	rg->Horiz();
	rg->Name(NULL);
	 group *rgp=new group;
	 rgp->Name(NULL);
      listview *lv=new listview;
      lv->Name("mimes");
      lv->Font(FONT_FIXED);
      lv->Callback(MIMEprefs_helpers);
      rgp->Add(lv);
     rg->Add(rgp);
    group *ctlgroup=new group;
    rg->Add(ctlgroup);
     ctlgroup->Name(NULL);
     ctlgroup->YDistance(4);
      group *mimegroup=new group;
      mimegroup->Name(NULL);
      mimegroup->Center();
      mimegroup->Horiz();
       input *ip=new input;
       ip->Name("mime1");
       ip->MaxChars(MIMELEN-2);
       ip->Callback(MIMEprefs_helpers);
       ip->ApplyKey('m');
       ip->HelpText("primary MIME type\n(eg. text)");
      mimegroup->Add(ip);
       text *mimetext=new text;
       mimetext->FixedText();
       mimetext->Text("/");
      mimegroup->Add(mimetext);
       ip=new input;
       ip->Name("mime2");
       ip->MaxChars(MIMELEN-2);
       ip->Callback(MIMEprefs_helpers);
       ip->HelpText("secondary MIME type\n(eg. plain)");
      mimegroup->Add(ip);
     ctlgroup->Add(mimegroup,"_Mime");
      choice *ch=new choice;
      ch->Name("command");
	  static char *chit[]={"show","save","internal helper","command",NULL};
      ch->Items(chit);
      ch->Callback(MIMEprefs_helpers);
      ch->HelpText("what to do if MIME type\nis found");
     ctlgroup->Add(ch,"_Do");
      group *sg1=new group;
      sg1->Name("shadow_input");
      sg1->Vert();
       fileinput *fip=new fileinput;
       fip->Name("start");
       fip->Callback(MIMEprefs_helpers);
	   fip->Lock();
	   fip->HelpText("command syntax:\n <command> [<parameters>] %s");
	   fip->ModeCommand();
	  sg1->Add(fip,"start");
	  group *sg2=new group;
	   sg2->Name("shadow_choice");
	   sg2->Vert();
	   ch=new choice;
	   ch->Name("hstart");
	   ch->Items(prghelper->helperarray());
	   ch->Lock();
       ch->Callback(MIMEprefs_helpers);
       ch->HelpText("choose a helper");
	  sg2->Add(ch,"helper");
//      sg1->AddShadow(sg2);
     ctlgroup->Add(sg1);
      sg1->AddShadow(sg2); // Xclasses problem!

     ctlgroup->Add(new ruler);
	  b=new fbutton;
	  b->Name("add");
	  b->Text("_add/replace");
	  b->Callback(MIMEprefs_helpers);
	 ctlgroup->Add(b,NULL);
	  b=new fbutton;
	  b->Name("del");
	  b->Text("_remove");
	  b->Callback(MIMEprefs_helpers);
	 ctlgroup->Add(b,NULL);

	int config_ok=0;
	listgroup *lg1=new listgroup;
	if (prgplugins->makeconfig(lg1))
	{
		g1=new bordergroup;
		g1->Name("plugins_setup");
		g1->Horiz();
		g1->XDistance(4);
		g1->Center();
		ccard->Add("Plugins setup",g1);
		g1->Add(lg1);
		config_ok=1;
	}

	g1=new bordergroup;
	g1->Name("helpers_setup");
	g1->Horiz();
	g1->XDistance(4);
	g1->Center();
	ccard->Add("Helpers setup",g1);
	 lg1=new listgroup;
	 prghelper->makeconfig(lg1);
	g1->Add(lg1);


	g1=new bordergroup;
	g1->Name("plugins_mgr");
	g1->Horiz();
	g1->XDistance(4);
	g1->Center();
	ccard->Add("Plugin manager",g1);
	 rubbergroup *bg2=new rubbergroup;
	 bg2->Horiz();
	 bg2->Name(NULL);
	  tvt1=new tv_mime("XCmail plugins");
	  tvt1->AddTail(new tv_loaded("loaded plugins"));
	  strcpy(buf,GLOBALCFGDIR);
	  strcat(buf,"/plugins");
	  xc_snprintf(buf2,sizeof(buf2),"global (%s)",buf);
	  tvt1->AddTail(new tv_dir(buf2,buf,1));
	  strcpy(buf,getenv("HOME"));
	  strcat(buf,"/."PROGRAMNAME"/plugins");
	  xc_snprintf(buf2,sizeof(buf2),"user (%s)",buf);
	  tvt1->AddTail(new tv_dir(buf2,buf,0));
	  tvt1->Open();
	  treeview *tv=new treeview;
	  tv->Name("pluginsdir");
	  tv->SetRoot(tvt1);
	  tv->Callback(MIMEprefs_tv);
	 bg2->Add(tv);
	  group *gp2=new group;
	  gp2->Name(NULL);
	  gp2->Vert();
	  gp2->YDistance(4);
	   output *op=new output;
	   op->Name("name");
	   op->Text("Name");
	  gp2->Add(op);
	   op=new output;
	   op->Name("type");
	   op->Text("Type");
	  gp2->Add(op);
	   op=new output;
	   op->Name("author");
	   op->Text("Author");
	  gp2->Add(op);
	   op=new output;
	   op->Name("filename");
	   op->Text("Filename");
	  gp2->Add(op);
	   checkbox *cb=new checkbox;
	   cb->Name("disable");
	   cb->Text("do _not load plugin");
	   cb->Callback(MIMEprefs_load);
	  gp2->Add(cb,"");
	  gp2->Add(new ruler);
	   text *tx=new text;
	   tx->Name("title1");
	   tx->Center();
	   tx->Text("Description");
	  gp2->Add(tx);
	   autotextbox *atb=new autotextbox;
	   atb->Name("descr");
	  gp2->Add(atb);
	 bg2->Add(gp2);
	g1->Add(bg2);


	aw->SetGroup(gp);
	MIMEprefs_helpers_list();
	MIMEprefs_tv((gadget*)FindObject(".plugins_mgr.pluginsdir"),0);
	aw->SetIcon(f_xicon1.pixmap(),f_xicon1.shadowmap());
	aw->Create();
	
	if (config_ok)
	{
		prgplugins->setupconfig();
	}
}
