/* 
 * This file is part of XCmail and is subject to the 
 * XCmail license agreement
 */

/* $Id: mimeclass.cxx,v 2.8 2002/07/04 13:24:00 schmitzj Exp $ */


#include "configure.h"

#include <Xclasses/Xclasses.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>
#include <sys/stat.h>
#include <unistd.h>

#include "mailer.h"
#include "encoder.h"
#include "helper.h"


extern char *current_plugin_directory;
int sortmime(struct node *n1,struct node *n2)
{
	int r=strcmp(((mime*)n1)->mime1,((mime*)n2)->mime1);
	if (!r)
	{
		r=strcmp(((mime*)n1)->mime2,((mime*)n2)->mime2);
	}
	if (!r)
	{
		r=strcmp(((mime*)n1)->suffix,((mime*)n2)->suffix);
	}
	return r;
}
int sortcap(struct node *n1,struct node *n2)
{
	int r=strcmp(((cap*)n1)->mime1[0]=='*'?"zzz":((cap*)n1)->mime1,((cap*)n2)->mime1[0]=='*'?"zzz":((cap*)n2)->mime1);
	if (!r)
	{
		r=strcmp(((cap*)n1)->mime2[0]=='*'?"zzz":((cap*)n1)->mime2,((cap*)n2)->mime2[0]=='*'?"zzz":((cap*)n2)->mime2);
	}
	return r;
}
int sortenc(struct node *n1,struct node *n2)
{
	return strcscmp(((encode*)n1)->name,((encode*)n2)->name);
}
int sorthelp(struct node *n1,struct node *n2)
{
	return strcscmp(((help_struct*)n1)->name,((help_struct*)n2)->name);
}

mimetype::mimetype(void) : dlist()
{
}
mimetype::~mimetype()
{
	clear();
}
void mimetype::clear(void)
{
	struct mime *n;

	while (n=(mime*)RemTail())
	{
		delete n;
	}
}
int mimetype::load(FILE *f)
{
	char buf[1000],m1[MIMELEN],m2[MIMELEN];
	char *c1,*c2,cb;

	while (!feof(f))
	{
		if (fgets(buf,sizeof(buf),f)>0)
		{
			if (buf[strlen(buf)-1]=='\n') buf[strlen(buf)-1]=0;
			c1=buf;
			c2=buf;
			if (*c1 && *c1!='#')
			{
				while (*c2 && *c2!='/') c2++;
				cb=*c2;
				*c2=0;
				strcpy(m1,c1);
				if (cb) c1=c2+1; else continue;
				c2=c1;
				while (*c2 && *c2!=' ' && *c2!='\t') c2++;
				cb=*c2;
				*c2=0;
				strcpy(m2,c1);
				if (cb) c1=c2+1; else continue;
				add(m1,m2,c1);
			}
		}
	}
	Sort(sortmime);
}
int mimetype::save(FILE *f)
{
	Sort(sortmime);
	struct mime *n=(mime*)head;
	char *m1=NULL,*m2=NULL;

	fprintf(f,"# "PROGRAMNAME" mime.types\n");
	fprintf(f,"# mime1/mime2   suffix suffix ....\n\n");
	while (n->base.next)
	{
		if (m1 && m2 && !strcmp(n->mime1,m1) && !strcmp(n->mime2,m2))
		{
			fprintf(f," %s",n->suffix);
		}
		else
		{
			if (m1) fprintf(f,"\n");
			m1=n->mime1;
			m2=n->mime2;
			fprintf(f,"%s/%s\t\t\t%s",n->mime1,n->mime2,n->suffix);
		}
		n=(mime*)n->base.next;
	}
	if (m1) fprintf(f,"\n");
}
void mimetype::add(char *m1,char *m2,char *suf)
{
	char *s1,*s2;
	char sbuf[strlen(suf)+2];
	struct mime *n;

	strcpy(sbuf,suf);
	sbuf[strlen(suf)+1]=0;
	s2=sbuf;

	do
	{
		while (*s2 && (*s2==' ' || *s2=='\t'))
		{
			s2++;
		}
		s1=s2;

		while (*s2 && *s2!=' ' && *s2!='\t')
		{
			s2++;
		}
		*s2=0; s2++;
	
		n=(mime*)find(s1,NULL,NULL);
		if (!n)
		{
			n=new mime;
			AddTail((node*)n);
		}
		strcpy(n->mime1,m1);
		strcpy(n->mime2,m2);
		strcpy(n->suffix,s1);
		strcpy(n->mimetype,m1);
		strcat(n->mimetype,"/");
		strcat(n->mimetype,m2);
	}
	while (*s2);
}
mime *mimetype::find(char *suf,char *m1,char *m2)
{
	if (!suf || strlen(suf)==0) return NULL;

	struct mime *n=(mime*)head;

	while (n->base.next)
	{
		if (!strcmp(n->suffix,suf))
		{
			if (m1) strcpy(m1,n->mime1);
			if (m2) strcpy(m2,n->mime2);
			return n;
		}
		n=(mime*)n->base.next;
	}
	return NULL;
}
mime *mimetype::findfull(char *f,char *m1,char *m2)
{
	if (!f || strlen(f)==0) return NULL;

	char *c=NULL,*c2=f;
	struct mime *n=(mime*)head;

	while (*c2)
	{
		if (*c2=='.') c=c2+1;
		c2++;
	}
	if (c)
	{
		while (n->base.next)
		{
			if (!strcmp(n->suffix,c))
			{
				if (m1) strcpy(m1,n->mime1);
				if (m2) strcpy(m2,n->mime2);
				return n;
			}
			n=(mime*)n->base.next;
		}
	}
	return NULL;
}
void mimetype::mimeinlistview(class listview *lv,char *m1,char *m2)
{
	struct mime *n=(mime*)head;
	char *old=NULL;
	int i=1;
		
	while (n->base.next)
	{
		if (!old || strcmp(n->mimetype,old))
		{
			old=n->mimetype;
			lv->AddAtEnd(n->mimetype,n->mimetype);
			if (!strcmp(m1,n->mime1) && !strcmp(m2,n->mime2))
			{
				lv->Selected(i);
			}
			i++;
		}
		n=(mime*)n->base.next;
	}
}

//--------------------------------------------------------------------

mailcap::mailcap(void) : dlist()
{
}
mailcap::~mailcap()
{
	clear();
}
void mailcap::clear(void)
{
	struct cap *n;

	while (n=(cap*)RemTail())
	{
		delete n;
	}
}
void mailcap::sort(void)
{
	Sort(sortcap);
}
int mailcap::load(FILE *f)
{
	char buf[1000],m1[MIMELEN],m2[MIMELEN];
	char *c1,*c2,cb;

	while (!feof(f))
	{
		if (fgets(buf,sizeof(buf),f)>0)
		{
			if (buf[strlen(buf)-1]=='\n') buf[strlen(buf)-1]=0;
			c1=buf;
			c2=buf;
			if (*c1 && *c1!='#')
			{
				while (*c2 && *c2!='/') c2++;
				cb=*c2;
				*c2=0;
				strcpy(m1,c1);
				if (cb) c1=c2+1; else continue;
				c2=c1;
				while (*c2 && *c2!=' ' && *c2!='\t') c2++;
				cb=*c2;
				*c2=0;
				strcpy(m2,c1);
				if (cb) c1=c2+1; else continue;
				add(m1,m2,c1);
			}
		}
	}
}
int mailcap::save(FILE *f)
{
	struct cap *n=(cap*)head;

	fprintf(f,"# "PROGRAMNAME" mailcap\n");
	fprintf(f,"# mime1/mime2 SHOW|SAVE|PREVIEW|COMMAND [command]\n\n");

	while (n->base.next)
	{
		fprintf(f,"%s/%s\t\t",n->mime1,n->mime2);
		if (n->handlemail==SHOW)
		{
			fprintf(f,"%s\n","SHOW");
		}
		else if (n->handlemail==SAVE)
		{
			fprintf(f,"%s\n","SAVE");
		}
		else if (n->handlemail==PREVIEW)
		{
			fprintf(f,"%s\t%s\n","PREVIEW",n->start);
		}
		else if (n->handlemail==COMMAND)
		{
			fprintf(f,"%s\t%s\n","COMMAND",n->start);
		}
		n=(cap*)n->base.next;
	}
}
void mailcap::add(char *m1,char *m2,char *cmd)
{
	char sbuf[COMMANDLEN];
	struct cap *n;
	enum HANDLEMAIL t;
	int done=0;
	char *c=cmd;

	n=(cap*)find(m1,m2,t,sbuf);
	if (!n || strcscmp(m1,n->mime1) || strcscmp(m2,n->mime2))
	{
		n=new cap;
		AddTail((node*)n);
	}
	strcpy(n->mime1,m1);
	strcpy(n->mime2,m2);
	n->start[0]=0;
	while (*c && (*c==' ' || *c=='\t')) c++;
	cmd=strdup(c);
	c=cmd;
	while (*c && *c!=' ' && *c!='\t') c++;
	if (!*c) done=1; else *c=0;
	c++;

	if (!strcmp(cmd,"SHOW"))
	{
		n->handlemail=SHOW;
	}
	else if (!strcmp(cmd,"SAVE"))
	{
		 n->handlemail=SAVE;
	}
	else if (!strcmp(cmd,"PREVIEW"))
	{
		n->handlemail=PREVIEW;
		char *c2=c; // fr kompatiebel zu < 0.10b5
		while (*c2 && *c2!='(') c2++;
		*c2=0;
	}
	else if (!strcmp(cmd,"COMMAND"))
	{
		n->handlemail=COMMAND;
	}
	else
	{
		n->handlemail=COMMAND;
	}
	if (!done)
	{
		while (*c && (*c==' ' || *c=='\t')) c++;
		strcpy(n->start,c);
	}
	::free(cmd);
}
void mailcap::add(char *m1,char *m2,HANDLEMAIL type,char *cmd)
{
	char sbuf[COMMANDLEN];
	struct cap *n;
	enum HANDLEMAIL t;
	int done=0;
	char *c=cmd;

	n=(cap*)find(m1,m2,t,sbuf);
	if (!n || strcscmp(m1,n->mime1) || strcscmp(m2,n->mime2))
	{
		n=new cap;
		AddTail((node*)n);
	}
	strcpy(n->mime1,m1);
	strcpy(n->mime2,m2);
	n->start[0]=0;

	n->handlemail=type;
	while (*c && (*c==' ' || *c=='\t')) c++;
	strcpy(n->start,c);
}
cap *mailcap::find(char *m1,char *m2,enum HANDLEMAIL &type,char *cmd,char *name)
{
	cap *r=NULL;
	if ((!strcscmp(m1,"application") && !strcscmp(m2,"octet-stream"))||!(r=find(m1,m2,type,cmd)))
	{
		char _m1[MIMELEN],_m2[MIMELEN];
		if (prgmime.findfull(name,_m1,_m2))
		{
			r=find(_m1,_m2,type,cmd);
		}
		else
			r=find(m1,m2,type,cmd);
	}
	else
		r=find(m1,m2,type,cmd);
	return r;
}
cap *mailcap::find(char *m1,char *m2,enum HANDLEMAIL &type,char *cmd)
{
	cmd[0]=0;
	struct cap *n=(cap*)head;

	if (!strlen(m1) && !strlen(m2)) return 0;

	while (n->base.next)
	{
		if (!strcscmp(n->mime1,m1) && !strcscmp(n->mime2,m2))
		{
			type=n->handlemail;
			strcat(cmd,n->start);
			return n;
		}
		n=(cap*)n->base.next;
	}

	n=(cap*)head;
	while (n->base.next)
	{
		if (!strcscmp(n->mime1,m1) && !strcscmp(n->mime2,"*"))
		{
			type=n->handlemail;
			strcat(cmd,n->start);
			return n;
		}
		n=(cap*)n->base.next;
	}

	n=(cap*)head;
	while (n->base.next)
	{
		if (!strcscmp(n->mime1,"*") && !strcscmp(n->mime2,m2))
		{
			type=n->handlemail;
			strcat(cmd,n->start);
			return n;
		}
		n=(cap*)n->base.next;
	}

	n=(cap*)head;
	while (n->base.next)
	{
		if (!strcscmp(n->mime1,"*") && !strcscmp(n->mime2,"*"))
		{
			type=n->handlemail;
			strcat(cmd,n->start);
			return n;
		}
		n=(cap*)n->base.next;
	}
	return 0;
}
void mailcap::del(char *m1,char *m2)
{
	struct cap *n=(cap*)head;

	while (n->base.next)
	{
		if (!strcscmp(n->mime1,m1) && !strcscmp(n->mime2,m2))
		{
			Remove((node*)n);
			delete n;
			return;
		}
		n=(cap*)n->base.next;
	}
}

//--------------------------------------------------------------------

static class encoder_command enc_fl;
encoding::encoding(void) : dlist()
{
	encarr=NULL;
}
encoding::~encoding()
{
	clear();
	delete *encarr;
}
void encoding::clear(void)
{
	struct encode *n;

	while (n=(encode*)RemTail())
	{
		delete n;
	}
}
void encoding::sort(void)
{
	Sort(sortenc);
}
char **encoding::encoderarray(void)
{
	if (!encarr)
	{
		if (Count())
		{
			encarr=new char*[Count()+1];
			int i=0;
			struct encode *n=(encode*)head;

			while (n->base.next)
			{
				if ((n->enc->flags & EF_ENCODE) && !strcmp(n->name,n->enc->name))
				{
					encarr[i]=n->name;
					i++;
				}
				n=(encode*)n->base.next;
			}
			encarr[i]=NULL;
		}
	}
	return encarr;
}
int encoding::load(FILE *f)
{
	char buf[200],*c1;

	while (!feof(f))
	{
		if (fgets(buf,sizeof(buf),f)>0)
		{
			if (buf[strlen(buf)-1]=='\n') buf[strlen(buf)-1]=0;
			c1=buf;
			if (*c1 && *c1!='#')
			{
				while (*c1 && *c1!=' ' && *c1!='\t') c1++;
				if (*c1)
				{
					*c1=0;
					c1++;
					add(buf,c1);
				}
			}
		}
	}
	Sort(sortenc);
}
int encoding::save(FILE *f)
{
	struct encode *n=(encode*)head;

	fprintf(f,"# "PROGRAMNAME" encoding\n");
	fprintf(f,"# encoding coder|INTERNAL decoder\n");
	fprintf(f,"# (use %%s for name place holder)\n\n");

	while (n->base.next)
	{
		fprintf(f,"%s\t\t",n->name);
		if (n->internal)
			fprintf(f,"INTERNAL\n");
		else
			fprintf(f,"\"%s\"\t\"%s\"\n",n->code,n->decode);
		n=(encode*)n->base.next;
	}
}
void encoding::set(struct encode *n,char *cmd)
{
	char *c1,*c2;
	int quote=0;
	n->internal=0;
	n->code[0]=0;
	n->decode[0]=0;

	c1=cmd;
	while (*c1 && (*c1==' ' || *c1=='\t')) c1++;
	if (*c1=='"')
	{
		c1++;
		quote=1;
	}
	if (!*c1) return;
	strcpy(n->code,c1);
	c2=n->code;
	while (*c2 && ((quote && *c2!='"') || (!quote && *c2!=' ' && *c2!='\t')))
	{
		c1++;
		c2++;
	}
	*c2=0;
	if (!*c1) return;

	quote=0;
	c1++;
	while (*c1 && (*c1==' ' || *c1=='\t')) c1++;
	if (*c1=='"')
	{
		c1++;
		quote=1;
	}
	if (!*c1) return;

	strcpy(n->decode,c1);
	c2=n->decode;
	while (*c2 && ((quote && *c2!='"') || (!quote && *c2!=' ' && *c2!='\t'))) c2++;
	*c2=0;
}
void encoding::add(char *enc,char *cmd)
{
	while (*cmd && (*cmd==' ' || *cmd=='\t')) cmd++;
	if (!strcncmp("INTERNAL",cmd)) return;

	struct encode *n=(encode*)head;

	while (n->base.next)
	{
		if (!strcscmp(n->name,enc))
		{
			set(n,cmd);
			return;
		}
		n=(encode*)n->base.next;
	}
	n=new encode;
	AddTail((node*)n);
	strcpy(n->name,enc);
	set(n,cmd);
	n->internal=0;
	n->isintern=0;
}
void encoding::add(char *enc,class encoder *e)
{
	char buf[strlen(enc)+1];
	char *p=buf,*c;
	int done=0;
	strcpy(buf,enc);

	while (c=strtok(p,";"))
	{
		struct encode *n=(encode*)head;
		p=NULL;
		done=0;
		while (*c==' ') c++;

		while (n->base.next && !done)
		{
			if (!strcscmp(n->name,c))
			{
				n->internal=1;
				n->enc=e;
				done=1;
			}
			n=(encode*)n->base.next;
		}
		if (!done)
		{
			n=new encode;
			AddTail((node*)n);
			strcpy(n->name,c);
			n->enc=e;
			n->internal=1;
			n->isintern=1;
		}
	}
}
class encoder *encoding::find(char *enc)
{
	struct encode *n=(encode*)head;

	while (n->base.next)
	{
		if (!strcscmp(n->name,enc))
		{
			if (!n->internal)
			{
				enc_fl.name=n->name;
				enc_fl.command(n->code,n->decode);
				return &enc_fl;
			}
			return n->enc;
		}
		n=(encode*)n->base.next;
	}
	return NULL;
}

//--------------------------------------------------------------------

encoder::encoder(char *fname,char *fauthor,int flag,char *finfo)
{
	if (!prgencode) prgencode=new encoding;
	if (fname) prgencode->add(fname,this);
	name=NULL;
	if (fname)
	{
		name=strdup(fname);
		fname=name;
		while (*fname && *fname!=';') fname++;
		*fname=0;
	}
	author=fauthor;
	information=finfo;
	flags=flag;
	filename=current_plugin_directory?strdup(current_plugin_directory):(char*)NULL;
}
encoder::~encoder()
{
	if (filename) free(filename);
	if (name) free(name);
	name=NULL;
}
char *encoder::encoder_name(void)
{
	return name;
}
char *encoder::encoder_author(void)
{
	return author;
}
char *encoder::encoder_information(void)
{
	return information;
}
char *encoder::encoder_filename(void)
{
	return filename;
}
void encoder::encoder_filename(char *f)
{
	if (filename) free(filename);
	filename=f?strdup(f):(char*)NULL;
}
char *encoder::data2mail(char *,int size,int &newsize,char *path,char *filename)
{
	return NULL;
}
char *encoder::mail2data(char *,int size,int &newsize,char *path,char *filename)
{
	return NULL;
}
void encoder::dispose(char *)
{
}
char *encoder::egets(char *to,int size,char **buf)
{
	char *c=to;
	int i=0;

	if (!**buf) return NULL;

	while (**buf && **buf!='\n' && **buf!='\r' && i<size)
	{
		*to=**buf;
		to++; (*buf)++;
		i++;
	}
	if (**buf=='\r') (*buf)++;
	if (**buf=='\n') (*buf)++;
	*to=0;
	return c;
}
void encoder::eputs(char *txt,char **buf)
{
	strcpy(*buf,txt);
	while (**buf) (*buf)++;
}

//--------------------------------------------------------------------

encoder_command::encoder_command(void) : encoder(NULL,NULL)
{
	codecom=NULL;
	decodecom=NULL;
}
encoder_command::~encoder_command()
{
}
void encoder_command::command(char *c,char *dc)
{
	codecom=c;
	decodecom=dc;
}
char *encoder_command::data2mail(char *d,int size,int &newsize,char *path,char *filename)
{
	char b1[100],b2[100],c[400],cm[400];
	FILE *file;
	struct stat st;

	xc_snprintf(b1,sizeof(b1),TMPDIR"/"PROGRAMNAME"_encode_from_%d",getpid());
	xc_snprintf(b2,sizeof(b2),TMPDIR"/"PROGRAMNAME"_encode_to_%d",getpid());
	xc_snprintf(cm,sizeof(cm),codecom,b1);
	xc_snprintf(c,sizeof(c),"%s >%s",cm,b2);

	if (file=fopen(b1,"w"))
	{
		fwrite(d,1,size,file);
		fclose(file);
		system(c);

		if (file=fopen(b2,"r"))
		{
			stat(b2,&st);
			newsize=st.st_size;
			d=new char[newsize];
			fread(d,1,newsize,file);
			fclose(file);
			remove(b1);
			remove(b2);
			return d;
		}
		remove(b1);
		remove(b2);
	}
	return NULL;
}
char *encoder_command::mail2data(char *d,int size,int &newsize,char *path,char *filename)
{
	char b1[100],b2[100],c[400],cm[400];
	FILE *file;
	struct stat st;

	xc_snprintf(b1,sizeof(b1),TMPDIR"/"PROGRAMNAME"_decode_from_%d",getpid());
	xc_snprintf(b2,sizeof(b2),TMPDIR"/"PROGRAMNAME"_decode_to_%d",getpid());
	xc_snprintf(cm,sizeof(cm),decodecom,b1);
	xc_snprintf(c,sizeof(c),"%s >%s",cm,b2);

	if (file=fopen(b1,"w"))
	{
		fwrite(d,1,size,file);
		fclose(file); printf("%s\n",c);
		system(c);

		if (file=fopen(b2,"r"))
		{
			stat(b2,&st);
			newsize=st.st_size;
			d=new char[newsize];
			fread(d,1,newsize,file);
			fclose(file);
			remove(b1);
			remove(b2);
			return d;
		}
		remove(b1);
		remove(b2);
	}
	return NULL;
}
void encoder_command::dispose(char *d)
{
	delete d;
}

//--------------------------------------------------------------------

helpers::helpers(void) : dlist()
{
	pbuf=NULL;
	harr=NULL;
}
helpers::~helpers()
{
	clear();
	if (pbuf)
	{
		delete pbuf;
		pbuf=NULL;
	}
}
void helpers::clear(void)
{
	struct help_struct *n;

	while (n=(help_struct*)RemTail())
	{
		delete n;
	}
	if (harr)
	{
		char **c=harr;
		while (*c)
		{
			::free(*c);
			c++;
		}
		delete [] harr;
		harr=NULL;
	}
}
void helpers::sort(void)
{
	Sort(sorthelp);
}
void helpers::makeconfig(class listgroup *g)
{
	if (Count())
	{
		struct help_struct *n=(help_struct*)head;

		while (n->base.next)
		{
			bordergroup *g1=new bordergroup;
			g1->Vert();
			g1->YDistance(3);
			g1->Title(n->name);
			 output *op=new output;
			 op->Text("Helper name");
			 op->Output(n->name);
			g1->Add(op);
//  			 op=new output;
//  			 op->Text("Author");
//  			 op->Output(n->help->author);
//  			g1->Add(op);
			 op=new output;
			 op->Text("usefull mime types");
			 op->Output(n->help->mimes);
			g1->Add(op);

			if (n->help->flags & HL_SETUP)
			{
				g1->Add(new ruler);
				text *txt=new text;
				txt->Name(NULL);
				txt->Center();
				txt->Text("Helper dependent settings");
				g1->Add(txt);
				n->configgp=n->help->makeconfig();
				g1->Add(n->configgp);
			}
			else
			{
				g1->Add(new ruler);
				n->configgp=NULL;
				text *t=new text;
				t->Center();
				t->Text("No extra settings for this helper.");
				g1->Add(t);
			}
// 		    if (n->help->information)
// 		    {
// 				g1->Add(new ruler);
// 				text *txt=new text;
// 				txt->Name(NULL);
// 				txt->Center();
// 				txt->Text("Description");
// 				g1->Add(txt);
// 				autotextbox *atb=new autotextbox;
// 				atb->AppendText(n->help->information);
// 				atb->Weight(40);
// 				g1->Add(atb);
// 			}
			g->Add(n->name,g1);
			n=(help_struct*)n->base.next;
		}
	}
}
void helpers::readconfig(void)
{
	if (Count())
	{
		struct help_struct *n=(help_struct*)head;

		while (n->base.next)
		{
			if (n->help->flags & HL_SETUP)
			{
				n->help->readconfig(n->configgp);
			}
			n->configgp=NULL;
			n=(help_struct*)n->base.next;
		}
	}
}
void helpers::readconfigfiles(void)
{
	if (Count())
	{
		struct help_struct *n=(help_struct*)head;

		while (n->base.next)
		{
			if (n->help->flags & HL_SETUP)
			{
				n->help->readconfigfile();
			}
			n=(help_struct*)n->base.next;
		}
	}
}
void helpers::writeconfigfiles(void)
{
	if (Count())
	{
		struct help_struct *n=(help_struct*)head;

		while (n->base.next)
		{
			if (n->help->flags & HL_SETUP)
			{
				n->help->writeconfigfile();
			}
			n=(help_struct*)n->base.next;
		}
	}
}
char **helpers::helperarray(void)
{
	if (!harr)
	{
		struct help_struct *n=(help_struct*)head;
		harr=new char*[Count()+2];
		char **c=harr;

		while (n->base.next)
		{
			*c=strdup(n->name);
			c++;
			n=(help_struct*)n->base.next;
		}
		*c=" ";
		c++;
		*c=NULL;
	}
	return harr;
}
void helpers::add(class helper *h)
{
	if (!find(h->name))
	{
		struct help_struct *n;
		n=new help_struct;
		AddHead((node*)n);
		strcpy(n->name,h->name);
		n->help=h;
	}
}
class helper *helpers::find(char *cmd)
{
	char c[strlen(cmd)+1],*c1;
	struct help_struct *n=(help_struct*)head;
	strcpy(c,cmd);
	c1=c;
	while (*c1 && *c1!='(')
	{
		if (*c1==' ') *c1=0;
		c1++;
	}
	*c1=0;

	while (n->base.next)
	{
		if (!strcscmp(n->name,c))
		{
			return n->help;
		}
		n=(help_struct*)n->base.next;
	}
	return NULL;
}

//--------------------------------------------------------------------

helper::helper(char *fname,char *fauthor,char *fmimes,int flag,char *finfo)
{
	if (!prghelper) prghelper=new helpers;
	name=fname;
	author=fauthor;
	mimes=fmimes;
	flags=flag;
	information=finfo;
	mail=NULL;
	partnb=0;
	if (fname) prghelper->add(this);
	filename=current_plugin_directory?strdup(current_plugin_directory):(char*)NULL;
}
helper::helper(class helper *h)
{
	flags=h->flags;
	name=h->name;
	author=h->author;
	mimes=h->mimes;
	information=NULL;
	mail=NULL;
	partnb=0;
	filename=NULL;
}
helper::~helper()
{
	if (filename) free(filename);
}
char *helper::helper_name(void)
{
	return name;
}
char *helper::helper_author(void)
{
	return author;
}
char *helper::helper_information(void)
{
	return information;
}
char *helper::helper_filename(void)
{
	return filename;
}
void helper::helper_filename(char *f)
{
	if (filename) free(filename);
	filename=f?strdup(f):(char*)NULL;
}
char *helper::egets(char *to,int size,char **buf)
{
	char *c=to;
	int i=0;

	if (!**buf) return NULL;

	while (**buf && **buf!='\n' && i<size)
	{
		*to=**buf;
		to++; (*buf)++;
		i++;
	}
	if (**buf=='\n') (*buf)++;
	*to=0;
	return c;
}
void helper::eputs(char *txt,char **buf)
{
	strcpy(*buf,txt);
	while (**buf) (*buf)++;
}
char *helper::getheader(char *h)
{
// 	if (mail)
// 	{
// 		return ((class mail*)mail)->getContentTag(h,partnb);
// 	}
	return NULL;
}
char *helper::mime1(void)
{
// 	if (mail)
// 	{
// 		return ((class mail*)mail)->submime1(partnb);
// 	}
	return NULL;
}
char *helper::mime2(void)
{
// 	if (mail)
// 	{
// 		return ((class mail*)mail)->submime2(partnb);
// 	}
	return NULL;
}

class helper *helper::clone(char *in,int insize)
{
	return NULL;
}

char *helper::go(int &newsize)
{
	newsize=0;
	return NULL;
}
int helper::makegadget(class visible *)
{
	return 0;
}
int helper::makemenu(class menu *,int menu)
{
	return 0;
}
void helper::menuhandler(class menu *,int mid)
{
}
group *helper::makeconfig(void)
{
	return NULL;
}
void helper::readconfig(group *)
{
}
void helper::readconfigfile(void)
{
}
void helper::writeconfigfile(void)
{
}
