/* 
 * This file is part of XCmail and is subject to the 
 * XCmail license agreement
 */

/* $Id: misc.cxx,v 2.22 2001/10/29 22:46:26 schmitzj Exp $ */


#include "configure.h"

#define USE_GNUREGEX 1

#include <ctype.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#include <pwd.h>
#include <sys/types.h>
#include <sys/utsname.h>
#include <netdb.h>

#ifdef USE_GNUREGEX
#include "xc_regex.h"
#else
#ifdef HAVE_REGEX_H
#define _REGEX_RE_COMP
#include <regex.h>
#endif
#ifdef HAVE_RE_COMP_H
#include <re_comp.h>
#endif
#endif

#include "mailer.h"
#include "descrypt.h"
#include "stringTokenizer.h"
#include "extraggs.h"

int strcncmp(char *s1,char *s2)
{ // strcasencmp
  register unsigned char *p1 = (unsigned char *) s1;
  register unsigned char *p2 = (unsigned char *) s2;
  register int ret=1;
  unsigned char c1;

  for (;*p1 && !(ret = (c1 = tolower(*p1)) - tolower(*p2)); p1++, p2++)
    if (c1 == '\0') break;
  return ret;
}
int strnncmp(char *s1,char *s2)
{ // strncmp
  register unsigned char *p1 = (unsigned char *) s1;
  register unsigned char *p2 = (unsigned char *) s2;
  register int ret=1;
  unsigned char c1;

  for (;*p1 && !(ret = (c1 = *p1) - *p2); p1++, p2++)
    if (c1 == '\0') break;
  return ret;
}

int strcscmp(char *s1,char *s2)
{ // strcasecmp
  register unsigned char *p1 = (unsigned char *) s1;
  register unsigned char *p2 = (unsigned char *) s2;
  register int ret=1;
  unsigned char c1;

  for (;!(ret = (c1 = tolower(*p1)) - tolower(*p2)); p1++, p2++)
    if (c1 == '\0') break;
  return ret;
}

int strscncmp(char *s1,char *s2)
{ // strsubcasencmp
  register char *p1 = (char *) s2;
  register int ret=1;

  for(;*p1 && (ret=strcncmp(s1,p1)); p1++) ;
  return ret;
}
int strsnncmp(char *s1,char *s2)
{ // strsubncmp
  register char *p1 = (char *) s2;
  register int ret=1;

  for(;*p1 && (ret=strnncmp(s1,p1)); p1++) ;
  return ret;
}

int strregcmp(char *s1,char *s2)
{
#ifdef USE_GNUREGEX
	xc_re_syntax_options=RE_SYNTAX_POSIX_EXTENDED;
	struct xc_re_pattern_buffer p;
	p.translate=NULL;
	p.fastmap=0;
	p.buffer=NULL;
	p.allocated=0;
	if (!xc_re_compile_pattern(s1,strlen(s1),&p))
	{
		int m=xc_re_match(&p,s2,strlen(s2),0,NULL);
		xc_regfree(&p);
		return m>0?0:1;
	}
	else
		return 1;
#else
	if (!re_comp(s1))
		return !re_exec(s2);
	else
		return 1;
#endif
}

int struhcmp(char *s1,char *s2)
{
	register unsigned char *p1 = (unsigned char *) s1;
	register unsigned char *p2 = (unsigned char *) s2;
	register int ret=1;
	unsigned char c1;

	if (programconfig.addresscmp_dns)
	{
		for (;!(ret = (c1 = tolower(*p1)) - tolower(*p2)); p1++, p2++)
			if (c1 == '\0' || c1=='@') break;
	}
	else
	{
		for (;!(ret = (c1 = tolower(*p1)) - tolower(*p2)); p1++, p2++)
			if (c1 == '\0') break;
	}

	if (!ret && programconfig.addresscmp_dns)
	{
		char *locadd=NULL;
		char *h=getHostFromEMail(s1);
		if (h && *h)
		{
			struct hostent *host_info;
			if (host_info=gethostbyname(h))
			{
				locadd=new char[host_info->h_length];
				memcpy(locadd,host_info->h_addr,host_info->h_length);
			}
		}

		h=getHostFromEMail(s2);
		if (h && *h)
		{
			struct hostent *host_info;
			if (locadd && (host_info=gethostbyname(h)))
			{
				ret=memcmp(locadd,host_info->h_addr,host_info->h_length);
			}
			else
			{
				for (;!(ret = (c1 = tolower(*p1)) - tolower(*p2)); p1++, p2++)
					if (c1 == '\0') break;
			}
		}
		else
		{
			for (;!(ret = (c1 = tolower(*p1)) - tolower(*p2)); p1++, p2++)
				if (c1 == '\0') break;
		}
		delete [] locadd;
	}
	return ret;
}

int strlistcmp(int (*cmp)(char *,char *),char *s1,char *s2)
{
	int l=strlen(s1);
	char *buf=new char[l+1];
	strcpy(buf,s1);
	char *c1=buf,*c2=buf;
	int ret=0;

	while (1)
	{
		if (*c1==',' || *c1==0)
		{
			char *c3=c1;
			int brk=*c1==0;
			*c1=0;
			while (*c2 && *c2==' ') c2++;
			do
			{
				c1--;
				if (*c1==' ') *c1=0;
			}
			while (c1>c2 && !*c1);
			ret=ret || !cmp(c2,s2);
			if (ret) break;
			c1=c3;
			c2=c1+1;
			if (brk) break;
		}
		c1++;
	}

	delete [] buf;
	return !ret;
}

void xc_strncpy(char *to,char *from,int i)
{
	*to=0;
	strncat(to,from,i-1);
}
// ------------------------------------------------------------------------
char *getPW(class appwindow *par)
{
	static char pw[40]="";
	appwindow aw;
	int x,y,w,h;

	if (!strlen(pw))
	{
		strdecodecpy(pw,programconfig.pgppassphrase);
	}
	par->WMPosition(x,y);
	par->Size(w,h);
	aw.ParentClass(par);
	aw.Name("passphrasewindow");
	aw.InitSize(110);
	if (!aw.Init())
	{
		return NULL;
	}
	aw.Adjust(x,y,w,h);
	aw.TransientFor(par);
	aw.Title("please enter");
	aw.IconTitle("");

	bordergroup sg;
	char buf[200];
	if (strlen(programconfig.pgpid))
		xc_snprintf(buf,sizeof(buf),"pass phrase of %s",programconfig.pgpid);
	else
		xc_snprintf(buf,sizeof(buf),"your pass phrase");
	sg.Title(buf);
	sg.Vert();
	sg.YDistance(10);
	 input pp;
	 pp.InvisibleMode();
	 pp.MaxChars(sizeof(pw)-1);
	 pp.Default(pw);
	 pp.ID(1);
	sg.Add(pp);
	 group g1;
	 g1.Horiz();
	 g1.DialogMode();
	  button b1;
	  b1.Text("_Ok");
	  b1.ID(1);
	 g1.Add(b1);
	  button b2;
	  b2.Text("_Cancel");
	  b2.ID(3);
	 g1.Add(b2);
	sg.Add(g1);

	aw.SetGroup(sg);
	aw.Create();
	aw.BlockExcept();
	pp.ActivateKey();
	b2.ApplyXKey(XK_Escape);

	int r=aw.NextEvent();
	if (r==EV_WINDOWCLOSE)
	{
		r=2;
	}
	else if (EV_GADGET)
	{
		r=aw.ID();
	}
	else
	{
		r=2;
	}
	aw.UnblockExcept();
	aw.Close();

	if (r==1)
	{
		pp.Input(pw);
		return pw;
	}
	return NULL;
}

// ------------------------------------------------------------------------
#define DES_KEY_MAGIC	"Amiga"
static class DESCrypt des;

int strencodecpy(char *to,char *from)
{
	if (from[0])
	{
		int dummy;
		char *p;

		des.SetKey(DES_KEY_MAGIC);

		if (p=des.Encode(from,&dummy))
		{
			char buf[3];
			to[0]=0;
			for(int i=0;i<dummy;i++)
			{
				xc_snprintf(buf,sizeof(buf),"%02x",(int)((unsigned char)p[i]));
				strcat(to,buf);
			}
			free(p);
		}
	}
	else
		to[0]=0;
	return 0;
}
int strdecodecpy(char *to,char *from)
{
	if (from[0])
	{
		char *p;
		des.SetKey(DES_KEY_MAGIC);

		char buf[3];
		char in[100];
		int l=strlen(from),blen=0;
		for(int i=0;i<l;i+=2)
		{
			buf[0]=from[i];
			buf[1]=from[i+1];
			buf[2]=0;
			in[blen++]=strtol(buf,NULL,16);
		}

		if (p=des.Decode(in,blen))
		{
			strcpy(to,p);
			free(p);
		}
	}
	else
		to[0]=0;
	return 0;
}

// ------------------------------------------------------------------------
int myfwrite(char *b,FILE *f)
{ // obsolete!
	int err=0, n=0;
	char *c1=b;
	while (*c1)
	{
		if (!strncmp(c1,"\nFrom",5))
		{
			err=err||fputc('\n',f)==EOF;
			err=err||fputc('>',f)==EOF;
			n+=2;
		}
		else
		{
			err=err||fputc(*c1,f)==EOF;
			n++;
		}
		c1++;
	}
	return err ? -1 : n;
}

int myfprintf(FILE *file, char *format, ...)
{
	static char buf[1000];
	va_list valist;

	va_start(valist,format);
	int n=xc_vsnprintf(buf,sizeof(buf), format, valist);
	va_end(valist);

	return (n<0||fputs(buf,file)<0)?-1:n;
}

// ------------------------------------------------------------------------
char *getusername(char *buf)
{
	char *u=getenv("USER");
	if (!u)
	{
		u=getenv("LOGNAME");
		if (!u)
		{
			struct passwd *pwd=getpwuid(getuid());
			if (!pwd)
			{
				fprintf(stderr,"Maybe you should get a decent Unix clone, looser! Use Linux!\n"); 
				u="looser";
			}
			else
			{
				u=pwd->pw_name;
			}
		}
	}
	if (buf)
	{
		strcpy(buf,u);
	}
	return u;
}

// ------------------------------------------------------------------------
int Alert(appwindow *aw, char *s, ...)
{

	static char buf[1000];
	va_list valist;

	va_start(valist,s);
	int n=xc_vsnprintf(buf,sizeof(buf), s, valist);
	va_end(valist);

	char *b = strchr(buf, '|');
	if (!b)
		b = "_Ok";
	else
		*(b++) = 0;

	if (n>0)
	{
		if (aw)
		{
			easy_dialog ed;
			ed.Text(buf);
			ed.Options(b);
			return ed.Start(aw);
		}
		else
		{
			fprintf(stderr, "xc-mail: Unable to open Alert!");
			fprintf(stderr, "xc-mail: Text:    \"%s\"\n", buf);
			fprintf(stderr, "xc-mail: Options: \"%s\"\n", b);
			fprintf(stderr, "xc-mail: choosing first option as default!\n\n");
			return 1;
		}
	}
	return -1;
}
int tAlert(appwindow *aw, char *s, ...)
{

	static char buf[1000];
	va_list valist;

	va_start(valist,s);
	int n=xc_vsnprintf(buf,sizeof(buf), s, valist);
	va_end(valist);

	char *b = strchr(buf, '|');
	if (!b)
		b = "_Ok";
	else
		*(b++) = 0;

	if (n>0)
	{
		if (aw)
		{
			autoeasy_dialog *ed=new autoeasy_dialog;
			ed->Text(buf);
			ed->Options(b);
			ed->WaitMode();
			return ed->Start(aw);
		}
		else
		{
			fprintf(stderr, "xc-mail: Unable to open Alert!");
			fprintf(stderr, "xc-mail: Text:    \"%s\"\n", buf);
			fprintf(stderr, "xc-mail: Options: \"%s\"\n", b);
			fprintf(stderr, "xc-mail: choosing first option an default!\n\n");
			return 1;
		}
	}
	return -1;
}

// ------------------------------------------------------------------------
char *getHostFromEMail(char *em)
{
	while (*em && *em++!='@');
	return em;
}

// ------------------------------------------------------------------------
char *strfilename(char *f)
{
	char *c1=f;
	while (*c1)
	{
		if (*c1=='/') f=c1+1;
		c1++;
	}
	return f;
}

// ------------------------------------------------------------------------
int developLevel(char *tok)
{
	char *l=getenv("XCMAIL_DEVLEVEL");
	int r=0;
	if (l)
	{
		tokenizer tk(l);
		for (int i=0;i<tk.count();i++)
		{
			if (!strcncmp(tk[i],tok))
			{
				return 1;
			}
		}
	}
	return r;
}

// ------------------------------------------------------------------------

int translateTimeZone2Offset(char offset[]);

// time_t is GMT,
// rest is localtime (maybe selectable in future versions: localtime, gmtime, ...)
time_t parseDate(char *date,int &d,int &m,int &y,int &h,int &min,int &sec)
{ // Thu, 13 Mar 1997 02:07:44 +0100
  // Thu 13 Mar 1997 02:07:44 +0100
  // Thu Mar 13 1997 02:07:44 +0100
  // Thu Mar 13 02:07:44 1997 +0100
  // Thu 13 Mar 02:07:44 1997 +0100
  // 13 Mar 02:07:44 1997 +0100
  // 13 Mar 1997 02:07:44 +0100
  // 13 Mar 02:07:44 +0100 1997

	char *pd=date;
	int offs = 0;

	d=1;
	m=1;
	y=1970;
	h=0;
	min=0;
	sec=0;

	tokenizer tk(pd);

	if (!isdigit(((char*)tk[0])[0]))
	{ // remove weekday
		tk.remove(0);
	}
	int ftk=1;

	if (isdigit(((char*)tk[0])[0]))
	{ // day - month
		d=atoi(tk[0]);
		ftk=1;
	}
	else
	{ // month day
		d=atoi(tk[1]);
		ftk=0;
	}

	pd=tk[ftk];
	if (toupper(*pd)=='J' && toupper(*(pd+1))=='A' && toupper(*(pd+2))=='N')
		m=1;
	else if (toupper(*pd)=='F' && toupper(*(pd+1))=='E' && toupper(*(pd+2))=='B')
		m=2;
	else if (toupper(*pd)=='M' && toupper(*(pd+1))=='A' && toupper(*(pd+2))=='R')
		m=3;
	else if (toupper(*pd)=='A' && toupper(*(pd+1))=='P' && toupper(*(pd+2))=='R')
		m=4;
	else if (toupper(*pd)=='M' && toupper(*(pd+1))=='A' && toupper(*(pd+2))=='Y')
		m=5;
	else if (toupper(*pd)=='J' && toupper(*(pd+1))=='U' && toupper(*(pd+2))=='N')
		m=6;
	else if (toupper(*pd)=='J' && toupper(*(pd+1))=='U' && toupper(*(pd+2))=='L')
		m=7;
	else if (toupper(*pd)=='A' && toupper(*(pd+1))=='U' && toupper(*(pd+2))=='G')
		m=8;
	else if (toupper(*pd)=='S' && toupper(*(pd+1))=='E' && toupper(*(pd+2))=='P')
		m=9;
	else if (toupper(*pd)=='O' && toupper(*(pd+1))=='C' && toupper(*(pd+2))=='T')
		m=10;
	else if (toupper(*pd)=='N' && toupper(*(pd+1))=='O' && toupper(*(pd+2))=='V')
		m=11;
	else if (toupper(*pd)=='D' && toupper(*(pd+1))=='E' && toupper(*(pd+2))=='C')
		m=12;

	// "year - time" or "time - year" ?
	// check for ":"
	pd=tk[2];
	ftk=2;

	if (pd[1]==':' || pd[2]==':')
	{ // time first, needs more checks
		ftk=2;
	}
	else
	{ // year first
		ftk=3;
		y=atoi(tk[2]);
		if (y<100) y+=1900;
	}

	pd=tk[ftk];
	if (isdigit(*pd))
	{
		sscanf(pd,"%d:%d:%d",&h,&min,&sec);
	}
	ftk++;
	pd=tk[ftk];
	// year or offset
	if (y==1970 && isdigit(pd[0]))
	{ // Year first
		y=atoi(pd);
		if (y<100) y+=1900;
		ftk++;
	}
	else if (y==1970 && isdigit(((char*)tk[ftk+1])[0]))
	{
		y=atoi(tk[ftk+1]);
		if (y<100) y+=1900;
	}
	else if (y==1970 && isdigit(((char*)tk[ftk+2])[0]))
	{
		y=atoi(tk[ftk+2]);
		if (y<100) y+=1900;
	}

	pd=tk[ftk];
	if (!isdigit (pd[1]))
		offs = translateTimeZone2Offset (pd);
	else
		offs = atoi (pd);

	return time2localtime(d,m,y,h,min,sec,offs);
}
time_t time2localtime(int &d,int &m,int &y,int &h,int &min,int &sec,int offset)
{
	struct tm ts, *tsP;
	time_t t;

	ts.tm_sec = sec;
	ts.tm_min = min;
	ts.tm_hour = h;
	ts.tm_mday = d;
	ts.tm_mon = m - 1;
	ts.tm_year = y - 1900;
	ts.tm_isdst = 0;
	t = mktime(&ts) - int_timezone						 	// now real sender time
			        - (offset%100)*60 - (offset/100)*3600;	// now GMT
// t is GMT!
	if (tsP = localtime(&t))
	{
		sec = tsP->tm_sec;
		min = tsP->tm_min;
		h = tsP->tm_hour;
		d = tsP->tm_mday;
		m = tsP->tm_mon + 1;
		y = tsP->tm_year + 1900;
	}
	return t;
}
int translateTimeZone2Offset(char offset[])
{
	/*
	 * Greenwich Mean Time (oder Zulu-Zeit ...)
	 */

	while (offset[0]=='(') offset++;
	if (offset[strlen(offset)-1]==')') offset[strlen(offset)-1]=0;

	if (!strcmp (offset, "GMT"))
		return 0;
	if (!strcmp (offset, "UTC"))
		return 0;

	/* 
	 * Mitteleuropaeische Winter Zeit 
	 */

	if (!strcmp (offset, "CET"))
		return +100;
	if (!strcmp (offset, "MET"))
		return +100;
	if (!strcmp (offset, "MEZ"))
		return +100;
	if (!strcmp (offset, "MES"))
		return +100;
	if (!strcmp (offset, "MEWT"))
		return +100;
	/* 
	 * Mitteleuropaeische Sommer Zeit
	 */

	if (!strcmp (offset, "CEST"))
		return +200;
	if (!strcmp (offset, "MEST"))
		return +200;
	if (!strcmp (offset, "MEDST"))
		return +200;
	if (!strcmp (offset, "METDST"))
		return +200;
	if (!strcmp (offset, "MET DST"))
		return +200;
	if (!strcmp (offset, "MESZ"))
		return +200;
	if (!strcmp (offset, "MDT"))
		return +200;
	if (!strcmp (offset, "MED"))
		return +200;
	if (!strcmp (offset, "CED"))
		return +200;
	if (!strcmp (offset, "MES"))
		return +200;
	
	/*
	 * Ein paar amerikanische Zeitzonen...
	 */
	
	if (!strcmp (offset, "EST"))
		return -500;
	if (!strcmp (offset, "CST"))
		return -600;
	if (!strcmp (offset, "MST"))
		return -700;
	if (!strcmp (offset, "PST"))
		return -800;
	if (!strcmp (offset, "YST"))
		return -900;
	if (!strcmp (offset, "HST"))
		return -1000;
	if (!strcmp (offset, "MDT"))
		return -600;
	if (!strcmp (offset, "PDT"))
		return -700;
	if (!strcmp (offset, "CDT"))
		return -500;
	if (!strcmp (offset, "EDT"))
		return -400;
	
		
	/*
	 * Noch einzufgen:	
	 *	UT
	 */
	 
	return 0;
		
} /* translateTimeZone2Offset */

static int idc=0;
void getUniqueID(char *unique_id)
{
	xc_snprintf(unique_id,UNIQUE_ID_LEN,"%04x%08x%02x",getpid(),time(NULL)-1000000000,idc++);
}
