/* 
 * This file is part of XCmail and is subject to the 
 * XCmail license agreement
 */

/* $Id: newmail.cxx,v 2.50 2002/07/31 11:32:16 schmitzj Exp $ */


#include "configure.h"

#include <Xclasses/Xclasses.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>
#include <ctype.h>
#include <time.h>
#include <sys/stat.h>
#include <netdb.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <fcntl.h>
#include <dirent.h>
#include <unistd.h>
#include <sys/types.h>
#include <setjmp.h>
#include <signal.h>

#ifdef HAVE_SYSENT_H
#include <sysent.h>
#endif

#include "mailer.h"
#include "encoder.h"
#include "address_book.h"
#include "spellchecker.h"
#include "replylist.h"
#include "maillist.h"
#include "sendmail.h"
#include "pluginsmgr.h"
#include "strbuf.h"
#include "signature.h"
#include "createmail.h"
#include "extraggs.h"

#include <sys/utsname.h>

static int opened=0;
static int allowMulti=1;
static int dead_letterc=0;

class nm_textbox : public textbox
{
	int count;
	int oldsize;
public:
	char *save;

	nm_textbox(void) : textbox()
	{
		save=NULL;
		count=0;
		oldsize=0;
	};
	~nm_textbox()
	{
		if (save) delete [] save;
	};
	int GActivate(XEvent *ev,int why)
	{
		if (why==GA_TICK && save)
		{
			if (--count<=0 && oldsize!=TextSize())
			{
				FILE *d;

				count=100;
				oldsize=TextSize();

				if (d=fopen(save,"w"))
				{
					char *b=new char[TextSize()+1];
					CopyText(b,TextSize()+1);
					fwrite(b,1,TextSize(),d);
					fclose(d);
					delete b;
				}
			}
		}
		return textbox::GActivate(ev,why);
	};
};

class minput : public abookinput
{
};

class addresslist
{
	int atype;
	char *ad;
public:
	addresslist(int t,char *a)
	{
		atype=t;
		ad=strdup(a);
	};
	~addresslist()
	{
		free(ad);
	};
	int g_atype(void)
	{
		return atype;
	};
	char *g_ad(void)
	{
		return ad;
	};
	void s_atype(int t)
	{
		atype=t;
	}
	void s_ad(char *a)
	{
		free(ad);
		ad=strdup(a);
	};
};

void newmail::genname(void)
{
	int l;

	if (ml)
	{
		l=strlen(ml->from())+strlen(ml->subject())+strlen(ml->messageid());
		objname=new char[l+5];

		if (mode & MAIL_REPLY) strcpy(objname,"re_");
		else if (mode & MAIL_FORWARD) strcpy(objname,"fw_");
		else if (mode & MAIL_RESEND) strcpy(objname,"rs_");
		strcat(objname,ml->from());
		strcat(objname,ml->subject());
		strcat(objname,ml->messageid());
	}
	else
	{
		static unsigned int cm=0;
		objname=new char[30];
		xc_snprintf(objname,30,"createmail_%x",cm++);
	}
	TranslateName(objname);
}

newmail::newmail(mail *m, int md) : decoder_abstract(), clonedplugins()
{
	char buf[300];
	newmail *nm;

	totalmulti=0;
	dead_letter=0;
	spellchecked=0;

	pluginsmenu=0;
	newmailpluginmenucount=0;

	allsigs=new allsignatures;
	sign=NULL;

	ml=m;
	mode=md;
	Name("newmail");
	ParentClass(readerwin);
	genname();
	opened++;

	if (ml)
	{
		ml->addreferer();
		if (nm=(newmail*)FindObject(readerwin.FullName(),objname))
		{
			nm->raise();
			delete this;
		}
		else
		{
			Name(objname);
			makewin();
			anyopen|=AO_NEWMAIL;
			fd=new file_dialog;
			fd->ParentClass(this);
		}
	}
	else
	{
		if (nm=(newmail*)FindObject(readerwin.FullName(),objname))
		{
			nm->raise();
			delete this;
		}
		else
		{
			Name(objname);
			makewin();
			anyopen|=AO_NEWMAIL;
			fd=new file_dialog;
			fd->ParentClass(this);
		}
	}
}
newmail::~newmail()
{
	if (sign) delete sign;
	delete allsigs;

	newmailplugin *np;
	while (np=(newmailplugin*)clonedplugins.RemTail())
	{
		np->finish(this);
		if (np->flags & PLF_CLONEUSE)
		{
			delete np;
		}
	}

	listview *lv=((listview*)FindObject(FullName(),".attachments"));
	struct attachments *mp;
	for(int i=totalmulti;i>0;i--)
	{
		mp=(attachments*)lv->GetData(i);
		freepart(mp);
		delete mp;
	}
	freeAddrList((multilistview*)FindObject(FullName(),".addressbook_popup.popbutton_gpopup.gpopupwindow.addresses"));
	DeleteObjectTree(FullName(),DOT_NOTBASE);
	opened--;
	delete objname;
	if (ml)
	{
		ml->freemem();
		ml->remreferer();
	}
	if (dead_letter)
	{
		dead_letterc=0;
		char *buf=new char[strlen(getenv("HOME"))+14];
		strcpy(buf,getenv("HOME"));
		strcat(buf,"/dead.letter");
		remove(buf);
		delete [] buf;
	}
	if (opened==0) anyopen&=~AO_NEWMAIL;
}

void newmail::freepart(struct attachments *mp)
{
	if (mp->body) free(mp->body);
	if (mp->mime) free(mp->mime);
	if (mp->name) free(mp->name);
	if (mp->filename)
	{
		remove(mp->filename);
		free(mp->filename);
	}
}
void newmail::quit(gadget *g,int i)
{
	((appwindow*)FindObject(FullName(),"window"))->Close();
	delete this;
}

void newmail::showtext(char *b,int size)
{
	textbox *txt=((textbox*)FindObject(FullName(),".mailbody"));
	int x,y,l;
	char lne[100];
	char *c1,*c21,*c22;

	txt->Cursor(x,y);
	int old_x=x,old_y=y;
	if (x>0) y++;

	if (mode & MAIL_FORWARD)
	{
		if (programconfig.quotesender)
		{
			char *s = ml->formtext(programconfig.forwardopen);
			txt->AppendText(s);
			while (*s)
			{
				if (*(s++) == '\n') y++;
			}
		}
		else
		{
			txt->AppendText("---Forwarded mail:\n");
		}
	}
	else if (mode & MAIL_REPLY)
	{
		if (programconfig.quotesender)
		{
			char *s = ml->formtext(programconfig.replyopen);
			txt->InsertText(s,0,y);
			while (*s)
			{
				if (*(s++) == '\n') y++;
			}
		}
		else
		{
			txt->InsertText("Quoted mail:\n",0,y);
		}
	}


	if (mode & MAIL_FORWARD)
	{
		txt->AppendText(b);
	}
	else if (mode & MAIL_RESEND)
	{
		txt->AppendText(b);
	}
	else if (mode & MAIL_REPLY)
	{
		xcm_strbuf sb;
		sb+=b;
		char *b2=sb();

		c1=lne;
		y++;
		while (*b2)
		{
			if (c1==lne)
			{
				if (*b2=='>')
					strcpy(c1,">");
				else
					strcpy(c1,"> ");
				l=strlen(c1);
				c1+=l;
				c22=b2;
				c21=c1;
			}
			if (*b2==' ' || *b2=='\n')
			{
				c22=b2;
				c21=c1;
			}
			if (l>80 || *b2=='\n')
			{
				if ((l>80 && *c21==' ')|| *b2=='\n')
				{
					*c21=0;
					strcat(lne,"\n");
				}
				else
				{
					*c1=0;
					c22=b2;
					strcat(lne,"\n");
				}
				txt->InsertText(lne,0,y);
				y++;
				c1=lne;
				b2=c22+1;
			}
			else
			{
				*c1=*b2;
				c1++;
				b2++;
				l++;
			}
		}
		if (l>0)
		{
			*c1=0;
			strcat(lne,"\n");
			txt->InsertText(lne,0,y);
			y++;
		}
	}

	if (mode & MAIL_FORWARD)
	{
		if (programconfig.quotesender)
			txt->AppendText(ml->formtext(programconfig.forwardclose));
		else
		 	txt->AppendText("---End forward\n");
	}
	else if (mode & MAIL_REPLY)
	{
		if (programconfig.quotesender)
			txt->InsertText(ml->formtext(programconfig.replyclose),0,y++);
		else
			txt->InsertText("---End quote\n",0,y++);
	}
//	txt->CursorTo(0,y);
	txt->CursorTo(old_x,old_y);
	txt->ActivateKey();

	((fbutton*)FindObject(FullName(),".quote"))->Lock();
	((menu*)FindObject(FullName(),".menu"))->LockItem(2,1);
}
void newmail::quote(void)
{
	if (ml->totalparts()<0)
	{
		ml->PGPdecode((appwindow*)FindObject(FullName(),"window"));
	}
	decode((appwindow*)FindObject(FullName(),"window"),1);
}
void newmail::insertfile(void)
{
	char dirbuf[600],filebuf[300];
	FILE *afile1;

	fd->LoadMode();
	fd->SingleSelect();
	if (fd->Start((appwindow*)FindObject(FullName(),"window")))
	{
		fd->GetFile(filebuf,sizeof(filebuf));
		fd->GetDir(dirbuf,sizeof(dirbuf));
		strcat(dirbuf,filebuf);
		if (afile1=fopen(dirbuf,"r"))
		{
			setvbuf(afile1,NULL,_IOFBF,2000);
			textbox *txt=((textbox*)FindObject(FullName(),".mailbody"));
			struct stat st;
			char *buf;
			stat(dirbuf,&st);
			
			if (buf=new char[st.st_size+1])
			{
				int x,y;

				txt->Cursor(x,y);
				if (x>0) y++;

				if (fread(buf,1,st.st_size,afile1))
				{
					buf[st.st_size]=0;
					txt->InsertText(buf,0,y);
				}
				txt->CursorTo(0,y);
			}
			fclose(afile1);
			txt->ActivateKey();
		}
	}
}
void newmail::saveas(void)
{
	char dirbuf[600],filebuf[300];
	FILE *afile1;

	fd->SaveMode();
	fd->SingleSelect();
	if (fd->Start((appwindow*)FindObject(FullName(),"window")))
	{
		fd->GetFile(filebuf,sizeof(filebuf));
		fd->GetDir(dirbuf,sizeof(dirbuf));
		strcat(dirbuf,filebuf);
		if (afile1=fopen(dirbuf,"w"))
		{
			setvbuf(afile1,NULL,_IOFBF,2000);
			textbox *txt=((textbox*)FindObject(FullName(),".mailbody"));
			int l=txt->TextSize();
			char *b=new char[l+2];
			if (b)
			{
				txt->CopyText(b,l+1);
				fwrite(b,sizeof(char),l,afile1);
				delete [] b;
			}
			fclose(afile1);
			txt->ActivateKey();
		}
	}
}
void newmail::spellchecker(void)
{
	SpellChecker *s = new SpellChecker((appwindow*)FindObject(FullName(),"window"));
	if (s) s->CheckMail();
}
void newmail::quotefile(void)
{
	char dirbuf[600],filebuf[300];
	FILE *afile1;

	fd->LoadMode();
	fd->SingleSelect();
	if (fd->Start((appwindow*)FindObject(FullName(),"window")))
	{
		fd->GetFile(filebuf,sizeof(filebuf));
		fd->GetDir(dirbuf,sizeof(dirbuf));
		strcat(dirbuf,filebuf);
		if (afile1=fopen(dirbuf,"r"))
		{
			setvbuf(afile1,NULL,_IOFBF,2000);
			textbox *txt=((textbox*)FindObject(FullName(),".mailbody"));
			int x,y,l;
			char lne[100],ba[100];
			char *c1,*c21,*c22,*b;

			txt->Cursor(x,y);
			if (x>0) y++;

			c1=lne;
			*c1=0;
			l=0;
			ba[sizeof(ba)-1]=0;
			while (fread(ba,sizeof(char),sizeof(ba)-1,afile1))
			{
				xcm_strbuf sb;
				sb+=ba;
				b=sb();
				while (*b)
				{
					if (c1==lne)
					{
						if (*b=='>')
							strcpy(c1,">");
						else
							strcpy(c1,"> ");
						l=strlen(c1);
						c1+=l;
						c22=b;
						c21=c1;
					}
					if (*b==' ' || *b=='\n')
					{
						c22=b;
						c21=c1;
					}
					if (l>80 || *b=='\n')
					{
						*c21=0;
						strcat(lne,"\n");
						txt->InsertText(lne,0,y);
						y++;
						c1=lne;
						b=c22+1;
					}
					else
					{
						*c1=*b;
						c1++;
						b++;
						l++;
					}
				}
			}
			if (l>0)
			{
				*c21=0;
				strcat(lne,"\n");
				txt->InsertText(lne,0,y);
				y++;
			}
			fclose(afile1);
			txt->CursorTo(0,y);
			txt->ActivateKey();
		}
	}
}
void newmail::menuhandler(gadget *g,int i)
{
	switch (((menu*)g)->Selection())
	{
		case 0x01000001: // Close
			quit(NULL,0);
			break;
		case 0x01000002: // Quit
			quit(NULL,0);
			quitprg();
			break;
		case 0x01000003: // Send
			send(NULL,0);
			break;
		case 0x01000004: // Save as
			saveas();
			break;
		case 0x02000001: // Quote
			quote();
			break;
		case 0x02000002: // Insert file
			insertfile();
			break;
		case 0x02000003: // Quotefile
			quotefile();
			break;
		case 0x02000004: // Check spelling
			spellchecker();
			break;
		case 0x02000005: // Extras
			if (((menu*)g)->State(2,5)==PRESSED)
				((appwindow*)FindObject(FullName(),"window"))->ShowExtra();
			else
				((appwindow*)FindObject(FullName(),"window"))->HideExtra();
			break;
#ifdef MAILLIST
#ifdef MAILLIST_TO
		case 0x03000001: // to maillist
			{
				addTo(MAILLIST_TO,1);
				addCc("",1);
				input *txt=((input *)FindObject(FullName(), ".subject"));
				txt->Default("");
				txt->ActivateKey();
				textbox *tb=(textbox*)FindObject(FullName(),".mailbody");
				tb->InsertText(MAILLIST_TEXT,0,0);
				tb->CursorTo(0,0);
			}
			break;
#endif
#endif
		case 0x03000002: // mail the authors
			{
				addTo("xcmail-dev@js-home.org",1);
				addCc("",1);
				input *txt=((input *)FindObject(FullName(), ".subject"));
				txt->Default(PROGRAMNAME" "VERSION":");
				txt->ActivateKey();
			}
			break;
		default:
			if (pluginsmenu && ((menu*)g)->Menu()>=pluginsmenu)
			{
				newmailpluginmenu[((menu*)g)->MID()-1].pl->handle(newmailpluginmenu[((menu*)g)->MID()-1].id,this);
			}
			if (((menu*)g)->Menu()==2)
			{
				if (((menu*)g)->MID()==10)
					addsignature((textbox*)FindObject(FullName(),".mailbody"));
				else if (((menu*)g)->MID()>10)
				{
					addsignature((textbox*)FindObject(FullName(),".mailbody"),allsigs->signatures[((menu*)g)->MID()-10]);
				}
			}
			break;
	}
}
class encoder *newmail::chooseEncoder(char *def,char *m1,char *m2,char *filename,char *retmime,int &compress)
{
	class encoder *ret;
	appwindow *aw=new appwindow;
	appwindow *a=(appwindow*)FindObject(FullName(),"window");
	int x,y,w,h,s,r;

	a->Position(x,y);
	a->Size(w,h);
	aw->ParentClass(a);
	aw->Name("encodingwindow");
	aw->InitSize(x+1,y+1,60);
	aw->Init();
	aw->Adjust(x,y,w,h);
	aw->TransientFor(a);
	aw->Title("Select encoding");
	aw->IconTitle("Select encoding");
	aw->Callback();
	aw->BlockExcept();

	spacegroup *bg=new spacegroup;

	bg->Name("egroup");
	bg->Border(10);
	bg->YDistance(10);
	bg->Vert();

	output *op=new output;
	 op->Text("Filename");
	 op->Output(filename);
	bg->Add(op);
	 group *g=new group;
	 g->Horiz();
	 g->Center();
	 g->Name(NULL);
	 if (programconfig.askmime)
		bg->Add(g,"Detected mime");
	 else
		bg->Add(g,"Mime type");
 	  op=new output;
 	  op->Output(m1);
 	 g->Add(op);
 	  text *txt=new text;
 	  txt->FixedText();
 	  txt->Text("/");
 	 g->Add(txt);
 	  op=new output;
 	  op->Output(m2);
 	 g->Add(op);

 	 listview *mimelv=NULL;
	 if (programconfig.askmime)
	 {
	 	 mimelv=new listview;
	 	 mimelv->ShowSelected();
	 	 mimelv->Name("mimetype");
	 	 prgmime.mimeinlistview(mimelv,m1,m2);
	 	bg->Add(mimelv,"Choose _mime type");
	 }

	choice *encodech=new choice;
	char **t=prgencode->encoderarray();
	s=0;
	 encodech->Items(t);
	 encodech->Selected(s);
	bg->Add(encodech,"_Encoding");

/*	choice *compressch=new choice;
	char *t2[]={"none","gzip","compress",NULL};
	 compressch->Items(t2);
	 compressch->Selected(0);
	 compressch->Name("com_press");
	bg->Add(compressch,"Compress");
	if (!programconfig.askcompress)
		compressch->Lock();
*/
	fbutton *b=new fbutton;
	 b->Text("_Ok");
	 b->Callback((gadget*)aw,(CALLBACK)&appwindow::Close);
	bg->Add(b);

	aw->SetGroup(bg);
	aw->Create();
	r=aw->NextEvent();

	ret=prgencode->find(t[encodech->Selected()]);
	if (mimelv)
	{
		strcpy(retmime,(char*)mimelv->GetData(mimelv->Selected()));
	}
	else
	{
		sprintf(retmime,"%s/%s",m1,m2); // so lassen....
	}
/*	compress=compressch->Selected(); */
	compress=0;

	aw->UnblockExcept();
	DeleteObjectTree(aw->FullName(),0);
	return ret;
}
void newmail::attachfile(char *filebuf,int delit)
{
	int err=0;
	char dirbuf[500],buf[500],dirbuf2[500],file2name[500];
	char m1[MIMELEN],m2[MIMELEN];
	FILE *afile1,*afile2;
	struct attachments *mp;
	listview *lv=((listview*)FindObject(FullName(),".attachments"));

	fd->GetDir(dirbuf2,sizeof(dirbuf2));
	fd->GetDir(dirbuf,sizeof(dirbuf));
	strcat(dirbuf,filebuf);
	mp=new attachments;
	mp->name=strdup(filebuf);
	mp->body=NULL;
	mp->mime=NULL;
	mp->filename=NULL;
	if (!prgmime.findfull(filebuf,m1,m2))
	{
		strcpy(m1,"application");
		strcpy(m2,"octet-stream");
	}
	xc_snprintf(file2name,sizeof(file2name),TMPDIR"/"PROGRAMNAME"_attachment_%d_%04x%08x_%s_%04x",totalmulti+1,getpid(),time(NULL),filebuf,getuid());
	strcpy(buf, file2name);
	if (afile1=fopen(dirbuf,"r"))
	{
		setvbuf(afile1,NULL,_IOFBF,2000);
		if (afile2=fopen(file2name,"w"))
		{
			struct stat sb;
			char *ebuf1,*ebuf2=NULL;
			int nsize,compress;
			class encoder *enc;

			setvbuf(afile2,NULL,_IOFBF,2000);
			stat(dirbuf,&sb);
			nsize=sb.st_size;
			mp->filename=strdup(buf);

			if (ebuf1=(char*)malloc(sb.st_size))
			{
				fread(ebuf1,1,sb.st_size,afile1);
				fclose(afile1);
				afile1=NULL;

				enc=chooseEncoder("base64",m1,m2,filebuf,buf,compress);
				mp->mime=strdup(buf);
				levelwindow *lw=new levelwindow(((appwindow*)FindObject(FullName(),"window")),"Encode...",5,"Step");

				if (enc)
				{
					lw->update(1,1);
					ebuf2=enc->data2mail(ebuf1,sb.st_size,nsize,dirbuf2,filebuf);
				}
				lw->update(2,2);
				if (ebuf2)
				{
					free(ebuf1);
					ebuf1=ebuf2;
				}
				lw->update(3,3);
				err = err ||
					myfprintf(afile2,"Content-Type: %s;\n",mp->mime)<0 ||
					myfprintf(afile2," name=%s\n",mp->name)<0 ||
					myfprintf(afile2,"Content-Description: File attachment\n")<0;
					
				if (enc)
					err=err||fprintf(afile2,"Content-Transfer-Encoding: %s\n",enc->encoder_name())<0;
				
				err = err ||
					myfprintf(afile2,"\n")<0 ||
					fwrite(ebuf1,1,nsize,afile2)!=nsize;
				
				err=fclose(afile2)!=0||err;
								
				lw->update(4,4);
				if (ebuf2)
					enc->dispose(ebuf2);
				else
					free(ebuf1);
				lw->update(5,5);

				if (err)
				{
	   				delete lw;
					char buf[200];
					easy_dialog ed;
					xc_snprintf(buf,sizeof(buf), "No space left on device to encode\nfile: %s", file2name);
					ed.Text(buf);
					ed.Options("_Ok");
					ed.Start((appwindow*)FindObject(FullName(),"window"));
					freepart(mp);
					delete mp;
				}
				else
				{
					char buf[200];
					xc_snprintf(buf,sizeof(buf),"%s (%s)",mp->name,mp->mime);
					lv->AddAtEnd(buf,(void*)mp);
   					totalmulti++;
	   				delete lw;
					if (delit) remove(dirbuf);
				}
			}
			else
			{
				fclose(afile2);
				easy_dialog ed;
				ed.Text("Out of memory to encode file.");
				ed.Options("_Ok");
				ed.Start((appwindow*)FindObject(FullName(),"window"));
				freepart(mp);
				delete mp;
			}
		}
		else
		{
			easy_dialog ed;
			ed.Text("Can't create temporary\ncopy in "TMPDIR".");
			ed.Options("_Ok");
			ed.Start((appwindow*)FindObject(FullName(),"window"));
			freepart(mp);
			delete mp;
		}
		if (afile1) fclose(afile1);
	}
	else
	{
		easy_dialog ed;
		xc_snprintf(buf,sizeof(buf),"File '%s'\nnot found.",dirbuf);
		ed.Text(buf);
		ed.Options("_Ok");
		ed.Start((appwindow*)FindObject(FullName(),"window"));
		freepart(mp);
		delete mp;
	}
}
void newmail::attahandler(gadget *g,int i)
{
	listview *lv=((listview*)FindObject(FullName(),".attachments"));
	struct attachments *mp;

	pstatus("");
	if (!strcmp(g->Name(),"attachments"))
	{
		((fbutton*)FindObject(FullName(),".delattach"))->Unlock();
	}
	else if (!strcmp(g->Name(),"addattach"))
	{
		fd->LoadMode();
		fd->MultiSelect();
		checkbox cb_delfile;
		group gp;
		 cb_delfile.Text("delete file(s) after adding");
		 cb_delfile.Name("delaftersend");
		gp.Add(cb_delfile);
		fd->InsertGroup(gp);
		if (fd->Start((appwindow*)FindObject(FullName(),"window")))
		{
			char filebuf[200];
			int i=0;
			pstatus("Attaching file...");
			readerwin.Update();
			if (!fd->IsSelectedMarked())
			{
				fd->GetFile(filebuf,sizeof(filebuf));
				attachfile(filebuf,cb_delfile.State()==PRESSED);
			}
			while (i=fd->NextMarkedFile(i,filebuf,sizeof(filebuf)))
			{
				attachfile(filebuf,cb_delfile.State()==PRESSED);
			}
			pstatus("");
			readerwin.Update();
		}
		fd->SingleSelect();
	}
	else if (!strcmp(g->Name(),"delattach"))
	{
		mp=(attachments*)lv->GetData(lv->Selected());
		lv->Remove(lv->Selected());
		freepart(mp);
		delete mp;
		((fbutton*)FindObject(FullName(),".delattach"))->Lock();
		totalmulti--;
	}
}
void newmail::addsignature(textbox *atxt,char *subname)
{
	if (sign)
	{ // alte sign. lschen
		int l=strlen(sign->get());

		class xcm_strbuf sigbuf;
		sigbuf(atxt->TextSize()+1);
		atxt->CopyText(sigbuf(),atxt->TextSize()+1);
		sigbuf.update();

		int x,y;
		atxt->Cursor(x,y);
		if (atxt->TextSize()-l>=0)
		{
			sigbuf.resize(atxt->TextSize()-l);
			atxt->ReplaceText(sigbuf());
		}
		else
		{
			return;
		}
		atxt->CursorTo(x,y);
		delete sign;
	}
	sign=new signature(subname);
	atxt->AppendText(sign->get());
}
void newmail::addAdd(void *d)
{
	char buf[MAXLENGTH+1],*t;
	t=(char*)d;
	if (t=abook.getEmailByAlias(t))
	{
		addByType(t);
	}
}
void newmail::inputhandler(gadget *g,int i)
{
	pstatus("");
	if (!strcmp(g->Name(),"addressfield"))
	{
		char buf[MAXLENGTH+1],*t;
		multilistview *mlv=(multilistview*)FindObject(FullName(),".alladdresses");
		minput *ip=(minput*)FindObject(FullName(),".addressfield");

		ip->Input(buf);
		if (buf[0])
		{
			if (mlv->Selected()>0)
			{
				chAdr(mlv->Selected(),buf);
				mlv->Selected(0);
			}
			else
			{
				mlv->Top(addByType(buf)-1);
			}
			ip->ActivateKey();
		}
		else
		{
			if (mlv->Selected()>0)
			{
				rmAdr(mlv->Selected());
				mlv->Selected(0);
				ip->ActivateKey();
			}
			else
				((input*)FindObject(FullName(),".subject"))->ActivateKey();
		}
		ip->Default("");
		if (((choice*)FindObject(FullName(),".address_type"))->Selected()==0)
		{
			((choice*)FindObject(FullName(),".address_type"))->Selected(1);
		}
	}
	else if (!strcmp(g->Name(),"address_type"))
	{
		multilistview *mlv=(multilistview*)FindObject(FullName(),".alladdresses");
		if (mlv->Selected()>0)
		{
			chAdr(mlv->Selected(),((choice*)FindObject(FullName(),".address_type"))->Selected());
		}
		minput *ip=(minput*)FindObject(FullName(),".addressfield");
		ip->ActivateKey();
	}
	else if (!strcmp(g->Name(),"alladdresses"))
	{
		multilistview *mlv=(multilistview*)FindObject(FullName(),".alladdresses");
		choice *ch=(choice*)FindObject(FullName(),".address_type");
		minput *ip=(minput*)FindObject(FullName(),".addressfield");

		if (mlv->DoubleClick())
		{
			ip->Default("");
			rmAdr(mlv->Selected());
			mlv->Selected(0);
		}
		else
		{
			addresslist *a=(addresslist*)mlv->GetData(mlv->Selected());
			if (a)
			{
				ch->Selected(a->g_atype());
				ip->Default(a->g_ad());
			}
		}
		ip->ActivateKey();
	}
	else if (!strcmp(g->Name(),"subject"))
	{
		((textbox*)FindObject(FullName(),".mailbody"))->ActivateKey();
	}
	else if (!strcmp(g->Name(),"addressbook_popup"))
	{
		multilistview *mlv=(multilistview*)FindObject(g->FullName(),".addresses");
		lv_forMarked(mlv,this,(LVWORKCAST)&newmail::addAdd);
		for(int i=0;i<mlv->Count();i++)
		{
			mlv->UnmarkItem(i+1);
		}
	}
}
void newmail::extrahandler(gadget *g,int)
{
	if (!strcmp(g->Name(),"activ"))
	{
		if (((checkbox*)g)->State()==PRESSED)
			((input*)FindObject(FullName(),".extramaingroup.asktext"))->Unlock();
		else
			((input*)FindObject(FullName(),".extramaingroup.asktext"))->Lock();
	}
}
void newmail::freeAddrList(multilistview *mlv)
{
	if (mlv)
	{
		for (int i=1;i<=mlv->Count();i++)
		{
			char *t=(char*)mlv->GetData(i);
			if (t) free(t);
			mlv->ChangeData(i,NULL);
		}
	}
}
class popbutton *newmail::getAddrList(void)
{
	struct column_spec cs[2];
	popbutton *pb=new popbutton;
	multilistview *mlv=new multilistview;
	char buf[400],*lst[2];

	cs[0].flags=CSF_SELECTABLE|CSF_USEWEIGHT|CSF_SCALEABLE|CSF_TITLE;
	cs[0].weight=40;
	cs[0].title="Name";
	cs[1].flags=CSF_SELECTABLE|CSF_USEWEIGHT|CSF_TITLE;
	cs[1].weight=60;
	cs[1].title="Address";
	mlv->Columns(2,cs);
	mlv->Name("addresses");
	mlv->MultiSelect();

	for (int i=1;i<=abook.Count();i++)
	{
		buf[0]=0;
		lst[0]=abook.getName(i,buf);
		lst[1]=abook.getField(i,AddressBook::EMAIL);
		mlv->AddAtEnd(lst,strdup(abook.getField(i,AddressBook::EMAIL)));
	}

	pb->Apply(mlv,"add address",350);
	pb->CloseGadget(mlv);
	pb->Text("...");
	pb->Weight(5);
	return pb;
}
static char *addr_types[]={"Reply-To","To","Cc","Bcc",NULL};
void newmail::rmAdr(int sel)
{
	multilistview *mlv=(multilistview*)FindObject(FullName(),".alladdresses");
	addresslist *a=(addresslist *)mlv->GetData(sel);
	if (a)
	{
		delete a;
		mlv->Remove(sel);
	}
}
void newmail::chAdr(int sel,char *address)
{
	multilistview *mlv=(multilistview*)FindObject(FullName(),".alladdresses");
	addresslist *a=(addresslist *)mlv->GetData(sel);
	if (a)
	{
		a->s_ad(abook.getEmailByAlias(address));
		mlv->Replace(sel,1,abook.getEmailByAlias(address));
	}
}
void newmail::chAdr(int sel,int type)
{
	multilistview *mlv=(multilistview*)FindObject(FullName(),".alladdresses");
	addresslist *a=(addresslist *)mlv->GetData(sel);
	if (a)
	{
		a->s_atype(type);
		mlv->Replace(sel,0,addr_types[type]);
	}
}
int newmail::addAdr(char *address,int type,int clearall)
{
	int ret=0;
	multilistview *mlv=(multilistview*)FindObject(FullName(),".alladdresses");
	if (type==0) clearall=1;
	if (clearall)
	{
		int c=0;
		while (c<mlv->Count())
		{
			addresslist *a=(addresslist *)mlv->GetData(c+1);
			if (a->g_atype()==type)
			{
				mlv->Remove(c+1);
				delete a;
			}
			else
				c++;
		}
	}

	if (address && address[0])
	{
		tokenizer aa(address,",;");
		for(int i=0;i<aa.count();i++)
        {
			int c=0,added=0;
			addresslist *na=new addresslist(type,abook.getEmailByAlias(aa[i]));

			while (c<mlv->Count())
			{
				addresslist *a=(addresslist *)mlv->GetData(c+1);
				if (a->g_atype()<=type)
					c++;
				else
				{
					char *data[2];
					data[0]=addr_types[type];
					data[1]=na->g_ad();
					if (data[1])
					{
						mlv->AddBefore(c+1,data,na);
						added=1;
						ret=c+1;
					}
					break;
				}
			}
			if (!added)
			{
				char *data[2];
				data[0]=addr_types[type];
				data[1]=na->g_ad();
				if (data[1])
				{
					mlv->AddAtEnd(data,na);
					ret=mlv->Count()+1;
				}
				else
					delete na;
			}
		}
		mlv->Selected(0);
	}
	return ret;
}
int newmail::addByType(char *txt,int t)
{
	int tp=t;
	if (t<0)
	{
		tp=((choice*)FindObject(FullName(),".address_type"))->Selected();
	}
	return addAdr(txt,tp,0);
}
void newmail::addReplyto(char *t,int clearall)
{
	addAdr(t,0,1);
}
void newmail::addTo(char *t,int clearall)
{
	addAdr(t,1,clearall);
}
void newmail::addCc(char *t,int clearall)
{
	addAdr(t,2,clearall);
}
void newmail::addBcc(char *t,int clearall)
{
	addAdr(t,3,clearall);
}
void newmail::pstatus(char *t)
{
	if (mstatus)
		mstatus->Text(t);
	else
		::pstatus(t);
}
void newmail::makewin(void)
{
	int x,y;
	appwindow *aw=new appwindow;
	readerwin.Position(x,y);
	aw->ParentClass(this);
	aw->Name("window");

	::pstatus("Open mailer window");

	aw->InitSize(x+(opened-1)*5,y+(opened-1)*5,70);
	if (!aw->Init())
	{
		delete this;
		::pstatus("Error in opening window!");
	}
	char buf[300];
	if (ml)
	{
		if (mode & MAIL_REPLY)
			strcpy(buf,"Reply");
		else if (mode & MAIL_FORWARD)
			strcpy(buf,"Forward");
		else if (mode & MAIL_RESEND)
			strcpy(buf,"Resend");
	}
	else
		strcpy(buf,"New mail");
	aw->Title(buf);
	aw->IconTitle(buf);
	aw->Callback(this,(CLOSECALLBACK)&newmail::quit);
	aw->Group(readerwin);
//	aw->TransientFor(readerwin.window());

	spacegroup *maingp=new spacegroup;
	maingp->Name("maingroup");
	maingp->Border(4);
	maingp->YDistance(3);
	maingp->Vert();

    fbutton *b;
	group *gp1=new group,*gp2=new group,*gp3;
    choice *ch;

	maingp->Add(gp1);
	 gp1->Name(NULL);
	 gp1->UseSmallest();
	 gp1->Weight(10);
	 gp1->Horiz();
	  gp2->Name(NULL);
	  gp2->Weight(110);
	  gp2->Vert();
	  gp2->YDistance(1);
	   text *txt1=new text;
	    txt1->FixedText();
	    txt1->Text(" ");
	  gp2->Add(txt1);
	   multilistview *mlv=new multilistview;
	   mlv->Name("alladdresses");
	   mlv->ShowSelected();
	   mlv->Weight(50);
	   column_spec ad_cols[2];
//	   ad_cols[0].flags=CSF_SELECTABLE|CSF_USEWEIGHT;
//	   ad_cols[0].weight=20;
	   ad_cols[0].flags=CSF_SELECTABLE|CSF_USEWIDTH;
	   ad_cols[0].width=70;
	   ad_cols[1].flags=CSF_SELECTABLE|CSF_USESHORTCUT;
	   mlv->Columns(2,ad_cols);
	   mlv->Callback(this,(CALLBACK)&newmail::inputhandler);
	  gp2->Add(mlv);
#ifdef HAS_LEFTGAGDET
	   lefttext *lt=new lefttext;
	   lt->Name(NULL);
#endif
	   gp3=new group;
	   gp3->Name(NULL);
	   gp3->Horiz();
	   gp3->UseSmallest();
		ch=new choice;
		ch->Name("address_type");
		ch->Items(addr_types);
	    ch->Weight(20);
	    ch->Callback(this,(CALLBACK)&newmail::inputhandler);
		ch->Selected(1);
#ifdef HAS_LEFTGAGDET
	   lt->LeftGadget(ch);
#else
	   gp3->Add(ch);
#endif
	    minput *mip=new minput;
	    mip->Name("addressfield");
	    mip->AlwaysMode();
	    mip->MaxChars(MAXLENGTH);
	    mip->Callback(this,(CALLBACK)&newmail::inputhandler);
	    mip->Weight(200);
	   gp3->Add(mip);
	    popbutton *pb=getAddrList();
	    pb->Name("addressbook_popup");
	    pb->Callback(this,(CALLBACK)&newmail::inputhandler);
	   gp3->Add(pb);
#ifdef HAS_LEFTGAGDET
	   lt->Gadget(gp3);
	  gp2->Add(lt);
#else
	  gp2->Add(gp3);
#endif
	   input *ip=new input;
	   ip->Name("subject");
	   ip->Text("Subject");
	   ip->AlwaysMode();
	   ip->MaxChars(MAXLENGTH);
	   if (ml)
	   {
	   	char buf[MAXLENGTH]="";

	   	if (mode & MAIL_REPLY)
	   	{
	   	 if (strcncmp("Re:", ml->subject()) != 0)  // beckerr
		  strcpy(buf,"Re: ");
		}
	   	strncat(buf,ml->subject(),sizeof(buf));
		if (mode & MAIL_FORWARD) strncat(buf," (fwd)",sizeof(buf));
	   	ip->Default(buf);
	   }
	   ip->Callback(this,(CALLBACK)&newmail::inputhandler);
	  gp2->Add(ip);
	   gp3=new group;
	   gp3->Name(NULL);
	   gp3->UseSmallest();
	   gp3->Horiz();
   	    ch=new choice;
 	    static char *vctxt[]={"private","work","none",NULL};
 	    ch->Name("vcard");
 	    ch->Items(vctxt);
 	    ch->Selected(programconfig.vcard_enable?(programconfig.vcard_work_default?1:0):2);
 	   gp3->Add(ch);
#ifdef USEPGP
	   gp3->Add(new ruler);
   	    ch=new choice;
 	    static char *ctxt[]={"Unencoded","PGP signed","PGP encoded","PGP signed & encoded",NULL};
 	    ch->Name("pgp");
 	    ch->Items(ctxt);
 	    ch->Selected(programconfig.pgpmode);
 	   gp3->Add(ch,"PGP");
  	    if (!programconfig.usepgp)
  	    {
  	      ch->Selected(0);
  		  ch->Lock();
  	    }
#endif
 	  gp2->Add(gp3,"vCard");
	 gp1->Add(gp2);
	  ruler *rl1=new ruler;
	  rl1->Vert();
	 gp1->Add(rl1);
	  gp2=new group;
	  gp2->Name(NULL);
	  gp2->Weight(50);
	  gp2->Vert();
	  gp2->YDistance(3);
	   text *txt2=new text;
	    txt2->Center();
	    txt2->Text("Attachments");
	   gp2->Add(txt2);
	   listview *atta=new listview;
	   atta->Name("attachments");
	   atta->Callback(this,(CALLBACK)&newmail::attahandler);
	   atta->ShowSelected();
	  gp2->Add(atta);
	   gp3=new group;
	   gp3->Horiz();
	   gp3->Name(NULL);
	    b=new fbutton;
	    b->Name("addattach");
	    b->Text("_Add...");
	    b->Font(FONT_SMALL);
	    b->Callback(this,(CALLBACK)&newmail::attahandler);
	   gp3->Add(b);
	    b=new fbutton;
	    b->Name("delattach");
	    b->Text("Del");
	    b->Font(FONT_SMALL);
	    b->Callback(this,(CALLBACK)&newmail::attahandler);
	    b->Lock();
	   gp3->Add(b);
	  gp2->Add(gp3);
	 gp1->Add(gp2);

	nm_textbox *atxt=new nm_textbox;
	 atxt->Name("mailbody");
	 atxt->MinCols(45);
	 atxt->MinRows(4);
	 atxt->MaxLineLength(programconfig.writelinelength);
	 atxt->Weight(180);
	maingp->Add(atxt);

	mstatus=new text;
	 mstatus->Font(FONT_SMALL);
	 mstatus->Text("");
	maingp->Add(mstatus);

	 group *ctl=new group;
	maingp->Add(ctl);
	 ctl->Horiz();
	 ctl->UseSmallest();
	 ctl->Name("controlgroup");
	  ruler *r;
	  b=new fbutton;
	  b->Name("send");
	  b->Text("Send mail (F9)");
	  b->ApplyXKey(XK_F9);
	  b->Callback(this,(CALLBACK)&newmail::send);
	 ctl->Add(b);

	  b=new fbutton;
	  b->Name("quote");
	  b->Text("_Quote");
	  b->Callback(this,(CALLBACK)&newmail::quote);
	  if (!ml || mode & MAIL_FORWARD) b->Lock();
	 ctl->Add(b);
	  b=new fbutton;
	  b->Name("close");
	  b->Text("Abort mail (Esc)");
	  b->ApplyXKey(XK_Escape);
	  b->Callback(this,(CALLBACK)&newmail::quit);
	 ctl->Add(b);
	aw->SetGroup(maingp);

	{
		newmailplugin *mp=NULL;

		while (mp=(newmailplugin*)prgplugins->nextOfType(plugins::PT_Newmailtool,mp))
		{
			if (mp->flags & PLF_CLONEUSE)
			{
				newmailplugin *m=mp->clone();
				clonedplugins.AddTail(m);
				m->init(this);
			}
			else
				mp->init(this);
		}
	}
	newmailpluginmenucount=::newmailpluginmenucount;
	for(int j=0;j<newmailpluginmenucount;j++)
		newmailpluginmenu[j]=::newmailpluginmenu[j];
	::newmailpluginmenucount=0;

	menu *mne=new menu;
	mne->Name("menu");
	mne->Callback(this,(CALLBACK)&newmail::menuhandler);
	mne->AddMenu("File");
	mne->Add("Save as...","s",4);
	mne->Add("Send","n",3);
	mne->Add("Close","w",1);
	mne->Add("Quit","q",2);
	mne->AddMenu("Message");
	mne->Add("Quote","u",1);
	mne->Add("Insert file...","i",2);
	mne->Add("Insert file quoted...",3);

	if (allsigs->signatures_count>1)
	{
		char cs[4];
		mne->AddSub("Signature");
		for (int i=0;i<allsigs->signatures_count;i++)
		{
			if (i<9)
				xc_snprintf(cs,sizeof(cs),"%d",i+1);
			else if (i==9)
				xc_snprintf(cs,sizeof(cs),"0");
			else
				cs[0]=0;
			mne->Add(allsigs->signatures[i],cs,10+i);
		}
		mne->EndSub();
	}

	mne->AddRuler();
	mne->AddSel("Extra fields","e",5);
	mne->AddRuler();
	mne->Add("Spell check","p",4);
	if (!ml || mode & MAIL_FORWARD) mne->LockItem(2,1);

	mne->AddHelpMenu("Xclasses");
#ifdef MAILLIST_TO
	mne->Add("Write to "PROGRAMNAME" mailing list",1);
	mne->Add("Mail the authors",2);
#else
	mne->Add("Mail the authors",2);
#endif

	{
		int f=0;
		for(int i=0;i<newmailpluginmenucount;i++)
		{
			if (!f)
			{
				mne->AddMenu("Extras");
				f=1;
				pluginsmenu=3;
			}
			if (newmailpluginmenu[i].shortcut)
			{
				mne->Add(newmailpluginmenu[i].text,newmailpluginmenu[i].shortcut,i+1);
			}
			else
			{
				mne->Add(newmailpluginmenu[i].text,i+1);
			}
		}
	}

	aw->SetMenu(mne);
	aw->SetIcon(f_xicon1.pixmap(),f_xicon1.shadowmap());

	maingp=new spacegroup;
	maingp->Name("extramaingroup");
	maingp->Vert();
	maingp->Border(4);
	maingp->Add(new ruler);
	 text *extratext=new text;
	 extratext->Center();
	 extratext->Text("Extras");
	 extratext->Font(FONT_GROUP);
	maingp->Add(extratext);
	 gp2=new group;
	 gp2->Name(NULL);
	 gp2->Horiz();
	 gp2->UseSmallest();
	 gp2->Center();
	  gp3=new group;
	  gp3->Name(NULL);
	  gp3->Vert();
	  gp3->Weight(150);
	   checkbox *cb=new checkbox;
	   cb->Name("activ");
	   cb->Text("Ask question before allow reading");
	   cb->State(RELEASED);
	   cb->Callback(this,(CALLBACK)&newmail::extrahandler);
	   cb->HelpText("Allows you to ask the reader a short question or warning\nbefore showing the text\n{FONT_SMALL}currently only supported by "PROGRAMNAME"!{/FONT}");
	  gp3->Add(cb,NULL);
	   ip=new input;
	   ip->Name("asktext");
	   ip->MaxChars(80);
	   ip->AlwaysMode();
	   ip->Lock();
	   ip->Font(FONT_FIXED);
	  gp3->Add(ip,"Text");
	  gp3->Add(new ruler);
	   cb=new checkbox;
	   cb->Name("disposition_notification");
	   cb->Text("Send disposition notification");
	   cb->HelpText("Currently only for test purpose:\nask the MUA of the recipient(s) to send a mail\nif this mail has been read - this is not supported\nby every mailers including the current XCmail!");
	   if (!ISSET(programconfig.replyto)) cb->Lock();
	  gp3->Add(cb,"");
	 gp2->Add(gp3);
	 gp2->Add(new ruler);
	  gp3=new group;
	  gp3->Name(NULL);
	  gp3->Vert();
	  gp3->Weight(50);
	   static char *pristr[]={"urgent","normal","non-urgent",NULL};
	   ch=new choice;
	   ch->Name("priority");
	   ch->Items(pristr);
	   ch->Selected(1);
	   ch->Weight(50);
	  gp3->Add(ch,"Priority");
	  gp3->Add(new ruler);
	   group *gp4=new group;
	   gp4->Name("xheader");
	   gp4->Vert();
	   group *gp5=new group,*gp6=NULL;
	   gp5->Name(NULL);
	   gp5->Horiz();
	   gp5->DialogMode();
	    for(int i=0;i<XHEADERCOUNT;i++)
	    {
	 	  checkbox *cb=new checkbox;
		  char buf[20];
		  xc_snprintf(buf,sizeof(buf),"%d",i);
	 	  cb->Name(buf);
		  cb->State(programconfig.xheader_mode[i]?RELEASED:PRESSED);
		  if (programconfig.xheader_name[i][0] && programconfig.xheader_value[i][0])
		  {
		 	  cb->Text(programconfig.xheader_name[i]);
			  cb->HelpText(programconfig.xheader_value[i]);
		  }
		  else
		  {
		 	  cb->Text("unused");
			  cb->Lock();
		  }
		  if (i%2==0)
		  {
		      if (gp6)
			  	gp5->Add(gp6);
		  	  gp6=new group;
			  gp6->Vert();
			  gp6->Name(NULL);
		  }
		  gp6->Add(cb);
	    }
	   	gp5->Add(gp6);
	   gp4->Add(gp5);
	  gp3->Add(gp4,"enable X-Header");
	 gp2->Add(gp3);
	maingp->Add(gp2);

	aw->SetExtraGroup(maingp);
	aw->ExtraBottom();

	aw->Create();
	if (programconfig.opennewmailextras)
	{
		mne->State(2,5,PRESSED);
		aw->ShowExtra();
	}
    if (ml && mode & MAIL_REPLY) addTo(ml->replyto());
    if (ml && mode & MAIL_RESEND) addTo(ml->to());
    if (ml && mode & MAIL_RESEND) addCc(ml->cc());
	if (ISSET(programconfig.defaultBCC))
		addBcc(programconfig.defaultBCC);
	if (ISSET(programconfig.replyto))
		addReplyto(programconfig.replyto);

	((input*)FindObject(FullName(),".addressfield"))->ActivateKey();
	if (mode & MAIL_FORWARD || mode & MAIL_RESEND)
	{ // add Parts 
		if (abs(ml->totalparts())>0)
		{
			struct attachments *mp;
			char buf[200],*b;
			
			if (ml->totalparts()<0)
			{
				ml->PGPdecode(aw);
			}
			for(int i=2;i<=abs(ml->totalparts());i++)
			{
				mp=new attachments;
				mp->body=strdup(ml->thesubmailtext(i));
				mp->name=strdup(ml->partname(i));
				mp->mime=strdup(ml->subcontenttype(i));
				mp->filename=NULL;
				b=ml->getContentTag("name=",i);
				if (b && strlen(b))
					xc_snprintf(buf,sizeof(buf),"%s (%s) (%s/%s)",ml->partname(i),b,ml->submime1(i),ml->submime2(i));
				else
					xc_snprintf(buf,sizeof(buf),"%s (%s/%s)",ml->partname(i),ml->submime1(i),ml->submime2(i));
		   		atta->AddAtEnd(buf,(void*)mp);
		   		totalmulti++;
			}
		}
	}
	atxt->CursorTo(0,0);
	if (mode & MAIL_RESEND || mode & MAIL_FORWARD ||(mode & MAIL_REPLY && programconfig.autoquote && !(mode & MAIL_DONTAUTOQUOTE)))
	{
		quote();
	}
	if (!(mode & MAIL_RESEND)) addsignature(atxt);

    if (ml && mode & MAIL_REPLY && programconfig.reply_maillist)
	{
		token *t;
		char *listname=ml->getTag("X-XCmail-List-Name:");
		if (listname)
		{
			t=new token(listname);
			listname=strdup((char*)*t);
			delete t;
		}
		char *listsender=ml->getTag("X-XCmail-List-Sender:");
		if (listsender) 
		{
			t=new token(listsender);
			listsender=strdup((char*)*t);
			delete t;
		}
		char *replyto=ml->getTag("Reply-To:");
		if (replyto)
		{
			t=new token(replyto);
			replyto=strdup((char*)*t);
			delete t;
		}
		if (listname && listname[0] && listsender && listsender[0] && strcscmp(ml->replyto(),listname))
		{
			easy_dialog ed;
			char buf[300];
			xc_snprintf(buf,sizeof(buf),
			            "This is a mail of the mailing list:\n'%s'.\n\nThe sender was\n'%s'.\n\n"
						"To whom would you like to reply?",listname,listsender); 
			Translate728Line(buf);
			ed.Text(buf);
			ed.Options("the _list|_both|_sender only|_reply address");
			int r=ed.Start(aw);
			switch (r)
			{
				case 1:
					strcpy(buf,listname);
					break;
				case 2:
					ml->parse_from_address(listsender);
					strcpy(buf,listsender);
					strcat(buf,", ");
					strcat(buf,listname);
					break;
				case 3:
					ml->parse_from_address(listsender);
					strcpy(buf,listsender);
					break;
				case 4:
					strcpy(buf,ml->replyto());
					break;
			}
			addTo(buf,1);
		}
		if (replyto) free(replyto);
		if (listsender) free(listsender);
		if (listname) free(listname);
	}
	
    if (ml && mode & MAIL_REPLY && programconfig.reply_group)
	{
		replylist ed;
		ed.Text("There have been other recipients in the original mail.\n"
				"Who should also receive a copy of this reply?\n");
		ed.Addresses(ml);
		ed.Start(aw,this);
	}

	if (!dead_letterc)
	{
		char *buf=new char[strlen(getenv("HOME"))+14];
		FILE *d;
		strcpy(buf,getenv("HOME"));
		strcat(buf,"/dead.letter");

		if (d=fopen(buf,"r"))
		{
			easy_dialog ed;
			ed.Text("An old mail text has been found.\nUse this again?");
			ed.Options("_Yes|_No");
			if (ed.Start(aw)==1)
			{
				char lne[200];
				atxt->ReplaceText("");
				while (fgets(lne,sizeof(lne)-1,d))
				{
					atxt->AppendText(lne);
				}
			}
			fclose(d);
			remove(buf);
		}
		atxt->save=buf;
		dead_letterc=1;
		dead_letter=1;
	}
	if (!checkExpire(aw))
	{
		maingp->Lock();
		((fbutton*)FindObject(FullName(),".close"))->Unlock();
		for(int i=1;i<10;i++) mne->LockItem(2,i);
		mne->LockItem(1,3);
	}
}
void newmail::raise(void)
{
	appwindow *aw=((appwindow*)FindObject(FullName(),"window"));
	aw->Raise();
	aw->Open();
}

void newmail::send(gadget *g,int i)
{
	createmail cm;
	easy_dialog ed;
	int l;
	appwindow *aw=((appwindow*)FindObject(FullName(),"window"));
	char buf[MAXLENGTH+1];

	if (!checkExpire(aw)) return;

	textbox *txt=((textbox*)FindObject(FullName(),".mailbody"));

#ifdef USEPGP
	if (programconfig.usepgp)
		cm.pgpmode(((choice*)FindObject(FullName(),".pgp"))->Selected());
#endif
	l=txt->TextSize();

	if (l==0)
	{
		ed.Text("You have no text\nentered to send.");
		ed.Options("_Send mail|_back to edit");
		if (ed.Start(aw)==2)
			return;
	}

	((input*)FindObject(FullName(),".addressfield"))->Input(buf);
	if (buf[0])
	{
		inputhandler((input*)FindObject(FullName(),".addressfield"),0);
	}
	tokenizer tk_to(NULL,","),tk_cc(NULL,","),tk_bcc(NULL,",");
	multilistview *mlv=(multilistview*)FindObject(FullName(),".alladdresses");
	for(int ai=0;ai<mlv->Count();ai++)
	{
		addresslist *a=(addresslist *)mlv->GetData(ai+1);
		switch (a->g_atype())
		{
			case 0: // Reply-To;
				cm.replyto(a->g_ad());
				break;
			case 1: // To;
				tk_to.append(new token(a->g_ad()));
				break;
			case 2: // Cc;
				tk_cc.append(new token(a->g_ad()));
				break;
			case 3: // Bcc;
				tk_bcc.append(new token(a->g_ad()));
				break;
		}
	}

	if (tk_to.count()==0)
	{
		ed.Text("You must specify\nan recipient in 'To:'.");
		ed.Options("_Close");
		ed.Start(aw);
		return;
	}
	cm.to(tk_to.string());
	cm.cc(tk_cc.string());
	cm.bcc(tk_bcc.string());


	((input*)FindObject(FullName(),".subject"))->Input(buf);
	if (buf[0])
	{
		cm.subject(buf);
	}
	else
	{
		ed.Text("You have no subject specified!");
		ed.Options("_Send mail|_back to edit");
		if (ed.Start(aw)==2)
			return;
	}

	aw->Block();
	aw->Update();
	aw->UpdateEvent();

	if (ml && (mode & MAIL_REPLY))
	{
		cm.replytomail(ml);
	}
    if (ml && mode & MAIL_FORWARD)
	{
		cm.forwardtomail(ml);
	}

	int prio=((choice*)FindObject(FullName(),".extramaingroup.priority"))->Selected();
	if (prio!=1)
	{
		cm.priority(prio);
	}
	xcm_strbuf header;

	if (ISSET(programconfig.replyto) && ((checkbox*)FindObject(FullName(),".extramaingroup.disposition_notification"))->State()==PRESSED)
	{
		header+="Disposition-Notification-To: ";
		header+=Translate827Line(programconfig.replyto,programconfig.encoding8bit);
		header+="\n";
	}

	if (((checkbox*)FindObject(FullName(),".extramaingroup.activ"))->State()==PRESSED)
	{
		char abuf[90];
		((input*)FindObject(FullName(),".extramaingroup.asktext"))->Input(abuf);
		if (abuf[0])
		{
			header+="X-AskBefore: ";
			header+=Translate827Line(abuf,programconfig.encoding8bit);
			header+="\n";
		}
	}

	for(int xhi=0;xhi<XHEADERCOUNT;xhi++)
	{
		char buf[40];
		if (programconfig.xheader_name[xhi][0] && programconfig.xheader_value[xhi][0])
		{
			xc_snprintf(buf,sizeof(buf),".extramaingroup.xheader.%d",xhi);
			if (((checkbox*)FindObject(FullName(),buf))->State()==PRESSED)
			{
				header+="X-";
				header+=programconfig.xheader_name[xhi];
				header+=": ";
				header+=programconfig.xheader_value[xhi];
				header+="\n";
			}
		}
	}
	cm.headers(header());

	switch (((choice*)FindObject(FullName(),".vcard"))->Selected())
	{
		case 0:
			cm.vcardtype(AddressBook::PRIVATE);
			break;
		case 1:
			cm.vcardtype(AddressBook::WORK);
			break;
		default:
			cm.vcardtype(AddressBook::NOVCARD);
			break;
	}

 	char *b=new char[l+3];
 	txt->CopyText(b,l+1);
 	b[l+1]=0;
 	b[l]='\n';
	cm.body(b);
	delete b;

	struct attachments **am=NULL;
	if (totalmulti>0)
	{
	 	listview *lv=((listview*)FindObject(FullName(),".attachments"));
		am=new attachments*[totalmulti];
		for(int i=0;i<totalmulti;i++)
		{
			am[i]=(attachments*)lv->GetData(i+1);
		}
		cm.attachments(totalmulti,am);
	}
	cm.referingwindow(aw);
	cm.outputobject(this);

	int r=cm.send();
	delete [] am;

	aw->Unblock();
	if (r>0)
	{
		quit(NULL,0);
	}
}
