/* 
 * This file is part of XCmail and is subject to the 
 * XCmail license agreement
 */

/* $Id: npop3get.cxx,v 2.66 2002/05/12 15:44:39 schmitzj Exp $ */


#include "configure.h"

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/types.h>
#include <utime.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <ctype.h>
#include <signal.h>
#include <netdb.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <fcntl.h>
#include <arpa/inet.h>
#include <sys/file.h>
#include <setjmp.h>
#include <stdarg.h>

#include "mailer.h"
#include "npop3get.h"
#include "xc_snprintf.h"
#include "stringTokenizer.h"
#include "newbutton.h"

//#define DEBUG

#define MAXSIZE user->popmaxsize


#define RET_EOT			 1	// End of transmission = '.\n'
#define RET_OK			 0	// operation results no error
#define RET_UNKNOWN		-1	// unknown error
#define RET_POPERROR	-2	// pop2 error = '-....'
#define RET_TIMEOUT		-3	// timeout
#define RET_NETWORK		-4	// network/connection error
#define RET_MEMORY		-5	// memory (malloc/new) error

#define TIMEOUT 60

#define POP3PORT 110

#ifdef HAVE_SIGLONGJMP
#define SETJMP(x)       sigsetjmp(x,1)
#define LONGJMP(x)      siglongjmp(x,1)
#elif defined(HAVE_LONGJMP)
#define SETJMP(x)       setjmp(x)
#define LONGJMP(x)      longjmp(x,1)
#endif

//Pop3User *Pop3Get::user;
sigjmp_buf Pop3Get::jmpbuffer;
int Pop3Get::instances=0;
dlist Pop3Get::boxstate;
time_t Pop3Get::ready4smtp=0;

#ifdef MAXSIZE
static int ignoreMaxsize=0;
#endif

static int get_smtpafterpop_server=0;

Pop3User::Pop3User(char *n) : node()
{
	_host[0]=0;
	host[0]=0;
	user[0]=0;
	pass[0]=0;
	dpass[0]=0;
	port=POP3PORT;
	popleave=0;
	popmaxsize=0;
	popdeleteonserver=0;
	smtpafterpop3=0;
	vfoldername[0]=0;
	vfolder *vv=folderlist->getfolder("INBOX");
	if (vv)
		strcpy(vfoldername,vv->g_id());
	name=strdup(n);

	loadprefs();
}
Pop3User::~Pop3User()
{
	free(name);
}
void Pop3User::loadprefs(void)
{
	configopts p3cfg[]={
	{"T"POP_HOST_LENSTR":pop_server",_host},
	{"T"POP_USER_LENSTR":pop_user",user},
	{"T"POP_PASS_LENSTR":pop_passwd",dpass},
	{"Eleave,remove:pop_leave",(char*)&popleave},
	{"I:pop_maxsize",(char*)&popmaxsize},
	{"B:pop_deleteonserver",(char*)&popdeleteonserver},
	{"B:pop_smtpafterpop3_onthis",(char*)&smtpafterpop3},
	{"T"POP_HOST_LENSTR":pop_folder",vfoldername},
	{NULL,NULL}};

	char wdir[300];
	xc_snprintf(wdir,sizeof(wdir),"~/."PROGRAMNAME"/pop3_%s",name);

	config cfg_file(wdir);

	cfg_file.SetItems(p3cfg);
	cfg_file.Read();
	strdecodecpy(pass,dpass);
	setvals();
	if (get_smtpafterpop_server)
		smtpafterpop3=0;
	if (smtpafterpop3)
		get_smtpafterpop_server=1;
}
void Pop3User::saveprefs(void)
{
	configopts p3cfg[]={
	{"T"POP_HOST_LENSTR":pop_server",_host},
	{"T"POP_USER_LENSTR":pop_user",user},
	{"T"POP_PASS_LENSTR":pop_passwd",dpass},
	{"Eleave,remove:pop_leave",(char*)&popleave},
	{"I:pop_maxsize",(char*)&popmaxsize},
	{"B:pop_deleteonserver",(char*)&popdeleteonserver},
	{"B:pop_smtpafterpop3_onthis",(char*)&smtpafterpop3},
	{"T"POP_HOST_LENSTR":pop_folder",vfoldername},
	{NULL,NULL}};

	strencodecpy(dpass,pass);

	char wdir[300];
	xc_snprintf(wdir,sizeof(wdir),"~/."PROGRAMNAME"/pop3_%s",name);

	config cfg_file(wdir);

	cfg_file.SetItems(p3cfg);
	cfg_file.Write();
}
void Pop3User::rmprefs(void)
{
	char wdir[300];
	xc_snprintf(wdir,sizeof(wdir),"~/."PROGRAMNAME"/pop3_%s",name);
	unlink(wdir);
}
void Pop3User::setvals(void)
{
	strcpy(host,_host);
	char *cP = strchr(host, ':');
	if (cP)
	{
		port = atol(cP+1);
		*cP = 0;
	}

	v=folderlist->getfolder_byid(vfoldername);

	if (!v)
	{
		vfolder *vv=folderlist->getfolder("INBOX");
		if (vv)
		{
			strcpy(vfoldername,vv->g_id());
			v=vv;
		}
	}
}

Pop3BoxState::Pop3BoxState(char *s, int t) : node()
{
	text=NULL;
	State(t);
	if (s) Name(s);
}

Pop3BoxState::~Pop3BoxState()
{
	if (text) delete [] text;
}

void Pop3BoxState::Name(char *s)
{
	if (text) delete [] text;
	text=NULL;
	if (s && (text=new char[strlen(s)+1],text))
	{
		strcpy(text,s);
	}
}

char *Pop3BoxState::Name(void)
{
	return text?text:(char*)"";
}

void Pop3BoxState::State(int t)
{
	state=t;
}

int Pop3BoxState::State(void)
{
	return state;
}

Pop3Get::Pop3Get(appwindow *a,Pop3User *u)
{
	popmode=POP_STATNLOAD;

	aw=NULL;
	beepdone=0;
	bytecount=0;
	bytes_read=0;
	error_text[0]=0;
	is_locked=0;
	mailcount=0;
	mailindex=0;
	mailsize=0;
	mailtext=NULL;
	mailused=0;
	mode=0;
	my_mclass=0;
	parentaw=a;
	retrieve=0;
	retrieved=0;
	send_spooled=0;
	sockfd=-1;
	tobigsize=0;
	uidl[0]=0;
	curhost_ip[0]=0;
	uidl_found=0;
	extraheadersadded=0;

	next_uidlhist=NULL;
	curr_uidlhist=NULL;
	dele_uidlhist=NULL;
	retr_uidlhist=NULL;

	u->setvals();
	user=u;
	ParentClass(parentaw);
	instances++;
}

Pop3Get::~Pop3Get()
{
	if (curr_uidlhist)	delete curr_uidlhist;
	if (next_uidlhist)	delete next_uidlhist;
	if (dele_uidlhist)	delete dele_uidlhist;
	if (retr_uidlhist)	delete retr_uidlhist;
	if (mailtext)		free(mailtext);
	if (aw)
	{
		aw->Close();
		DeleteObjectTree(aw->FullName(),0);
		aw=NULL;
	}
	if (sockfd>=0)
	{
		gadget::RemoveFD(sockfd);
		close(sockfd);
	}
	instances--;
	if (popmode!=POP_BEFORESMTP)
	{
		nextpop3check(user,popmode);
	}
}

void Pop3Get::in_timeout(int)
{
	LONGJMP(Pop3Get::jmpbuffer);
}

int Pop3Get::read_line(char *buf, int len)
{
	int err=0;
	error_text[0]=0;
	
	if (SETJMP(Pop3Get::jmpbuffer) != 0)
	{
		return RET_TIMEOUT;
	}
	else
	{
		signal(SIGALRM,Pop3Get::in_timeout);
		alarm(TIMEOUT);
		int r=0;
		while (r<len-1)
		{
			if (recv(sockfd,buf+r++,1,0)<0)
			{
				r=-1;
				break;
			}
			if (buf[r-1]=='\n') break;

			if (r==1 && buf[0]=='-') err=1;

			if (err && r<sizeof(error_text)-1)
			{
				error_text[r-1]=buf[r-1];
				error_text[r]=0;
			}
		}
		alarm(0);
		signal(SIGALRM,SIG_DFL);

		if (r>=0)
		{
			buf[r]=0;
			int i=r-1;
			
			if (buf[i]!='\n')
			{
#ifdef DEBUG
				printf("%i ignr: ", mode); fflush(stdout);
#endif
				char c;
				signal(SIGALRM,Pop3Get::in_timeout);
				alarm(TIMEOUT);
				while (recv(sockfd,&c,1,0)==1 && c!='\n')
				{
					if (err && r<sizeof(error_text)-1)
					{
						error_text[r]=c;
						error_text[r+1]=0;
					}
					r++;
#ifdef DEBUG
					printf("%c", (unsigned char)c); fflush(stdout);
#endif
				}
#ifdef DEBUG
				printf("\n");
#endif
				alarm(0);
				signal(SIGALRM,SIG_DFL);
			}
			bytes_read+=r;
			
			while (buf[i]=='\n') buf[i--]=0;
			while (buf[i]=='\r') buf[i--]=0;
			buf[++i]=0;
#ifdef DEBUG
			printf("%i recv: %s\n",mode,buf);
#endif
			return RET_OK;
		}
	}
	return RET_UNKNOWN;
}

int Pop3Get::read_result(void)
{
	char buf[2];
	return read_result(buf,sizeof(buf));
}

int Pop3Get::read_result(char *buf, int len)
{
	int r=read_line(buf, len);
	if (!r)
	{
		switch (buf[0])
		{
			case '-': return RET_POPERROR;
			case '+': return RET_OK;
		}
	}
	return r;
}

int Pop3Get::read_multiline(char *buf, int len)
{
	int r=read_line(buf, len);
	if (!r && buf[0]=='.' && !buf[1])
	{
		return RET_EOT;
	}
	return r;
}

int Pop3Get::write(char *s, ...)
{
	int r=RET_NETWORK;
	char buf[1000];
	va_list valist;
	va_start(valist,s);
	
	int n=xc_vsnprintf(buf, sizeof(buf), s, valist);
	va_end(valist);

#ifdef DEBUG
	printf("%i send: %s\n",mode,buf);
#endif

//	fcntl(sockfd,F_SETFL,O_NONBLOCK);
	if (send(sockfd,buf,n,0)==n)
	{
		r=RET_OK;
	}
//	fcntl(sockfd,F_SETFL,O_SYNC);
	return r;
}


int Pop3Get::connect(void)
{
	int i;
	struct hostent *hinfo;
	struct sockaddr_in caddr,saddr;

	if (hinfo=gethostbyname(user->host), !hinfo)
	{
		return RET_NETWORK;
	}
	char *hip = inet_ntoa(*(struct in_addr *)hinfo->h_addr);
	strcpy(curhost_ip,hip);
	
//	if (popmode!=POP_STATONLY)
	{
		next_uidlhist=new UIDLhistory(user->host,user->user,HISTORYFILE,this);
		if (next_uidlhist)
		{
			next_uidlhist->Read();
			next_uidlhist->Clean();
		}
//		if (programconfig.popuseuidl)
		{
			// update code -------------------------------------------
			UIDLhistory *old_current_uidlhist=new UIDLhistory(hip,user->user,HISTORYFILE,this);
			if (old_current_uidlhist)
			{
				old_current_uidlhist->Read();
				old_current_uidlhist->RemoveFile();
			}
			// update code -------------------------------------------


			// UIDL from last download
			curr_uidlhist=new UIDLhistory(user->host,user->user,HISTORYFILE,this);
			if (curr_uidlhist)
			{
				curr_uidlhist->Read();
				if (old_current_uidlhist)
				{
					curr_uidlhist->Conc(old_current_uidlhist);
				}
			}


			// update code -------------------------------------------
			if (old_current_uidlhist)
			{
				delete old_current_uidlhist;
			}
			dele_uidlhist=new UIDLhistory(NULL,NULL,REMOVEFILE,this);
			if (dele_uidlhist)
			{
				dele_uidlhist->RemoveFile();
				delete dele_uidlhist;
				dele_uidlhist=NULL;
			}
			retr_uidlhist=new UIDLhistory(hip,user->user,RETRIEVEFILE,this);
			if (retr_uidlhist)
			{
				retr_uidlhist->RemoveFile();
				delete retr_uidlhist;
				retr_uidlhist=NULL;
			}
			// update code -------------------------------------------


			// UIDL from messages to delete from server
			// we use only one global list or we have to
			// ask DNS before every folder saving!
			// OK, no longer required, no all information is inside the mail!
			dele_uidlhist=new UIDLhistory(user->host,user->user,REMOVEFILE,this);
			if (dele_uidlhist)
			{
				dele_uidlhist->Read();
			}

			// UIDL from messages to retrieve from server
			retr_uidlhist=new UIDLhistory(user->host,user->user,RETRIEVEFILE,this);
			if (retr_uidlhist)
			{
				retr_uidlhist->Read();
			}
		}
		
	}
	pstatus("Connecting %s...",user->host);

	saddr.sin_family=hinfo->h_addrtype;
	memcpy((char*)&saddr.sin_addr.s_addr,hinfo->h_addr,hinfo->h_length);
	saddr.sin_port=htons(user->port);
	for(i=0;i<8;i++) saddr.sin_zero[i]=0;

	if (sockfd=socket(AF_INET,SOCK_STREAM,0), sockfd<0)
	{
		return RET_NETWORK;
	}
	caddr.sin_family=AF_INET;
	caddr.sin_addr.s_addr=htonl(INADDR_ANY);
	caddr.sin_port=htons(0);
	for(i=0;i<8;i++) caddr.sin_zero[i]=0;

	if (bind(sockfd,(struct sockaddr*)&caddr,sizeof(caddr))<0)
	{
		return RET_NETWORK;
	}

	if (::connect(sockfd, (struct sockaddr*)&saddr, sizeof(saddr))<0)
	{
		return RET_NETWORK;
	}
	gadget::AddReadFD(sockfd,this,(FDHANDLER)&Pop3Get::read_handler);

	return RET_OK;
}

void Pop3Get::disconnect(void)
{
	if (sockfd>=0)
	{
		close(sockfd);
		gadget::RemoveFD(sockfd);
		sockfd=-1;
	}
}

int Pop3Get::mailtext_append(char *buf)
{
	int l=strlen(buf);

	if (!mailtext)
	{
		mailtext=(char*)malloc(l+1000);
		if (!mailtext) return RET_MEMORY;
		mailtext[0]=0;
		mailsize=l+1000;
		mailused=0;
		if (strncmp(buf, "From ",5)!=0)
		{
			time_t now=time(NULL);
			mailused=xc_snprintf(mailtext,mailsize,"From pop3.%s@%s %s",user->user,curhost_ip,ctime(&now));
		}
	}
	else if (mailused+l>=mailsize-1)
	{
		int newsize=mailsize+l+1000;
		mailtext=(char*)realloc(mailtext,newsize);
		if (!mailtext) return RET_MEMORY;
		mailsize=newsize;
	}
	strcpy(mailtext+mailused,buf);
	mailused+=l;
	strcpy(mailtext+mailused,"\n");
	mailused++;

	return OK;
}

int Pop3Get::retrieve_or_ignore(int fd)
{
	int r=0;
	if (retrieve)
	{
// 		if (popmode==POP_TESTONLY)
// 		{
// 			retrieved++;
// 			mailindex++;
// 			if (mailindex < mailcount)
// 			{
// 				mode=40;	// get next mail
// 				r=read_handler(fd);
// 			}
// 			else
// 			{
// 				mode=90; // terminate
// 				r=write("stat\r\n");
// 			}
// 		}
// 		else
		{
			if (!aw)
			{
				show_state(popmode==POP_INTERACTIVE);
			}
			tobigsize=0;

			char buf[300];
			if (!write("list %d\r\n",mailindex+1) && !read_result(buf,sizeof(buf)))
			{
				char *c=buf+4;
				while (*c && *c!=' ')c++;
				tobigsize=atol(c);
				bytecount=tobigsize;
				if (tobigsize<=MAXSIZE*1024 || MAXSIZE==0 || (retr_uidlhist && retr_uidlhist->Scan(uidl)))
				{
					tobigsize=0;
				}
			}

#ifdef MAXSIZE
			if (!ignoreMaxsize && (!retr_uidlhist || !retr_uidlhist->Scan(uidl)))
			{
				// this is a little bit dirty :-) (rb)
				char buf[300];
				if (tobigsize<=MAXSIZE*1024 || MAXSIZE==0)
				{
					tobigsize=0;
				}
			}
#endif
			if (tobigsize)
			{
				pstatus("Retrieving message %i partial",mailindex+1);
				r=write("top %d 0\r\n",mailindex+1);
			}
			else
			{
				pstatus("Retrieving message %i",mailindex+1);
				r=write("retr %d\r\n",mailindex+1);
				if (next_uidlhist)
				{
					next_uidlhist->Add(uidl);
				}
				if (retr_uidlhist)
				{
					retr_uidlhist->Remove(uidl);
				}
			}
			uidl_found=0;
			extraheadersadded=0;
			mode=60;
		}
	}
	else
	{
		if (dele_uidlhist && dele_uidlhist->Scan(uidl))
		{
			mode=80; // delete mail
			pstatus("Deleting message %d...",mailindex+1);
			r=write("dele %d\r\n",mailindex+1);
			dele_uidlhist->Remove(uidl);
			next_uidlhist->Remove(uidl);
			next_uidlhist->SetInvalid();
		}
		else
		{
			if (next_uidlhist)
			{
				next_uidlhist->Add(uidl);
			}
			pstatus("Ignoring message %i", mailindex+1);
			if (user->popleave==0 || tobigsize>0)
			{
				mailindex++;
				if (mailindex < mailcount)
				{
					mode=40;	// get next mail
					pstatus("Looking up next mail...");
					return read_handler(fd);
				}
				else
				{
					mode=90; // terminate
				    pstatus("Finishing session...");
					r=write("stat\r\n");
				}
			}
			else
			{
				mode=80; // delete mail
				pstatus("Deleting message...");
				r=write("dele %d\r\n",mailindex+1);
			}
		}
	}
	return r;
}
int Pop3Get::read_handler(int fd)
{
	if (islocked()) return 0;
	
	char buf[3000];
	int done=0;
	int r=0;

	switch(mode)
	{
	
		case 0:
			r=read_result();
			// +OK QPOP (version 2.2) at ... starting.
			if (!r)
			{
				if (send_spooled)
				{
					sendspooled();
				}
				pstatus("Sending username...");
				r=write("user %s\r\n", user->user);
			}
			mode=10;
			break;

		case 10:
			r=read_result();
			// +OK Password required for ...
			if (!r)
			{
				pstatus("Sending password...");
				r=write("pass %s\r\n", user->pass);
			}
			mode=20;
			break;
		
		case 20:
			r=read_result();
			// +OK beckers has 1 message (1475 octets).
			if (!r)
			{
				if (programconfig.smtpafterpop3 && user->smtpafterpop3)
				{
					ready4smtp=time(NULL);
					sendspooledNOW();
				}
				if (popmode==POP_BEFORESMTP)
				{
					r=write("quit\r\n");
					mode=91;
				}
				else
				{
					pstatus("Testing for new mail...");
					r=write("stat\r\n");
					mode=30;
				}
			}
			else
				mode=30;
			break;
			
		case 30:
			r=read_result(buf,sizeof(buf));
			// +OK 12 750684
			if (!r)
			{
				char *c;
				for (c=buf+4; *c && *c!=' '; c++);
				mailcount=atol(buf+4);
				foldersize=atol(c);

				if (curr_uidlhist->IsValid() && curr_uidlhist->Count()==mailcount)
				{
					pstatus("Folder stats not changed since last check - so no new mail");
					if ((!dele_uidlhist || dele_uidlhist->Count()==0) && (!retr_uidlhist || retr_uidlhist->Count()==0))
					{
						mode=91;
						r=write("quit\r\n");
						break;
					}
				}
// 				else if (popmode==POP_STATONLY)
// 				{
// 					pstatus("folder stats have changed since last check");
// 					retrieved=1;
// 					mode=91;
// 					r=write("quit\r\n");
// 					break;
// 				}
			}
			if (!mailcount)
			{
				mode=90; // terminate
			    pstatus("Finishing session...");
				r=write("stat\r\n");
				break;
			}
			else
			{
				mode=40;
			}
			// no break !!!!!!!!!
			
		case 40:
			if (!r)
			{
				pstatus("Testing message %d...",mailindex+1);
				istatus(0,(mailindex+1)*100/mailcount,"Message %i of %i",mailindex+1,mailcount);
				istatus(1,0);
				bytes_read=0;
				
				r=write("uidl %d\r\n",mailindex+1);
				mode=50;
			}
			break;
		
		case 50:
			r=read_result(buf,sizeof(buf));
			// +OK 1 fc43fb2671e1e5a7a3968c96dc77c43c
			if (!r)
			{
				int i,j;
				char *c;
					
				// extract uidl
				for (c=buf,i=0,j=0; *c && i<sizeof(uidl)-1; c++)
				{
					if (j>=2) uidl[i++]=*c;
					if (*c==' ') j++;
				}
				uidl[i]=0;
				
				// uidl in:
				//   retr_uidlhist -> retrieve
				//   curr_uidlhist -> don't retrieve
				//   else          -> retrieve
				//
				retrieve=1;
				if (!retr_uidlhist || !retr_uidlhist->Scan(uidl))
				{
					if (curr_uidlhist)
					{
						retrieve=!curr_uidlhist->Scan(uidl);
					}
				}
			}
			return retrieve_or_ignore(fd);

// 		case 51:
// 			r=read_result(buf,sizeof(buf));
// 			// +OK 631 octets
// 			mode=52;
// 			
// 		case 52:
// 			// <head>
// 			if (read_multiline(buf,sizeof(buf))==0)
// 			{
// 				r=mailtext_append(buf);
// 				mode=52;
// 			}
// 			else
// 			{
// 				if (mailtext)
// 				{
// 					retrieve=1;
// 					node *n;
// 					mail *m=new mail(mailtext);
// 					m->addreferer();
// 					if (mclass)
// 					{
// 						for (n=mclass->allmails.head; n->next; n=n->next)
// 						{
// 							if (*m==*((mail*)n->text))
// 							{
// 								retrieve=0;
// 								break;
// 							}
// 						}
// 					}
// 					m->remreferer();
// 					mailtext=NULL;
// 				}
// 				return retrieve_or_ignore(fd);
// 			}
// 			break;

		case 60:
			r=read_result(buf,sizeof(buf));
			// +OK 631 octets
			if (!r)
			{
//				bytecount=atol(buf+4);
				if (bytecount)
				{
					istatus(1,0, "0 of %i Bytes", bytecount);
				}
				else
				{
					istatus(1,0, "0 of ??? Bytes");
				}
			}
			mode=70;
			break;
			
		case 70:
			// <mail>
			if (read_multiline(buf,sizeof(buf))==0)
			{
				static int old_value=-1;
				static int old_bytes_read=-1;
				if (bytecount)
				{
					if (bytes_read>bytecount)
					{
						bytes_read=bytecount;
					}
				
					int value=bytes_read*100/bytecount;

					if (value!=old_value || bytes_read>(old_bytes_read+10240))
					{
						istatus(1,value, "%i of %i Bytes",bytes_read,bytecount);
						old_value=value;
						old_bytes_read=bytes_read;
					}

				}
				else
				{
					if (bytes_read>old_value+100)
					{
						istatus(1,0, "%i of ??? Bytes",bytes_read);
						old_value=bytes_read;
					}
				}
				
				
				// hack to preevent missing X-UIDL-Header in
				// retrieven mail
				//
				if (uidl[0] && !uidl_found)
				{
					if (strcncmp("X-UIDL:", buf)==0)
					{
						uidl_found=1;
					}
					else if (buf[0]==0 || buf[0]=='\n' || (buf[0]=='\r' && buf[1]=='\n'))
					{
						char buf2[100];
						xc_snprintf(buf2,sizeof(buf2),"X-UIDL: %s", uidl);
						uidl_found=1;
						r=mailtext_append(buf2);
					}
				}
				if (!extraheadersadded && (buf[0]==0 || buf[0]=='\n' || (buf[0]=='\r' && buf[1]=='\n')))
				{
					char buf2[100];
					extraheadersadded=1;
					xc_snprintf(buf2,sizeof(buf2),"X-POPCfg: %s",user->name);
					r=mailtext_append(buf2);
					xc_snprintf(buf2,sizeof(buf2),"X-POPHost: %s",user->host);
					r=mailtext_append(buf2);
					xc_snprintf(buf2,sizeof(buf2),"X-POPUser: %s",user->user);
					r=mailtext_append(buf2);
				}

				if (strncmp("From: ",buf,6)==0)
				{
					pheader(0, "%s", buf+6);
				}
				else if (strncmp("Subject: ",buf,9)==0)
				{
					pheader(1, "%s", buf+9);
				}
				// don't support mime-encoding when receiving partial
				// message -> delete lines containing "Content-Type: "
				//
				char *b=buf;
				while (*b && (*b==' ' || *b=='\t')) b++;
				if (tobigsize && !strcncmp("Content-Type: ",buf))
				{
					r=mailtext_append("Content-Type: text/x-xcmail-pop3size");
				}
				else if (tobigsize && !strcncmp("boundary",b))
				{
				}
				else
				{
					r=mailtext_append(buf);
				}
				mode=70;
			}
			else
			{
				istatus(1,100, "%i of %i Bytes",bytes_read,bytes_read);

				if (next_uidlhist)
				{
					next_uidlhist->Add(uidl);
				}
				if (mailtext)
				{
					if (tobigsize)
					{
#ifdef MAXSIZE
						mailtext_append("@ This message has not been retrieved, because it");
						xc_snprintf(buf,sizeof(buf),"@ is bigger then %dKB!",MAXSIZE); mailtext_append(buf);
						xc_snprintf(buf,sizeof(buf),"@ Size: %d Octets",tobigsize); mailtext_append(buf);
						mailtext_append("@ ");
						mailtext_append("@ To retrieve this message, you must enable XCmail's");
						mailtext_append("@ text/x-xcmail-pop3size MIME helper.");
						mailtext_append("");
						token *t;
						t=new token(uidl); xc_snprintf(buf,sizeof(buf),"UIDL:%s",t->tokenize()); mailtext_append(buf); delete t;
						t=new token(curhost_ip); xc_snprintf(buf,sizeof(buf),"HIP:%s",t->tokenize()); mailtext_append(buf); delete t;
						t=new token(user->user); xc_snprintf(buf,sizeof(buf),"USER:%s",t->tokenize()); mailtext_append(buf); delete t;
						t=new token(user->host); xc_snprintf(buf,sizeof(buf),"HOST:%s",t->tokenize()); mailtext_append(buf); delete t;
						xc_snprintf(buf,sizeof(buf),"MAXSIZE:%d",MAXSIZE); mailtext_append(buf);
						xc_snprintf(buf,sizeof(buf),"SIZE:%d",tobigsize); mailtext_append(buf);
						mailtext_append("");
#endif
					}
					mail *m=new mail(mailtext);
					m->status(m->status() | STATUS_POPMAIL | STATUS_XCPOP3NEW);
					user->v->appendmail_default(m);
					if (!beepdone)
					{
						::newmailarrived(0,user->v);
						beepdone=1;
					}
// 					if (next_uidlhist)
// 					{
// 						next_uidlhist->Write();
// 					}
					mailtext=NULL;
					retrieved++;
				}
				
				pheader(0);
				pheader(1);
				
				if (user->popleave==0 || tobigsize>0)
				{
					mailindex++;
					if (mailindex < mailcount)
					{
						mode=40;	// get next mail
						pstatus("Looking up next mail...");
						return read_handler(fd);
					}
					else
					{
						mode=90; // terminate
					    pstatus("Finishing session...");
						r=write("stat\r\n");
					}
				}
				else
				{
					mode=80; // delete mail
					pstatus("Deleting message...");
					r=write("dele %d\r\n",mailindex+1);
					if (next_uidlhist)
					{
						next_uidlhist->Remove(uidl);
						next_uidlhist->SetInvalid();
					}
				}
			}
			break;

		case 80:
			r=read_result();
			// +OK Message 1 has been deleted.
			if (!r)
			{
				mailindex++;
				if (mailindex < mailcount)
				{
					mode=40;	// get next mail
					pstatus("Looking up next mail...");
					return read_handler(fd);
				}
				else
				{
					mode=90;	// terminate
				    pstatus("Finishing session...");
					r=write("stat\r\n");
				}
			}
			break;

		
		case 90:
			r=read_result(buf,sizeof(buf));
			// +OK 12 750684
			if (!r)
			{
				r=write("quit\r\n");
				mode=91;
			}
			break;

		case 91:
			r=read_result();
			// +OK Pop server at arrakis.home.local signing off.
			if (!r)
			{
			    pstatus("Closing connection...");
				done=1;
			}
			else
				mode=92;
			break;

		case 92:
			r=write("quit\r\n");
			mode=91;
			break;

		case 93: // use abort
			done=1;
			r=0;
			break;
			
		default:
			fprintf(stderr, "POP3: unknown mode %d", mode);
			r=-1;
			break;
			
	}
	if (done || r)
	{
		disconnect();

		if (r)
		{
			char *c = error_text;
			
			while (*c && *c!=' ') c++;
			while (*c && *c==' ') c++;
			
			pstatus("Error");
			tAlert(parentaw, "Pop3 error: \n%s",c[0]?c:"unknown error");
		}
		else
		{
			if (dele_uidlhist)
			{
				dele_uidlhist->Clean();
			}
			finish_successfully();
		}
		delete this;
	}
	return r;
}

void Pop3Get::finish_successfully(void)
{
//	int state=POPSTATE_UNKNOWN;
//	char *fn;
	
	if (aw)
	{
		istatus(0,100);
		istatus(1,100);
		pheader(0);
		pheader(1);
	}
	
	switch(popmode)
	{
		case POP_INTERACTIVE:
//		case POP_SILENT:
			pstatus();
			if (retrieved==0)
			{
				::pstatus("You have no new mail!");
				if (next_uidlhist && !next_uidlhist->IsValid())
				{
					next_uidlhist->WriteInvalid();
				}
			}
			else
			{
				::pstatus("You have %d new mail%s",retrieved,retrieved==1?"!":"s!");
				if (next_uidlhist)
				{
					next_uidlhist->Write();
				}
			}
			if (dele_uidlhist)
			{
				dele_uidlhist->Write();
			}
			if (retr_uidlhist)
			{
				retr_uidlhist->Clean();
				retr_uidlhist->Write();
			}
			break;
			
//		case POP_TESTONLY:
// 		case POP_STATONLY:
// 			if (retrieved>0)
// 			{
// 				if (next_uidlhist)
// 				{
// 					next_uidlhist->Write();
// 				}
// 			}
// 			else
// 			{
// 				pstatus("You have no new mail!");
// 			}
// 			if (dele_uidlhist)
// 			{
// 				dele_uidlhist->Write();
// 			}
// 			if (retr_uidlhist)
// 			{
// 				retr_uidlhist->Clean();
// 				retr_uidlhist->Write();
// 			}
// 			break;

		case POP_STATNLOAD:
			if (retrieved==0)
			{
				pstatus("You have no new mail!");
				if (next_uidlhist && !next_uidlhist->IsValid())
				{
					next_uidlhist->WriteInvalid();
				}
			}
			else
			{
				if (next_uidlhist)
				{
					next_uidlhist->Write();
				}
			}
			if (dele_uidlhist)
			{
				dele_uidlhist->Write();
			}
			if (retr_uidlhist)
			{
				retr_uidlhist->Clean();
				retr_uidlhist->Write();
			}
			break;

		case POP_BEFORESMTP:
			break;

		default: fprintf(stderr,"unknown mode - this should not happen (%d)\n",retrieved);
			break;
	}
}

void Pop3Get::okCB(gadget *g,int i)
{
// 	if (aw)
// 		aw->Close();
// 
// 	((input*)FindObject(".popwindow.popserver"))->Input(user->_host);
// 	((input*)FindObject(".popwindow.popuser"))->Input(user->user);
// 	((input*)FindObject(".popwindow.poppassword"))->Input(user->pass);
// 	send_spooled=((checkbox*)FindObject(".popwindow.sendspooled"))->State()==PRESSED;
// #ifdef MAXSIZE
// 	ignoreMaxsize=((checkbox*)FindObject(".popwindow.ignoreMaxsize"))->State()==PRESSED;
// #endif
// 
// 	if (aw)
// 		DeleteObjectTree(aw->FullName(),0);
// 	aw=NULL;
// 
// 	user->host[0]=0;
// 	user->port=POP3PORT;
// 	strncat(user->host, user->_host, POP_HOST_LEN);
// 	char *cP = strchr(user->host, ':');
// 	if (cP)
// 	{
// 		user->port = atol(cP+1);
// 		*cP = 0;
// 	}
// 	
// 	show_state(1);
// 	
// 	mode=0;
// 	if (!connect())
// 	{
// 		pstatus("Connected.");
// 	}
// 	else
// 	{
// 		tAlert(parentaw,"Unable to connect %s!",user->host);
// 		delete this;
// 		return;
// 	}
// 	if (send_spooled) sendspooled(1);
}

void Pop3Get::breakCB(void)
{
	if (mode==93) return;

	lock();
	if (aw && Alert(aw, "Are you sure?|Yes|No")==1)
	{
		mode=93;	// user abort mode
		((gadget*)FindObject(".popstate.Cancel"))->Lock();
		
		// we have to do this, because the next uid-list in incomplet!
		if (next_uidlhist && curr_uidlhist)
		{
			next_uidlhist->Conc(curr_uidlhist);
			next_uidlhist->Write(); 
		}
		
//		finish_successfully();
//		delete this;
	}
	unlock();
}

void Pop3Get::cancelCB(void)
{
	delete this;
}

void Pop3Get::pheader(int id, char *s, ...)
{
	if (!aw || id>1 || id<0) return;

	static char *name[2]={".popstate.from", ".popstate.subject" };
	static char buf[2][200];
	buf[id][0]=0;
		
	if (s)
	{
		va_list valist;
		va_start(valist,s);	
		xc_vsnprintf(buf[id], sizeof(buf[id]), s, valist);
		va_end(valist);
	}
	Translate728Line(buf[id]);
	((output*)FindObject(name[id]))->Output(buf[id]);
}

void Pop3Get::pstatus(char *s, ...)
{
	static char buf[100];
	buf[0]=0;

	if (s)
	{
		va_list valist;
		va_start(valist,s);
		xc_vsnprintf(buf, sizeof(buf), s, valist);
		va_end(valist);
	}
	if (aw)
	{
		((output*)FindObject(".popstate.state"))->Output(buf);
		XFlush(((output*)FindObject(".popstate.state"))->display());
	}
	else
	{
		::pstatus(buf);
	}
}

void Pop3Get::istatus(int id, int value, char *s, ...)
{
	if (!aw || id>1 || id<0) return;

	static char *name[2] = { ".popstate.overall", ".popstate.message" };
	static char buf[2][200];

	buf[id][0]=0;
		
	if (s)
	{
		va_list valist;
		va_start(valist,s);	
		xc_vsnprintf(buf[id], sizeof(buf[id]), s, valist);
		va_end(valist);
	}
	((indicator*)FindObject(name[id]))->Value(value,buf[id]);
}

void Pop3Get::show_state(int forceopen)
{
	int x,y,w,h;

	if (!forceopen && programconfig.no_pop3statewindow)
	{
		aw=NULL;
		return;
	}

	aw=new appwindow;

	parentaw->Position(x,y);
	parentaw->Size(w,h);

	aw->ParentClass(parentaw);
	aw->InitSize(x+w/2-50 /*200*/,y/*+h/2-50*/,400,100);
	if (!aw->Init()) return;

	aw->Adjust(x-w/3,y-10,w,h,ADJUST_LEFT|ADJUST_BOTTOM);
	aw->Name("popstate");
	aw->Title("Pop3 State");
	aw->IconTitle("Pop3");
	aw->SetIcon(f_xicon1.pixmap(),f_xicon1.shadowmap());
	
	spacegroup *g=new spacegroup;
	g->Name(NULL);
	g->Border(4);
	g->YDistance(4);
	g->Vert();
		output *o=new output;
		o->Name("from");
		o->Font(FONT_SMALL);
		o->Output("--");
	g->Add(o,"From");
		o=new output;
		o->Name("subject");
		o->Font(FONT_SMALL);
		o->Output("--");
	g->Add(o,"Subject");
		indicator *indi=new indicator;
		indi->Name("overall");
		indi->Font(FONT_SMALL);
		indi->Weight(0);
	g->Add(indi);
		indi=new indicator;
		indi->Name("message");
		indi->Font(FONT_SMALL);
		indi->Weight(0);
	g->Add(indi);
		o=new output;
		o->Name("state");
		o->Font(FONT_SMALL);
		o->Output("--");
	g->Add(o,"State");
		button *b=new button;
		b->Name("Cancel");
		b->Text("Cancel");
		b->Callback(this,(CALLBACK) &Pop3Get::breakCB);
	g->Add(b);
	aw->SetGroup(g);
	aw->Create();
	aw->TransientFor(parentaw);
	aw->Callback(this,(CLOSECALLBACK) &Pop3Get::breakCB);
}

void Pop3Get::Start(int pm)
{
	popmode=pm;
	if (pm==POP_BEFORESMTP)
	{
		if (time(NULL)-ready4smtp<60*5)
		{
			sendspooledNOW();
			return;
		}
	}

	if (instances>1)
	{
		if (pm==POP_BEFORESMTP)
		{
			if (time(NULL)-ready4smtp<60*5)
			{
				sendspooledNOW();
			}
		}
		else
		{
			pstatus("Already in use, aborting new try");
		}
		delete this;
		return;
	}


	// send spooled mail by default?
	send_spooled=(programconfig.popsendspooled && spooled);

	if (popmode==POP_INTERACTIVE)
	{
		if (!aw)
		{
			show_state(1);
		}
	}
	if (!connect())
	{
		pstatus("Connected.");
	}
	else
	{
		pstatus("Unable to connect %s!",user->host);
		delete this;
		return;
	}
	return;	

// 	int x,y,w,h;
// 
// 	aw=new appwindow;
// 
// 	parentaw->Position(x,y);
// 	parentaw->Size(w,h);
// 
// 	aw->ParentClass(parentaw);
// 	aw->InitSize(10);
// 	if (!aw->Init()) return;
// 
// 	aw->Adjust(x,y,w,h,ADJUST_CENTER|ADJUST_MIDDLE);
// 	aw->Name("popwindow");
// 	aw->Title("Pop3 authentification");
// 	aw->IconTitle("Pop3");
// 	aw->TransientFor(readerwin);
// 	aw->Callback(this,(CLOSECALLBACK) &Pop3Get::cancelCB);
// 	aw->SetIcon(f_xicon1.pixmap(),f_xicon1.shadowmap());
// 
// 	spacegroup *g=new spacegroup;
// 	g->Name(NULL);
// 	g->Border(4);
// 	g->YDistance(4);
// 	g->Vert();
// 		input *hostip=new input;
// 		hostip->Name("popserver");
// 		hostip->Text("POP3 server");
// 		hostip->MaxChars(POP_HOST_LEN);
// 		hostip->Default(user->_host);
// 	g->Add(hostip);
// 		input *userip=new input;
// 		userip->Name("popuser");
// 		userip->Text("Username");
// 		userip->MaxChars(POP_USER_LEN);
// 		userip->Default(user->user);
// 	g->Add(userip);
// 		input *passwdip=new input;
// 		passwdip->Name("poppassword");
// 		passwdip->Text("Password");
// 		passwdip->MaxChars(POP_PASS_LEN);
// 		passwdip->InvisibleMode();
// 		passwdip->Default(user->pass);
// 		passwdip->Callback(this,(CALLBACK)&Pop3Get::okCB);
// 	g->Add(passwdip);
// 		group *g2=new group;
// 		g2->Name(NULL);
// 		g2->Vert();
// #ifdef MAXSIZE
// 		 char cb2buf[50];
// 		 checkbox *c2=new checkbox;
// 		 xc_snprintf(cb2buf,sizeof(cb2buf),"ignore maxsize settings of %d KB",MAXSIZE);
// 		 c2->Name("ignoreMaxsize");
// 		 c2->Text(cb2buf);
// 		 c2->State(ignoreMaxsize?PRESSED:RELEASED);
// 		g2->Add(c2);
// #endif
// 		 checkbox *c1=new checkbox;
// 		 c1->Name("sendspooled");
// 		 c1->Text("send spooled mail");
// 		 c1->HelpText("send spooled mail before transfering mail from server");
// 		 if (!spooled)
// 		 {
// 			c1->State(RELEASED);
// 			c1->Lock();
// 		 }
// 		 else
// 		 {
// 			c1->State(PRESSED);
// 		 }
// 		g2->Add(c1);
// 	g->Add(g2,"");
// 	g->Add(new ruler);
// 		group *g3=new group;
// 		g3->Horiz();
// 		g3->DialogMode();
// 			button *b=new button;
// 			b->Text("_Ok");
// 			b->Callback(this,(CALLBACK) &Pop3Get::okCB);
// 		g3->Add(b);
// 			b=new button;
// 			b->Text("_Cancel");
// 			b->Callback(this,(CALLBACK) &Pop3Get::cancelCB);
// 			b->ApplyXKey(XK_Escape);
// 		g3->Add(b);
// 	g->Add(g3);
// 	aw->SetGroup(g);
// 	aw->Create();
// 	aw->TransientFor(parentaw);
}

void Pop3Get::lock(void)
{
	is_locked++;
}

void Pop3Get::unlock(void)
{
	if (--is_locked<0) is_locked=0;
}

int Pop3Get::islocked(void)
{
	return is_locked;
}

// ------------------------------------------------------------------------


pUIDLhistory::pUIDLhistory(char *m, char *u, char *p, Pop3Get *p3) : dlist()
{
	mailhost=strdup(m?m:"nowhere");
	prefix=strdup(p?p:".unknown-uidl:");
	user=strdup(u?u:"nobody");
	p3g=p3;
	
	char *h=getenv("HOME");
	int ln=strlen(h)+strlen(prefix)+strlen(mailhost)+strlen(user)+5;
	valid=1;
	
	if (fname=(char*)malloc(ln*sizeof(char)), fname)
	{
		xc_snprintf(fname,ln*sizeof(char),"%s/%s%s@%s",h,prefix,user,mailhost);
	}
	if (ifname=(char*)malloc((ln+8)*sizeof(char)), ifname)
	{
		xc_snprintf(ifname,(ln+8)*sizeof(char),"%s/%s%s@%s_invalid",h,prefix,user,mailhost);
	}
}
pUIDLhistory::~pUIDLhistory(void)
{
	Clean();
	free(mailhost);
	free(user);
	free(fname);
}
void pUIDLhistory::pstatus(char *s)
{
	if (p3g)
	{
		p3g->pstatus(s);
	}
	else
	{
		::pstatus(s);
	}
}
void pUIDLhistory::Read(void)
{
	char lne[200];
	FILE *f;
	if (f=fopen(fname,"r"), f)
	{
		int ucc=0;
	    pstatus("Reading UILD history...");
		setvbuf(f,NULL,_IOFBF,2000);
		while (fgets(lne,sizeof(lne),f)>0)
		{
			int ln=strlen(lne)-1;
			if (lne[ln]=='\n')
			{
				lne[ln]=0;
			}
			Add(lne);
			if (--ucc<=0)
			{
				ucc=50;
				appwindow::UpdateEvent();
			}
		}
		fclose(f);
	}
	if (f=fopen(ifname,"r"), f)
	{
		valid=0;
		fclose(f);
	}
	else
		valid=1;
}
void pUIDLhistory::Write(void)
{
	FILE *f;
	unlink(ifname);
	if (f=fopen(fname,"w"), f)
	{
		int ucc=0;
	    pstatus("Writing UILD history...");
		setvbuf(f,NULL,_IOFBF,2000);
		for (node *n=head; n->next; n=n->next)
		{
			fprintf(f,"%s\n",n->text);
			if (--ucc<=0)
			{
				ucc=50;
				appwindow::UpdateEvent();
			}
		}
		fclose(f);
	    pstatus("Done.");
	}
}
void pUIDLhistory::Clean(void)
{
	int ucc=0;
	node *n;
	while (n=RemTail())
	{
		free(n->text);
		delete n;
		if (--ucc<=0)
		{
			ucc=50;
			appwindow::UpdateEvent();
		}
	}
	valid=1;
}
int pUIDLhistory::Scan(char *u)
{ // exists: true - else: false
	return FindText(u,head)?1:0;
}
void pUIDLhistory::Add(char *u)
{
	if (!FindText(u,head))
	{
		node *n=new node;
		if (n)
		{
			n->text=strdup(u);
			if (n->text) AddTail(n); else delete n;
		}
	}
}
void pUIDLhistory::Remove(char *u)
{
	node *n;
	if (n=FindText(u,head))
	{
		dlist::Remove(n);
		free(n->text);
		delete n;
	}
}
void pUIDLhistory::Conc(pUIDLhistory *h)
{
	Conc(*h);
}
void pUIDLhistory::Conc(pUIDLhistory &h)
{
	int ucc=0;
	for (node *n=h.head; n->next; n=n->next)
	{
		Add(n->text);
		if (--ucc<=0)
		{
			ucc=50;
			appwindow::UpdateEvent();
		}
	}
} 
void pUIDLhistory::Minus(pUIDLhistory *h)
{
	Minus(*h);
}
void pUIDLhistory::Minus(pUIDLhistory &h)
{ // this - h
	int ucc=0;
	node *n1=head,*n2=NULL;
	while (n1->next)
	{
		n2=n1->next;
		if (h.Scan(n1->text))
		{
			dlist::Remove(n1);
			free(n1->text);
			delete n1;
		}
		if (--ucc<=0)
		{
			ucc=50;
			appwindow::UpdateEvent();
		}
		n1=n2;
	}
} 
int pUIDLhistory::Count(void)
{
	return dlist::Count();
}
void pUIDLhistory::RemoveFile(void)
{
	unlink(fname);
	unlink(ifname);
}
int pUIDLhistory::IsValid(void)
{
	return valid;
}
void pUIDLhistory::SetInvalid(void)
{
	valid=0;
}
void pUIDLhistory::WriteInvalid(void)
{
	if (valid)
	{
		unlink(ifname);
	}
	else
	{
		FILE *f;
		if (f=fopen(ifname,"w"))
		{
			fclose(f);
		}
	}
}

// ---

UIDLhistory::UIDLhistory(char *m, char *u, char *p, Pop3Get *p3)
{
	m1=new pUIDLhistory(m,u,p,p3);	// old hist
	m2=new pUIDLhistory(m,u,p,p3);	// working hist
	m3=new pUIDLhistory(m,u,p,p3);	// loaded before writing new
}
UIDLhistory::~UIDLhistory(void)
{
	delete m3;
	delete m2;
	delete m1;
}
void UIDLhistory::Read(void)
{
	m1->Read();
	appwindow::UpdateEvent();
	m2->Read();
}
void UIDLhistory::Write(void)
{
	m3->Clean();
	m3->Read();
	m3->Minus(m1);
	appwindow::UpdateEvent();
	m3->Conc(m2);
	m3->Write();
}
void UIDLhistory::Clean(void)
{
	m2->Clean();
}
int UIDLhistory::Scan(char *u)
{ // exists: true - else: false
	return m2->Scan(u);
}
void UIDLhistory::Add(char *u)
{
	m2->Add(u);
}
void UIDLhistory::Remove(char *u)
{
	m2->Remove(u);
}
void UIDLhistory::Conc(UIDLhistory *h)
{
	Conc(*h);
}
void UIDLhistory::Conc(UIDLhistory &h)
{
	m2->Conc(h.m2);
} 
int UIDLhistory::Count(void)
{
	return m2->Count();
}
void UIDLhistory::RemoveFile(void)
{
	m2->RemoveFile();
}
int UIDLhistory::IsValid(void)
{
	return m2->IsValid();
}
void UIDLhistory::SetInvalid(void)
{
	m2->SetInvalid();
}
void UIDLhistory::WriteInvalid(void)
{
	m2->WriteInvalid();
}

// -------------------------------------------------------------------

static int pop_count=0;
dlist pop3userlist;
extern class newbutton f_readnew;
void nextpop3check(Pop3User *o,int pm)
{
	Pop3User *u=(Pop3User*)pop3userlist.head;
	if (o)
	{
		u=(Pop3User*)o->next;
	}
	if (u->next)
	{
		Pop3Get *p = new Pop3Get(&readerwin,u);
		if (p)
		{
			p->Start(pm);
		}
	}
	else
		pop_count=0;
}
void checkpop3(int now)
{ // 0: count, 1: now, -1: now delayed
	if (!Pop3Get::InAction())
	{
		if (now>0 || (++pop_count>=programconfig.popcheckdelay*600))
		{
			nextpop3check(NULL,now?POP_INTERACTIVE:POP_STATNLOAD);
		}
		else if (now<0)
		{
			pop_count=programconfig.popcheckdelay*600-10;
		}
		static int oldval=-1;
		int val=pop_count/(programconfig.popcheckdelay*6);
		if (val!=oldval)
		{
			f_readnew.Value(oldval=val);
		}
	}
}
void smtpafterpop3(void)
{
	Pop3Get *p = NULL;
	Pop3User *u=(Pop3User*)pop3userlist.head;
	while (u->next)
	{
		if (u->smtpafterpop3)
		{
			p=new Pop3Get(&readerwin,u);
			break;
		}
		u=(Pop3User*)u->next;
	}

	if (!p && ((Pop3User*)pop3userlist.head)->next)
	{
		p=new Pop3Get(&readerwin,(Pop3User*)pop3userlist.head);
		((Pop3User*)pop3userlist.head)->smtpafterpop3=1;
	}
	if (p)
	{
		p->Start(POP_BEFORESMTP);
	}
}

Pop3User *findPop3User(char *name)
{
	Pop3User *u=(Pop3User*)pop3userlist.head;
	while (u->next)
	{
		if (!strcmp(name,u->name))
		{
			return u;
		}
		u=(Pop3User*)u->next;
	}
	return NULL;
}
Pop3User *findPop3User(char *host,char *user)
{
	Pop3User *u=(Pop3User*)pop3userlist.head;
	while (u->next)
	{
		if (!strcmp(host,u->host) && !strcmp(user,u->user))
		{
			return u;
		}
		u=(Pop3User*)u->next;
	}
	return NULL;
}
void rmPop3User(char *name)
{
	rmPop3User(findPop3User(name));
}
void rmPop3User(Pop3User *u)
{
	pop3userlist.Remove(u);
	u->rmprefs();
	delete u;
}
Pop3User *addPop3User(char *name)
{
	char dummy[40];
	if (!name)
	{
		name=dummy;
		xc_snprintf(dummy,sizeof(dummy),"pop3_%x",time(NULL));
	}
	Pop3User *u=new Pop3User(name);
	pop3userlist.AddTail(u);
	return u;
}
void readPop3Prefs(void)
{
	get_smtpafterpop_server=0;
	tokenizer tk(programconfig.popservers,",");
	for(int i=0;i<tk.count();i++)
	{
		if (((char*)tk[i])[0])
			addPop3User((char*)tk[i]);
	}
}
void writePop3Prefs(void)
{
	tokenizer tk(NULL,",");
	Pop3User *u=(Pop3User*)pop3userlist.head;
	while (u->next)
	{
		tk.append(new token(u->name));
		u->saveprefs();
		u=(Pop3User*)u->next;
	}
	strcpy(programconfig.popservers,tk.string());
}
void verifyPop3beforeSMTP(Pop3User *uu)
{
	if (uu->smtpafterpop3)
	{
		Pop3User *u=(Pop3User*)pop3userlist.head;
		while (u->next)
		{
			if (u!=uu)
				u->smtpafterpop3=0;
			u=(Pop3User*)u->next;
		}
	}
}
