/* 
 * This file is part of XCmail and is subject to the 
 * XCmail license agreement
 */

/* $Id: npop3get.h,v 2.13 2002/05/12 15:44:39 schmitzj Exp $ */


#ifndef NPOP3GET
#define NPOP3GET

#define POP_HOST_LEN		100
#define POP_HOST_LENSTR		"100"
#define POP_USER_LEN		50
#define POP_USER_LENSTR		"50"
#define POP_PASS_LEN		50
#define POP_PASS_LENSTR		"50"

#define POP_INTERACTIVE		0	// obsolete
#define POP_SILENT			1
#define POP_TESTONLY		2	// obsolete
#define POP_STATONLY		3	// obsolete
#define POP_STATNLOAD		4
#define POP_BEFORESMTP		5

#define POPSTATE_UNKNOWN	0	// obsolete
#define POPSTATE_UPTODATE	1	// obsolete
#define POPSTATE_NEWMAIL	2	// obsolete

#define HISTORYSIZE			2000


class Pop3User : public node
{
		char dpass[POP_PASS_LEN*4+1];
	public:
		char *name;
		char _host[POP_HOST_LEN+1];
		char host[POP_HOST_LEN+1];
		char user[POP_USER_LEN+1];
		char pass[POP_PASS_LEN+1];
		char vfoldername[POP_HOST_LEN+1];
		int port;
		int popleave;
		int popmaxsize;
		int popdeleteonserver;
		int smtpafterpop3;
		class vfolder *v;
		Pop3User(char *name);
		~Pop3User();
		void loadprefs(void);
		void saveprefs(void);
		void rmprefs(void);
		void setvals(void);
};


class Pop3BoxState : public node
{
		int state;
	public:
		Pop3BoxState(char *s, int t=POPSTATE_UNKNOWN);
		~Pop3BoxState();
		
		void  Name(char *);
		char* Name(void);
		void  State(int);
		int   State(void);
};

class Pop3Get : public Xclasses
{
	void pstatus(char *s=NULL, ...);
	void pheader(int id, char *s=NULL, ...);
	void istatus(int id, int value, char *s=NULL, ...);
		
friend class pUIDLhistory;

	static int		instances;
	static sigjmp_buf jmpbuffer;
	static dlist	boxstate;
	static time_t	ready4smtp;

	Pop3User 		*user;
	
	appwindow*		parentaw;
	appwindow*		aw;
//	mailclass*		mclass;	// no longer used!
	
	class UIDLhistory*	curr_uidlhist;
	class UIDLhistory*	next_uidlhist;
	class UIDLhistory*	dele_uidlhist;
	class UIDLhistory*	retr_uidlhist;
	
	int   			sockfd;
	char			curhost_ip[32];
	char			uidl[256];
	char			error_text[256];
	int				send_spooled;
	int 			beepdone;
	int				mailindex;
	int				mailcount;
	int				foldersize;
	int				bytecount;
	int				my_mclass;
	int				tobigsize;
	
	char*			mailtext;
	int				mailsize;
	int				mailused;

	int				is_locked;

	int				retrieved;
	int				retrieve;
	int				uidl_found;
	int				extraheadersadded;
	int				bytes_read;
	int				popmode;

	int			mode;

	static void in_timeout(int);
	int connect(void);
	void disconnect(void);

	int write(char *, ...);

	int read_line(char *, int);
	int read_multiline(char *, int);
	int read_result(void);
	int read_result(char *, int);
	int read_handler(int fd);
	int mailtext_append(char *);
	int retrieve_or_ignore(int fd);
	void finish_successfully(void);
		
	void okCB(gadget *g,int i);
	void cancelCB(void);
	void breakCB(void);
	
	void show_state(int forceopen=0);

	void lock(void);
	void unlock(void);
	int  islocked(void);
		
public:
	Pop3Get(appwindow *awP,Pop3User *);
	~Pop3Get();
		
	void Start(int =POP_INTERACTIVE);
//	static Pop3User *LastUser(void) {return user;};
	static int InAction(void) {return instances?1:0;};
};

class pUIDLhistory : private dlist
{
	Pop3Get* p3g;

	char *prefix;
	char *user;
	char *mailhost;
	char *fname;
	char *ifname;

	void pstatus(char *);
	int valid;
	
public:
	pUIDLhistory(char *, char *, char * =NULL, Pop3Get* =NULL); // mailhost, user, prefix (opt), Pop3Get (opt)
	~pUIDLhistory(void);

	int  Scan(char *);
	void Conc(pUIDLhistory&);
	void Conc(pUIDLhistory*);
	void Minus(pUIDLhistory&);
	void Minus(pUIDLhistory*);
	void Add(char *);
	void Remove(char *u);

	void Read(void);
	void Write(void);
	void Clean(void);

	int Count(void);
	int IsValid(void);
	void SetInvalid(void);
	void WriteInvalid(void);

	void RemoveFile(void);
};

class UIDLhistory
{
	pUIDLhistory *m1;	
	pUIDLhistory *m2;
	pUIDLhistory *m3;

public:
	UIDLhistory(char *, char *, char * =NULL, Pop3Get* =NULL); // mailhost, user, prefix (opt), Pop3Get (opt)
	~UIDLhistory(void);

	int  Scan(char *);
	void Conc(UIDLhistory&);
	void Conc(UIDLhistory*);
	void Add(char *);
	void Remove(char *u);

	void Read(void);
	void Write(void);
	void Clean(void);

	int Count(void);
	int IsValid(void);
	void SetInvalid(void);
	void WriteInvalid(void);

	void RemoveFile(void);
};

extern dlist pop3userlist;
void nextpop3check(Pop3User *,int pm=POP_STATNLOAD);
void checkpop3(int now=0);
void smtpafterpop3(void);
Pop3User *findPop3User(char *name);
Pop3User *findPop3User(char *host,char *user);
void rmPop3User(char *name);
void rmPop3User(Pop3User *);
Pop3User *addPop3User(char *name=NULL);
void readPop3Prefs(void);
void writePop3Prefs(void);
void verifyPop3beforeSMTP(Pop3User *);

#endif
