/* 
 * This file is part of XCmail and is subject to the 
 * XCmail license agreement
 */

/* $Id: pgp.cxx,v 2.3 2000/05/25 07:29:34 schmitzj Exp $ */


#include "configure.h"

#include <string.h>
#include <strings.h>
#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/time.h>
#include <sys/types.h>
#ifdef HAVE_SYS_SELECT_H
#include <sys/select.h>
#endif

#include "mailer.h"
#include "pgp.h"

enum PGPerrors PGPerrno=PGP_OK;

static void PGPclosewindow(appwindow *a)
{
	a->Close();
	DeleteObjectTree(a->FullName(),0);
}
static void PGPclosewindowGG(gadget *g,int i)
{
	appwindow *a=(appwindow*)g->ParentClassType("appwindow");
	a->Close();
	DeleteObjectTree(a->FullName(),0);
}
static char *PGPid(void)
{
	if (strlen(programconfig.pgpid))
	{
		static char buf[55];
		strcpy(buf,"-u \"");
		strcat(buf,programconfig.pgpid);
		strcat(buf,"\"");
		return buf;
	}
	return "";
}

// appwindow, command, PW, PW-Pipe, ERR-Pipe
static char *runPGP(appwindow *aw,char *buf,char *pwb,int *fds,int *fds2)
{
	char *t=NULL;
	FILE *f;

	if (f=popen(buf,"r"))
	{
		static int pgpwinname=0;
		int l,ma=0,mu=0,mx,cc;
		char cs[20];
		char nme[300];
		fd_set readfds,creadfds;
		appwindow *maw=new appwindow;
		spacegroup *mgp1=new spacegroup;
		group *mgp2=new group;
		textbox *mtb=new textbox;
		fbutton *mb=new fbutton;

		if (fds) write(fds[1],pwb,strlen(pwb));

		int x,y,w,h;

		aw->WMPosition(x,y);
		aw->Size(w,h);

		xc_snprintf(nme,sizeof(nme),"pgp_status_window_%x",pgpwinname++);
		maw->Name(nme);
		maw->ParentClass(aw);
		maw->InitSize(40);
		maw->Init();
		maw->Adjust(x,y,w,h,ADJUST_TOP|ADJUST_CENTER);
		maw->TransientFor(aw);
		maw->Title("PGP output");

		mgp1->Vert();
		mgp1->YDistance(4);
		mgp1->Border(4);
		 mtb->ReadOnly();
		 mtb->AlwaysShowLast();
		 mtb->MinCols(60);
		 mtb->MinRows(5);
		mgp1->Add(mtb);
		 mgp2->Horiz();
		 mgp2->DialogMode();
		  mb->Name("closebutton");
		  mb->Text(" _Close ");
		  mb->Callback(PGPclosewindowGG);
		  mb->Lock();
		 mgp2->Add(mb);
		mgp1->Add(mgp2);
		maw->Callback(PGPclosewindow);
		maw->SetGroup(mgp1);
		maw->Create();

		FD_ZERO(&readfds);
		FD_SET(fds2[0],&readfds);	fcntl(fds2[0],F_SETFL,O_NONBLOCK);
		FD_SET(fileno(f),&readfds);	fcntl(fileno(f),F_SETFL,O_NONBLOCK);
		mx=fds2[0];
		if (fileno(f)>mx) mx=fileno(f);
		creadfds=readfds;

		appwindow::UpdateEvent();
		cc=0;
		cs[0]=0;

		int r,din=1,dst=1;
		int timeoutc=0;
		struct timeval tv;
		tv.tv_sec = 1;
		tv.tv_usec = 0;
		while ((din || dst) && (r=select(mx+1,&creadfds,NULL,NULL,&tv))>=0)
		{ // printf("TOC: %d, %d (%d/%d)\n",timeoutc,r,FD_ISSET(fileno(f),&creadfds),FD_ISSET(fds2[0],&creadfds));
			if (cc>5)
			{
				cs[cc]=0;
				mtb->AppendText(cs);
				cc=0;
				appwindow::UpdateEvent();
			}
			if (r==0)
			{
				if (++timeoutc>2)
				{
					din=0;
					dst=0;
				}
			}
			else if (r>0)
				timeoutc=0;
			if (din && FD_ISSET(fileno(f),&creadfds))
			{
				if (fgets(buf,sizeof(buf)-1,f)>0)
				{
					l=strlen(buf);
					if (mu+l+1>ma)
					{
						ma+=l+1+1000;
						t=(char*)realloc(t,ma);
					}
					strcpy(&t[mu],buf);
					mu+=l;
				}
				else
				{
					din=0;
					FD_CLR(fileno(f),&readfds);
				}
			}
			if (dst && FD_ISSET(fds2[0],&creadfds))
			{
				if (read(fds2[0],&cs[cc],1)>0)
					cc++;
				else
				{
					dst=0;
					FD_CLR(fds2[0],&readfds);
				}
			}
			creadfds=readfds;
		}
		if (cc>0)
		{
			cs[cc]=0;
			mtb->AppendText(cs);
			cc=0;
		}
		pclose(f);
		if (mu>0)
		{
			PGPerrno=PGP_OK;
			if (programconfig.pgpleaveopen)
				mb->Unlock();
			else
			{
				maw->Close();
				DeleteObjectTree(maw->FullName(),0);
			}
		}
		else
		  mb->Unlock();
	}
	else
		PGPerrno=PGP_CANTDECODE;
	return t;
}

int PGPcode(class appwindow *aw,char *tfile,char *tofile,char *head,char *bound,int md)
{
	// md = 1: sign
	//      2: code
	//      3: code & sign

	FILE *f;
	char *to,*t;
	struct stat st;
	PGPerrno=PGP_OK;

	stat(tofile,&st);
	if (f=fopen(tofile,"r"))
	{
		if (to=new char[st.st_size+1])
		{
			fread(to,1,st.st_size,f);
			to[st.st_size]=0;
			fclose(f);
			t=to;
			while (*t)
			{
				if (*t=='\n' || *t=='\r') *t=' ';
				t++;
			}
		}
		else
		{
			fclose(f);
			PGPerrno=PGP_NOMEMORY;
			return 1;
		}
	}
	else
	{
		PGPerrno=PGP_CANTREAD;
		return 1;
	}
	if (md==1)
	{
		char *pw=getPW(aw);
		if (pw && (t=PGPsign(aw,tfile,pw)))
		{
			char tfile2[strlen(tfile)+10];
			strcpy(tfile2,tfile);
			strcat(tfile2,"_copy");
			remove(tfile2);
			rename(tfile,tfile2);
			if (f=fopen(tfile,"w"))
			{
				setvbuf(f,NULL,_IOFBF,2000);
				fwrite(head,strlen(head),1,f);
				fputc('\n',f);

				fprintf(f,"This is the preamble area of a multipart message.\n"
						 "Mail readers that understand multipart format\n"
						 "should ignore this preamble.\n\n"
						 "If you are reading this text, you might want to\n"
						 "consider changing to a mail reader that understands\n"
						 "how to properly display multipart messages.\n(like the cool "PROGRAMNAME")\n\n");

				fprintf(f,"--%s\n",bound);
				{
					FILE *f2;
					char tbuf[500];
					int l;
					if (f2=fopen(tfile2,"r"))
					{
						setvbuf(f2,NULL,_IOFBF,2000);
						while ((l=fread(tbuf,1,sizeof(tbuf),f2))>0)
						{
							fwrite(tbuf,1,l,f);
						}
						fclose(f2);
					}
				}
				fprintf(f,"\n");

				fprintf(f,"--%s\n",bound);
				fprintf(f,"Content-Type: application/pgp-signature\n\n");
				fwrite(t,strlen(t),1,f);
				fprintf(f,"\n--%s--\n",bound);
				fclose(f);
			}
			else
				PGPerrno=PGP_CANTWRITE;
			free(t);
			remove(tfile2);
		} else PGPerrno=PGP_CANTSIGN;
	}
	else if (md==2)
	{
		if (t=PGPcode(aw,tfile,to))
		{
			if (f=fopen(tfile,"w"))
			{
				setvbuf(f,NULL,_IOFBF,2000);
				fwrite(head,strlen(head),1,f);
				fputc('\n',f);

				fprintf(f,"This is the preamble area of a multipart message.\n"
						 "Mail readers that understand multipart format\n"
						 "should ignore this preamble.\n\n"
						 "If you are reading this text, you might want to\n"
						 "consider changing to a mail reader that understands\n"
						 "how to properly display multipart messages.\n(like the cool "PROGRAMNAME")\n\n");

				fprintf(f,"--%s\n",bound);
				fprintf(f,"Content-Type: application/pgp-encrypted\n\n");
				fprintf(f,"Version: 1\n\n");

				fprintf(f,"--%s\n",bound);
				fprintf(f,"Content-Type: application/octet-stream\n\n");
				fwrite(t,strlen(t),1,f);
				fprintf(f,"\n--%s--\n",bound);
				fclose(f);
			}
			else
				PGPerrno=PGP_CANTWRITE;
			free(t);
		} else PGPerrno=PGP_CANTENCODE;
	}
	else if (md==3)
	{
		char *pw=getPW(aw);
		if (pw && (t=PGPcodesign(aw,tfile,pw,to)))
		{
			if (f=fopen(tfile,"w"))
			{
				setvbuf(f,NULL,_IOFBF,2000);
				fwrite(head,strlen(head),1,f);
				fputc('\n',f);

				fprintf(f,"This is the preamble area of a multipart message.\n"
						 "Mail readers that understand multipart format\n"
						 "should ignore this preamble.\n\n"
						 "If you are reading this text, you might want to\n"
						 "consider changing to a mail reader that understands\n"
						 "how to properly display multipart messages.\n(like the cool "PROGRAMNAME")\n\n");

				fprintf(f,"--%s\n",bound);
				fprintf(f,"Content-Type: application/pgp-encrypted\n\n");
				fprintf(f,"Version: 1\n\n");

				fprintf(f,"--%s\n",bound);
				fprintf(f,"Content-Type: application/octet-stream\n\n");
				fwrite(t,strlen(t),1,f);
				fprintf(f,"\n--%s--\n",bound);
				fclose(f);
			}
			else
				PGPerrno=PGP_CANTWRITE;
			free(t);
		} else PGPerrno=PGP_CANTSIGN;
	}
	delete to;
	return (PGPerrno!=PGP_OK)?1:0;
}
char *PGPsign(class appwindow *aw,char *txt,char *pw)
{
	char pwb[30];
	char *t=NULL;
	char buf[450];
	int fds[2],fds2[2];

	pipe(fds);
	pipe(fds2);

	strcpy(pwb,pw);
	strcat(pwb,"\n");

	xc_snprintf(buf,sizeof(buf),"PGPPASSFD=%d;export PGPPASSFD;%s +batchmode -afs %s <%s 2>&%d",fds[0],programconfig.pgpcommand,PGPid(),txt,fds2[1]);
	t=runPGP(aw,buf,pwb,fds,fds2);

	close(fds2[0]);
	close(fds2[1]);
	close(fds[0]);
	close(fds[1]);
	return t;
}
char *PGPcode(class appwindow *aw,char *txt,char *to)
{
	char *t=NULL;
	char buf[450+strlen(to)];
	int fds[2],fds2[2];

//	pipe(fds);
	pipe(fds2);

	xc_snprintf(buf,sizeof(buf),"%s +batchmode -aef %s %s <%s 2>&%d",programconfig.pgpcommand,PGPid(),to,txt,fds2[1]);
	t=runPGP(aw,buf,"",NULL,fds2);

	close(fds2[0]);
	close(fds2[1]);
//	close(fds[0]);
//	close(fds[1]);
	return t;
}

char *PGPcodesign(class appwindow *aw,char *txt,char *pw,char *to)
{
	char *t=NULL;
	char buf[450+strlen(to)];
	char pwb[30];
	int fds[2],fds2[2];

	pipe(fds);
	pipe(fds2);

	strcpy(pwb,pw);
	strcat(pwb,"\n");

	xc_snprintf(buf,sizeof(buf),"PGPPASSFD=%d;export PGPPASSFD;%s +batchmode -asef %s %s <%s 2>&%d",fds[0],programconfig.pgpcommand,PGPid(),to,txt,fds2[1]);
	t=runPGP(aw,buf,pwb,fds,fds2);

	close(fds2[0]);
	close(fds2[1]);
	close(fds[0]);
	close(fds[1]);
	return t;
}

// ------------------------------------------------------------------------

int PGPcheckSign(class appwindow *aw,char *txt,char *sign)
// 1: Ok
{
	int r=0;
	FILE *f;
	char *t=NULL;
	char buf[450];
	int fds2[2];
	char fname[200];

	xc_snprintf(fname,sizeof(fname),"%s/%s_%x.pgp",TMPDIR,PROGRAMNAME,time(NULL));
	remove(fname);
	if (f=fopen(fname,"w"))
	{
		fwrite(sign,strlen(sign),1,f);
		fclose(f);
	}
	else
	{
		PGPerrno=PGP_CANTWRITE;
		return r;
	}

	pipe(fds2);

	xc_snprintf(buf,sizeof(buf),"%s +batchmode -f %s <%s 2>&%d",programconfig.pgpcommand,PGPid(),fname,fds2[1]);

	t=runPGP(aw,buf,"",NULL,fds2);

	close(fds2[0]);
	close(fds2[1]);
	remove(fname);

	if (t)
	{
		register char *c1=t;
		register char *c2=txt;
		while (*c1 && (r=(*c1==*c2)))
		{
			c1++;
			c2++;
		}
		free(t);
	}
	return r;
}

char *PGPdecode(class appwindow *aw,char *txt,char *pw,int &sign)
{
	FILE *f;
	char *t=NULL;
	char buf[450];
	char pwb[30];
	int fds[2],fds2[2];
	char fname[200];

	xc_snprintf(fname,sizeof(fname),"%s/%s_%x.pgp",TMPDIR,PROGRAMNAME,time(NULL));
	remove(fname);
	if (f=fopen(fname,"w"))
	{
		fwrite(txt,strlen(txt),1,f);
		fclose(f);
	}
	else
	{
		PGPerrno=PGP_CANTWRITE;
		return NULL;
	}

	pipe(fds);
	pipe(fds2);

	xc_snprintf(buf,sizeof(buf),"PGPPASSFD=%d;export PGPPASSFD;%s +batchmode -f %s <%s 2>&%d",fds[0],programconfig.pgpcommand,PGPid(),fname,fds2[1]);
	strcpy(pwb,pw);
	strcat(pwb,"\n");

	t=runPGP(aw,buf,pwb,fds,fds2);

	close(fds2[0]);
	close(fds2[1]);
	close(fds[0]);
	close(fds[1]);
	remove(fname);
	return t;
}

void PGPaddkey(class appwindow *aw,char *txt)
{
	int r=0;
	FILE *f;
	char *t=NULL;
	char buf[450];
	int fds2[2];
	char fname[200];

	xc_snprintf(fname,sizeof(fname),"%s/%s_%x.key",TMPDIR,PROGRAMNAME,time(NULL));
	remove(fname);
	if (f=fopen(fname,"w"))
	{
		fwrite(txt,strlen(txt),1,f);
		fclose(f);
	}
	else
	{
		PGPerrno=PGP_CANTWRITE;
		return;
	}

	pipe(fds2);

	xc_snprintf(buf,sizeof(buf),"%s +batchmode -ka %s 2>&%d",programconfig.pgpcommand,fname,fds2[1]);

	t=runPGP(aw,buf,"",NULL,fds2);

	close(fds2[0]);
	close(fds2[1]);
	remove(fname);
}
