/* 
 * This file is part of XCmail and is subject to the 
 * XCmail license agreement
 */

/* $Id: plugins.cxx,v 2.15 2002/01/14 19:09:20 schmitzj Exp $ */


#include "configure.h"

#include "mailer.h"
#include "plugins.h"
#include "pluginsmgr.h"
#include "strbuf.h"
#include "createmail.h"
#include "stringTokenizer.h"

#include <stdio.h>
#include <string.h>
#include <strings.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <unistd.h>
#include <errno.h>
#include "folder.h"

char *current_plugin_directory="(buildin)";

#ifdef PLUGINS
#ifdef HAVE_DLFCN_H
#include <dlfcn.h>
#endif

static dlist pluginlist;

static void addplugin(char *dir,char *name)
{
	char buf[400];
	struct node *n;
	void *dl;

	strcpy(buf,dir);
	strcat(buf,"/");
	strcat(buf,name);
	current_plugin_directory=buf;

	if (developLevel("SHOWPLUGINS"))
	{
		fprintf(stderr,"Check plugin %s\n",buf);
	}

#ifdef HAVE_DLOPEN_FLAGS
	if (dl=dlopen(buf,RTLD_GLOBAL|RTLD_LAZY))
#else
	if (dl=dlopen(buf,1))
#endif
	{
		n=new node;
		n->text=(char*)dl;
		pluginlist.AddTail(n);
		if (developLevel("SHOWPLUGINS"))
		{
			fprintf(stderr,"Loaded plugin %s\n",buf);
		}
	}
}

void readplugins(void)
{
	char buf[400],pbuf[100];
	DIR *d;
	struct dirent *de;
	int l;

	strcpy(buf,getenv("HOME"));
	strcat(buf,"/."PROGRAMNAME"/plugins");

	tokenizer t(programconfig.disabledPlugins,",");
	
	if ((!developLevel("LOCALPLUGINS")) && (d=opendir(buf)))
	{
		while (de=readdir(d))
		{
			l=strlen(de->d_name);
			if (l>2 && de->d_name[l-1]=='o' && de->d_name[l-2]=='.')
			{
				strcpy(pbuf,"USER::");
				strncat(pbuf,de->d_name,sizeof(pbuf));
				if (!t.find(pbuf))
				{
					addplugin(buf,de->d_name);
				}
			}
		}
		closedir(d);
	}
	else
	{
		if (errno==ENOENT)
		{
			mkdir(buf,S_IRUSR|S_IWUSR|S_IXUSR);
		}
	}

	strcpy(buf,GLOBALCFGDIR);
	strcat(buf,"/plugins");

	if ((!developLevel("GLOBALPLUGINS")) && (d=opendir(buf)))
	{
		while (de=readdir(d))
		{
			l=strlen(de->d_name);
			if (l>2 && de->d_name[l-1]=='o' && de->d_name[l-2]=='.')
			{
				strcpy(pbuf,"GLOBAL::");
				strncat(pbuf,de->d_name,sizeof(pbuf));
				if (!t.find(pbuf))
				{
					addplugin(buf,de->d_name);
				}
			}
		}
		closedir(d);
	}
}

void freeplugins(void)
{
	struct node *n;

	while (n=pluginlist.RemTail())
	{
		dlclose((void*)n->text);
		delete n;
	}
}

#else

void readplugins(void)
{
}
void freeplugins(void)
{
}

#endif

pluginmgr *prgplugins=NULL;

static char *types[]={	"Unknown",
						"Helper",
						"Encoder",
						"Main enhancer",
						"Read enhancer",
						"Write enhancer",
						"Filter spec",
						"Filter condition",
						"Filter job",
						NULL};

pluginmgr::pluginmgr(void) : pluginslist()
{
}
pluginmgr::~pluginmgr()
{
	clear();
}
void pluginmgr::clear(void)
{
	while (pluginslist.RemTail()) ;
}

plugins *pluginmgr::next(plugins *old)
{
	if (prgplugins!=this) return prgplugins->next(old);

	plugins *p;
	if (!old) p=(plugins*)pluginslist.head; else p=(plugins*)old->next;
	if (p->next)
	{
		return p;
	}
	return NULL;
}
plugins *pluginmgr::nextOfType(enum plugins::PLUGINTYPE t, plugins *old)
{
	if (prgplugins!=this) return prgplugins->nextOfType(t,old);

	plugins *p;
	if (!old) p=(plugins*)pluginslist.head; else p=(plugins*)old->next;
	while (p->next)
	{
		if (p->type==t)
		{
			return p;
		}
		p=(plugins*)p->next;
	}
	return NULL;
}
plugins *pluginmgr::find(char *name)
{
	if (prgplugins!=this) return prgplugins->find(name);

	plugins *p;
	p=(plugins*)pluginslist.head;
	while (p->next)
	{
		if (!strcmp(p->name,name))
		{
			return p;
		}
		p=(plugins*)p->next;
	}
	return NULL;
}
char *pluginmgr::typeName(plugins *p)
{
	return types[p->type];
}
char *pluginmgr::typeName(enum plugins::PLUGINTYPE t)
{
	return types[t];
}

void pluginmgr::add(plugins *p)
{
	if (prgplugins!=this) { prgplugins->add(p); return; }

	if (!pluginslist.FindText(p->plugin_name(),pluginslist.head))
	{
		pluginslist.AddTail(p);
		p->text=p->plugin_name();
	}
}
void pluginmgr::remove(plugins *p)
{
	if (prgplugins!=this) { prgplugins->remove(p); return; }

	if (pluginslist.FindText(p->plugin_name(),pluginslist.head))
	{
		pluginslist.Remove(p);
	}
}
void pluginmgr::readconfigfiles(void)
{
	if (prgplugins!=this) { prgplugins->readconfigfiles(); return; }

	if (pluginslist.Count())
	{
		plugins *n=(plugins*)pluginslist.head;

		while (n->next)
		{
			if (n->flags & PLF_CONFIGURABLE)
			{
				n->readconfigfile();
			}
			n=(plugins*)n->next;
		}
	}
}
void pluginmgr::writeconfigfiles(void)
{
	if (prgplugins!=this) { prgplugins->writeconfigfiles(); return; }

	if (pluginslist.Count())
	{
		plugins *n=(plugins*)pluginslist.head;

		while (n->next)
		{
			if (n->flags & PLF_CONFIGURABLE)
			{
				n->writeconfigfile();
			}
			n=(plugins*)n->next;
		}
	}
}
void pluginmgr::readconfig(void)
{
	if (prgplugins!=this) { prgplugins->readconfig(); return; }

	if (pluginslist.Count())
	{
		plugins *n=(plugins*)pluginslist.head;

		while (n->next)
		{
			if (n->flags & PLF_CONFIGURABLE)
			{
				n->readconfig(n->configgp);
			}
			n->configgp=NULL;
			n=(plugins*)n->next;
		}
	}
}
void pluginmgr::setupconfig(void)
{
	if (prgplugins!=this) { prgplugins->setupconfig(); return; }

	if (pluginslist.Count())
	{
		plugins *n=(plugins*)pluginslist.head;

		while (n->next)
		{
			if (n->flags & PLF_CONFIGURABLE)
			{
				n->setupconfig(n->configgp);
			}
			n=(plugins*)n->next;
		}
	}
}
int pluginmgr::makeconfig(listgroup *g)
{
	if (prgplugins!=this) { return prgplugins->makeconfig(g); }

	int cc=0;
	if (pluginslist.Count())
	{
		plugins *n=(plugins*)pluginslist.head;

		while (n->next)
		{
			if (!(n->flags & PLF_INVISIBLE))
			{
				cc++;
				bordergroup *g1=new bordergroup;
				g1->Vert();
				g1->YDistance(3);
				g1->Title(n->name);
				 output *op=new output;
				 op->Text("Plugin name");
				 op->Output(n->name);
				g1->Add(op);
  				 op=new output;
  				 op->Text("Type");
  				 op->Output(types[n->type]);
  				g1->Add(op);
  				 op=new output;
  				 op->Text("Author");
  				 op->Output(n->author);
  				g1->Add(op);

				if (n->flags & PLF_CONFIGURABLE)
				{
					g1->Add(new ruler);
					text *txt=new text;
					txt->Name(NULL);
					txt->Center();
					txt->Text("Plugin dependent settings");
					g1->Add(txt);
					n->configgp=n->makeconfig();
					g1->Add(n->configgp);
				}
				else
				{
					g1->Add(new ruler);
					n->configgp=NULL;
					text *t=new text;
					t->Center();
					t->Text("No extra settings for this plugin.");
					g1->Add(t);
				}
// 		    	if (n->information)
// 		    	{
// 					g1->Add(new ruler);
// 					text *txt=new text;
// 					txt->Name(NULL);
// 					txt->Center();
// 					txt->Text("Description");
// 					g1->Add(txt);
// 					autotextbox *atb=new autotextbox;
// 					atb->AppendText(n->information);
// 					atb->Weight(40);
// 					g1->Add(atb);
// 				}
				g->Add(n->name,g1);
			}
			n=(plugins*)n->next;
		}
	}
	return cc;
}
void pluginmgr::makemenu(menu *m)
{
	if (prgplugins!=this) { prgplugins->makemenu(m); return; }

}

// -------------------------------------------------------------------------

plugins::plugins(char *fname,char *fauthor,enum PLUGINTYPE t,unsigned int fflags,char *finfo) : callable()
{
	if (fname && developLevel("LISTPLUGINS"))
	{
		fprintf(stderr,"Added plugin %s\n",fname);
	}

	if (!prgplugins) prgplugins=new pluginmgr;
	name=fname;
	author=fauthor;
	type=t;
	flags=fflags;
	information=finfo;
	filename=current_plugin_directory?strdup(current_plugin_directory):(char*)NULL;
	if (fname) prgplugins->add(this);
}
plugins::plugins(class plugins *h) : callable()
{
	flags=h->flags;
	name=h->name;
	author=h->author;
	information=NULL;
	type=h->type;
	filename=NULL;
}
plugins::~plugins()
{
	if (filename) free(filename);
}
char *plugins::plugin_name(void)
{
	return name;
}
char *plugins::plugin_author(void)
{
	return author;
}
char *plugins::plugin_information(void)
{
	return information;
}
char *plugins::plugin_filename(void)
{
	return filename;
}
void plugins::plugin_filename(char *f)
{
	if (filename) free(filename);
	filename=strdup(f);
}
char *plugins::programversion(void)
{
	return VERSION;
}
char *plugins::settingspath(void)
{
	static xcm_strbuf path;
	path.clear();

	char *h=getenv("HOME");
	if (h)
	{
		path+=h;
		path+="/."PROGRAMNAME"/";
	}
	return path();
}

char *plugins::egets(char *to,int size,char **buf)
{
	char *c=to;
	int i=0;

	if (!**buf) return NULL;

	while (**buf && **buf!='\n' && i<size)
	{
		*to=**buf;
		to++; (*buf)++;
		i++;
	}
	if (**buf=='\n') (*buf)++;
	*to=0;
	return c;
}
void plugins::eputs(char *txt,char **buf)
{
	strcpy(*buf,txt);
	while (**buf) (*buf)++;
}
class plugins *plugins::clone(void)
{
	fprintf(stderr,"WARNING: Plugin %s clone call not redirected!\n",name);
	return NULL;
}
group *plugins::makeconfig(void)
{
	return NULL;
}
void plugins::readconfig(group *)
{
}
void plugins::setupconfig(group *)
{
}
void plugins::readconfigfile(void)
{
}
void plugins::writeconfigfile(void)
{
}
int plugins::newmail(char *to,char *cc,char *subject,char *body)
{
	createmail cm;
	cm.to(to);
	cm.cc(cc);
	cm.subject(subject);
	cm.body(body);
	return cm.send();
}
int plugins::replymail(mail *m,char *subject,char *body)
{
	createmail cm;
	cm.replytomail(m);
	cm.to(m->replyto());
	cm.subject(subject);
	cm.body(body);
	return cm.send();
}
int plugins::forwardmail(mail *m,char *to,char *subject)
{
	int dummy;
	createmail cm;
	cm.forwardtomail(m);
	cm.to(to);
	cm.subject(subject);
	cm.body(m->mailbody(dummy,0));
	return cm.send();
}
class folders *plugins::folderlist(void)
{
	return ::folderlist;
}
class iolist *plugins::iolist(void)
{
	return ::allios;
}

// -------------------------------------------------------------------------

mainplugin::mainplugin(char *name,char *author,unsigned int flags=0,char *information=NULL) : plugins(name,author,PT_Maintool,flags,information)
{
}
mainplugin::mainplugin(class mainplugin *m) : plugins(m)
{
}
mainplugin::~mainplugin()
{
}
mainplugin *mainplugin::clone(void)
{
	return NULL;
}

struct mainpluginmenustruct mainpluginmenu[MAINPLUGINMENUMAX];
int mainpluginmenucount=0;
void mainplugin::addmenu(unsigned int flag,int id,char *text,char *popuptext,char*shortcut)
{
	if (mainpluginmenucount<MAINPLUGINMENUMAX)
	{
		mainpluginmenu[mainpluginmenucount].text		=text;
		mainpluginmenu[mainpluginmenucount].popuptext	=popuptext;
		mainpluginmenu[mainpluginmenucount].shortcut	=shortcut;
		mainpluginmenu[mainpluginmenucount].id			=id;
		mainpluginmenu[mainpluginmenucount].pl			=this;
		mainpluginmenu[mainpluginmenucount].general		=flag&MPMENU_GENERAL?1:0;
		mainpluginmenu[mainpluginmenucount].marked		=flag&MPMENU_MARKED?1:0;
		mainpluginmenu[mainpluginmenucount].selected	=flag&MPMENU_SELECTED?1:0;

		mainpluginmenucount++;
	}
}

void mainplugin::init(void)
{
}
void mainplugin::finish(void)
{
}
void mainplugin::handle(int id,mail *)
{
}
void mainplugin::handle(int id)
{
}

// -------------------------------------------------------------------------

newmailplugin::newmailplugin(char *name,char *author,unsigned int flags=0,char *information=NULL) : plugins(name,author,PT_Newmailtool,flags,information)
{
}
newmailplugin::newmailplugin(class newmailplugin *m) : plugins(m)
{
}
newmailplugin::~newmailplugin()
{
}
newmailplugin *newmailplugin::clone(void)
{
	return NULL;
}

struct newmailpluginmenustruct newmailpluginmenu[NEWMAILPLUGINMENUMAX];
int newmailpluginmenucount=0;
void newmailplugin::addmenu(int id,char *text,char *shortcut)
{
	if (newmailpluginmenucount<NEWMAILPLUGINMENUMAX)
	{
		newmailpluginmenu[newmailpluginmenucount].text=text;
		newmailpluginmenu[newmailpluginmenucount].shortcut=shortcut;
		newmailpluginmenu[newmailpluginmenucount].id=id;
		newmailpluginmenu[newmailpluginmenucount].pl=this;

		newmailpluginmenucount++;
	}
}

void newmailplugin::init(class Xclasses *)
{
}
void newmailplugin::finish(class Xclasses *)
{
}
void newmailplugin::handle(int id,class Xclasses *)
{
}

// -------------------------------------------------------------------------

showmailplugin::showmailplugin(char *name,char *author,unsigned int flags=0,char *information=NULL) : plugins(name,author,PT_Showmailtool,flags,information)
{
}
showmailplugin::showmailplugin(class showmailplugin *m) : plugins(m)
{
}
showmailplugin::~showmailplugin()
{
}
showmailplugin *showmailplugin::clone(void)
{
	return NULL;
}

struct showmailpluginmenustruct showmailpluginmenu[SHOWMAILPLUGINMENUMAX];
int showmailpluginmenucount=0;
void showmailplugin::addmenu(int id,char *text,char *shortcut)
{
	if (showmailpluginmenucount<SHOWMAILPLUGINMENUMAX)
	{
		showmailpluginmenu[showmailpluginmenucount].text=text;
		showmailpluginmenu[showmailpluginmenucount].shortcut=shortcut;
		showmailpluginmenu[showmailpluginmenucount].id=id;
		showmailpluginmenu[showmailpluginmenucount].pl=this;

		showmailpluginmenucount++;
	}
}

void showmailplugin::init(class Xclasses *)
{
}
void showmailplugin::finish(class Xclasses *)
{
}
void showmailplugin::handle(int id,class Xclasses *)
{
}
