/* 
 * This file is part of XCmail and is subject to the 
 * XCmail license agreement
 */

/* $Id: ps_printer.cxx,v 2.9 1999/03/26 16:36:29 schmitzj Exp $ */


#include "configure.h"

#include "mailer.h"
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sys/types.h>
#include <unistd.h>
#include <limits.h>
#include <locale.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h> 
#include <sys/utsname.h>
#include <string.h>
#include "ps_printer.h"

struct utsname sysInfo;


int printMail::translateCharacter (FILE *out, unsigned char c) {

	if (c == '\0')
		return 0;

	if (c == '\n') {
		fprintf ( out, ") printline\n");
		PageLines ++;
 		characters = 0;
		if (PageLines >= linesperpage)
			return NEW_PAGE;
		fprintf ( out, "(");
		return 0;
	} /* if */

	if (c == '\t') {
	
		int anzChars;
		int i;
		
		anzChars = 4 - (characters - ((characters/4)*4));

		for (i=0; i < anzChars; i++ )	
			fprintf ( out, " ");
		return 0;
	} /* if */


    if (c >= ' ' && c < 0177) {
		if (c == '(' || c == ')' || c == '\\')
	    	fprintf ( out, "\\");
		fprintf ( out, "%c", c);
		characters ++;
		return 0;
    }

    if (c > 0177) {
		fprintf( out, "\\%o", c);
		characters ++;
		return 0;
    }
    
    if (c < ' ') {
		fprintf( out, "^");
		if ((c = c + '@') == '(' || c == ')' || c == '\\')
		    fprintf( out, "\\");
		fprintf( out, "%c", c);
		characters ++;
    }
    else if (c == 0177) {
		fprintf( out,"^?");
		characters ++;
	} else {
		if (c == '(' || c == ')' || c == '\\')
			fprintf( out,"\\");
		fprintf( out, "%c", c);
		characters ++;
	}

    return 1;
    
} /* translateCharacter */


/********************************************************/

void printMail::printPSFooter (FILE *out) {

	fprintf (out, "%%%% EOF\n");
}

/********************************************************/

// #define FONT			"Helvetica"
// #define FONTBOLD		"Helvetica-Bold"
#define FONT		"Times-Roman"
#define FONTBOLD	"Times-Bold"

void printMail::printPageFooter (FILE *out) {

	fprintf ( out, "%% \n");
	fprintf ( out, "%% Here starts a new Page.\n");
	fprintf ( out, "%% \n");
  	fprintf ( out, "/ly1 %d def\n", 44);
	fprintf ( out, "/ly2 %f def\n", 44.5);
	fprintf ( out, "/y0 %d def\n", 35);
	fprintf ( out, "headerunderline\n");
	fprintf ( out, "x0 10 moveto");
	fprintf ( out, "/fs 10 def\n");
	fprintf ( out, "/"FONT" findfont\n");
	fprintf ( out, "fs scalefont\n");
	fprintf ( out, "setfont\n");
	fprintf ( out, "%d (Page %d/%d) stringwidth pop sub "
		, (max_x - 40), pages, ((lines + linesperpage -1)/linesperpage));
	fprintf ( out, "y0 ");		
	fprintf ( out, "moveto\n");
	fprintf ( out, "(Page %d/%d) show\n"
		, pages, ((lines + linesperpage - 1)/ linesperpage));
	fprintf ( out, "(Printed by: %s@%s with %s %s)printline\n", user, host
		, PROGRAMNAME, VERSION);
    fprintf ( out, "showpage\n");
	fprintf ( out, "%% Page %d\n", pages);
	fprintf ( out, "%% \n");
	
	pages ++;
	PageLines = 0;

} /* printPageFoooter */

/********************************************************/

void printMail::setSize (enum PAPERSIZE size) {

	if (size == DINA4) {
		max_x = 595;
		max_y = 839;
	} else if (size == LETTER) {
		max_x = 612;
		max_y = 792;
	} else if (size == LEGAL) {
		max_x = 612;
		max_y = 1008; 
	} else {
		/* 
		 * Default:
		 *	DIN A4
		 */
		max_x = 595;
		max_y = 839;
	}
}

/********************************************************/

/* 
 * Initialisation of the Postscript-Code
 */

void printMail::printPSHeader(FILE *out) {
	
	time_t zeit;
	
	zeit = time(NULL);

	fprintf ( out, "%%!PS-Adobe-3.0\n");
	fprintf ( out, "%%%%Creator: %s %s\n", PROGRAMNAME, VERSION);
	fprintf ( out, "%%%%CreationDate: %s", ctime (&zeit)); 
	fprintf ( out, "%%%%Pages: %d\n",
		((lines + linesperpage - 1)/ linesperpage));
	fprintf ( out, "%%%%DocumentFonts: Courier "FONT"\n");
	fprintf ( out, "%%%%EndComments\n");
	fprintf ( out, "%% Lines: %d\n", lines);
	fprintf ( out, "%% LinesPerPage: %d\n", linesperpage);
	fprintf ( out, "%% by XCmail team!\n");

	fprintf ( out, "%% Function for Printing-Lines relativ to x0 y0\n");
	fprintf ( out, "/printline {\n");
	fprintf ( out, "	x0 y0 moveto\n");
	fprintf ( out, "	show\n");
	fprintf ( out, "	/y0 y0 fs sub def\n");
	fprintf ( out, "} def\n");
	
	fprintf ( out, "%% Function to draw a box from lx1, ly1 to lx2, ly2\n");
	fprintf ( out, "/headerunderline {\n");
	fprintf ( out, "	newpath\n");
	fprintf ( out, "	lx1 ly1 moveto\n");
	fprintf ( out, "	lx2 ly1 lineto\n");
	fprintf ( out, "	lx2 ly2 lineto\n");
	fprintf ( out, "	lx1 ly2 lineto\n");
	fprintf ( out, "	closepath fill\n");
	fprintf ( out, "} def\n");

	fprintf ( out, "%% Function to draw a box from lx1, y0 to lx2, ly2\n");
	fprintf ( out, "/headerfill {\n");
	fprintf ( out, "	newpath\n");
 	fprintf ( out, "	lx1 2 sub y0 moveto\n");
 	fprintf ( out, "	lx2 2 add y0 lineto\n");
 	fprintf ( out, "	lx2 2 add ly2 lineto\n");
 	fprintf ( out, "	lx1 2 sub ly2 lineto\n");
	fprintf ( out, "	closepath fill\n");
	fprintf ( out, "} def\n");

	fprintf ( out, "%% Function to draw a rectangle from lx1, y0 to lx2, ly2\n");
	fprintf ( out, "/headerdraw {\n");
	fprintf ( out, "	newpath\n");
	fprintf ( out, "	lx1 2 sub y0 moveto\n");
	fprintf ( out, "	lx2 2 add y0 lineto\n");
	fprintf ( out, "	lx2 2 add ly2 lineto\n");
	fprintf ( out, "	lx1 2 sub ly2 lineto\n");
	fprintf ( out, "	closepath stroke\n");
	fprintf ( out, "} def\n");

	// print text and calc max
	fprintf(out, "/printmax {\n");
	fprintf(out, "  dup\n");
	fprintf(out, "  printline\n");
	fprintf(out, "  stringwidth pop\n");
	fprintf(out, "  2 copy\n");
	fprintf(out, "  lt {2 1 roll} if\n");
	fprintf(out, "  pop\n");
	fprintf(out, "} def\n");

	fprintf ( out, "%% Initialisation\n");
	fprintf ( out, "/x0 %d def\n", left);
	fprintf ( out, "/fs %d def\n",fontsize);
	fprintf ( out, "/lx1 x0 def\n");
	fprintf ( out, "/lx2 %d def\n", (max_x - 40));
	fprintf ( out, "/ly1 %d def\n"
			, (max_y - unuseableHeader - headerSize));
	fprintf ( out, "/ly2 %f def\n"
			, (max_y - unuseableHeader - headerSize));

	fprintf ( out, "%% Enabeling ISOLatin1-Fonts\n");
	fprintf ( out, "/"FONT" findfont\n");
	
	fprintf ( out, "dup length dict begin\n");
	fprintf ( out, "	{1 index /FID ne {def}{pop pop} ifelse}forall\n");
	fprintf ( out, "	/Encoding ISOLatin1Encoding def\n");
	fprintf ( out, "	currentdict\n");
	fprintf ( out, "end\n");
	
	fprintf ( out, "/"FONT" exch definefont pop\n");

	fprintf ( out, "/"FONTBOLD" findfont\n");
	
	fprintf ( out, "dup length dict begin\n");
	fprintf ( out, "	{1 index /FID ne {def}{pop pop} ifelse}forall\n");
	fprintf ( out, "	/Encoding ISOLatin1Encoding def\n");
	fprintf ( out, "	currentdict\n");
	fprintf ( out, "end\n");
	
	fprintf ( out, "/"FONTBOLD" exch definefont pop\n");


	fprintf ( out, "/Courier findfont\n");
	
	fprintf ( out, "dup length dict begin\n");
	fprintf ( out, "	{1 index /FID ne {def}{pop pop} ifelse}forall\n");
	fprintf ( out, "	/Encoding ISOLatin1Encoding def\n");
	fprintf ( out, "	currentdict\n");
	fprintf ( out, "end\n");
	
	fprintf ( out, "/Courier exch definefont pop\n");

	fprintf ( out, "/Courier findfont\n");
	fprintf ( out, "fs scalefont\n");
	fprintf ( out, "setfont\n");
	fprintf ( out, "106 45 {dup mul exch dup mul add 1.0 exch sub} setscreen\n");
	
} /* printPSHeader */
/********************************************************/

/* 
 * Function to print a Page Prologue
 */

void printMail::printPageHeader (mail *m, FILE *out) {
	
	fprintf ( out, "%%%%Page: %d %d\n", pages , pages);
	fprintf ( out, "%% \n");
	fprintf ( out, "%% Page Prologue\n");	
	fprintf ( out, "%% \n");
	fprintf ( out, "/y0 %d def\n"
			, (max_y - unuseableHeader));
	fprintf ( out, "/ly1 %d def\n"
			, (max_y - unuseableHeader - headerSize - 2));
	fprintf ( out, "/ly2 %f def\n"
			, (max_y - unuseableHeader - headerSize - 2.5));
	fprintf ( out, "0.88 setgray\nheaderfill\n");
//	fprintf ( out, "0 setgray\nheaderdraw\n0 setgray\n");
//	fprintf ( out, "0 setgray\nheaderunderline\n0 setgray\n");

// xface
	if (m->xface()[0])
	{
		int size=30;
		fprintf(out,"gsave\n");
 		fprintf(out,"1 setgray\nlx2 10 add y0 moveto\n");
 		fprintf(out,"0 -%d rlineto -%d 0 rlineto 0 %d rlineto\n",size,size+10,size);
 		fprintf(out,"closepath fill\n0 setgray\n");
		fprintf(out,"lx2 %d sub y0 %d sub translate\n",size-2,size-2+1);
		fprintf(out,"%d %d scale\n",size-2,size-2);
		fprintf(out,"/pix 6 string def\n");
		fprintf(out,"48 48 1\n");
		fprintf(out,"[48 0 0 -48 0 48]\n");
		fprintf(out,"{currentfile pix readhexstring pop}\n");
		fprintf(out,"image\n");
		for(int i=0;i<6*48;i++)
		{
			fprintf(out,"%02x",bits[i]^0xff);
			if (!((i+1)%6)) fprintf(out,"\n");
		}
		
		fprintf(out,"\n\ngrestore\n");
	}	
//
	fprintf ( out, "0 setgray\nheaderdraw\n0 setgray\n");

	fprintf ( out, "/fs %d def\n",headerfont);
	fprintf ( out, "/y0 y0 fs sub def\n");
	fprintf ( out, "/"FONT" findfont\n");
	fprintf ( out, "fs scalefont\n");
	fprintf ( out, "setfont\n");	

// print headers
	fprintf (out, "/pout {\n0\n");
// subject
	fprintf ( out, "(");
	for (int i=0; i< strlen (subjectHeader); i++)
		translateCharacter ( out,(unsigned char) subjectHeader[i]);
	fprintf ( out, ") printmax\n");

// from
	fprintf ( out, "(");
	for (int i=0; i< strlen (fromHeader); i++)
		translateCharacter ( out,(unsigned char) fromHeader[i]);
	fprintf ( out, ") printmax\n");

// to
	fprintf ( out, "(");
	for (int i=0; i< strlen (toHeader); i++)
		translateCharacter ( out,(unsigned char) toHeader[i]);
	fprintf ( out, ") printmax\n");

// cc
	if (cc[0])
	{
		fprintf ( out, "(");
		for (int i=0; i< strlen (ccHeader); i++)
			translateCharacter ( out,(unsigned char) ccHeader[i]);
		fprintf ( out, ") printmax\n");
	}

// send date
	fprintf ( out, "(");
	for (int i=0; i< strlen (sendDateHeader); i++)
		translateCharacter ( out,(unsigned char) sendDateHeader[i]);
	fprintf ( out, ") printmax\n");

// maxlen on stack
	fprintf ( out, "x0 add\n");
	fprintf ( out, "8 add\n");
	fprintf ( out, "} def\n");
	fprintf (out, "/xzwi pout def\n");
	fprintf ( out, "/y0 %d def\n", (max_y - unuseableHeader));

// print header text
// subject
	fprintf ( out, "/"FONTBOLD" findfont\n");
	fprintf ( out, "fs scalefont\n");
	fprintf ( out, "setfont\n");	

	fprintf ( out, "/y0 y0 fs sub def\n");
	fprintf ( out, "xzwi y0 moveto\n");

	fprintf ( out, "(");
	for (int i=0; i< strlen (subject); i++)
		translateCharacter ( out,(unsigned char) subject[i]);
	fprintf ( out, ") show\n");

// from
	fprintf ( out, "/"FONT" findfont\n");
	fprintf ( out, "fs scalefont\n");
	fprintf ( out, "setfont\n");	

	fprintf ( out, "/y0 y0 fs sub def\n");
	fprintf ( out, "xzwi y0 moveto\n");

	fprintf ( out, "(");
	for (int i=0; i< strlen (from); i++)
		translateCharacter ( out,(unsigned char) from[i]);
	fprintf ( out, ") show\n");

// to
	fprintf ( out, "/"FONT" findfont\n");
	fprintf ( out, "fs scalefont\n");
	fprintf ( out, "setfont\n");	

	fprintf ( out, "/y0 y0 fs sub def\n");
	fprintf ( out, "xzwi y0 moveto\n");

	fprintf ( out, "(");
	for (int i=0; i< strlen (to); i++)
		translateCharacter ( out,(unsigned char) to[i]);
	fprintf ( out, ") show\n");

// cc
	if (cc[0])
	{
		fprintf ( out, "/y0 y0 fs sub def\n");
		fprintf ( out, "xzwi y0 moveto\n");

		fprintf ( out, "(");
		for (int i=0; i< strlen (cc); i++)
			translateCharacter ( out,(unsigned char) cc[i]);
		fprintf ( out, ") show\n");
	}

// send date
	fprintf ( out, "/y0 y0 fs sub def\n");
	fprintf ( out, "xzwi y0 moveto\n");
	fprintf ( out, "/yzwi y0 %d sub def\n",border);

	fprintf ( out, "(");
	for (int i=0; i< strlen (sendDate); i++)
		translateCharacter ( out,(unsigned char) sendDate[i]);
	fprintf ( out, ") show\n");

	fprintf ( out, "/fs %d def\n",fontsize);
	fprintf ( out, "xzwi yzwi moveto\n0 setgray\n");

	fprintf ( out, "/Courier findfont\n");
	fprintf ( out, "fs scalefont\n");
	fprintf ( out, "setfont\n");	
	fprintf ( out, "/y0 %d def\n"
			, (max_y - unuseableHeader - headerSize - fontsize - border));
	fprintf ( out, "(");
	
} /* printPageHeader */

/********************************************************/

void printMail::insertNewLines (char *str) {	

	int i = 0;
	int j = 0;
	
	while ('\0' != str[i]) {
		
		if ('\n' == str[i]) {
			j = 0;
			lines++;
		} /* if */
		
		if (j>79) {
			for  (j=i; str[j]!=' ' && (j > i-79); j--)
				;
				
			if (str[j] == ' ') {
		        str[j] = '\n';
		        j = 80 - j;
        	} else {
    			str[i] = '\n';
    			j = 0;
	        } /* if */		

			lines++;
		} /* if */
		
		i++;
		j++;
		
	} /* while */
		
} /* newLine */

/********************************************************/

void printMail::printMailSizeSet (mail *m, FILE *fp, char *text) {

	char *zwi1;
	char *zwi2;

	left = 60;
	right = 60;
	unuseableHeader = 25;
	unuseableFooter = 25;
	fontsize = 10;
	headerfont=12;
	headerSize = 0; //40;
	footerSize = 40;
	border=5;
	pages = 1;
	lines = 0;
	PageLines = 0;
	toHeader = strdup("To:");
	ccHeader = strdup("Cc:");
	fromHeader = strdup ("From:");
	subjectHeader = strdup ("Subject:");
	sendDateHeader = strdup ("Send Date:");
	
	uname (&sysInfo);
	
	if (!(host = new char[(strlen (sysInfo.nodename)+1)])) {
		fprintf (stderr, "Error: No more memory!\n"); 
		return ;
	}

	user = getusername ();
	strcpy (host, sysInfo.nodename);

	zwi1 = m->from();
	zwi2 = m->from_email ();
	
	from = new char[(strlen (zwi1) + strlen (zwi2) +4)];
	xc_snprintf (from,strlen(zwi1)+strlen(zwi2)+4, "%s <%s>", zwi1, zwi2);
	
	subject = m->subject();
	sendDate = m->date();
	to = m->to();
	cc = m->cc();
	headerSize=4*headerfont;
	if (cc[0]) headerSize+=headerfont;

	linesperpage = (max_y - unuseableHeader - headerSize - footerSize 
		- unuseableFooter - border) / fontsize - 1;

	char *mailBody = strdup (text);

	insertNewLines (mailBody);

	if (m->xface()[0])
		xface(m->xface());

	printPSHeader (fp);
	printPageHeader (m, fp);
	
	
	for (int i=0; i < strlen (mailBody); i++) {
		if (NEW_PAGE == translateCharacter (fp, (unsigned char) mailBody[i])) {
			printPageFooter (fp);
			printPageHeader (m, fp);
		}
	}

	fprintf ( fp, ") printline\n");
	fprintf ( fp, "%%%%Pages: %d\n", pages);

	printPageFooter (fp);
	printPSFooter (fp);

	free (toHeader); 
	free (fromHeader); 
	delete [] from;
	free (subjectHeader); 
	free (sendDateHeader);
	delete [] host;

} /* printMailSizeSet */

void printMail::xface(char *str)
{
	char *tmp_str=NULL;
	int len=strlen(str);

	// Make sure that buffer is big enough.
	int buffer_size=len+4096;
	char *buffer=new char[buffer_size];
    
	// Put header data in the buffer.
    strcpy(buffer, str);
    
    /*
     *  Uncompress the face.
    */

    if (uncompface(buffer) < 0)
    {
    	delete [] buffer;
		return;
    }
	int i,val,j;
	char *p=buffer;
	for(i=0;i<144 && p;i++)
	{
		if (!*p)
		{
			delete [] buffer;
			return;
		}
	
		// Skip leading spaces.
		while(*p==' ' || *p=='\t' || *p=='\n')
		{
			p++;
		}
		sscanf(p,"0x%x",&val);

		bits[i*2+1]=val & 0xff;
		val=val>>8;
		bits[i*2]=val;

		// Skip just past the next comma.
		p=strchr(p, ',');
		p++;
	}
	// Take the image data and create a bitmap.
	delete [] buffer;
}

/********************************************************/

printMail::printMail (mail *m, FILE *fp, char *text, enum PAPERSIZE size) {
	
	char *old;
	
	old = setlocale (LC_NUMERIC, "C");
	
	setSize (size);
	
	printMailSizeSet (m, fp, text);
	
	setlocale (LC_NUMERIC, old);
	
} /* printMail */
