/* 
 * This file is part of XCmail and is subject to the 
 * XCmail license agreement
 */

/* $Id: spellchecker.cxx,v 2.6 1999/12/15 18:08:44 schmitzj Exp $ */


#include "configure.h"

#include <Xclasses/Xclasses.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <sys/time.h>

#include <signal.h>
#include <dirent.h>

#include <stdarg.h>
#include <unistd.h>
#include <fcntl.h>

#include "config.h"
#include "mailer.h"
#include "spellchecker.h"
#include "xc_snprintf.h"

//#define DEBUG 1

/*
int _pipe(int x[], char *f, int l)
{
	int y=::pipe(x);
	fprintf(stderr,"(%d)%s:%d: pipe()->%d,%d\n",getpid(),f,l,x[0],x[1]);
	return y;
}

int _close(int x, char *f, int l)
{
	int r=close(x);
	fprintf(stderr,"(%d)%s:%d: close(%d)=%d\n",getpid(),__FILE__,__LINE__,x,r);
	return r;
}

#define pipe(x)  _pipe(x,__FILE__,__LINE__)
#define close(x) _close(x,__FILE__,__LINE__)
*/

//--------------------------------------------------------------------

#define WORD_DELIMITER	" \t\n.,/`'!|@#$%^&*()-=+{}[]\\\":;<>?~1234567890_"
#define URL_SPECIALS	"-/#?&=+~1234567890_.@"
#define EMAIL_SPECIALS	"-/=+1234567890_."

//--------------------------------------------------------------------

#define Object(t,n) ((t*)FindObject(FullName(),n))
#define ForeignObject(t,a,n) (a? (t*)FindObject(a->FullName(),n) : (t*)NULL)

char SpellChecker::libdir[MAXLENGTH] = "";
char *SpellChecker::dicts[MAXDICTS+1] = { NULL };


static int save_pipe(int x[])
{
	x[0]=x[1]=-1;
	return pipe(x);
}

static int save_close(int &x)
{
	int r=0;
	if (x>=0)
	{
		r=close(x);
		x=-1;
	}
	return r;
}

//--------------------------------------------------------------------

class nm_spacer : public gadget
{
	public:
		int Create(void)
		{
			Background(col_background());
			Mode(GM_ONEEXPOSE);
			return gadget::Create();
		};
		void GExpose(XEvent *)
		{
		};
};

//--------------------------------------------------------------------

SpellChecker::SpellChecker(appwindow *p) : Xclasses("SpellChecker")
{	
	aw = NULL;
	lang = NULL;
	altlang = NULL;
	
	misspelled[0] = 0;
	subject[0] = 0;
	body = NULL;
	sp = NULL;
	
	in_subject = 0;
	curx = 0;
	cury = 0;

	parent = p;
	ParentClass(p);
	
	ispell_available = 1;
	send_after_check = 0;
	
	newmail_obj = NULL;
	newmail_gadget = NULL;
	newmail_parm = 0;

	if (parent)
		fcntl(ConnectionNumber(parent->display()),F_SETFD,1);

}

SpellChecker::~SpellChecker()
{
	DeleteObjectTree(FullName(),DOT_NOTBASE);
	if (lang) delete lang;
	if (altlang) delete altlang;
	if (body) delete[] body;
}

//---------------------------------------------------------------------

void SpellChecker::closeCB(appwindow *aw)
{
	aw->Close();
	if (parent) parent->Unblock();
	aw = NULL;
	
	if (send_after_check)
		newmail_obj->send(newmail_gadget, newmail_parm);

	delete this;
}

void SpellChecker::dictCB(gadget *g,int i)
{

	if (!g) return;

	Speller **spellerPP=NULL;

	if (strcmp(g->Name(), "dict") == 0)
	{
		spellerPP = &lang;
	}
	else if (strcmp(g->Name(), "altdict") == 0)
	{
		spellerPP = &altlang;
	}
	
	if (spellerPP)
	{
		choice *ch = (choice *)g;
		char *dname = dicts[ch->Selected()];

		if (!*spellerPP || strcmp(dname, (*spellerPP)->Dict()) != 0)
		{
			delete *spellerPP;
			
			if (strcmp("---", dname) != 0)
			{
				*spellerPP = new Speller(dname);
			}
			if (Check(misspelled))
			{
			
				// in this dictionary the former misspelled
				// word is correct, so ignore it!
				//
				ignoreword();
				CheckMail();
			}
		}
	}
}

void SpellChecker::ignoreCB(gadget *, int)
{
	ignoreword();
	CheckMail();
}

void SpellChecker::replaceCB(gadget *, int)
{
	input *inc = Object(input, ".correction");

	if (inc)
	{
		char iword[MAXWORDLEN];
		inc->Input(iword);
		int diff = strlen(iword) - strlen(Misspelled());

		if (in_subject)
		{
			input *in = ForeignObject(input, parent, ".subject");
			if (in)
			{
				char tsub[MAXLENGTH] = "";
				
				strncat(tsub, subject, curx);
				strcat(tsub, iword);
				strncat(tsub, subject+ecurx, MAXLENGTH-ecurx);
				
				in->Default(tsub);
				strcpy(subject, tsub);
				
				curx += diff;
				ecurx += diff;
				sp += diff;
			}
		}
		else
		{
			textbox *te = ForeignObject(textbox, parent, ".mailbody");
			if (te)
			{
				te->CursorTo(curx, cury);
				te->MarkTo(ecurx, cury);
				te->ReplaceMarkedText(iword);
				
				curx += diff;
				ecurx += diff;
			}
		}
	}
	ignoreword();
	CheckMail();
}

void SpellChecker::lernCB(gadget *, int)
{
	if (lang)
	{
		lang->LernWord(Misspelled());
	}
	ignoreword();
	CheckMail();
}

void SpellChecker::finishCB(gadget *, int)
{
	if (aw) closeCB(aw);
}

void SpellChecker::correctionCB(gadget *aw, int i)
{
	replaceCB(aw, i);
}

void SpellChecker::suggestionsCB(gadget *aw, int i)
{
	if (!lang) return;
	input *in = Object(input, ".correction");
	if (in)
	{
		int sel = ((listview*)aw)->Selected();
		if (sel > 0)
		{
			sel--;
			if (sel < lang->Suggestions())
			{
				in->Default(lang->Suggestion(sel));
				if (aw->DoubleClick()) replaceCB(aw, i);
			}
			else if (altlang)
			{
				if (lang->Suggestions())
				{
					sel -= lang->Suggestions()+1;
				}
				if (sel < altlang->Suggestions())
				{
					in->Default(altlang->Suggestion(sel));
					if (aw->DoubleClick()) replaceCB(aw, i);
				}
			}
		}
	}
}

//---------------------------------------------------------------------

void SpellChecker::createdialog(void)
{

	if (aw) return;

	int x,y,w,h;
	
	aw = new appwindow;
	if (!aw) return;

	readerwin.Position(x,y);
	readerwin.Size(w,h);

	aw->ParentClass(this);
	aw->Name("spellwindow");
	aw->InitSize(30);

	if (!aw->Init())
	{
		delete aw;
		pstatus("Error in opening window!");
		return;
	}
	if (parent)
	{
		aw->TransientFor(parent);
		parent->Block();
	}
	aw->Title(PROGRAMNAME": Spellchecker");
	aw->IconTitle("Spell");
	aw->Adjust(x+6,y-6,w,h,ADJUST_TOP|ADJUST_RIGHT);

	spacegroup *g = new spacegroup;
	g->Horiz();
	g->Name("maingroup");
	g->YDistance(5);
	g->XDistance(5);
	g->Border(5);
		group *g1 = new group;
		g1->YDistance(4);
		g1->Vert();
		g1->Name(NULL);
			output *tf1 = new output;
			tf1->Name("misspelled");
			tf1->Text("Misspelled");
		g1->Add(tf1);
			input *tf2 = new input;
			tf2->Name("correction");
			tf2->Text("Correction");
			tf2->MaxChars(40);
			tf2->Callback(this,(CALLBACK) &SpellChecker::correctionCB);
		g1->Add(tf2);
			listview *lv = new listview;
			lv->Name("suggestions");
			lv->Callback(this,(CALLBACK) &SpellChecker::suggestionsCB);
		g1->Add(lv,"Suggestions" );
		g1->Add(new ruler);
			choice *ch1 = new choice;
			ch1->Name("dict");
			ch1->Items(dicts);
			ch1->Callback(this,(CALLBACK) &SpellChecker::dictCB);
			ch1->HelpText("select the current dictionary/language and\nrespell the misspelled word");
		g1->Add(ch1, "Dictionary");

	if (altlang)
	{
			choice *ch2 = new choice;
			ch2->Name("altdict");
			ch2->Items(dicts);
			ch2->Callback(this,(CALLBACK) &SpellChecker::dictCB);
			ch2->HelpText("select the alternative dictionary/language and\nrespell the misspelled word");
		g1->Add(ch2, "Alternative");
	}

	g->Add(g1);
		group *g2 = new group;
		g2->YDistance(4);
		g2->Vert();
		g2->Weight(40);
		g2->Name(NULL);
			fbutton *fb1 = new fbutton;
			fb1->Name("ignore");
			fb1->Text("_Ignore");
			fb1->Callback(this,(CALLBACK) &SpellChecker::ignoreCB);
			fb1->HelpText("Ignore the current misspelled word\nand go ahead");
		g2->Add(fb1);
			fbutton *fb2 = new fbutton;
			fb2->Name("replace");
			fb2->Text("_Replace");
			fb2->Callback(this,(CALLBACK) &SpellChecker::replaceCB);
			fb2->HelpText("Replace the mispelled word with the\ncurrent correction"); 
		g2->Add(fb2);
			fbutton *fb3 = new fbutton;
			fb3->Name("lern");
			fb3->Text("_Learn");
			fb3->Callback(this,(CALLBACK) &SpellChecker::lernCB);
			fb3->HelpText("Put the mispelled word into the user dictionary");
		g2->Add(fb3);
		g2->Add(new nm_spacer);
		g2->Add(new nm_spacer);
			fbutton *fb4 = new fbutton;
			fb4->Name("finish");
			fb4->Text("_Finish");
			fb4->Callback(this,(CALLBACK) &SpellChecker::finishCB);
			fb4->HelpText("Finish correction");
		g2->Add(fb4);
		g2->Add(new nm_spacer);
	g->Add(g2);
	aw->SetGroup(g);
	aw->Callback(this,(CLOSECALLBACK) &SpellChecker::closeCB);
	aw->Create();
}

char *SpellChecker::Misspelled(void)
{
	return misspelled;
}

void SpellChecker::ignoreword(void)
{
	while (*sp && strchr(WORD_DELIMITER, *sp) == NULL)
	{
		sp++;
		curx++;
	}
}

void SpellChecker::SendAfterCheck(newmail *o, gadget *g, int p)
{
	send_after_check = 1;
	newmail_obj = o;
	newmail_gadget = g;
	newmail_parm = p;
}

void SpellChecker::CheckMail(void)
{
	if (!body)
	{
		int len;
		input *in = ForeignObject(input, parent, ".subject");
		textbox *te = ForeignObject(textbox, parent, ".mailbody");
		
		if (!in || !te) return;

		// subject
		//
		in->Input(subject);

		// body
		//
		len  = te->TextSize();
		body = new char[len + 2];
		if (!body) return;
		te->CopyText(body, len + 1);

		// init
		//
		in_subject = 1;
		sp = (unsigned char *)subject;
		curx = 0;
		cury = 0;
	}
	
	static char iword[MAXWORDLEN];

	while (*sp || in_subject)
	{
		int i = 0;
	
		if (!*sp && in_subject)
		{
			in_subject = 0;
			sp = (unsigned char *)body;
			curx = 0;
			cury = 0;
		}
			
		while (*sp && strchr(WORD_DELIMITER, *sp) != NULL)
		{
			if (*sp == '\n')
			{
				cury++;
				curx=0;

				if (!in_subject
					&& programconfig.spellignorequoted
					&& strncmp("\n>", (char*)sp, 2) == 0)
				{
					
					int k=1;
					while (sp[k] == '>') k++;
					
					if (sp[k] == ' ')
					{
						sp++; 
						while (*sp && *sp != '\n')
						{
							curx++; sp++;
						}
						if (*sp) continue;
						else     break;
					}
				}

				if (!in_subject
					&& programconfig.spellignoresig
					&& strncmp("\n-- \n", (char*)sp, 5) == 0)
				{
					sp+=strlen((char*)sp);
					break;
				}
			}
			else
			{
				curx++;
			}
			sp++;
		}
		if (!*sp && !in_subject) break;
	
		unsigned char *esp = sp;
		ecurx = curx;
		
		while (*sp && strchr(WORD_DELIMITER, *esp) == NULL)
		{
			if (i < MAXWORDLEN-1) iword[i++] = *esp;
			esp++;
			ecurx++;
		}
		iword[i] = 0;

		if (programconfig.spellignorespecials)
		{
		
			if ((*esp == ')' && curx+cury > 0 && sp[-1] == '(') || (*esp == ':'))
			{
				static char *tag[] = {
					"aw",
					"fwd",
					"re",
					"btw",
					"imho",
					"rtfm",
					"lotfl",
					NULL };
				int t;

				for (t = 0; tag[t]; t++)
				{
					if (strcncmp(tag[t], iword) == 0)
					{
						iword[0] = 0;
						break;
					}
				}
			}
		}

		// ignore URLS
		//
		if (programconfig.spellignoreurl)
		{
			if (*esp == ':' &&
				((curx == 0 && cury == 0) || strchr(WORD_DELIMITER, sp[-1]) != NULL))
			{

				static char *tag[] = {
					"http",
					"mailto",
					"ftp",
					"gopher",
					"file",
					NULL };
				int t;

				for (t = 0; tag[t]; t++)
				{
					if (strcncmp(tag[t], iword) == 0)
					{
						ecurx++;
						esp++;
						while (*esp)
						{
							if (strchr(URL_SPECIALS, *esp) == NULL &&
								strchr(WORD_DELIMITER, *esp) != NULL)
								break;
							ecurx++; esp++;
						}
						iword[0] = 0;
						break;
					}
				}
			}
			else if (*esp == '@' &&
				strchr(WORD_DELIMITER, esp[1]) == NULL &&
				((curx == 0 && cury == 0) || strchr(WORD_DELIMITER, sp[-1]) != NULL))
			{
				ecurx++;
				esp++;
				while (*esp)
				{
					if (strchr(EMAIL_SPECIALS, *esp) == NULL &&
						strchr(WORD_DELIMITER, *esp) != NULL)
						break;
					ecurx++; esp++;
				}
				iword[0] = 0;
			}
		}
		
		if (iword[0] && !Check(iword))
		{
			if (in_subject)
			{
				input *in = ForeignObject(input, parent, ".subject");
				if (in)
				{
					in->Cursor(curx);
					in->MarkTo(ecurx);
				}
			}
			else
			{
				textbox *te = ForeignObject(textbox, parent, ".mailbody");
				if (te)
				{
					te->CursorTo(curx, cury);
					te->MarkTo(ecurx, cury);
				}
			}
			return;
		}
		if (!ispell_available)
			break;
		
		sp = esp;
		curx = ecurx;
	}
	if (aw) {
		Alert(aw,"No more mistakes found!");
		closeCB(aw);
	}
	else
	{
		Alert(parent,"No mistakes found!");
	}

}

int SpellChecker::Check(char *iword)
{
	
	int j;
	
	if (!ispell_available) return 1;
	if (!scandictdir())
	{
		ispell_available = 0;
		return 1;
	}
		
	if (!lang && !altlang)
	{	// true on first calling
	
		char *s = programconfig.spelllanguage;
		char t[strlen(s)];
		int i,j;
		
		while (*s && *s == ' ')
		{
			s++;
		}
			
		for (i = 0; s[i] && s[i] != ','; i++)
		{
			t[i] = s[i];
		}
		for (j = i-1; j > 0 && t[j] == ' '; j++)
			;
		t[j+1]=0;

		if (t[0]) lang = new Speller(t);

		while (s[i] && (s[i] == ' ' || s[i] == ','))
			i++;
		
		if (s[i])
		{
			s=s+i;

			for (i = 0; s[i] && s[i] != ','; i++)
				t[i]=s[i];

			for (j = i-1; j > 0 && t[j] == ' '; j++)
				;
			t[j+1]=0;

			if (t[0]) altlang = new Speller(t);
		}

		// select first language, if no specified
		//	
		if (!lang) lang = new Speller(dicts[0]);
		
		if (!altlang)
		{
			
			lang->Check("dummy");
			
			if (!lang->Available())
			{
				Alert(parent,
					"Dictionary '%s' not available!\n"
					"Unable to continue checking!",
					 lang->Dict());
				ispell_available = 0;
				return 2;
			}
			
		}
		else
		{
			lang->Check("dummy");
			altlang->Check("dummy");
			
			if (!lang->Available() && altlang->Available())
			{
				Alert(parent,
					"Primary Dictionary '%s' not available!\n"
					"Using secoundary dictionary '%s'!",
					 lang->Dict(), altlang->Dict());
				
				delete lang; lang=altlang; altlang=NULL;
				
			}
			else if (!lang->Available() && !altlang->Available())
			{
				Alert(parent,
					"Primary Dictionary '%s' not available!\n"
					"Alternative Dictionary '%s' not available!\n"
					"Unable to continue!",
					 lang->Dict(), altlang->Dict());
				ispell_available = 0;
				return 1;
			}
			else if (!lang->Available())
			{
				Alert(parent,
					"Primary Dictionary '%s' not available!\n"
					"Unable to continue!",
					 lang->Dict());
				ispell_available = 0;
				return 1;
			}
			else if (!altlang->Available())
			{
				Alert(parent,
					"Alternative Dictionary '%s' not available!\n"
					"Using primary only.",
					 altlang->Dict());
				delete altlang; altlang=NULL;
			}
		}
	}
	
	// test word
	//
	if (!lang) return 2;
	if (lang->Check(iword))	return 1;
	if (altlang && altlang->Check(iword)) return 1;

	// save misspelled word
	//
	if (iword != misspelled)
	{
		misspelled[0] = 0;
		strncat(misspelled, iword, sizeof(SpellChecker::misspelled)-1);
	}
		
	// create/raise dialog
	//
	if (aw)
	{
		aw->Raise();
	}
	else
	{
		createdialog();
	}

	// select korrekt dictionary in popup
	//
	choice *ch = Object(choice, ".dict");
	if (ch)
	{
		for (j = 0; dicts[j]; j++)
			if (strcmp(dicts[j], lang->Dict()) == 0)
			{
				ch->Selected(j);
				break;
			}
	}	

	ch = Object(choice, ".altdict");
	if (ch)
	{
		if (!altlang)
		{
			ch->Selected(-1);
		}
		else
		{
			for (j = 0; dicts[j]; j++)
				if (strcmp(dicts[j], altlang->Dict()) == 0)
				{
					ch->Selected(j);
					break;
				}
		}
	}	

	// insert misspelled word
	//
	output *o = Object(output, ".misspelled");
	if (o) o->Output(iword);

	// write misspelled word to input-gadget
	//
	input *i = Object(input, ".correction");
	if (i) i->Default(iword);
	
	// insert al suggestions
	//
	listview *l = Object(listview, ".suggestions");
	if (l)
	{
		l->RemoveAll();
		
		for (j = 0; j < lang->Suggestions(); j++)
			l->AddAtEnd(lang->Suggestion(j));
			
		if (j && altlang && altlang->Suggestions())
		{
			l->AddAtEnd("----------------------");
			l->LockItem(j+1);
			for (j = 0; j < altlang->Suggestions(); j++)
				l->AddAtEnd(altlang->Suggestion(j));
			
		}
	}
	return 0;
}

static int cmp(const void *t1, const void *t2)
{
	return strcscmp(*(char**)t1, *(char**)t2);
}

int SpellChecker::scandictdir(void)
{
	DIR *dfd;
	int i = 0;
	int err = 0;
	char wdir[300];
	struct dirent *dptr;
	struct stat sbuf;

	// looking up dictionary path
	//
	if (!libdir[0])
	{
		int apipe[2]={-1,-1};
		
		if (pipe(apipe) >= 0)
		{
			int child = fork();
				
			if (child == 0)
			{
				// connect stdout
				//
				close(1);
				dup(apipe[1]);
				save_close(apipe[0]);
				execlp(programconfig.spellpath, programconfig.spellpath, "-vv", NULL);
				write(apipe[1], "^ERR\n", 5);
				_exit(127);
				
			}
			else if (child < 0)
			{
				err = 1;
				save_close(apipe[0]);
				save_close(apipe[1]);
			}
			else
			{
				unsigned char c;
				char buf[MAXLENGTH];
				int ret;
				int i = 0;
				fd_set fd;
				struct timeval tv;
				
				save_close(apipe[1]);

				tv.tv_sec = 10;
				tv.tv_usec = 0;
				
				FD_ZERO(&fd);
				FD_SET(apipe[0], &fd);
				
				if (!select(apipe[0]+1, &fd, NULL, NULL, &tv))
					err = 1;
				else
				{
					while (ret = read(apipe[0], &c, 1), ret == 1)
					{

						if (i == 0 && (c == ' ' || c  == '\t'))
							continue;

						if (c == '\n')
						{
							buf[i] = 0;
							i = 0;

							if (strcmp("^ERR", buf) == 0)
							{
								err = 1;
							}
							else if (strncmp("LIBDIR =", buf, 8) == 0)
							{
								char *s = buf+8;

								while (*s && *s != '"')
									s++;
								if (*s) s++;

								int j = 0;

								while (*s && *s != '"')
									libdir[j++] = *(s++);
								libdir[j++] = 0;
							}
							continue;
						}
						if (i < sizeof(buf)-1) ((unsigned char *)buf)[i++] = c;
					}
				}
				save_close(apipe[0]);
				waitpid(child, NULL, 0);
			}
		}
		else
		{
			err = 1;
		}
	}
	if (err)
	{
		Alert(parent,
			"Unable to execute '%s'!",
			 programconfig.spellpath);
		return !err;
	}
	if (!libdir[0])
		err = 1;

	// looking up dictionaries
	//
	if (libdir[0] && !dicts[1])
	{
		if (dfd = opendir(libdir), dfd)
		{
			while (dptr = readdir(dfd), dptr)
			{
				char *c = strrchr(dptr->d_name, '.');

				if (c && c != dptr->d_name && strcmp(c, ".hash") == 0)
				{
					xc_snprintf(wdir, sizeof(wdir), "%s/%s", libdir, dptr->d_name);
					stat(wdir, &sbuf);
					if (S_ISREG(sbuf.st_mode) && !S_ISDIR(sbuf.st_mode))
					{
// TH : added 1 to size allocated 
						dicts[i] = new char [c - dptr->d_name + 1];
						if (dicts[i])
						{
							dicts[i][0]=0;
							strncat(dicts[i],  dptr->d_name, c - dptr->d_name);
							i++;
						}
					}
				}
			}
			closedir(dfd);
			qsort(dicts,i,sizeof(char*),cmp);
		}
		if (!i) err = 1;
	}

	if (err)
	{
		Alert(parent,
			"Unable to locate ispell dictionaries in '%s':\n"
			"No spell checking available!",
			 libdir);
		err = 1;
	}
	
	return !err;
}

char **SpellChecker::GetDictList(char *ispell, appwindow *aw)
{
	SpellChecker *sc=new SpellChecker(aw);
	SpellChecker::libdir[0]=0;
	char *temp=NULL;
	char **d=NULL;
	
	if (ispell)
	{
		temp = new char [strlen(programconfig.spellpath)+1];
		if (!temp)
		{
			Alert(aw, "no memory available!");
			return NULL;
		}
		strcpy(temp,programconfig.spellpath);
		strcpy(programconfig.spellpath,ispell);
	}
	if (sc && sc->scandictdir())
	{
		delete sc;
		d=SpellChecker::dicts;
	}
	if (temp)
	{
		strcpy(programconfig.spellpath,temp);
		delete [] temp;
	}
	SpellChecker::libdir[0]=0;
	return d;
}

//----------------------------------------------------------------------

Speller::Speller(char *d)
{

	dict[0] = 0;
	ispell_started = 0;
	
	strcpy(format, "latin1");
	if (d) strcpy(dict,d);

	FD_ZERO(&fdset);
	nfds = 0;
}

Speller::~Speller() {
	stop_ispell();
}

int Speller::Check(char *word)
{
	if (!ispell_started)
		start_ispell();

	if (ispell_started && word) 
	{
		
		if (word[0])
		{
			write_line(word);
#ifdef DEBUG
			fprintf(stderr, "-> %s\n", word);
#endif
			read_line(spellbuf, sizeof(spellbuf)-1);
#ifdef DEBUG
			fprintf(stderr, "<- %s\n", spellbuf);
#endif
			if (!spellbuf[0]) return 1;

			char smallbuf[3];
			while (read_line(smallbuf, 2) > 0)
			{
				if (smallbuf[0] != spellbuf[0])
				{
					spellbuf[0] = '?';
				}
			}

			switch (spellbuf[0])
			{
				case '*':
				case '+':
				case '-':
					return 1;
				
				case '&':
				case '?':
					return 0;
			}
		}
	}
	return 0;
}

int Speller::LernWord(char *iword)
{

	if (!iword ||
		!iword[0] ||
		strlen(iword)>sizeof(Speller::spellbuf)-2)
		return 0;

	if (!ispell_started)
		start_ispell();

	if (ispell_started)
	{
		xc_snprintf(spellbuf, sizeof(spellbuf), "*%s", iword);
		write_line(spellbuf);
		write_line("#");
	}
	return 1;
}

int Speller::Suggestions(void)
{
	if (spellbuf[0] == '&')
	{
		char *c = spellbuf+1;

		if (*c) c++;
		while(*c && *c != ' ')
			c++;
		if (*c) c++;

		return atol(c);
	}
	return 0;
}

char *Speller::Suggestion(int i)
{
	static char iword[MAXWORDLEN];
	
	if (spellbuf[0] == '&')
	{
		char *c = spellbuf+1;

		if (*c) c++;
		while(*c && *c != ':')
			c++;
		if (*c) c++;
		if (*c) c++;
		
		while (i > 0)
		{
			while (*c && *c != ',')
				c++;
			if (*c) c++;
			if (*c) c++;
			i--;
		}
		
		i = 0;
		while (*c && *c != ',')
			iword[i++] = *(c++);
		iword[i] = 0;
		
		return iword;
	}
	return "";
}

char *Speller::Dict(void)
{
	return dict;
}

int Speller::read_line(void)
{
	return read_line(NULL, 0);
}

int Speller::read_line(char *s, int len)
{
	int				ret, found = 0;
	unsigned char	c;
	unsigned char*	to = (unsigned char *)s;
	struct timeval	tv;
	
	if (!ispell_started) return -1;
	
	tv.tv_sec = 30;
	tv.tv_usec = 0;
	
	if (select(nfds, &fdset, NULL, NULL, &tv) <= 0)
	{
		s[0] = 0;
		stop_ispell();
		return -1;
	}
	if (FD_ISSET(pipe3fd[0], &fdset))
	{
		while (ret = read(pipe3fd[0], &c, 1), ret == 1 && c != '\n')
		{
			if (len > 0 && to)
			{
				*(to++) = c;
				found++;
				len--;
			}
		}
		if (to) *to=0;
	}
	else if (FD_ISSET(pipe2fd[0], &fdset))
	{
		while (ret = read(pipe2fd[0], &c, 1), ret == 1 && c != '\n')
		{
			if (len > 0 && to)
			{
				*(to++) = c;
				found++;
				len--;
			}
		}
		if (to) *to=0;
	}
	return ret>0 ? found : -1;
}

int Speller::write_line(char *s)
{
	int ret;
	int err = 0;
	int len = strlen(s);
	err = err || write(pipe1fd[1], s, len) != len;
	err = err || write(pipe1fd[1], "\n", 1) != 1;
	return !err;
}

int Speller::start_ispell(void)
{
	if (save_pipe(pipe1fd) < 0)
		return 0;

	if (save_pipe(pipe2fd) < 0)
	{
		save_close(pipe1fd[0]);
		save_close(pipe1fd[1]);
		return 0;
	}

	if (save_pipe(pipe3fd) < 0)
	{
		save_close(pipe1fd[0]);
		save_close(pipe1fd[1]);
		save_close(pipe2fd[0]);
		save_close(pipe2fd[1]);
		return 0;
	}

	ispell_child = fork();
	if (ispell_child == 0)
	{
		// connect stdin
		//
		close(0);
		dup(pipe1fd[0]);
		
		// connect stdout
		//
		close(1);
		dup(pipe2fd[1]);
		
		// connect stderr
		//
		close(2);
		dup(pipe3fd[1]);
		
		// close pipe ends
		//
		save_close(pipe1fd[1]);
		save_close(pipe2fd[0]);
		save_close(pipe3fd[0]);

		// start ispell
		//
		if (dict[0])
		{
			if (format[0])
			{
				execlp(programconfig.spellpath, programconfig.spellpath, "-T", format, "-d", dict, "-a", NULL);
			}
			else
			{
				execlp(programconfig.spellpath, programconfig.spellpath, "-d", dict, "-a", NULL);
			}
		}
		else
		{
			if (format[0])
			{
				execlp(programconfig.spellpath, programconfig.spellpath, "-T", format, "-a", NULL);
			}
			else
			{
				execlp(programconfig.spellpath, programconfig.spellpath, "-a", NULL);
			}
		}
		
		close(0);
		close(1);
		close(2);
		save_close(pipe1fd[0]);
		save_close(pipe2fd[1]);
		save_close(pipe3fd[1]);
		
		_exit(127);
	}
	
	// close pipe ends
	//
	save_close(pipe1fd[0]);
	save_close(pipe2fd[1]);

	if (ispell_child < 0)
	{
		save_close(pipe1fd[1]);
		save_close(pipe2fd[0]);
		save_close(pipe3fd[0]);
		return 0;
	}

	ispell_started = 1;

	FD_ZERO(&fdset);
	FD_SET(pipe2fd[0], &fdset);
	FD_SET(pipe3fd[0], &fdset);

	nfds = pipe2fd[0];
	if (pipe3fd[0] > nfds)
		nfds = pipe3fd[0];
	nfds++;
	
	// read first line
	//	
	if (read_line(spellbuf, sizeof(spellbuf)-1) < 0)
	{
		stop_ispell();
		ispell_started = 0;
		return 0;
	}
	if (strncmp(spellbuf, "Can't open", 10) == 0)
	{
		stop_ispell();
		ispell_started = 0;
		return 0;
	}
	if (strstr(spellbuf, "unrecognized formatter type") != NULL)
	{
		stop_ispell();
		if (format[0])
		{
			format[0] = 0;
			return start_ispell();
		}
	}
	return 1;
}

int Speller::stop_ispell(void)
{
	int status;

	if (ispell_started)
	{
		kill(ispell_child, SIGTERM);
		waitpid(ispell_child, &status, 0);
		save_close(pipe1fd[1]);
		save_close(pipe2fd[0]);
		save_close(pipe3fd[0]);
		return 1;
	}
	return 0;
}

int Speller::Available(void)
{
	return ispell_started;
}
