/* 
 * This file is part of XCmail and is subject to the 
 * XCmail license agreement
 */

/* $Id: spellchecker.h,v 2.3 1999/01/27 20:01:34 beckerr Exp $ */


#ifdef HAVE_SYS_SELECT_H
#include <sys/select.h>
#endif

#define MAXDICTS	64		// max. number of dictionaries
#define MAXWORDLEN	64		// max. number of characters per word
#define SHORTSTRING	48		// length of short strings

class Speller;

class SpellChecker : public Xclasses {

	private:
		appwindow *parent;					// pointer to parent
		appwindow *aw;						// pointer to gadget tree
		
		Speller *lang;						// current speller
		Speller *altlang;					// alternative speller

		char misspelled[MAXWORDLEN];		// last misspelled word

		char subject[MAXLENGTH];			// subject of parent mail
		char *body;							// body of parent mail

		int in_subject;						// spelling the subject now?
		unsigned char *sp;					// pointer to current spell position
		int  curx;							// x curser position in parents window
		int  cury;							// y curser position in parents window
		int  ecurx;							// x cursor position behind the misspelled word

		static char libdir[MAXLENGTH];		// path to ispell libraries
		static char *dicts[MAXDICTS+1];		// available libraries

		int ispell_available;				// ispell available? (!=0 if yes)
		int send_after_check;				// send mail after checking
		
		newmail*	newmail_obj;			// Parameter to call send Callback
		gadget*		newmail_gadget;
		int 		newmail_parm;


		void closeCB(appwindow *);
		void dictCB(gadget *, int);
		void ignoreCB(gadget *,int);
		void replaceCB(gadget *, int);
		void lernCB(gadget *, int);
		void finishCB(gadget *, int);
		void correctionCB(gadget *, int);
		void suggestionsCB(gadget *, int);

		void createdialog(void);
		int  scandictdir(void);
		
		void ignoreword(void);

	public:
		SpellChecker(appwindow * = NULL);
		~SpellChecker();

		int  Check(char *);					// check one word (returns 1 if correct, 0 else)
		void CheckMail(void);				// check mail in parent window
		
		char *Misspelled(void);				// returns last misspelled word

		void SendAfterCheck(newmail *, gadget *, int);	
											// send parent mail after check
		static char **GetDictList(char *ispell=NULL, appwindow *aw = NULL);
};


class Speller {

	private:

		int ispell_started;				// =1 if ispell is running, =0 else
		int ispell_child;				// ispell PID

		int pipe1fd[2];					// pipe to ispell
		int pipe2fd[2];					// pipe from ispell
		int pipe3fd[2];					// pipe from ispell (stderr)

		fd_set fdset;					// used by select
		int nfds;						// used by select
		
		char dict[SHORTSTRING];			// name of dictionary to use
		char format[SHORTSTRING];		// name of formater (normaly 'latin1' or empty)
		char spellbuf[MAXLENGTH];		// line returned from ispell
		
		int start_ispell(void);			// start ispell and open pipes
		int stop_ispell(void);			// stop ispell and close pipes
		
		int read_line(char *, int);		// receive line from ispell
		int read_line(void);			// ignore line from ispell
		
		int write_line(char * =NULL);	// send line to ispell

	public:
		// constructor:
		//  'd' is the name of the dictionary to use
		//  (e.g. english, deutsch, ...)
		Speller(char *d = NULL);
		~Speller();

		// check word 'w':
		//  returns '1' if word is correct
		//  returns '0' if word is not correct
		//
		int Check(char *w);

		// correct string 'w':
		//  returns '1' if word is correct
		//  returns '0' if word is not correct
		//
		int Correction(char *w);
		
		// returns the number of near misses
		//
		int Suggestions(void);
		
		// returns missed words
		//
		char* Suggestion(int);

		// lern word (!= 0 on suggess)
		//
		int  LernWord(char *iword);
		
		// returns current dict name
		//
		char *Dict(void);

		// ispell available
		//
		int Available(void);
		
};
