/* 
 * This file is part of XCmail and is subject to the 
 * XCmail license agreement
 */

/* $Id: stringTokenizer.cxx,v 2.31 2002/09/09 13:26:29 schmitzj Exp $ */


#include "configure.h"

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "stringTokenizer.h"


token::token(char *s)
{
	ign_quote1=1;
	ign_quote2=0;
	tokstr=NULL;
	text=tokstr;
	return_buf=NULL;
	set_token(s);
}

token::token(char *s, int size)
{
	ign_quote1=1;
	ign_quote2=0;
	tokstr=NULL;
	text=tokstr;
	return_buf=NULL;
	set_token(s,size);
}

token::token(token &t)
{
	if (this == &t) return;
	ign_quote1=t.ign_quote1;
	ign_quote2=t.ign_quote2;
	tokstr=strdup(t.tokstr?t.tokstr:(char*)"");
	text=tokstr;
	return_buf=NULL;
}

token& token::operator=(token &t)
{
	if (this == &t) return *this;
	ign_quote1=t.ign_quote1;
	ign_quote2=t.ign_quote2;
	tokstr=strdup(t.tokstr?t.tokstr:(char*)"");
	text=tokstr;
	if (return_buf)
	{
		free(return_buf);
		return_buf=NULL;
	}
	return *this;
}
token& token::operator=(char *s)
{
	if (return_buf)
	{
		free(return_buf);
		return_buf=NULL;
	}
	set_token(s);
	return *this;
}
token::~token()
{
	if (return_buf) free(return_buf);
	if (tokstr) free(tokstr);
}

void token::set_token(char *s, int size)
{
	char *quote1=NULL;
	char *quote2=NULL;

	if (!s) return;
	
	if (size<0)
	{
		tokstr=strdup(s);
		text=tokstr;
		if (!tokstr) return;
	}
	else
	{
		tokstr=(char*)malloc(size+1);
		text=tokstr;
		if (!tokstr) return;
		*tokstr=0;
		strncat(tokstr,s,size);
	}

	char *c=tokstr;
	
	while (*c)
	{
		switch(*c)
		{
			case '\'':
				if (!ign_quote1 && !quote2)
				{
					if (quote1)
					{
						if (quote1==tokstr && c[1]==0)
						{
							*c=0;
							strcpy(quote1,quote1+1); c--;
							strcpy(c,c+2); c-=2;
						}
						quote1=NULL;
					}
					else
						quote1=c;
				}
				break;

			case '"':
				if (!ign_quote2 && !quote1)
				{
					if (quote2)
					{
						if (quote2==tokstr && c[1]==0)
						{
							*c=0;
							strcpy(quote2,quote2+1); c--;
							strcpy(c,c+2); c-=2;
						}
						quote2=NULL;
					}
					else
						quote2=c;
				}
				break;

			case '\\':
				switch(c[1])
				{
					case '"':
					case '\'':
					case '\\':
						strcpy(c,c+1);
						break;

					case 'n':
						strcpy(c,c+1);
						*c='\n';
						break;

					case 't':
						strcpy(c,c+1);
						*c='\t';
						break;

					case 'r':
						strcpy(c,c+1);
						*c='\r';
						break;

					default:
						strcpy(c,c+2);
						c--;
						break;
				}
				break;
		}
		c++;
	}
}

token::operator const char *() const
{
	return this && tokstr ? tokstr : "";
}
token::operator char *() const
{
	return this && tokstr ? tokstr : (char*)"";
}

int token::isempty(void)
{
	return !tokstr || !*tokstr;
}

const char *token::tokenize(char *delim,int ignbslash) const
{
	if (!this || !tokstr) return "";
	if (return_buf)
	{
		return return_buf;
	}

	((token *)this)->return_buf=(char*)malloc(strlen(tokstr?tokstr:(char*)"")*2+3);
	if (!return_buf) return "";
	
	char *c=tokstr,
		 *d=return_buf;
	
	*(d++)='"';
	
	int use_quotes=0;
	
	while (*c)
	{
		switch(*c)
		{
			case '"':
			case '\\':
				if (!ignbslash)
					*(d++)='\\';
				*(d++)=*c;
				break;
			
			case '\r':
				if (!ignbslash)
					*(d++)='\\';
				*(d++)='r';
				break;
				
			case '\n':
				if (!ignbslash)
					*(d++)='\\';
				*(d++)='n';
				break;

			case '\t':
				if (!ignbslash)
					*(d++)='\\';
				*(d++)='t';
				break;
			
			default:
				if (delim && !use_quotes && strchr(delim, *c))
				{
					use_quotes=1;
				}
				*(d++)=*c;
		}
		c++;
	}
	if (use_quotes || return_buf+1==d)
	{
		*(d++)='"';
		*d=0;
	}
	else
	{
		*d=0;
		strcpy(return_buf,return_buf+1);
	}
	return return_buf;
}

token& token::reduce(char *rdc)
{
	if (tokstr && rdc)
	{
		char *c;
		
		for (c=tokstr; *c; c++)
		{
			if (strchr(rdc,*c))
			{
				char *c2=c+1;
				while (*c2 && strchr(rdc,*c2)) c2++;			
				if (c2>c+1)	strcpy(c+1,c2);
				*c=rdc[0];
			}
		}
	}
	return *this;
}

tokenizer::tokenizer(const char *str, char *dlm, char *rdc) : dlist()
{
	ign_quote1=1;
	ign_quote2=0;
	delim = dlm ? strdup(dlm) : (char*)NULL;
	reduc = rdc ? strdup(rdc) : (char*)NULL;
	return_buf = NULL;
	tokenize((char*)str);
}

tokenizer::~tokenizer()
{
	if (delim) free(delim);
	if (reduc) free(reduc);
	if (return_buf) free(return_buf);
	removeall();
}

void tokenizer::tokenize(char *s)
{
	if (!s) return;

	if (!*s)
	{
		token *t=new token(s);
		if (t) AddTail(t);
		return;
	}
	while (*s)
	{
		if (reduc && strchr(reduc,*s))
		{
			s++;
			continue;
		}
		char *start=s;
		int quote1=0;
		int quote2=0;
		while (*s)
		{
			if (!quote1 && !quote2 && (!delim || strchr(delim,*s)))
				break;

			switch(*s)
			{
				case '\\':
					if (*(s+1))
						s++;
					break;
				case '\'':
					if (!ign_quote1 && !quote2) quote1=1-quote1;
					break;

				case '"':
					if (!ign_quote2 && !quote1) quote2=1-quote2;
					break;
			}
			s++;
		}
		char *end=s-1;
		if (reduc)
		{
			while (end>start && strchr(reduc,*end))
			{
				end--;
			}
		}
		token *t=new token(start,end-start+1);
		if (t) AddTail(t);
		if (delim && *s)
		{
			s++;
		}

	}
}

tokenizer& tokenizer::removeall(void)
{
	token *t;
	while (t=(token*)RemTail())
	{
		delete t;
	}
	return *this;
}

token *tokenizer::get(int i)
{
	return (token*)Find(i+1);
}
token *tokenizer::find(char *t)
{
	return (token*)FindText(t);
}

tokenizer& tokenizer::set(int i, token* t)
{
	return set(i,*t);
}

int tokenizer::count(void)
{
	return Count();
}

tokenizer& tokenizer::set(int i, const token& t)
{
	if (i>=count())
	{
		count(i+1);
	}
	*get(i)=(token&)t;
	return *this;
}

tokenizer& tokenizer::remove(int i)
{
	Remove(get(i));
	return *this;
}
tokenizer& tokenizer::remove(token *t)
{
	Remove(t);
	return *this;
}
tokenizer& tokenizer::removeempty(void)
{
	int i;
	for (i=0;i<count();)
	{
		if (get(i)->isempty())
		{
			remove(i);
		}
		else i++;
	}
	return *this;
}
tokenizer& tokenizer::insert(token* t, int i)
{
	if (i<count())
	{
		Insert(t,get(i));
	}
	else
	{
		AddTail(t);
	}
	return *this;
}
tokenizer& tokenizer::insert(const token& t, int i)
{
	return insert((token *)&t,i);
}
tokenizer& tokenizer::append(token* t)
{
	AddTail(t);
	return *this;
}
tokenizer& tokenizer::append(const token& t)
{
	return append((token *)&t);
}

char *tokenizer::list(char *devider,char *delim2,int ign_bslash)
{
	if (return_buf)
	{
		free(return_buf);
	}
	int chs=0;
	int i;
	int l=devider?strlen(devider):1;
	
	for (i=count()-1; i>=0; i--)
	{
		chs+=strlen(get(i)->tokenize(delim2?delim2:delim,ign_bslash))+l;
	}
	if (!(return_buf=(char*)malloc(chs)))
	{
		return (char*)"";
	}
	char *d=return_buf;
	for (i=0;i<count();i++)
	{
		const char *v=get(i)->tokenize(delim2?delim2:delim,ign_bslash);
		strcpy(d,v);
		d+=strlen(v);
		if (i<count()-1)
		{
			if (devider)
			{
				strcpy(d,devider);
				d+=l;
			}
			else if (delim)
			{
				*(d++)=delim[0];
			}
		}
	}
	*d=0;
	return return_buf;
}

tokenizer& tokenizer::string(char *s)
{
	removeall();
	tokenize(s);
	return *this;
}

char *tokenizer::string(void)
{
		return list();
}

tokenizer& tokenizer::reduce(void)
{
	int i;
	for (i=0; i<count(); i++)
	{
		get(i)->reduce(reduc);
	}
	return *this;
}

tokenizer& tokenizer::reduce(int i)
{
	token *t=get(i);
	if (t)
	{
		get(i)->reduce(reduc);
	}
	return *this;
}

token &tokenizer::operator[](int i)
{
	static token empty;
	token *t=get(i);
	return t?*t:empty;
}
	
tokenizer& tokenizer::count(int i)
{
	if (i>count())
	{ // add
		int j;
		for (j=count();j<i;j++)
		{
			token *t=new token("");
			if (t) AddTail(t);
		}
	}
	else if (i<count())
	{ // rem
		token *t;
		while (count()>i && (t=(token*)RemTail()))
		{
			delete t;
		}
	}
	return *this;
}

void tokenizer::debugOutput(void)
{
	printf("Count: %d\n",count());
	for(int i=0;i<count();i++)
	{
		printf("%d: *%s*\n",i,(char*)*get(i));
	}
}


rextokenizer::rextokenizer(const char *str, const char *rex, int srch)
	:tokenizer(NULL,NULL,NULL)
{
	search=srch;
	xc_re_syntax_options=RE_SYNTAX_POSIX_EXTENDED;
	pattern.translate=NULL;
	pattern.fastmap=0;
	pattern.buffer=NULL;
	pattern.allocated=0;
	
	xc_re_compile_pattern(rex,strlen(rex),&pattern);
	
	tokenize((char*)str);
}
rextokenizer::~rextokenizer()
{
	xc_regfree(&pattern);
}
void rextokenizer::tokenize(char *s)
{
	struct xc_re_registers r;
	r.num_regs=0;
	r.start=NULL;
	r.end=NULL;
	int l=strlen(s);
	if ((search && xc_re_search(&pattern,s,l,0,l,&r)>0) ||
		(!search && xc_re_match(&pattern,s,l,0,&r)>0))
	{
		int i;
		for (i=1;i<r.num_regs;i++)
		{
			if (r.start[i]>=0 || r.end[i]>=0)
			{
				token *t=new token(s+r.start[i],r.end[i]-r.start[i]);
				if (t) AddTail(t);
			}
		}
	}
}
