/* 
 * This file is part of XCmail and is subject to the 
 * XCmail license agreement
 */

/* $Id: stringTokenizer.h,v 2.23 2000/07/23 19:14:04 schmitzj Exp $ */

#ifndef STRING_TOKENIZER_H
#define STRING_TOKENIZER_H

#include "configure.h"
#include "xc_regex.h"

#include <Xclasses/Xclasses.h>

class token : public node
{
	int ign_quote1:1,
		ign_quote2:1;
	char *tokstr;
	char *return_buf;
	
	void set_token(char *s, int size=-1);

public:
	operator const char *() const;
	operator char *() const;
	token(char *chararray, int size);
	token(char *string=NULL);
	token(token&);
	
	virtual ~token();

	int isempty(void);

	token& operator=(token &);
	token& operator=(char *);

	const char *tokenize(char *delim=NULL,int ignbslash=0) const;
	token& reduce(char *rdc);
};

class tokenizer : public dlist
{
	int ign_quote1:1,
		ign_quote2:1;
	char *delim;
	char *reduc;
	char *return_buf;

	virtual void tokenize(char *);
	
public:
	tokenizer(const char *str=NULL, char *dlm=" \t", char *rdc=" \t");
	virtual ~tokenizer();
	
	token *get(int);
	token *find(char *);
	tokenizer& set(int, token*);
	tokenizer& set(int, const token&);

	tokenizer& remove(int);
	tokenizer& remove(token *);
	tokenizer& removeempty(void);
	tokenizer& removeall(void);
	tokenizer& insert(token*, int); 		// before
	tokenizer& insert(const token&, int);	// before
	tokenizer& append(token*);
	tokenizer& append(const token&);

	char *list(char *devider=NULL,char *delim=NULL,int ign_bslash=0);
		// make string, use DEVIDER between tokens instead of 1st DLM,
		// DELIM is new delimiter to make TOKEN eg. use quotes if space included
		// ign_bslash disables adding of \ before quotes, linefeets, etc.
	char *string(void);
	
	tokenizer& string(char *);

	tokenizer& reduce(void);		// to 1st char
	tokenizer& reduce(int);	
	
	token &operator[](int i);

	int count(void);
	tokenizer& count(int);

	void debugOutput(void);
};

class rextokenizer : public tokenizer
{
	int search;
	struct xc_re_pattern_buffer pattern;
	virtual void tokenize(char *);
public:
	rextokenizer(const char *str=NULL, const char *rex="", int srch=0);
	virtual ~rextokenizer();
};

#endif
