/* $Id: xbmread.c,v 2.1 1999/01/15 21:16:41 schmitzj Exp $ */


#include "xface.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define MAX_LINE 500
static int ReadBitmapStream(FILE *stream, int *widthP, int *heightP, char **dataP)
{
    char line[MAX_LINE]="", name_and_type[MAX_LINE]="";
    char* ptr=NULL;
    char* t=NULL;
    int version10=0, raster_length=0, v=0;
    int bytes=0, bytes_per_line=0, padding=0;
    int c1=0, c2=0, value1=0, value2=0;
    int hex_table[256];

    *widthP = *heightP = -1;
	*dataP=NULL;

    for ( ; ; )
	{
	if ( fgets( line, MAX_LINE, stream ) == NULL )
	    return 1;
	if ( strlen( line ) == MAX_LINE - 1 )
	    return 1;

	if ( sscanf( line, "#define %s %d", name_and_type, &v ) == 2 )
	    {
	    if ( ( t = rindex( name_and_type, '_' ) ) == NULL )
			t = name_and_type;
	    else
			++t;
	    if ( ! strcmp( "width", t ) )
			*widthP = v;
	    else if ( ! strcmp( "height", t ) )
			*heightP = v;
	    continue;
	    }
	
	if ( sscanf( line, "static short %s = {", name_and_type ) == 1 ) /* } */
	    {
	    version10 = 1;
	    break;
	    }
	if ( sscanf( line, "static char %s = {", name_and_type ) == 1 ) /* } */
	    {
	    version10 = 0;
	    break;
	    }
	}
 
    if ( *widthP == -1 )
	    return 1;
    if ( *heightP == -1 )
	    return 1;

    padding = 0;
    if ( ((*widthP % 16) >= 1) && ((*widthP % 16) <= 8) && version10 )
		padding = 1;

    bytes_per_line = (*widthP+7)/8 + padding;
    
    raster_length =  bytes_per_line * *heightP;
    *dataP = (char*) malloc( raster_length );
    if ( *dataP == (char*) 0 )
	    return 1;

    /* Initialize hex_table. */
    for ( c1 = 0; c1 < 256; ++c1 )
		hex_table[c1] = 256;
    hex_table['0'] = 0;
    hex_table['1'] = 1;
    hex_table['2'] = 2;
    hex_table['3'] = 3;
    hex_table['4'] = 4;
    hex_table['5'] = 5;
    hex_table['6'] = 6;
    hex_table['7'] = 7;
    hex_table['8'] = 8;
    hex_table['9'] = 9;
    hex_table['A'] = 10;
    hex_table['B'] = 11;
    hex_table['C'] = 12;
    hex_table['D'] = 13;
    hex_table['E'] = 14;
    hex_table['F'] = 15;
    hex_table['a'] = 10;
    hex_table['b'] = 11;
    hex_table['c'] = 12;
    hex_table['d'] = 13;
    hex_table['e'] = 14;
    hex_table['f'] = 15;

    if ( version10 )
	for ( bytes = 0, ptr = *dataP; bytes < raster_length; bytes += 2 )
    {
	    while ( ( c1 = getc( stream ) ) != 'x' )
	        if ( c1 == EOF )
			    return 1;
	    c1 = getc( stream );
	    c2 = getc( stream );
	    if ( c1 == EOF || c2 == EOF )
		    return 1;
	    value1 = ( hex_table[c1] << 4 ) + hex_table[c2];
	    if ( value1 >= 256 )
		    return 1;
	    c1 = getc( stream );
	    c2 = getc( stream );
	    if ( c1 == EOF || c2 == EOF )
		    return 1;
	    value2 = ( hex_table[c1] << 4 ) + hex_table[c2];
	    if ( value2 >= 256 )
		    return 1;
	    *ptr++ = value2;
	    if ( ( ! padding ) || ( ( bytes + 2 ) % bytes_per_line ) )
	        *ptr++ = value1;
    }
    else
	for ( bytes = 0, ptr = *dataP; bytes < raster_length; ++bytes )
    {
	    /*
	    ** Skip until digit is found.
	    */
	    for ( ; ; )
		{
			c1 = getc( stream );
			if ( c1 == EOF )
			    return 1;
			value1 = hex_table[c1];
			if ( value1 != 256 )
			    break;
		}
	    /*
	    ** Loop on digits.
	    */
	    for ( ; ; )
		{
			c2 = getc( stream );
			if ( c2 == EOF )
			    return 1;
			value2 = hex_table[c2];
			if ( value2 != 256 )
		    {
			    value1 = (value1 << 4) | value2;
		    	if ( value1 >= 256 )
				    return 1;
		    }
			else if ( c2 == 'x' || c2 == 'X' )
			    if ( value1 == 0 )
					continue;
			    else return 1;
		        else break;
		}
		*ptr++ = value1;
	}
	return 0;
}
int ReadBitmapFile(char *name, unsigned int *widthP, unsigned int *heightP, unsigned char **dataP)
{
	FILE *f;
	int r=1;
	if (f=fopen(name,"r"))
	{
		r=ReadBitmapStream(f,(int*)widthP,(int*)heightP,(char**)dataP);
		if (r) free(*dataP);
		fclose(f);
	}
	return r;
}
