/* 
 * This file is part of XCmail and is subject to the 
 * XCmail license agreement
 */

/* $Id: xfacegg.cxx,v 2.1 1999/01/07 16:09:06 beckerr Exp $ */


#include "configure.h"

#include <string.h>
#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif
#include "mailer.h"
#include "xfacegg.h"

xfacegg::xfacegg(void) : Ddraw(this), gadget(), icon(), ishadow()
{
	Name("xfacegg");
	BorderSize(1);
	pix=0;
	tmp_str=NULL;
}
xfacegg::~xfacegg()
{
	if (pix)
	{
		XFreePixmap(display(),pix);
		pix=0;
	}
}
void xfacegg::XFace(char *str)
{
	if (pix)
	{
		XFreePixmap(display(),pix);
		pix=0;
	}
    if (!str || !strlen(str))
    {
    	draw();
		return;
    }
	if (!win)
	{
		tmp_str=str;
		return;
	}
	tmp_str=NULL;
	int len=strlen(str);

	// Make sure that buffer is big enough.
	int buffer_size=len+4096;
	char *buffer=new char[buffer_size];
    
	// Put header data in the buffer.
    strcpy(buffer, str);
    
    /*
     *  Uncompress the face.
    */

    if (uncompface(buffer) < 0)
    {
    	delete [] buffer;
		return;
    }
	int i,val,j;
	char *p=buffer;
	unsigned char bits[512];
	for(i=0;i<144 && p;i++)
	{
		if (!*p)
		{
			delete [] buffer;
			return;
		}
	
		// Skip leading spaces.
		while(*p==' ' || *p=='\t' || *p=='\n')
		{
			p++;
		}
		sscanf(p,"0x%x",&val);
		bits[i*2+1]=0;
		for(j=0;j<8;j++)
		{
			bits[i*2+1]|=((val&(1<<j))>>j)<<(7-j);
		}
		val=val>>8;
		bits[i*2]=0;
		for(j=0;j<8;j++)
		{
			bits[i*2]|=((val&(1<<j))>>j)<<(7-j);
		}
	
		// Skip just past the next comma.
		p=strchr(p, ',');
		p++;
	}
	// Take the image data and create a bitmap.
	delete [] buffer;
    
    pix=XCreateBitmapFromData(display(),win,(char*)bits,48,48);
	draw();
}
void xfacegg::XFaceStr(char *str)
{
	if (pix)
	{
		XFreePixmap(display(),pix);
		pix=0;
	}
    if (!str || !strlen(str))
    {
    	draw();
		return;
    }
	if (!win)
	{
		tmp_str=str;
		return;
	}
	tmp_str=NULL;

	int i,val,j;
	char *p=str;
	unsigned char bits[512];
	for(i=0;i<144 && p;i++)
	{
		if (!*p)
		{
			return;
		}
	
		// Skip leading spaces.
		while(*p==' ' || *p=='\t' || *p=='\n')
		{
			p++;
		}
		sscanf(p,"0x%x",&val);
		bits[i*2+1]=0;
		for(j=0;j<8;j++)
		{
			bits[i*2+1]|=((val&(1<<j))>>j)<<(7-j);
		}
		val=val>>8;
		bits[i*2]=0;
		for(j=0;j<8;j++)
		{
			bits[i*2]|=((val&(1<<j))>>j)<<(7-j);
		}
	
		// Skip just past the next comma.
		p=strchr(p, ',');
		p++;
	}
	// Take the image data and create a bitmap.
    
    pix=XCreateBitmapFromData(display(),win,(char*)bits,48,48);
	draw();
}
int xfacegg::Create(void)
{ 
	Background(col_background());
        
	if (gadget::Create())
	{
		Mode(GM_ONEEXPOSE);
		icon.Size(48,48);
		icon.CreatePixmap();
		ishadow.Size(48,48);
		ishadow.CreatePixmap(1);
		if (tmp_str) XFace(tmp_str);
		return 1;
	} 
	return 0;
}
void xfacegg::Free(void)
{
	if (pix) XFreePixmap(display(),pix);
	pix=0;
	gadget::Free();
	ishadow.Free();
	icon.Free();
}
void xfacegg::GExpose(XEvent *event)
{
	if (event->type!=NoExpose && event->type!=GraphicsExpose)
	{
		draw();
	}
}
void xfacegg::GSetLimits(void)
{
	minwidth=RealSize()*2+48;
	minheight=RealSize()*2+48;
	maxwidth=RealSize()*2+48;
	maxheight=RealSize()*2+48;
}

void xfacegg::draw(void)
{
	if (win)
	{
		if (pix)
		{
			borderDown(this,0,0,width,height);
			XSetForeground(display(),gc,col_black());
			XSetBackground(display(),gc,col_shinebackground());
//			XFillRectangle(display(),win,gc,RealSize(),RealSize(),48,48);
			XCopyPlane(display(),pix,win,gc,0,0,48,48,RealSize(),RealSize(),1);
		}
		else
		{
			borderDown(this,0,0,width,height);
			XSetForeground(display(),gc,col_background());
			XFillRectangle(display(),win,gc,RealSize(),RealSize(),48,48);
			f_xlogo.draw_normal(this,RealSize(),RealSize(),48,48);
		}

		if (pix)
		{
			XSetForeground(display(),icon.gc(),col_shadow());
			XSetBackground(display(),icon.gc(),col_shinebackground());
			XCopyPlane(display(),pix,icon.pixmap(),icon.gc(),0,0,48,48,0,0,1);

			XSetForeground(display(),ishadow.gc(),col_shadow());
			XSetBackground(display(),ishadow.gc(),col_shinebackground());
			XCopyPlane(display(),pix,ishadow.pixmap(),ishadow.gc(),0,0,48,48,0,0,1);
		}
		else
		{
			XSetForeground(display(),icon.gc(),col_background());
			XFillRectangle(display(),icon.pixmap(),icon.gc(),0,0,48,48);

			XSetForeground(display(),ishadow.gc(),col_background());
			XFillRectangle(display(),ishadow.pixmap(),ishadow.gc(),0,0,48,48);
		}
		icon.Font("-*-*-medium-r-*-*-12-*-*-*-m-*-*-*");
		ishadow.Font("-*-*-medium-r-*-*-12-*-*-*-m-*-*-*");
		int l=icon.StringLength(PROGRAMNAME);
		icon.Foreground(col_black());
		for(int x=-1;x<=1;x++)
			for(int y=-1;y<=1;y++)
				icon.DrawString((48-l)/2+x,48-icon.FontHeight()+icon.FontAscent()+y,PROGRAMNAME);
		icon.Foreground(col_white());
		icon.DrawString((48-l)/2,48-icon.FontHeight()+icon.FontAscent(),PROGRAMNAME);

		l=ishadow.StringLength(PROGRAMNAME);
		ishadow.Foreground(col_black());
		for(int x=-1;x<=1;x++)
			for(int y=-1;y<=1;y++)
				ishadow.DrawString((48-l)/2+x,48-ishadow.FontHeight()+ishadow.FontAscent()+y,PROGRAMNAME);
		ishadow.Foreground(col_white());
		ishadow.DrawString((48-l)/2,48-ishadow.FontHeight()+ishadow.FontAscent(),PROGRAMNAME);
	}
}
